&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v7r11
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Method-Library 
/*------------------------------------------------------------------------
Description:    Method Library Template
Arguments:      <none>
Comments:       
    Builds a treeview from a temp-table. Just need a frame to parent the treeview into
                
    Expects that the temp-table is already fully built (doesn't currently support lazy loading of the data)
    
    The node id can be any datatype - normally expecting to be a character or integer.
    Note the variable names assume character (e.g. cNodeId)
    
    Use the following preprocessors if necessary in the parent procedure to override the defaults
    
&GLOBAL-DEFINE ttNode <Temp-Table>
&GLOBAL-DEFINE id_dataType <id field DataType>    
&GLOBAL-DEFINE id <id field>
&GLOBAL-DEFINE parentId <parent id field>
&GLOBAL-DEFINE nodeText <field to be used as node text>
&GLOBAL-DEFINE fTreeViewParent <frame to place the treeview in>
    
    NB.  The following fields need to be added to the temp-table:
    
    FIELD nodeTextHandle AS HANDLE
    FIELD nodeOpenerImageHandle AS HANDLE
    FIELD nodeFrameHandle AS HANDLE
    
    potentially, these could be moved to a separate hash-table, and then this method library could be
    used directly on DB tables.  The only link we really need is from the ID to the nodeTextHandle
    anyway.
    
    Enhancements:
        Add leaf node support?  E.g. specify logic to determine if a leaf node. 
            Currently empty parents are by default shown as leaf documents.
          
------------------------------------------------------------------------*/
/*          This .W file was created with the Progress UIB.             */
/*----------------------------------------------------------------------*/
/* ***************************  Definitions  ************************** */
/* Parameters Definitions ---                                           */
&SCOPED-DEFINE TextSpacingPixels 18
&SCOPED-DEFINE LevelOffsetPixels 20
/* Preprocessor Defaults */
&IF DEFINED(ttNode) = 0 &THEN 
    &SCOPED-DEFINE ttNode ttNode
&ENDIF
&IF DEFINED(id_dataType) = 0 &THEN 
    &SCOPED-DEFINE id_dataType CHARACTER
&ENDIF
&IF DEFINED(id) = 0 &THEN 
    &SCOPED-DEFINE id id
&ENDIF
&IF DEFINED(parentId) = 0 &THEN 
    &SCOPED-DEFINE parentId parentId
&ENDIF
&IF DEFINED(nodeText) = 0 &THEN 
    &SCOPED-DEFINE nodeText nodeText
&ENDIF
&IF DEFINED(fTreeViewParent) = 0 &THEN 
    &SCOPED-DEFINE fTreeViewParent fTreeViewParent
&ENDIF
/* CAST_id function - converts from a CHARACTER to whatever the id datatype is. */
/* If Progress had a "CHARACTER" function instead of "STRING", then this wouldn't be necessary - would always use the datatype as a CAST */
&IF '{&id_dataType}' = 'CHARACTER'
&THEN
    &SCOPED-DEFINE CAST_id
&ELSE
    &SCOPED-DEFINE CAST_id {&id_dataType}
&ENDIF
DEFINE VARIABLE LcTreeVirtualRootNodeId AS {&id_dataType} NO-UNDO. /* The virtual parent of all the displayed nodes - normally <blank>, associated with the <root> frame */
DEFINE VARIABLE LhTreeSelectedText AS HANDLE NO-UNDO.
DEFINE VARIABLE LhTreeFocusPixelButton AS HANDLE NO-UNDO. /* Where the focus resides when the treeview is in use */
DEFINE VARIABLE LhTreeFocusSwapPixelButton AS HANDLE NO-UNDO. /* Used in "trick" to force the frame to reposition when using cursor to move selected row */
    
/* Check to see if this is included in something else, if not, define dummy frame and temp-table */
&IF DEFINED(Procedure-Type) = 0 &THEN
DEFINE TEMP-TABLE {&ttNode}
    FIELD {&id} AS {&id_dataType}
    FIELD {&parentId} AS {&id_dataType}
    FIELD {&nodeText} AS CHARACTER
    FIELD nodeTextHandle AS HANDLE
    FIELD nodeOpenerImageHandle AS HANDLE
    FIELD nodeFrameHandle AS HANDLE
    INDEX idx-1 parentId nodeText
    .
DEFINE FRAME {&fTreeViewParent}.
&ENDIF
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 
/* ********************  Preprocessor Definitions  ******************** */

/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME

/* *********************** Procedure Settings ************************ */
&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Method-Library
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: INCLUDE-ONLY
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS
/* *************************  Create Window  ************************** */
&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Method-Library ASSIGN
         HEIGHT             = 9.81
         WIDTH              = 38.72.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _INCLUDED-LIB Method-Library 
/* ************************* Included-Libraries *********************** */
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

 

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Method-Library 

/* ***************************  Main Block  *************************** */
/* Ensure focus in {&fTreeViewParent} always go to the button - so user can tab off */
ON ENTRY OF FRAME {&fTreeViewParent} ANYWHERE DO:
    
    IF VALID-HANDLE(LhTreeFocusPixelButton) /* AND FOCUS <> LhTreeFocusPixelButton */
    THEN APPLY 'ENTRY' TO LhTreeFocusPixelButton.
END.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

/* **********************  Internal Procedures  *********************** */
&IF DEFINED(EXCLUDE-PTreeCreateNodeFrame) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeCreateNodeFrame Method-Library 
PROCEDURE PTreeCreateNodeFrame :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       Creating the root refreshes the whole tree.
  
               Coding note:
               Use Widget:NAME to point to the node id.
                   Widget:PRIVATE-DATA used to point to another widget handle
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER IcNodeId AS {&id_dataType} NO-UNDO.
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    
    DEFINE BUFFER child-{&ttNode} FOR {&ttNode}.
    DEFINE BUFFER grandchild-{&ttNode} FOR {&ttNode}.
    
    DEFINE VARIABLE hFrame AS HANDLE NO-UNDO.
    DEFINE VARIABLE iNodeCount AS INTEGER NO-UNDO.
    DEFINE VARIABLE iTextIdx AS INTEGER NO-UNDO INITIAL 0. /* Zero based */
    DEFINE VARIABLE hImage AS HANDLE NO-UNDO.
    DEFINE VARIABLE hLine AS HANDLE NO-UNDO.
    DEFINE VARIABLE hHorizontalLine AS HANDLE NO-UNDO.
    DEFINE VARIABLE cImageName AS CHARACTER NO-UNDO.
    
    DEFINE VARIABLE iMaxTextWidthPixels AS INTEGER NO-UNDO.
    DEFINE VARIABLE cWidgetPoolName AS CHARACTER NO-UNDO. /* possibly more efficient as a var */
    cWidgetPoolName = STRING(THIS-PROCEDURE).
    FIND {&ttNode} WHERE {&ttNode}.{&id} = IcNodeId NO-ERROR.
    FOR EACH child-{&ttNode} WHERE child-{&ttNode}.{&parentId} = IcNodeId:
        iNodeCount = iNodeCount + 1.
        
        iMaxTextWidthPixels = MAXIMUM(iMaxTextWidthPixels,FONT-TABLE:GET-TEXT-WIDTH-PIXELS(" " + child-{&ttNode}.{&nodeText} + " ",10)).
    END.
    IF NOT AVAILABLE({&ttNode})
    THEN DO: /* <Root> node */
        DELETE WIDGET-POOL cWidgetPoolName NO-ERROR.
        LcTreeVirtualRootNodeId = IcNodeId.
        /* Create nothing if no nodes */
        IF iNodeCount = 0
        THEN RETURN.
        CREATE WIDGET-POOL cWidgetPoolName PERSISTENT.
        CREATE FRAME hFrame IN WIDGET-POOL cWidgetPoolName
            ASSIGN SCROLLABLE = FALSE
                   BOX = FALSE
                   FRAME = FRAME {&fTreeViewParent}:HANDLE
                   WIDTH-PIXELS = iMaxTextWidthPixels + 34 /* FRAME fTreeViewParent:WIDTH-PIXELS - 6 */
                   HEIGHT-PIXELS = iNodeCount * {&TextSpacingPixels}
                   /* Y = -1 * iNodeCount * {&TextSpacingPixels} /* Create the new child frame "hidden" until the parent frame has been resized - to prevent flashing */ */
                   HIDDEN = TRUE /* Trying to prevent flashing */
                   FONT = 10
                   NAME = "<ROOT>"
                   TRIGGERS:
                       ON CURSOR-UP ANYWHERE PERSISTENT RUN PTrigger_CursorUp IN THIS-PROCEDURE.
                       ON CURSOR-DOWN ANYWHERE PERSISTENT RUN PTrigger_CursorDown IN THIS-PROCEDURE.
                   END.
        /* Create a dummy button to receive focus when tabbing into the treeview, and from which the cursor up, cursor down and return keys will work */
        CREATE BUTTON LhTreeFocusPixelButton IN WIDGET-POOL cWidgetPoolName
            ASSIGN  /* This was for original static position button - but that didn't help with cursoring off frame 
                    FRAME = FRAME {&fTreeViewParent}:FRAME
                    X = FRAME {&fTreeViewParent}:X
                    Y = FRAME {&fTreeViewParent}:Y
                    */
                    FRAME = FRAME {&fTreeViewParent}:HANDLE
                    X = 0
                    Y = 0
                    
                    WIDTH-PIXELS = 1
                    HEIGHT-PIXELS = 1
                    SENSITIVE = TRUE
                    HIDDEN = FALSE
                    TRIGGERS:
                        ON CURSOR-UP PERSISTENT RUN PTrigger_CursorUp IN THIS-PROCEDURE.
                        ON CURSOR-DOWN PERSISTENT RUN PTrigger_CursorDown IN THIS-PROCEDURE.
                        ON CHOOSE PERSISTENT RUN PTrigger_Return IN THIS-PROCEDURE.
                    END.
         
         /* LhTreeFocusPixelButton:MOVE-AFTER-TAB-ITEM(FRAME {&fTreeViewParent}:HANDLE). - required for static position button */
        
         /* Create another dummy button to swap the focus between - used in trick to move frame when
         using cursors to move out of frame view */
         CREATE BUTTON LhTreeFocusSwapPixelButton IN WIDGET-POOL cWidgetPoolName
            ASSIGN  
                    FRAME = FRAME {&fTreeViewParent}:FRAME
                    X = FRAME {&fTreeViewParent}:X
                    Y = FRAME {&fTreeViewParent}:Y
                    
                    WIDTH-PIXELS = 1
                    HEIGHT-PIXELS = 1
                    HIDDEN = FALSE /* Required for refreshes of tree (hidden otherwise, which stops the trick from working) */
                    SENSITIVE = FALSE /* disabling means it can't be tabbed into. it's only used in trick */
                    .
    END.
    ELSE DO:
        CREATE FRAME hFrame IN WIDGET-POOL cWidgetPoolName
            ASSIGN SCROLLABLE = FALSE
                   /* SCROLLABLE = TRUE */
                   BOX = FALSE
                   FRAME = {&ttNode}.nodeTextHandle:FRAME
                   WIDTH-PIXELS = iMaxTextWidthPixels + 34 /* {&ttNode}.nodeTextHandle:FRAME:WIDTH-PIXELS - ({&LevelOffsetPixels} + 2) /* Not sure why, but need 2 extra pixels now */ */
                   HEIGHT-PIXELS = iNodeCount * {&TextSpacingPixels}
                   FONT = 10
                   HIDDEN = TRUE /* Trying to prevent flashing */
                   NAME = STRING({&ttNode}.{&id})
                   TRIGGERS:
                       ON CURSOR-UP ANYWHERE PERSISTENT RUN PTrigger_CursorUp IN THIS-PROCEDURE.
                       ON CURSOR-DOWN ANYWHERE PERSISTENT RUN PTrigger_CursorDown IN THIS-PROCEDURE.
                   END.
            
        ASSIGN {&ttNode}.nodeFrameHandle = hFrame.
    END.
        
    /* Background line */
    CREATE IMAGE hLine IN WIDGET-POOL cWidgetPoolName
        ASSIGN PARENT = hFrame:BACKGROUND
               X = 4
               Y = (IF AVAILABLE({&ttNode}) THEN 0 ELSE 6)
               HEIGHT-PIXELS = {&TextSpacingPixels} * (iNodeCount - 1) + (IF AVAILABLE({&ttNode}) THEN 6 ELSE 0) + 4.
               .
    hLine:LOAD-IMAGE("graphics/treeview/dottedVerticalLine.png").
    FOR EACH child-{&ttNode} WHERE child-{&ttNode}.{&parentId} = IcNodeId:
        CREATE 
            TEXT child-{&ttNode}.nodeTextHandle IN WIDGET-POOL cWidgetPoolName
            ASSIGN  FORMAT = "x(256)"
                    SCREEN-VALUE = " " + TRIM(child-{&ttNode}.{&nodeText}) + " "
                    FRAME = hFrame
                    X = 34
                    Y = ((iTextIdx) * {&TextSpacingPixels}) + 1
                    SENSITIVE = TRUE
                    WIDTH-PIXELS = FONT-TABLE:GET-TEXT-WIDTH-PIXELS(" " + TRIM(child-{&ttNode}.{&nodeText}) + " ",hFrame:FONT)
                    NAME = STRING(child-{&ttNode}.{&id})
            TRIGGERS:
                ON MOUSE-SELECT-DBLCLICK PERSISTENT RUN PTrigger_TextDblClick IN THIS-PROCEDURE.
                /* ON MOUSE-SELECT-CLICK PERSISTENT RUN PTrigger_TextClick IN THIS-PROCEDURE. */
                ON MOUSE-SELECT-DOWN PERSISTENT RUN PTrigger_TextDown IN THIS-PROCEDURE.
            END.
        CREATE
            IMAGE hImage IN WIDGET-POOL cWidgetPoolName
            ASSIGN FRAME = hFrame
                   X = 17
                   Y = ((iTextIdx) * {&TextSpacingPixels}) + 2
                   SENSITIVE = TRUE
                   PRIVATE-DATA = STRING(child-{&ttNode}.nodeTextHandle)
                TRIGGERS:
                    ON MOUSE-SELECT-DOWN PERSISTENT RUN PTrigger_ImageDown IN THIS-PROCEDURE.
                    ON MOUSE-SELECT-DBLCLICK PERSISTENT RUN PTrigger_ImageDblClick IN THIS-PROCEDURE.
                END.
        
        cImageName = ?.
        cImageName = DYNAMIC-FUNCTION("FcNodeImageName" IN THIS-PROCEDURE, BUFFER child-{&ttNode}) NO-ERROR.
        IF cImageName = ?
        THEN DO:
            IF CAN-FIND(FIRST grandchild-{&ttNode} WHERE grandchild-{&ttNode}.{&parentId} = child-{&ttNode}.{&id})
            THEN cImageName = "folder".
            ELSE cImageName = "doc".
        END.
        ELSE cImageName = LC(cImageName).
        hImage:LOAD-IMAGE("graphics/treeview/" + cImageName + ".png").
                
        IF CAN-FIND(FIRST grandchild-{&ttNode} WHERE grandchild-{&ttNode}.{&parentId} = child-{&ttNode}.{&id})
        THEN DO:
            CREATE
                IMAGE child-{&ttNode}.nodeOpenerImageHandle IN WIDGET-POOL cWidgetPoolName
                ASSIGN FRAME = hFrame
                       X = 0
                       Y = ((iTextIdx) * {&TextSpacingPixels}) + 5
                       SENSITIVE = TRUE
                       NAME = STRING(child-{&ttNode}.{&id})
                TRIGGERS:
                    ON MOUSE-SELECT-DOWN PERSISTENT RUN PTrigger_OpenerClick IN THIS-PROCEDURE.
                    ON MOUSE-SELECT-DBLCLICK PERSISTENT RUN PTrigger_OpenerClick IN THIS-PROCEDURE.
                END.
            child-{&ttNode}.nodeOpenerImageHandle:LOAD-IMAGE("graphics/treeview/plus.png").
        END.
        ELSE DO:
            CREATE
                IMAGE hHorizontalLine IN WIDGET-POOL cWidgetPoolName
                ASSIGN FRAME = hFrame
                       X = 3
                       Y = ((iTextIdx) * {&TextSpacingPixels}) + 10
                       SENSITIVE = TRUE
                TRIGGERS:
                    /* TODO */
                END.
            
            hHorizontalLine:LOAD-IMAGE("graphics/treeview/dottedHorizontalLine.png").
        END.
        
        iTextIdx = iTextIdx + 1.
    END.
    IF AVAILABLE({&ttNode}) /* Not root - so resize parent */
    THEN DO:
        RUN PTreeResizeFrame(hFrame:FRAME,
                         {&ttNode}.nodeTextHandle,
                         {&ttNode}.nodeFrameHandle:HEIGHT-PIXELS,
                         {&ttNode}.nodeFrameHandle:WIDTH-PIXELS).
        ASSIGN hFrame:X = {&LevelOffsetPixels}
               hFrame:Y = {&ttNode}.nodeTextHandle:Y + ({&TextSpacingPixels} - 1) /* Frames just don't seem to overlay on another */
               hFrame:VISIBLE = TRUE
               hFrame:SENSITIVE = TRUE
               .
    END.
    ELSE DO:
        /* Force re-assign !! (Progress issue) otherwise get errors about frame not fitting */
        FRAME {&fTreeViewParent}:HEIGHT-PIXELS = FRAME {&fTreeViewParent}:HEIGHT-PIXELS.
        FRAME {&fTreeViewParent}:WIDTH-PIXELS = FRAME {&fTreeViewParent}:WIDTH-PIXELS.
        FRAME {&fTreeViewParent}:VIRTUAL-HEIGHT-PIXELS = MAXIMUM(FRAME {&fTreeViewParent}:HEIGHT-PIXELS,hFrame:HEIGHT-PIXELS + 2).
        FRAME {&fTreeViewParent}:VIRTUAL-WIDTH-PIXELS = MAXIMUM(FRAME {&fTreeViewParent}:WIDTH-PIXELS,hFrame:WIDTH-PIXELS + 7).
                
        ASSIGN hFrame:X = 4
               hFrame:Y = 0
               hFrame:VISIBLE = TRUE
               hFrame:SENSITIVE = TRUE
               .
    END.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTreeExpandNode) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeExpandNode Method-Library 
PROCEDURE PTreeExpandNode :
/*------------------------------------------------------------------------------
  Purpose:     Ensure that a specific parent node has been expanded.
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER IcNodeId AS {&id_dataType} NO-UNDO.
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    FIND {&ttNode} WHERE {&ttNode}.{&id} = IcNodeId NO-LOCK.
   
    /* Is the node frame already visible - then leave */
    IF VALID-HANDLE({&ttNode}.nodeFrameHandle) AND {&ttNode}.nodeFrameHandle:VISIBLE THEN LEAVE.
    ELSE DO:
        IF NOT VALID-HANDLE({&ttNode}.nodeTextHandle) OR NOT {&ttNode}.nodeTextHandle:VISIBLE
        THEN RUN PTreeExpandNode({&ttNode}.{&parentId}). /* Recurse up */
        
        RUN PTreeToggleNode(BUFFER {&ttNode}).
    END.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTreeMoveFocusPixelButton) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeMoveFocusPixelButton Method-Library 
PROCEDURE PTreeMoveFocusPixelButton :
/*------------------------------------------------------------------------------
  Purpose:     Keep the focus button next to the widget being used -
               this prevents strange issues when tabbing onto the treeview,
               when opening/closing branches when the selected row goes off the screen
               or when using the scrollbar (if it appears).
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    
    DEFINE INPUT PARAMETER IhWidget AS HANDLE NO-UNDO.
    DEFINE VARIABLE iY AS INTEGER NO-UNDO.
    DEFINE VARIABLE hWidget AS HANDLE NO-UNDO.
    /* Calculate the Y value of the widget with respect to the container frame */
    
    hWidget = IhWidget.
    DO WHILE hWidget <> FRAME {&fTreeViewParent}:HANDLE:
        iY = iY + hWidget:Y.
        hWidget = hWidget:FRAME.
    END.
    
    IF LhTreeFocusPixelButton:Y > iY /* Moving up, position button on the high side */
    THEN LhTreeFocusPixelButton:Y = iY.
    ELSE LhTreeFocusPixelButton:Y = iY + 14. /* Moving down, position button on the low side */
        
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTreeResizeFrame) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeResizeFrame Method-Library 
PROCEDURE PTreeResizeFrame :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  
  Notes:       Width - we just have to ensure we can still fit the width in
                - we just get bigger for now, we would have to check all the child widths to see if we can shrink.
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER IhFrame AS HANDLE NO-UNDO.
    DEFINE INPUT PARAMETER IhChild AS HANDLE NO-UNDO.
    DEFINE INPUT PARAMETER iResizeHeightPixels AS INTEGER NO-UNDO.
    DEFINE INPUT PARAMETER iChildFrameWidthPixels AS INTEGER NO-UNDO.
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    DEFINE VARIABLE hWidget AS HANDLE NO-UNDO.
    DEFINE VARIABLE lLineResized AS LOGICAL     NO-UNDO.
    /* Special case is the Container fTreeViewParent frame.
       This has 2 children:
          The <root> frame which parents all other frames
          The focus pixel button.
          
       We assume that the focus pixel button has already been moved in triggers to a position where it can't be forced off the container frame.
       Therefore the only thing we need to do for fTreeViewParent is ensure that the virual height is set correctly
    */
    IF IhFrame = FRAME {&fTreeViewParent}:HANDLE /* Container frame */
    THEN DO:
        IF iResizeHeightPixels > 0 /* Growing */
        THEN ASSIGN IhFrame:VIRTUAL-HEIGHT-PIXELS = MAXIMUM(IhFrame:HEIGHT-PIXELS,IhChild:HEIGHT-PIXELS + iResizeHeightPixels + 7)
                    IhFrame:VIRTUAL-WIDTH-PIXELS = MAXIMUM(IhFrame:WIDTH-PIXELS,iChildFrameWidthPixels + {&LevelOffsetPixels} + 2)
                    .
        ELSE IhFrame:VIRTUAL-HEIGHT-PIXELS = MAXIMUM(IhFrame:HEIGHT-PIXELS,IhChild:HEIGHT-PIXELS + 7). /* Child has already been shrunk */
    END.
          
    ELSE DO: /* Not container frame - i.e. all other frames */
        IF iResizeHeightPixels > 0 
        THEN DO: /* Getting bigger - Resize parents first */
            RUN PTreeResizeFrame(IhFrame:FRAME, IhFrame, iResizeHeightPixels, MAXIMUM(IhFrame:WIDTH-PIXELS,iChildFrameWidthPixels + {&LevelOffsetPixels})).
            IhFrame:HEIGHT-PIXELS = IhFrame:HEIGHT-PIXELS + iResizeHeightPixels.
            IhFrame:WIDTH-PIXELS = MAXIMUM(IhFrame:WIDTH-PIXELS,iChildFrameWidthPixels + {&LevelOffsetPixels}).
        END.
       
        /* move everything below the child up or down */
        hWidget = IhFrame:FIRST-CHILD. /* Field group */
        hWidget = hWidget:FIRST-CHILD. /* Proper widget - feckin' progress */
        
        DO WHILE VALID-HANDLE(hWidget):
    
            IF hWidget:Y > IhChild:Y + 5 /* + 5 to ignore the openers */
            /* But not the frame */
            AND NOT (hWidget:TYPE = "FRAME" AND hWidget:Y = IhChild:Y + ({&TextSpacingPixels} - 1)) /* because of frame overlay issue */
            THEN DO: 
                hWidget:Y = hWidget:Y + iResizeHeightPixels.
                
                /* Resize line if necessary */
                IF NOT lLineResized
                THEN DO:
                    IF VALID-HANDLE(IhFrame:BACKGROUND:FIRST-CHILD)
                    THEN DO:
                        IhFrame:BACKGROUND:FIRST-CHILD:HEIGHT-PIXELS = IhFrame:BACKGROUND:FIRST-CHILD:HEIGHT-PIXELS + iResizeHeightPixels.
                        IhFrame:BACKGROUND:FIRST-CHILD:LOAD-IMAGE("graphics/treeview/dottedVerticalLine.png").
                    END.
    
                    lLineResized = TRUE.
                END.
            END.
            hWidget = hWidget:NEXT-SIBLING.
        END.
            
        IF iResizeHeightPixels < 0 
        THEN DO: /* Getting Smaller - children have been resized first, so next shrink this frame's height and then its parent's */
            IhFrame:HEIGHT-PIXELS = IhFrame:HEIGHT-PIXELS + iResizeHeightPixels.
            RUN PTreeResizeFrame(IhFrame:FRAME, IhFrame, iResizeHeightPixels,MAXIMUM(IhFrame:WIDTH-PIXELS,iChildFrameWidthPixels + {&LevelOffsetPixels})).
        END.
    END.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTreeSelectNextNode) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeSelectNextNode Method-Library 
PROCEDURE PTreeSelectNextNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    
    DEFINE INPUT PARAMETER IlForwards AS LOGICAL NO-UNDO.
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    DEFINE VARIABLE cNodeId AS {&id_dataType}   NO-UNDO.
    DEFINE VARIABLE cNextNodeId AS {&id_dataType}   NO-UNDO INITIAL ?.
    DEFINE VARIABLE cParentNodeId AS {&id_dataType} NO-UNDO.
    IF NOT VALID-HANDLE(LhTreeSelectedText)
    THEN DO:
        FIND FIRST {&ttNode} WHERE {&ttNode}.{&parentId} = LcTreeVirtualRootNodeId NO-ERROR.
        IF AVAILABLE({&ttNode})
        THEN RUN PTreeSelectNode({&ttNode}.{&id}).
    END.
    ELSE DO:
        cNodeId = {&CAST_id}(LhTreeSelectedText:NAME).
        IF IlForwards
        THEN DO: 
            /* Do we have any children visible? */
            FIND FIRST {&ttNode} WHERE {&ttNode}.{&parentId} = cNodeId NO-ERROR.
    
            IF AVAILABLE({&ttNode})
            AND VALID-HANDLE({&ttNode}.nodeTextHandle)
            AND {&ttNode}.nodeTextHandle:VISIBLE
            THEN cNextNodeId = {&ttNode}.{&id}.
    
            /* else look for next sibling or parent's next sibling */
            ELSE REPEAT WHILE(cNodeId <> LcTreeVirtualRootNodeId):
                FIND {&ttNode} WHERE {&ttNode}.{&id} = cNodeId NO-ERROR.
                cParentNodeId = {&ttNode}.{&parentId}.
                FIND NEXT {&ttNode} WHERE {&ttNode}.{&parentId} = cParentNodeId NO-ERROR.
                IF AVAILABLE {&ttNode} 
                THEN DO:
                    cNextNodeId = {&ttNode}.{&id}.
                    LEAVE.
                END.
                ELSE cNodeId = cParentNodeId.
            END.
        END.
        ELSE DO: /* Backwards */
            /* Do I have a previous sibling? */
            FIND {&ttNode} WHERE {&ttNode}.{&id} = cNodeId NO-ERROR.
            cParentNodeId = {&ttNode}.{&parentId}.
            FIND PREV {&ttNode} WHERE {&ttNode}.{&parentId} = cParentNodeId NO-ERROR.
    
            /* If so, what's its last visible child (or itself)? */
            IF AVAILABLE {&ttNode}
            THEN DO:
                cNodeId = {&ttNode}.{&id}.
    
                REPEAT:
                    FIND LAST {&ttNode} WHERE {&ttNode}.{&parentId} = cNodeId NO-ERROR.
                    IF AVAILABLE({&ttNode})
                    AND VALID-HANDLE({&ttNode}.nodeTextHandle)
                    AND {&ttNode}.nodeTextHandle:VISIBLE
                    THEN cNodeId = {&ttNode}.{&id}.
                    ELSE LEAVE.
                END.
                
                cNextNodeId = cNodeId.
            END.
    
            /* If not, who's my parent? */
            ELSE DO:
                IF cParentNodeId <> LcTreeVirtualRootNodeId
                THEN cNextNodeId = cParentNodeId.
            END.
        END.
    
        IF cNextNodeId <> ?
        THEN RUN PTreeSelectNode(cNextNodeId).
    END.

END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTreeSelectNode) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeSelectNode Method-Library 
PROCEDURE PTreeSelectNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER IcNodeId AS {&id_dataType} NO-UNDO.
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    DEFINE VARIABLE hOriginalFocus AS HANDLE NO-UNDO. /* For focus trick */
    
    /* If node isn't present or visible, then select parent node */
    FIND {&ttNode} WHERE {&ttNode}.{&id} = IcNodeId NO-ERROR.
    IF NOT AVAILABLE {&ttNode} THEN RETURN.
    IF NOT VALID-HANDLE({&ttNode}.nodeTextHandle) OR NOT {&ttNode}.nodeTextHandle:VISIBLE
    THEN RUN PTreeExpandNode({&ttNode}.{&parentId}).
        
    /* By thie time, the nodeTextHandle should be valid, regardless! */
    IF LhTreeSelectedText <> {&ttNode}.nodeTextHandle
    THEN DO:
        IF VALID-HANDLE(LhTreeSelectedText) /* Refresh might have been run */
        THEN DO:
            LhTreeSelectedText:BGCOLOR = ?.
            LhTreeSelectedText:FGCOLOR = ?.
        END.
    
        LhTreeSelectedText = {&ttNode}.nodeTextHandle.
        LhTreeSelectedText:BGCOLOR = 1.
        LhTreeSelectedText:FGCOLOR = 15.
        
        RUN PTreeMoveFocusPixelButton(LhTreeSelectedText).
        
        /* Trick to force the selected text into view (if it isn't already) - by swapping the focus out and into the focus pixel button */
        /* NB, can't find a Progress way of identifying where a frame is scrolled to, nor manipulating the scrolled frame, but this trick works */
        IF LhTreeFocusSwapPixelButton:VISIBLE
        THEN DO:
            hOriginalFocus = FOCUS.
            LhTreeFocusSwapPixelButton:SENSITIVE = TRUE.
            APPLY 'ENTRY' TO LhTreeFocusSwapPixelButton.
            LhTreeFocusSwapPixelButton:SENSITIVE = FALSE.
            APPLY 'ENTRY' TO FRAME {&fTreeViewParent}.
            APPLY 'ENTRY' TO hOriginalFocus.
        END.
                
        RUN PNodeSelected IN THIS-PROCEDURE (INPUT IcNodeId) NO-ERROR.  /* Hook for selecting a node */
    END.
    
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTreeToggleNode) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeToggleNode Method-Library 
PROCEDURE PTreeToggleNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE PARAMETER BUFFER {&ttNode} FOR {&ttNode}.
    
    DEFINE BUFFER child-{&ttNode} FOR {&ttNode}.
    
    /* Move dummy button to toggling node, at start */
    RUN PTreeMoveFocusPixelButton({&ttNode}.nodeTextHandle).
    IF VALID-HANDLE({&ttNode}.nodeFrameHandle)
    THEN DO:
        IF {&ttNode}.nodeFrameHandle:VISIBLE
        THEN DO: /* Hide and shrink */
            {&ttNode}.nodeFrameHandle:HIDDEN = TRUE.
            {&ttNode}.nodeFrameHandle:Y = -1 - {&ttNode}.nodeFrameHandle:HEIGHT-PIXELS.     /* Hide it away */
            IF VALID-HANDLE({&ttNode}.nodeOpenerImageHandle)
            THEN {&ttNode}.nodeOpenerImageHandle:LOAD-IMAGE("graphics/treeview/plus.png").

            RUN PTreeResizeFrame({&ttNode}.nodeTextHandle:FRAME,
                             {&ttNode}.nodeTextHandle,
                             -1 * {&ttNode}.nodeFrameHandle:VIRTUAL-HEIGHT-PIXELS,
                             {&ttNode}.nodeFrameHandle:WIDTH-PIXELS).
        END.
        ELSE DO: /* Grow and view */
            RUN PTreeResizeFrame({&ttNode}.nodeTextHandle:FRAME,
                             {&ttNode}.nodeTextHandle,
                             {&ttNode}.nodeFrameHandle:VIRTUAL-HEIGHT-PIXELS,
                             {&ttNode}.nodeFrameHandle:WIDTH-PIXELS).
            
            
            {&ttNode}.nodeFrameHandle:Y = {&ttNode}.nodeTextHandle:Y + ({&TextSpacingPixels} - 2). /* Move back from whence it came */
            {&ttNode}.nodeFrameHandle:VISIBLE = TRUE.
            IF VALID-HANDLE({&ttNode}.nodeOpenerImageHandle)
            THEN {&ttNode}.nodeOpenerImageHandle:LOAD-IMAGE("graphics/treeview/minus.png").
        END.
    END.
    ELSE IF CAN-FIND(FIRST child-{&ttNode} WHERE child-{&ttNode}.{&parentId} = {&ttNode}.{&id})
    THEN DO:
        RUN PTreeCreateNodeFrame({&ttNode}.{&id}).
        {&ttNode}.nodeOpenerImageHandle:LOAD-IMAGE("graphics/treeview/minus.png").
    END.
    ELSE.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_CursorDown) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_CursorDown Method-Library 
PROCEDURE PTrigger_CursorDown :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    RUN PTreeSelectNextNode(TRUE). /* Next */
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_CursorUp) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_CursorUp Method-Library 
PROCEDURE PTrigger_CursorUp :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    
    RUN PTreeSelectNextNode(FALSE). /* Previous */
    
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_ImageDblClick) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_ImageDblClick Method-Library 
PROCEDURE PTrigger_ImageDblClick :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       Have to use intermediatory hTextHandle var because Progress is sh!te
------------------------------------------------------------------------------*/
    
    DEFINE VARIABLE hTextHandle AS HANDLE NO-UNDO.
    hTextHandle = HANDLE(SELF:PRIVATE-DATA).
    APPLY "MOUSE-SELECT-DBLCLICK" TO hTextHandle.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_ImageDown) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_ImageDown Method-Library 
PROCEDURE PTrigger_ImageDown :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       Have to use intermediatory hTextHandle var because Progress is sh!te
------------------------------------------------------------------------------*/
    
    DEFINE VARIABLE hTextHandle AS HANDLE NO-UNDO.
    hTextHandle = HANDLE(SELF:PRIVATE-DATA).
    APPLY "MOUSE-SELECT-DOWN" TO hTextHandle.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_OpenerClick) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_OpenerClick Method-Library 
PROCEDURE PTrigger_OpenerClick :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    
    FIND {&ttNode} WHERE id = {&CAST_id}(SELF:NAME).
    
    RUN PTreeToggleNode(BUFFER {&ttNode}).
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_Return) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_Return Method-Library 
PROCEDURE PTrigger_Return :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    IF VALID-HANDLE(LhTreeSelectedText)
    THEN APPLY 'MOUSE-SELECT-DBLCLICK' TO LhTreeSelectedText.
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_TextDblClick) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_TextDblClick Method-Library 
PROCEDURE PTrigger_TextDblClick :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE BUFFER {&ttNode} FOR {&ttNode}.
    
    FIND {&ttNode} WHERE {&ttNode}.{&id} = {&CAST_id}(SELF:NAME).
    
    RUN PTreeToggleNode(BUFFER {&ttNode}).
    RUN PNodeDblClicked IN THIS-PROCEDURE (INPUT {&ttNode}.{&id}) NO-ERROR.  /* Hook for double-clicking a node */
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
&IF DEFINED(EXCLUDE-PTrigger_TextDown) = 0 &THEN
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTrigger_TextDown Method-Library 
PROCEDURE PTrigger_TextDown :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    
    RUN PTreeSelectNode(SELF:NAME).
    
END PROCEDURE.
/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME
&ENDIF
