/*
 * Copyright 2016-2017 Golden Code Development Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"),  to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.ui.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in emain.p).
 */
public class Emain
{
   logical fwdUserLogout = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "fwd-user-logout", new logical(false));

   handle fwdCustomApis = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "fwd-custom-apis", new handle());

   logical fwdEmbeddedMode = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "fwd-embedded-mode", true, new logical(true));

   logical fwdInitialized = UndoableFactory.logical(false);

   /**
    * External procedure (converted to Java from the 4GL source code
    * in emain.p).
    */
   public void execute()
   {
      externalProcedure(Emain.this, new Block((Body) () -> 
      {
         OnPhrase[] onPhrase0 = new OnPhrase[]
         {
            new OnPhrase(Condition.QUIT, Action.LEAVE, "blockLabel0"),
            new OnPhrase(Condition.STOP, Action.LEAVE, "blockLabel0"),
            new OnPhrase(Condition.ERROR, Action.LEAVE, "blockLabel0"),
            new OnPhrase(Condition.ENDKEY, Action.LEAVE, "blockLabel0")
         };
         
         doBlock(TransactionType.SUB, "blockLabel0", onPhrase0, new Block((Body) () -> 
         {
            LogicalTerminal.waitFor(null, new EventList("close", thisProcedure()));
         }));
         
         if ((fwdInitialized).booleanValue())
         {
            if (_not(fwdUserLogout))
            {
               ControlFlowOps.invoke("fwdLogout");
            }
            
            /* if we reached this part, it means we have finished - send the notification signa*/
            ControlFlowOps.remoteCall("{ \"api\":\"notifyDisconnect\" }");
         }
      }));
   }

   public void doLogin(final integer _messageId, final character _username, final character _password)
   {
      integer messageId = UndoableFactory.initInput(_messageId);
      character username = UndoableFactory.initInput(_username);
      character password = UndoableFactory.initInput(_password);
      
      internalProcedure(new Block((Body) () -> 
      {
         if ((SecurityOps.setUserId(username, password)).booleanValue())
         {
            fwdInitialized.assign(true);
            ControlFlowOps.remoteCall("{ \"api\":\"p2jInitialized\" }");
         }
         else
         {
            ControlFlowOps.remoteCall("{ \"api\":\"showError\" , \"error\":\"Invalid username or password\" }");
            quit();
         }
      }));
   }

   public character fwdGetAdmversion()
   {
      return function("fwdGetADMVersion", character.class, new Block((Body) () -> 
      {
         returnNormal("ADM2.2");
      }));
   }
}
