/*
 * Copyright 2016-2017 Golden Code Development Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"),  to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.hotel.ui.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo.hotel.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.KeyReader.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.util.ErrorManager.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in reservations-frame.w).
 */
public class ReservationsFrame
{
   public static final String FMT_STR_1 = ">>>>>";

   public static final String FMT_STR_2 = "X(256)";

   RoomType.Buf roomType = RecordBuffer.define(RoomType.Buf.class, "hotel", "roomType", "room-type");

   Room.Buf room = RecordBuffer.define(Room.Buf.class, "hotel", "room", "room");

   Reservation.Buf reservation = RecordBuffer.define(Reservation.Buf.class, "hotel", "reservation", "reservation");

   ReservationsFrameFmain fmainFrame = GenericFrame.createFrame(ReservationsFrameFmain.class, "fmain");

   /* Connected Databases 
             hotel            PROGRESS
   */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /* adecomm/appserv.i checks to see if the persistent procedure adecomm/as-utils.w
                        has been started up, and if not, it starts it and then makes
                        is a SUPER procedure of the calling procedure.
                        
      Created: 11/26/97 by Ross Hunter
                                                                                    */   
   handle appSrvUtils = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "appsrvutils", true, new handle());

   integer filterRoom = TypeFactory.integer((long) 0);

   character findGuest = TypeFactory.character();

   /* Query definitions                                                    */   
   final QueryWrapper query0 = new QueryWrapper("reservationsBrowse", true);

   /* These preprocessors tell at compile time which properties can
        be retrieved directly from the temp-table */   
   /* include properties from smart */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in reservations-frame.w).
    */
   public void execute()
   {
      /* For {get/set} */
      handle ghAdmprops = TypeFactory.handle();
      /* Handle of prop t-t */
      handle ghAdmpropsBuf = TypeFactory.handle();
      /*  and its buffer    */
      logical glAdmloadFromRepos = TypeFactory.logical();
      /* _UIB-CODE-BLOCK-END */
      /* _UIB-CODE-BLOCK-END */
      /* ********************  Preprocessor Definitions  ******************** */
      /* _UIB-PREPROCESSOR-BLOCK-END */
      /* *********************** Procedure Settings ************************ */
      /* Settings for THIS-PROCEDURE
         Type: Method-Library
         Allow: 
         Frames: 0
         Add Fields to: Neither
         Other Settings: INCLUDE-ONLY
       */
      /* *************************  Create Window  ************************** */
      /* DESIGN Window definition (used by the UIB) 
        CREATE WINDOW Method-Library ASSIGN
               HEIGHT             = 7.86
               WIDTH              = 60.
      _* END WINDOW DEFINITION *_
                                                                              */
      /* ************************* Included-Libraries *********************** */
      /* If this container has no visualization 
         -- is either defined to be 'virtual' or is a SmartFrame with no frame --
         (This can be used simply by opening a SmartFrame and deleting the frame.) 
         If virtual then it is simply a container for non-visual objects; 
         so skip visual.i, and include smart.i or appserver.i depending on the  
         appserver aware preprocessor defined for Appserver aware objects in 
         adecomm/appserv.i */
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /*--------------------------------------------------------------------------
          File        : visual.i
          Purpose     : Defines basic properties and other info for visual objects
      
          Syntax      :
      
          Description :
      
          Modified    : May 19, 1999 Version 9.1A
          Modified    : 11/13/2001      Mark Davies (MIP)
                        Check for valid handle of any gsh... variables
        ------------------------------------------------------------------------*/
      /*          This .i file was created with the Progress UIB.             */
      /*----------------------------------------------------------------------*/
      /* ***************************  Definitions  ************************** */
      /* _UIB-CODE-BLOCK-END */
      /* ********************  Preprocessor Definitions  ******************** */
      /* _UIB-PREPROCESSOR-BLOCK-END */
      /* *********************** Procedure Settings ************************ */
      /* Settings for THIS-PROCEDURE
         Type: Include
         Allow: 
         Frames: 0
         Add Fields to: Neither
         Other Settings: INCLUDE-ONLY
       */
      /* *************************  Create Window  ************************** */
      /* DESIGN Window definition (used by the UIB) 
        CREATE WINDOW Include ASSIGN
               HEIGHT             = 8
               WIDTH              = 60.
      _* END WINDOW DEFINITION *_
                                                                              */
      /* ************************* Included-Libraries *********************** */
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /*-------------------------------------------------------------------------
          File        : appserver.i
          Purpose     : Basic Method Library for the ADMClass appserver.
        
          Syntax      : {src/adm2/appserver.i}
      
          Description :
        
          Created     : 17 Nov 2000
          Author      : Roland de Pijper, Progress Software BV                
          Modified    : 24 Dec 2001 
                        Added to the Commercial product 
                       (Renamed to appserver and made part of existing classes) 
      -------------------------------------------------------------------------*/
      /*          This .W file was created with the Progress UIB.             */
      /*----------------------------------------------------------------------*/
      /* ***************************  Definitions  ************************** */
      /* _UIB-CODE-BLOCK-END */
      /* ********************  Preprocessor Definitions  ******************** */
      /* _UIB-PREPROCESSOR-BLOCK-END */
      /* *********************** Procedure Settings ************************ */
      /* Settings for THIS-PROCEDURE
         Type: Method-Library
         Allow: 
         Frames: 0
         Add Fields to: Neither
         Other Settings: INCLUDE-ONLY
       */
      /* *************************  Create Window  ************************** */
      /* DESIGN Window definition (used by the UIB) 
        CREATE WINDOW Method-Library ASSIGN
               HEIGHT             = 8
               WIDTH              = 60.
      _* END WINDOW DEFINITION *_
                                                                              */
      /* ************************* Included-Libraries *********************** */
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /*--------------------------------------------------------------------------
          Library     : smart.i - NEW V9 version of top-level SmartObject include
      
          Modified    : May 22, 2000 -- Version 9.1B
        ------------------------------------------------------------------------*/
      /*          This .i file was created with the Progress UIB.             */
      /*----------------------------------------------------------------------*/
      /* ***************************  Definitions  ************************** */
      /* If smart.i has already been included, skip everything 
            (matching ENDIF is at the end of the file). */
      /* Define the preprocessor that identifies the basic "class" of the object;
           ADMClass will remain undefined here only if the object uses no sub-class
           include files. */
      /* If this object is of type "smart", i.e., uses no lower class include
           files, then include the property file here. Otherwise this will be
           done in the class include file of the base class of the object. */
      /* Window or Frame handle (not used, but kept for backwards comaptibility) */
      handle ghContainer = TypeFactory.handle();
      character cObjectName = TypeFactory.character();
      integer iStart = TypeFactory.integer();
      /* Note: This ENDIF matches the adm-smart definition at the top of file.
         Do not delete it and if another procedure or function is defined which
         occurs later in the file, move it to the end of that. */
      /* _UIB-CODE-BLOCK-END */
      /* _UIB-CODE-BLOCK-END */
      /* ***************************  Main Block  *************************** */
      character cAppService = TypeFactory.character();
      character cAsdivision = TypeFactory.character();
      character cServerOperatingMode = TypeFactory.character();
      /* _ADM-CODE-BLOCK-START _CUSTOM _INCLUDED-LIB-CUSTOM CUSTOM */
      /*********************************************************************
      * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
      * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
      * below.  All Rights Reserved.                                       *
      *                                                                    *
      * The Initial Developer of the Original Code is PSC.  The Original   *
      * Code is Progress IDE code released to open source December 1, 2000.*
      *                                                                    *
      * The contents of this file are subject to the Possenet Public       *
      * License Version 1.0 (the "License"); you may not use this file     *
      * except in compliance with the License.  A copy of the License is   *
      * available as of the date of this notice at                         *
      * http://www.possenet.org/license.html                               *
      *                                                                    *
      * Software distributed under the License is distributed on an "AS IS"*
      * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
      * should refer to the License for the specific language governing    *
      * rights and limitations under the License.                          *
      *                                                                    *
      * Contributors:                                                      *
      *                                                                    *
      *********************************************************************/
      /*------------------------------------------------------------------------
          File        : appservercustom.i
          Purpose     : References the start of the custom super procedure.
                        Allows properties initialization.
      
          Syntax      : {src/adm2/custom/appservercustom.i}
      
          Description :
      
          Created     : 01/24/2001
          Notes       : Referenced in {src/adm2/appserver.i}
        ----------------------------------------------------------------------*/
      /*          This .W file was created with the Progress AppBuilder.      */
      /*----------------------------------------------------------------------*/
      /* ***************************  Definitions  ************************** */
      /* _UIB-CODE-BLOCK-END */
      /* ********************  Preprocessor Definitions  ******************** */
      /* _UIB-PREPROCESSOR-BLOCK-END */
      /* *********************** Procedure Settings ************************ */
      /* Settings for THIS-PROCEDURE
         Type: Include
         Allow: 
         Frames: 0
         Add Fields to: Neither
         Other Settings: INCLUDE-ONLY
       */
      /* *************************  Create Window  ************************** */
      /* DESIGN Window definition (used by the UIB) 
        CREATE WINDOW Include ASSIGN
               HEIGHT             = 15
               WIDTH              = 60.
      _* END WINDOW DEFINITION *_
                                                                              */
      /* ***************************  Main Block  *************************** */
      /* Starts here the custom super procedure 
         Uncomment to run it */
      /*RUN start-super-proc ("adm2/custom/appservercustom.p":U).*/
      /* _UIB-CODE-BLOCK-END */
      /* _ADM-CODE-BLOCK-END */
      /* _UIB-CODE-BLOCK-END */
      /* _UIB-CODE-BLOCK-END */
      /* ***************************  Main Block  *************************** */
      character cFields = TypeFactory.character();
      
      externalProcedure(ReservationsFrame.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         query0.addBuffer(reservation, true);
         query0.addBuffer(room, true);
         query0.addBuffer(roomType, true);
         fmainFrame.openScope();
         RecordBuffer.openScope(reservation, room, roomType);
         
         if (_not(appSrvUtils.isValid()))
         {
            ControlFlowOps.invokePersistentSet("adecomm/as-utils.w", appSrvUtils);
         }
         
         thisProcedure().unwrapCommonProcedure().addSuperProcedure(appSrvUtils);
         /*------------------------------------------------------------------------
         
           File: 
         
           Description: from cntnrfrm.w - ADM2 SmartFrame Template
         
           Input Parameters:
               <none>
         
           Output Parameters:
               <none>
         
          
         ------------------------------------------------------------------------*/
         /*          This .W file was created with the Progress AppBuilder.      */
         /*----------------------------------------------------------------------*/
         /* Create an unnamed pool to store all the widgets created 
              by this procedure. This is a good default which assures
              that this procedure's triggers and internal procedures 
              will execute in this procedure's storage, and that proper
              cleanup will occur on deletion of the procedure. */
         WidgetPool.create();
         
         FrameElement[] elementList0 = new FrameElement[]
         {
            new Element(new FieldReference(reservation, "roomNum"), fmainFrame.widgetReservationsBrowseRoomNumColumn()),
            new Element(new FieldReference(reservation, "guest"), fmainFrame.widgetReservationsBrowseGuestColumn()),
            new Element(new FieldReference(reservation, "numGuests"), fmainFrame.widgetReservationsBrowseNumGuestsColumn()),
            new Element(new FieldReference(reservation, "startDate"), fmainFrame.widgetReservationsBrowseStartDateColumn()),
            new Element(new FieldReference(reservation, "endDate"), fmainFrame.widgetReservationsBrowseEndDateColumn()),
            new HeaderElement(() -> date.differenceNum(reservation.getEndDate(), reservation.getStartDate()), fmainFrame.widgetReservationsBrowseExpr24Column()),
            new Element(new FieldReference(roomType, "description"), fmainFrame.widgetReservationsBrowseDescriptionColumn()),
            new Element(new FieldReference(reservation, "price"), fmainFrame.widgetReservationsBrowsePriceColumn())
         };
         /* Browse definitions                                                   */
         fmainFrame.widgetReservationsBrowse().registerQuery(query0, elementList0);
         
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: SmartFrame
            Allow: Basic,Browse,DB-Fields,Query,Smart
            Container Links: Data-Target,Data-Source,Page-Target,Update-Source,Update-Target
            Other Settings: PERSISTENT-ONLY APPSERVER
          */
         /* This procedure should always be RUN PERSISTENT.  Report the error,  */
         /* then cleanup and return.                                            */
         if (_not(thisProcedure().unwrapProcedure().isPersistent()))
         {
            messageBox("reservations-frame.w should only be RUN PERSISTENT.", ALERT_ERROR, BTN_OK, (String) null);
            returnNormal();
         }
         
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW fFrameWin ASSIGN
                  HEIGHT             = 15.38
                  WIDTH              = 125.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ************************* Included-Libraries *********************** */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*-------------------------------------------------------------------------
             Library     : containr.i  
             Purpose     : Default V9 Main Block code and Method Procedures
                           for ADM Container procedures.
           
             Syntax      : {src/adm2/containr.i}
         
             Modified    : February 18, 2001 Version 9.1C 
             
             Note: !!!   : Method Libraries are maintained manually for 
                           conditional inclusion.        
         -------------------------------------------------------------------------*/
         /***********************  DEFINITIONS  ***********************************/
         /* If this is a SmartFrame with no Frame, then re-identify it as a 'virtual'
            -- i.e, non-visual -- container. */
         /* make the window's max size the session size, but do it no-error 
            as window-name may be current-window with no window (serverside sbo) */
         /* Avoid this if current-window = {Window-name} already. (a suppressed window) */
         /* The current-window will be set to {&window-name} later for a normal window */
         if (_isNotEqual(currentWindow(), currentWindow()))
         {
            /* only change if the max-width is default */
            if (_isEqual(currentWindow().unwrapWindow().getMaxWidth(), currentWindow().unwrapWidget().getVirtWidth()))
            {
               silent(() -> currentWindow().unwrapWindow().setMaxWidth(minus(LogicalTerminal.getWidthChars(), 1)));
            }
            
            /* only change if the max-height is default */
            if (_isEqual(currentWindow().unwrapWindow().getMaxHeight(), currentWindow().unwrapWidget().getVirtHeight()))
            {
               silent(() -> currentWindow().unwrapWindow().setMaxHeight(minus(LogicalTerminal.getHeightChars(), 1)));
            }
         }
         
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         /* Note that adm-props-from-repository is intended to be replaced with
              a variable for flexible override, so it cannot be used for conditional 
              compile.  */
         if ((gshRepositoryManager.isValid()).booleanValue())
         {
            if (_not(new logical(ControlFlowOps.invokeDynamicFunctionInWithMode("prepareInstance", gshRepositoryManager, "II", concat(valueOf(thisProcedure()), ",", valueOf(fmainFrame.asWidgetHandle()), ",", valueOf(fmainFrame.widgetReservationsBrowse().asWidgetHandle())), sourceProcedure()))))
            {
               stop();
            }
            
            ghAdmpropsBuf.assign(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))));
            glAdmloadFromRepos.assign(ghAdmpropsBuf.isValid());
         }
         
         /* VALID-HANDLE(gshRepositoryManager) */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            TempTableBuilder.create(ghAdmprops);
            ghAdmprops.unwrapTempTable().setCanUndo(new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectName"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectVersion"), new character("CHAR"), new integer(0), new character(), new character("ADM2.2"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectType"), new character("CHAR"), new integer(0), new character(), new character("SmartFrame"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerType"), new character("CHAR"), new integer(0), new character(), new character("FRAME"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PropertyDialog"), new character("CHAR"), new integer(0), new character(), new character("adm2/support/visuald.w"));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryObject"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceProperties"), new character("CHAR"), new integer(0), new character(), new character("LogicalObjectName,PhysicalObjectName,DynamicObject,RunAttribute, HideOnInit,DisableOnInit,ObjectLayout"));
            /* NOTE: Any need to support &User-Supported-Links?? */
            ghAdmprops.unwrapTempTable().addNewField(new character("SupportedLinks"), new character("CHAR"), new integer(0), new character(), new character("Data-Target,Data-Source,Page-Target,Update-Source,Update-Target"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerHidden"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectInitialized"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            /*  "hidden" is a logical concept. */
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectHidden"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideOnInit"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("UIBMode"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerSource"), new character("HANDLE"));
            /* Note that datavis adds some events that are only required for updating objects */
            /* isUpdateActive event is needed only for data, datavis and container classes,
                 but we currently define it here.. (this simplifies the logic for datavis, 
                 which may or may not inherit from container)
                 confirmOk and confirmCancel only for container ans datavis */
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("initializeObject,hideObject,viewObject,destroyObject,enableObject,confirmExit,confirmCancel,confirmOk,isUpdateActive"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSource"), new character("HANDLE"));
            /* Note that DataSourceEvents is overidden in data.i since some of these 
                 events are intended for visual targets; queryPosition, fetchDataSet, 
                 assignMaxdataguess and the most resent addition confirmUndo and 
                 confirmCommit. These two are NOT dataSourceEvents for SDOs as the message 
                 should not to be published to objects not affected by the commit/undo. */
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("dataAvailable,queryPosition,updateState,deleteComplete,fetchDataSet,confirmContinue,confirmCommit,confirmUndo,assignMaxGuess,isUpdatePending"));
            ghAdmprops.unwrapTempTable().addNewField(new character("TranslatableProperties"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectPage"), new character("INT"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("DBAware"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("DesignDataObject"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSourceNames"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataTargetEvents"), new character("CHARACTER"), new integer(0), new character(), new character("updateState,rowObjectState,fetchBatch,LinkState"));
            ghAdmprops.unwrapTempTable().addNewField(new character("LogicalObjectName"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalObjectName"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("LogicalVersion"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalVersion"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DynamicObject"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RunAttribute"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ChildDataKey"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ParentDataKey"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataLinksEnabled"), new character("LOGICAL"), new integer(), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("InactiveLinks"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceId"), new character("DECIMAL"));
            /* SuperProcedure management */
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedureMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedureHandle"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutPosition"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ClassName"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RenderingProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ThinRenderingProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("Label"), new character("CHAR"), new integer(0), new character(), new unknown());
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("AppService"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASDivision"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ASHandle"), new character("HANDLE"), new integer(0), new character(), new unknown());
            /* Set to true on the first connection */
            ghAdmprops.unwrapTempTable().addNewField(new character("ASHasStarted"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASInfo"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASInitializeOnRun"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASUsePrompt"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("BindSignature"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("BindScope"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerOperatingMode"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerFileName"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerFirstCall"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("NeedContext"), new character("LOGICAL"), new integer(0), new character(), new unknown());
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectLayout"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutOptions"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectEnabled"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutVariable"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DefaultLayout"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DisableOnInit"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("EnabledObjFlds"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("EnabledObjHdls"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("FieldSecurity"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("SecuredTokens"), new character("CHARACTER"), new integer(0), new character(), new unknown());
            /* Initial must be unknown. */
            ghAdmprops.unwrapTempTable().addNewField(new character("AllFieldHandles"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("AllFieldNames"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("MinHeight"), new character("DECIMAL"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("MinWidth"), new character("DECIMAL"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("ResizeHorizontal"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ResizeVertical"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectSecured"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectTranslated"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("PopupButtonsInFields"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorInfoBG"), new character("INTEGER"), new integer(0), new character(), new integer(10));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorInfoFG"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorWarnBG"), new character("INTEGER"), new integer(0), new character(), new integer(3));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorWarnFG"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorErrorBG"), new character("INTEGER"), new integer(0), new character(), new integer(12));
            ghAdmprops.unwrapTempTable().addNewField(new character("ColorErrorFG"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("BGColor"), new character("INTEGER"), new integer(0), new character(), new integer(12));
            ghAdmprops.unwrapTempTable().addNewField(new character("FGColor"), new character("INTEGER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FieldPopupMapping"), new character("CHARACTER"), new integer(0), new character(), new character(""));
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("CurrentPage"), new character("INT"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("PendingPage"), new character("INT"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerTargetEvents"), new character("CHAR"), new integer(0), new character(), new character("exitObject,okObject,cancelObject,updateActive"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerToolbarSource"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerToolbarSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("toolbar,okObject,cancelObject"));
            ghAdmprops.unwrapTempTable().addNewField(new character("OutMessageTarget"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PageNTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("PageSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("FilterSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateSource"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitSource"), new character("HANDLE"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("commitTransaction,undoTransaction"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitTargetEvents"), new character("CHAR"), new integer(0), new character(), new character("rowObjectState"));
            ghAdmprops.unwrapTempTable().addNewField(new character("StartPage"), new character("INT"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RunMultiple"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("WaitForObject"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DynamicSDOProcedure"), new character("CHAR"), new integer(0), new character(), new character("adm2/dyndata.w"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RunDOOptions"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("InitialPageList"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            /* comma delimited list of pages to construct at startup or * for all or empty for just start page and page 0 */
            ghAdmprops.unwrapTempTable().addNewField(new character("WindowFrameHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("Page0LayoutManager"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("MultiInstanceSupported"), new character("LOGICAL"), new integer(), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("MultiInstanceActivated"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SavedContainerMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SdoForeignFields"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationSource"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationTarget"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PrimarySdoTarget"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("fetchFirst,fetchNext,fetchPrev,fetchLast,startFilter"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CallerWindow"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CallerProcedure"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CallerObject"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DisabledAddModeTabs"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ReEnableDataLinks"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("WindowTitleViewer"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateActive"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectsCreated"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceNames"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ClientNames"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainedDataObjects"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainedAppServices"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataContainer"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("HasDynamicProxy"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideOnClose"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideChildContainersOnClose"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("HasObjectMenu"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("RequiredPages"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RemoveMenuOnHide"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
         }
         
         ghContainer.assign(fmainFrame.asWidgetHandle());
         
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 8.62
                  WIDTH              = 52.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ************************* Included-Libraries *********************** */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/smart.p"));
         }
         
         /* Use the old adm definition */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            /* Now create the one record in this property temp-table, and store its
                   handle in ADM-DATA. The CHR(1) delimiters are to set aside spots
                   for UserProperties and UserLinks. */
            ghAdmprops.unwrapTempTable().tempTablePrepare("ADMProps");
            ghAdmpropsBuf.assign(ghAdmprops.unwrapTempTable().defaultBufferHandle());
            ghAdmpropsBuf.unwrapBuffer().bufferCreate();
            /* ... and save it. This ADM object is now open for business! */
            thisProcedure().unwrapADMData().setADMData(concat(valueOf(ghAdmpropsBuf), chr(1), chr(1)));
            /* from this point  {&ADM-PROPS-DEFINED} is true for all cases */
            cObjectName.assign("LogicalObjectName,PhysicalObjectName,DynamicObject,RunAttribute, HideOnInit,DisableOnInit,ObjectLayout");
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("InstanceProperties"));
            ghProp.unwrapBufferField().changeValue(cObjectName);
            cObjectName.assign(replaceAll(thisProcedure().unwrapFile().getFileName(), "\\", "/"));
            iStart.assign(plus(lastIndexOf(cObjectName, "/"), 1));
            cObjectName.assign(substring(cObjectName, iStart, minus(lastIndexOf(thisProcedure().unwrapFile().getFileName(), "."), iStart)));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
            ghProp.unwrapBufferField().changeValue(cObjectName);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
            ghProp.unwrapBufferField().changeValue(fmainFrame.asWidgetHandle());
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
            ghProp.unwrapBufferField().changeValue(new character("SmartFrame"));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerType"));
            ghProp.unwrapBufferField().changeValue(new character("FRAME"));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalVersion"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalObjectName"));
            ghProp.unwrapBufferField().changeValue((_isNotEqual("", "") ? new character("") : thisProcedure().unwrapFile().getFileName()));
         }
         
         /* Starts super procedure */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/appserver.p"));
         }
         
         /* May have been retrieved from repository */
         /* {get Prop var [handle]} */ 
         cAppService.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAppService", targetProcedure()));
         
         /* NOTE: If REMOTE = true, we are running on an AppServer, so ignore the 
              AppService (partition) value, which is meaningful only to the client. 
              Also set the ASDivision property to Server.
              Note that AsDivision is set to 'Client' or blank in connectServer upon 
              in appserver.p */
         if ((SessionUtils.isRemote()).booleanValue())
         {
            cAppService.assign("");
            cAsdivision.assign("Server");
            cServerOperatingMode.assign(toUpperCase(SessionUtils.getServerOperatingMode()));
         }
         else
         {
            if (_isEqual(cAppService, ""))
            {
               cAppService.assign("");
            }
         }
         
         if (_isEqual(cAsdivision, ""))
         {
            cServerOperatingMode.assign("NONE");
         }
         else
         {
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setASDivision", targetProcedure(), "I", cAsdivision);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ServerOperatingMode"));
         ghProp.unwrapBufferField().changeValue(cServerOperatingMode);
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setAppService", targetProcedure(), "I", cAppService);
         
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/visual.p"));
         }
         
         cFields.assign(replaceAll("reselFilters filterRoom findGuest searchButton reservationsBrowse addButton updateButton deleteButton checkIn", " ", ","));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
         ghProp.unwrapBufferField().changeValue(cFields);
         
         /* _ADM-CODE-BLOCK-START _CUSTOM _INCLUDED-LIB-CUSTOM CUSTOM */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File        : visualcustom.i
             Purpose     : References the start of the custom super procedure.
                           Allows properties initialization.
         
             Syntax      : {src/adm2/custom/visualcustom.i}
         
             Description :
         
             Created     : 06/03/1999
             Notes       : Referenced in {src/adm2/visual.i}
           ----------------------------------------------------------------------*/
         /*          This .W file was created with the Progress AppBuilder.      */
         /*----------------------------------------------------------------------*/
         /* ***************************  Definitions  ************************** */
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Include
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Include ASSIGN
                  HEIGHT             = 15
                  WIDTH              = 60.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Starts here the custom super procedure 
            Uncomment to run it */
         /*RUN start-super-proc ("adm2/custom/visualcustom.p":U).*/
         /* _UIB-CODE-BLOCK-END */
         /* _ADM-CODE-BLOCK-END */
         if ((gshSessionManager.isValid()).booleanValue())
         {
            registerTrigger(new EventList("HELP", fmainFrame.asWidget(), true), ReservationsFrame.this, new Trigger((Body) () -> 
            {
               ControlFlowOps.invokeInWithMode("contextHelp", gshSessionManager, "II", thisProcedure(), focus());
            }));
         }
         
         registerTrigger(new EventList("CTRL-PAGE-UP", fmainFrame.asWidget(), true), ReservationsFrame.this, new Trigger((Body) () -> 
         {
            ControlFlowOps.invokeInWithMode("processAction", targetProcedure(), "I", new character("CTRL-PAGE-UP"));
         }));
         
         registerTrigger(new EventList("CTRL-PAGE-DOWN", fmainFrame.asWidget(), true), ReservationsFrame.this, new Trigger((Body) () -> 
         {
            ControlFlowOps.invokeInWithMode("processAction", targetProcedure(), "I", new character("CTRL-PAGE-DOWN"));
         }));
         
         /* _UIB-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/containr.p"));
            ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", targetProcedure(), new character("Add"), new character("ContainerSourceEvents"), new character("initializeDataObjects"));
            
            /* {fn <func-name> [handle] */ 
            if (_not(matchesList(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getSupportedLinks", targetProcedure()), "containertoolbar-target")))
            {
               ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", targetProcedure(), new character("Add"), new character("SupportedLinks"), new character("ContainerToolbar-Target"));
            }
         }
         /* if not adm-load-from-repos */
         /* Best default for GUI applications - this will apply to the whole session: */
         pauseBeforeHide(0);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("WindowFrameHandle"));
         ghProp.unwrapBufferField().changeValue(fmainFrame.asWidgetHandle());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataContainer"));
         ghProp.unwrapBufferField().changeValue(new logical(true));
         fmainFrame.setHidden(new logical(true));
         /* _RUN-TIME-ATTRIBUTES-END */
         /* Setting information for Queries and Browse Widgets fields            */
         /* Query rebuild information for FRAME fMain
              _Options          = ""
              _Query            is NOT OPENED
         */
         /* FRAME fMain */
         /* Query rebuild information for BROWSE reservationsBrowse
              _TblList          = "hotel.reservation,hotel.room OF hotel.reservation,hotel.room-type OF hotel.room"
              _Options          = "NO-LOCK"
              _Where[1]         = "reservation.end-date >= today"
              _FldNameList[1]   > hotel.reservation.room-num
         "reservation.room-num" "Room Num" ? "integer" ? ? ? ? ? ? no ? no no "11.2" yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[2]   > hotel.reservation.guest
         "reservation.guest" ? ? "character" ? ? ? ? ? ? no ? no no "22.2" yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[3]   > hotel.reservation.num-guests
         "reservation.num-guests" "Num Guests" ? "integer" ? ? ? ? ? ? no ? no no ? yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[4]   > hotel.reservation.start-date
         "reservation.start-date" "Check-in" ? "date" ? ? ? ? ? ? no ? no no "10.6" yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[5]   > hotel.reservation.end-date
         "reservation.end-date" "Check-out" ? "date" ? ? ? ? ? ? no ? no no "12.2" yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[6]   > "_<CALC>"
         "reservation.end-date - reservation.start-date" "Days" ? ? ? ? ? ? ? ? no ? no no "5.4" yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[7]   > hotel.room-type.description
         "room-type.description" "Room Type" ? "character" ? ? ? ? ? ? no ? no no ? yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _FldNameList[8]   > hotel.reservation.price
         "reservation.price" "Room Price" "$ >>>,>>9.99" "decimal" ? ? ? ? ? ? no ? no no "15.8" yes no no "U" "" "" "" "" "" "" 0 no 0 no no
              _Query            is OPENED
         */
         /* BROWSE reservationsBrowse */
         /* ************************  Control Triggers  ************************ */
         registerTrigger(new EventList("CHOOSE", fmainFrame.widgetAddButton()), TriggerBlock0.class, ReservationsFrame.this);
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("CHOOSE", fmainFrame.widgetCheckIn()), TriggerBlock1.class, ReservationsFrame.this);
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("CHOOSE", fmainFrame.widgetDeleteButton()), TriggerBlock2.class, ReservationsFrame.this, true);
         
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("ANY-KEY", fmainFrame.widgetFilterRoom()), ReservationsFrame.this, new Trigger((Body) () -> 
         {
            if (_isEqual(lastKey(), keyCode("RETURN")))
            {
               ControlFlowOps.invoke("refresh-reservations");
            }
         }));
         
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("ANY-KEY", fmainFrame.widgetFindGuest()), ReservationsFrame.this, new Trigger((Body) () -> 
         {
            if (_isEqual(lastKey(), keyCode("RETURN")))
            {
               ControlFlowOps.invoke("refresh-reservations");
            }
         }));
         
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("CHOOSE", fmainFrame.widgetReselFilters()), ReservationsFrame.this, new Trigger((Body) () -> 
         {
            filterRoom.assign(0);
            findGuest.assign("");
            
            FrameElement[] elementList1 = new FrameElement[]
            {
               new Element(filterRoom, fmainFrame.widgetFilterRoom()),
               new Element(findGuest, fmainFrame.widgetFindGuest())
            };
            
            fmainFrame.display(elementList1);
            
            ControlFlowOps.invoke("refresh-reservations");
         }));
         
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("CHOOSE", fmainFrame.widgetSearchButton()), ReservationsFrame.this, new Trigger((Body) () -> 
         {
            ControlFlowOps.invoke("refresh-reservations");
         }));
         
         /* _UIB-CODE-BLOCK-END */
         registerTrigger(new EventList("CHOOSE", fmainFrame.widgetUpdateButton()), TriggerBlock3.class, ReservationsFrame.this);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
            cType.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    *    Purpose:  Clone the current adm property temp-table<br>
    * Parameters: <br>
    *      Notes:  This is used to clone the dynamic property record for static <br>
    *              objects that does not call deleteProperties from destroyObject<br>
    *              in the target-procedure. This way the properties are kept alive <br>
    *              until destroyObject is completed without creating a memory leak,<br>
    *              since this TT is created in the object's widget-pool.
    */
   public void admCloneProps()
   {
      handle hReposBuffer = TypeFactory.handle();
      handle hPropTable = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hReposBuffer.assign(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))));
         
         if (_and(hReposBuffer.isValid(), () -> isNotEqual(hReposBuffer.unwrap().name(), "ADMProps")))
         {
            silent(() -> hReposBuffer.unwrapBuffer().findFirst(concat("WHERE Target = WIDGET-H(\"", valueOf(thisProcedure()), "\")")));
            
            if ((hReposBuffer.unwrapBuffer().available()).booleanValue())
            {
               TempTableBuilder.create(hPropTable);
               hPropTable.unwrapTempTable().createTableLike(hReposBuffer);
               hPropTable.unwrapTempTable().tempTablePrepare("ADMProps");
               hBuffer.assign(hPropTable.unwrapTempTable().defaultBufferHandle());
               hBuffer.unwrapBuffer().bufferCreate();
               hBuffer.unwrapBuffer().bufferCopy(hReposBuffer);
               /* delete the dynamic's property record */
               /* {fn <func-name> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionIn(logical.class, "deleteProperties", targetProcedure());
               /* ... and save it. This ADM object is now open for business! */
               thisProcedure().unwrapADMData().setADMData(concat(valueOf(hBuffer), chr(1), chr(1)));
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Procedure to start a super proc if it's not already running, <br>
    *              and to add it as a super proc in any case.<br>
    * Parameters:  Procedure name to make super.<br>
    * Notes:       NOTE: This presumes that we want only one copy of an ADM<br>
    *              super procedure running per session, meaning that they are <br>
    *              stateless and "multi-threaded". This is intended to be the case<br>
    *              for ours, but may not be true for all super procs.<br>
    *           -  The LAST-SUPER-PROCEDURE-PROP preprocessor allows classes<br>
    *              to specify a property of char or handle that stores<br>
    *              super-procedures that need to be kept at the bottom of the list <br>
    *              (defined by data.i and sbo.i if they are extended by other classes)
    */
   public void startSuperProc(final character _pcProcName)
   {
      character pcProcName = TypeFactory.initInput(_pcProcName);
      handle hProc = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hProc.assign(ProcedureManager.firstProcedure());
         
         loopLabel0:
         while (_and(hProc.isValid(), () -> isNotEqual(hProc.unwrapFile().getFileName(), pcProcName)))
         {
            hProc.assign(hProc.unwrap().getNextSibling());
         }
         
         if (_not(hProc.isValid()))
         {
            ControlFlowOps.invokePersistentSet((pcProcName).toStringMessage(), hProc);
         }
         
         thisProcedure().unwrapCommonProcedure().addSuperProcedure(hProc, ProcedureManager.SEARCH_TARGET);
         returnNormal();
      }));
   }

   /**
    * *********************  Internal Procedures **********************
    */
   public void admCreateObjects()
   {
      internalProcedure(new Block());
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     DISABLE the User Interface<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Here we clean-up the user-interface by deleting<br>
    *              dynamic widgets we have created and/or hide <br>
    *              frames.  This procedure is usually called when<br>
    *              we are ready to "clean-up" after running.
    */
   public void disableUi()
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* Hide all frames. */
         fmainFrame.hide();
         
         if ((thisProcedure().unwrapProcedure().isPersistent()).booleanValue())
         {
            HandleOps.delete(thisProcedure());
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     ENABLE the User Interface<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Here we display/view/enable the widgets in the<br>
    *              user-interface.  In addition, OPEN all queries<br>
    *              associated with each FRAME and BROWSE.<br>
    *              These statements here are based on the "Other <br>
    *              Settings" section of the widget Property Sheets.
    */
   public void enableUi()
   {
      internalProcedure(new Block((Body) () -> 
      {
         FrameElement[] elementList2 = new FrameElement[]
         {
            new Element(filterRoom, fmainFrame.widgetFilterRoom()),
            new Element(findGuest, fmainFrame.widgetFindGuest())
         };
         
         fmainFrame.display(elementList2);
         
         FrameElement[] elementList3 = new FrameElement[]
         {
            new WidgetElement(fmainFrame.widgetReselFilters()),
            new WidgetElement(fmainFrame.widgetFilterRoom()),
            new WidgetElement(fmainFrame.widgetFindGuest()),
            new WidgetElement(fmainFrame.widgetSearchButton()),
            new WidgetElement(fmainFrame.widgetReservationsBrowse()),
            new WidgetElement(fmainFrame.widgetAddButton()),
            new WidgetElement(fmainFrame.widgetUpdateButton()),
            new WidgetElement(fmainFrame.widgetDeleteButton()),
            new WidgetElement(fmainFrame.widgetCheckIn())
         };
         
         fmainFrame.enable(elementList3);
         
         query0.assign(new CompoundQuery().initialize(true, false));
         query0.addComponent(new AdaptiveQuery().initialize(reservation, "reservation.endDate >= ?", null, "reservation.roomNum asc, reservation.startDate asc, reservation.endDate asc", "WHOLE-INDEX,room-num-date", new Object[]
         {
            (P2JQuery.Parameter) () -> date.today()
         }, LockType.NONE));
         query0.addComponent(new AdaptiveQuery().initialize(room, ((String) null), null, "room.roomNum asc", "WHOLE-INDEX,room-num", reservation, LockType.NONE));
         query0.addComponent(new AdaptiveQuery().initialize(roomType, ((String) null), null, "roomType.roomType asc", "WHOLE-INDEX,type", room, LockType.NONE));
         query0.open();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void refreshReservations()
   {
      integer roomNum = UndoableFactory.integer((Long) null);
      character guestName = UndoableFactory.character(null);
      
      internalProcedure(new Block((Body) () -> 
      {
         roomNum.assign(fmainFrame.widgetFilterRoom().inputValue());
         
         if (_isEqual(roomNum, 0))
         {
            roomNum.setUnknown();
         }
         
         if (_and(notUnknown(fmainFrame.widgetFindGuest().inputValue()), () -> isNotEqual(trim((character) fmainFrame.widgetFindGuest().inputValue()), "")))
         {
            guestName.assign(concat("*", trim((character) fmainFrame.widgetFindGuest().inputValue()), "*"));
         }
         else
         {
            guestName.setUnknown();
         }
         
         query0.assign(new CompoundQuery().initialize(true, false));
         query0.addComponent(new AdaptiveQuery().initialize(reservation, "reservation.endDate >= ? and case when ? then reservation.roomNum = ? else true end and case when upper(reservation.guest) is not null then matches(upper(reservation.guest), upper(reservation.guest), true) else true end", null, "reservation.roomNum asc, reservation.startDate asc, reservation.endDate asc", "WHOLE-INDEX,room-num-date", new Object[]
         {
            (P2JQuery.Parameter) () -> date.today(),
            notUnknown(roomNum),
            roomNum
         }, LockType.NONE));
         query0.addComponent(new AdaptiveQuery().initialize(room, ((String) null), null, "room.roomNum asc", "WHOLE-INDEX,room-num", reservation, LockType.NONE));
         query0.addComponent(new AdaptiveQuery().initialize(roomType, ((String) null), null, "roomType.roomType asc", "WHOLE-INDEX,type", room, LockType.NONE));
         query0.open();
      }));
   }

   public handle fwdGetMainFrame()
   {
      return function("fwdGetMainFrame", handle.class, new Block((Body) () -> 
      {
         returnNormal(fmainFrame.asWidgetHandle());
      }));
   }

   public character fwdGetAdmversion()
   {
      return function("fwdGetADMVersion", character.class, new Block((Body) () -> 
      {
         returnNormal("ADM2.2");
      }));
   }

   public class TriggerBlock0
   extends Trigger
   {
      logical needRefresh = UndoableFactory.logical();

      rowid rid = UndoableFactory.rowid();

      rowid createdReservationRid = UndoableFactory.rowid();

      public void body()
      {
         ControlFlowOps.invokeWithMode("update-reservation-dialog.w", "IOO", rid, needRefresh, createdReservationRid);
         
         if ((needRefresh).booleanValue())
         {
            ControlFlowOps.invoke("refresh-reservations");
            silent(() -> query0.repositionByID(createdReservationRid));
         }
      }
   }

   public class TriggerBlock1
   extends Trigger
   {
      logical needRefresh = UndoableFactory.logical();

      rowid rid = UndoableFactory.rowid();

      rowid createdStayRid = UndoableFactory.rowid();

      public void body()
      {
         if (_not(reservation.available()))
         {
            messageBox("There are no selected reservations!", ALERT_INFO, BTN_OK, (String) null);
            returnConsume();
         }
         
         doBlock(TransactionType.FULL, "blockLabel0", new Block((Body) () -> 
         {
            ControlFlowOps.invokeWithMode("update-stay-dialog.w", "IOOIIIII", rid, needRefresh, createdStayRid, new logical(false), reservation.getRoomNum(), reservation.getStartDate(), reservation.getEndDate(), reservation.getGuest());
            
            if ((needRefresh).booleanValue())
            {
               NamedEventManager.publish("refresh-stays", thisProcedure());
            }
            else
            {
               undoLeave("blockLabel0");
            }
         }));
      }
   }

   public class TriggerBlock2
   extends Trigger
   {
      logical ret = UndoableFactory.logical();

      public void body()
      {
         if (_not(reservation.available()))
         {
            messageBox("There are no selected reservations!", ALERT_INFO, BTN_OK, (String) null);
            returnConsume();
         }
         
         messageBox(new Object[]
         {
            concat(new character("Are you sure you want to delete reservation of "), valueOf((character) new FieldReference(reservation, "guest").getValue())),
            "?"
         }, false, new AccessorWrapper(ret), ALERT_QUESTION, BTN_YES_NO, "Deletion confirmation");
         
         if (_not(ret))
         {
            returnConsume();
         }
         
         new FindQuery(reservation, (String) null, null, "reservation.roomNum asc, reservation.startDate asc, reservation.endDate asc", LockType.EXCLUSIVE).current();
         reservation.deleteRecord();
         ControlFlowOps.invoke("refresh-reservations");
      }
   }

   public class TriggerBlock3
   extends Trigger
   {
      logical needRefresh = UndoableFactory.logical();

      rowid rid = UndoableFactory.rowid();

      rowid unk = UndoableFactory.rowid();

      public void body()
      {
         if (_not(reservation.available()))
         {
            messageBox("There are no selected reservations!", ALERT_INFO, BTN_OK, (String) null);
            returnConsume();
         }
         
         rid.assign(reservation.rowID());
         ControlFlowOps.invokeWithMode("update-reservation-dialog.w", "IOO", rid, needRefresh, unk);
         
         if ((needRefresh).booleanValue())
         {
            ControlFlowOps.invoke("refresh-reservations");
            silent(() -> query0.repositionByID(rid));
         }
      }
   }
}
