/*
 * Copyright 2016-2017 Golden Code Development Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"),  to deal in the Software without
 * restriction, including without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo._temp.*;
import com.goldencode.hotel.dmo.hotel.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.ArrayAssigner.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;
import static com.goldencode.p2j.util.ErrorManager.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in roomtypedo.w).
 */
public class Roomtypedo
{
   Rowobjupd_1_1.Buf rowobjupd = TemporaryBuffer.define(Rowobjupd_1_1.Buf.class, "rowobjupd", "RowObjUpd", false, false);

   /* if not adm-load-from-repos */   
   Rowobject_1_1.Buf rowobject = TemporaryBuffer.define(Rowobject_1_1.Buf.class, "rowobject", "RowObject", false);

   RoomType.Buf roomType = RecordBuffer.define(RoomType.Buf.class, "hotel", "roomType", "room-type");

   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /* adecomm/appserv.i checks to see if the persistent procedure adecomm/as-utils.w
                        has been started up, and if not, it starts it and then makes
                        is a SUPER procedure of the calling procedure.
                        
      Created: 11/26/97 by Ross Hunter
                                                                                    */   
   handle appSrvUtils = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "appsrvutils", true, new handle());

   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* Db-Required definitions. */   
   /* Internal Tables (found by Frame, Query & Browse Queries)             */   
   /* Definitions for QUERY Query-Main                                     */   
   /* Custom List Definitions                                              */   
   /* List-1,List-2,List-3,List-4,List-5,List-6                            */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* ************************  Function Prototypes ********************** */   
   /* TODO: is the openQuery override needed?
   {&DB-REQUIRED-START}
   
   &ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD openQuery dTables  _DB-REQUIRED
   FUNCTION openQuery RETURNS LOGICAL
     ( _* parameter-definitions *_ )  FORWARD.
   
   _* _UIB-CODE-BLOCK-END *_
   &ANALYZE-RESUME
   
   {&DB-REQUIRED-END}
   */   
   /* ***********************  Control Definitions  ********************** */   
   /* Query definitions                                                    */   
   final QueryWrapper query0 = new QueryWrapper("Query-Main", true);

   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB)
     CREATE WINDOW dTables ASSIGN
            HEIGHT             = 1.85
            WIDTH              = 46.57.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*
   Description:
       Super procedure general functions
       Includes an extension of start-super-proc which also copies super
       procedures from THIS-PROCEDURE to a newly-ran super proc
       copy of CMS cosuprmli0.i
       
   Arguments:
           
   Comments:
   
   ************************************************************************/   
   /* ***************************  Definitions  ************************** */   
   /*
   Description: Defines debug global variables for adbgmsg0.i
   ************************************************************************/   
   character goisDebug = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "goisdebug", true, new character(""));

   handle ghDebugWindowProc = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "ghdebugwindowproc", true, new handle());

   handle lhClientFunctionsSuperProcedure = TypeFactory.handle();

   HandleExpr0 handleExpr0 = new HandleExpr0();

   HandleExpr1 handleExpr1 = new HandleExpr1();

   HandleExpr2 handleExpr2 = new HandleExpr2();

   HandleExpr3 handleExpr3 = new HandleExpr3();

   HandleExpr4 handleExpr4 = new HandleExpr4();

   HandleExpr5 handleExpr5 = new HandleExpr5();

   /* These preprocessors tell at compile time which properties can
        be retrieved directly from the temp-table */   
   /* include properties from smart */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   final QueryWrapper query1 = new QueryWrapper("qDataQuery", true);

   /**
    * External procedure (converted to Java from the 4GL source code
    * in roomtypedo.w).
    */
   public void execute()
   {
      /* For {get/set} */
      handle ghAdmprops = TypeFactory.handle();
      /* Handle of prop t-t */
      handle ghAdmpropsBuf = TypeFactory.handle();
      /*  and its buffer    */
      logical glAdmloadFromRepos = TypeFactory.logical();
      character cObjectName = TypeFactory.character();
      integer iStart = TypeFactory.integer();
      /* Note: This ENDIF matches the adm-smart definition at the top of file.
         Do not delete it and if another procedure or function is defined which
         occurs later in the file, move it to the end of that. */
      /* _UIB-CODE-BLOCK-END */
      /* _UIB-CODE-BLOCK-END */
      /* ***************************  Main Block  *************************** */
      character cAppService = TypeFactory.character();
      character cAsdivision = TypeFactory.character();
      character cServerOperatingMode = TypeFactory.character();
      character cQueryString = TypeFactory.character();
      handle hRowObject = TypeFactory.handle();
      handle hDataQuery = TypeFactory.handle();
      character cColumns = TypeFactory.character();
      character cDataFieldDefs = TypeFactory.character();
      
      externalProcedure(Roomtypedo.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         ProcedureManager.registerFunctionHandle("FcCalculateStringValue", handleExpr5);
         ProcedureManager.registerFunctionHandle("FcOrderStringList", handleExpr4);
         ProcedureManager.registerFunctionHandle("FlIsExistingDirectory", handleExpr3);
         ProcedureManager.registerFunctionHandle("FcClearCombo", handleExpr2);
         ProcedureManager.registerFunctionHandle("FcString_RemoveNonNumeric", handleExpr1);
         ProcedureManager.registerFunctionHandle("FcConvDataType", handleExpr0);
         query0.addBuffer(roomType, true);
         query1.addBuffer(rowobject, true);
         RecordBuffer.openScope(roomType, rowobject, rowobjupd);
         
         if (_not(appSrvUtils.isValid()))
         {
            ControlFlowOps.invokePersistentSet("adecomm/as-utils.w", appSrvUtils);
         }
         
         thisProcedure().unwrapCommonProcedure().addSuperProcedure(appSrvUtils);
         /* Create an unnamed pool to store all the widgets created
              by this procedure. This is a good default which assures
              that this procedure's triggers and internal procedures
              will execute in this procedure's storage, and that proper
              cleanup will occur on deletion of the procedure. */
         WidgetPool.create();
         
         /* ************************  Frame Definitions  *********************** */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: SmartDataObject
            Allow: Query
            Frames: 0
            Add Fields to: Neither
            Other Settings: PERSISTENT-ONLY APPSERVER DB-AWARE
          */
         /* This procedure should always be RUN PERSISTENT.  Report the error,  */
         /* then cleanup and return.                                            */
         if (_not(thisProcedure().unwrapProcedure().isPersistent()))
         {
            messageBox("roomtypedo.w should only be RUN PERSISTENT.", ALERT_ERROR, BTN_OK, (String) null);
            returnNormal();
         }
         
         if (_not(lhClientFunctionsSuperProcedure.isValid()))
         {
            lhClientFunctionsSuperProcedure.assign(ProcedureManager.firstProcedure());
            
            loopLabel1:
            while (_and(lhClientFunctionsSuperProcedure.isValid(), () -> isNotEqual(lhClientFunctionsSuperProcedure.unwrapFile().getFileName(), "aclifli0.p")))
            {
               lhClientFunctionsSuperProcedure.assign(lhClientFunctionsSuperProcedure.unwrap().getNextSibling());
            }
            
            if (_not(lhClientFunctionsSuperProcedure.isValid()))
            {
               ControlFlowOps.invokePersistentSet("aclifli0.p", lhClientFunctionsSuperProcedure);
            }
            
            thisProcedure().unwrapCommonProcedure().addSuperProcedure(lhClientFunctionsSuperProcedure, ProcedureManager.SEARCH_TARGET);
         }
         
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         /* Note that adm-props-from-repository is intended to be replaced with
              a variable for flexible override, so it cannot be used for conditional 
              compile.  */
         if ((gshRepositoryManager.isValid()).booleanValue())
         {
            if (_not(new logical(ControlFlowOps.invokeDynamicFunctionInWithMode("prepareInstance", gshRepositoryManager, "II", valueOf(thisProcedure()), sourceProcedure()))))
            {
               stop();
            }
            
            ghAdmpropsBuf.assign(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))));
            glAdmloadFromRepos.assign(ghAdmpropsBuf.isValid());
         }
         
         /* VALID-HANDLE(gshRepositoryManager) */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            TempTableBuilder.create(ghAdmprops);
            ghAdmprops.unwrapTempTable().setCanUndo(new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectName"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectVersion"), new character("CHAR"), new integer(0), new character(), new character("ADM2.2"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectType"), new character("CHAR"), new integer(0), new character(), new character("SmartDataObject"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerType"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("PropertyDialog"), new character("CHAR"), new integer(0), new character(), new character("adm2/support/datad.w"));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryObject"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceProperties"), new character("CHAR"), new integer(0), new character(), new character("AppService,ASUsePrompt,ASInfo,ForeignFields,RowsToBatch,CheckCurrentChanged,RebuildOnRepos,ServerOperatingMode,DestroyStateless,DisconnectAppServer,ObjectName,UpdateFromSource,ToggleDataTargets,OpenOnInit,PromptOnDelete,PromptColumns"));
            /* NOTE: Any need to support &User-Supported-Links?? */
            ghAdmprops.unwrapTempTable().addNewField(new character("SupportedLinks"), new character("CHAR"), new integer(0), new character(), new character("Data-Source,Data-Target,Navigation-Target,Update-Target,Commit-Target,Commit-Source"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerHidden"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectInitialized"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            /*  "hidden" is a logical concept. */
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectHidden"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("HideOnInit"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("UIBMode"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerSource"), new character("HANDLE"));
            /* Note that datavis adds some events that are only required for updating objects */
            /* isUpdateActive event is needed only for data, datavis and container classes,
                 but we currently define it here.. (this simplifies the logic for datavis, 
                 which may or may not inherit from container)
                 confirmOk and confirmCancel only for container ans datavis */
            ghAdmprops.unwrapTempTable().addNewField(new character("ContainerSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("initializeObject,hideObject,viewObject,destroyObject,enableObject,confirmExit,confirmCancel,confirmOk,isUpdateActive"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSource"), new character("HANDLE"));
            /* Note that DataSourceEvents is overidden in data.i since some of these 
                 events are intended for visual targets; queryPosition, fetchDataSet, 
                 assignMaxdataguess and the most resent addition confirmUndo and 
                 confirmCommit. These two are NOT dataSourceEvents for SDOs as the message 
                 should not to be published to objects not affected by the commit/undo. */
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("dataAvailable,queryPosition,updateState,deleteComplete,fetchDataSet,confirmContinue,confirmCommit,confirmUndo,assignMaxGuess,isUpdatePending"));
            ghAdmprops.unwrapTempTable().addNewField(new character("TranslatableProperties"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ObjectPage"), new character("INT"), new integer(0), new character(), new integer(0));
            ghAdmprops.unwrapTempTable().addNewField(new character("DBAware"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("DesignDataObject"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataSourceNames"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataTargetEvents"), new character("CHARACTER"), new integer(0), new character(), new character("updateState,rowObjectState,fetchBatch,LinkState"));
            ghAdmprops.unwrapTempTable().addNewField(new character("LogicalObjectName"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalObjectName"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("LogicalVersion"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalVersion"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DynamicObject"), new character("LOGICAL"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RunAttribute"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ChildDataKey"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ParentDataKey"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataLinksEnabled"), new character("LOGICAL"), new integer(), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("InactiveLinks"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("InstanceId"), new character("DECIMAL"));
            /* SuperProcedure management */
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedureMode"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SuperProcedureHandle"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("LayoutPosition"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ClassName"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("RenderingProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ThinRenderingProcedure"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("Label"), new character("CHAR"), new integer(0), new character(), new unknown());
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("AppService"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASDivision"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ASHandle"), new character("HANDLE"), new integer(0), new character(), new unknown());
            /* Set to true on the first connection */
            ghAdmprops.unwrapTempTable().addNewField(new character("ASHasStarted"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASInfo"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASInitializeOnRun"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("ASUsePrompt"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("BindSignature"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("BindScope"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerOperatingMode"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerFileName"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerFirstCall"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("NeedContext"), new character("LOGICAL"), new integer(0), new character(), new unknown());
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("BaseQuery"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataColumns"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("FirstRowNum"), new character("INT"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("LastRowNum"), new character("INT"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FirstResultRow"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("LastResultRow"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataColumnsByTable"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationSource"), new character("CHAR"));
            ghAdmprops.unwrapTempTable().addNewField(new character("NavigationSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("fetchFirst,fetchNext,fetchPrev,fetchLast,startFilter"));
            ghAdmprops.unwrapTempTable().addNewField(new character("FilterWindow"), new character("CHAR"));
            ghAdmprops.unwrapTempTable().addNewField(new character("FilterSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ForeignFields"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryPosition"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("BufferHandles"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryRowIdent"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("OpenOnInit"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("ForeignValues"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("CheckLastOnOpen"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryString"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryColumns"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("LastDBRowIdent"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("AssignList"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdatableColumns"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdatableColumnsByTable"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("Tables"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("PhysicalTables"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("TempTables"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DBNames"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("KeyFields"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("KeyTableId"), new character("CHARACTER"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("FetchOnOpen"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataIsFetched"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("FilterActive"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("FilterAvailable"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("SkipTransferDBRow"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FetchHasAudit"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FetchHasComment"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FetchAutoComment"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("EntityFields"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RequiredProperties"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("TransferChildrenForAll"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("PositionForClient"), new character("CHARACTER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("AuditEnabled"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("NewBatchInfo"), new character("CHAR"), new integer(0), new character(), new unknown());
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            ghAdmprops.unwrapTempTable().addNewField(new character("RowObject"), new character("HANDLE"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RowObjUpd"), new character("HANDLE"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RowObjectTable"), new character("HANDLE"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RowObjUpdTable"), new character("HANDLE"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("AutoCommit"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataHandle"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataQueryString"), new character("CHAR"), new integer(0), new character(), new character("FOR EACH RowObject"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CurrentRowid"), new character("ROWID"));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateSource"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CurrentUpdateSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitSourceEvents"), new character("CHAR"), new integer(0), new character(), new character("commitTransaction,undoTransaction"));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitTarget"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("CommitTargetEvents"), new character("CHAR"), new integer(0), new character(), new character("rowObjectState"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataModified"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("RowsToBatch"), new character("INT"), new integer(0), new character(), new integer(200));
            /* Rows per AppServer xfer */
            ghAdmprops.unwrapTempTable().addNewField(new character("CheckCurrentChanged"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataQueryBrowsed"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("RebuildOnRepos"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("RowObjectState"), new character("CHAR"), new integer(0), new character(), new character("NoUpdates"));
            ghAdmprops.unwrapTempTable().addNewField(new character("StatelessSavedProperties"), new character("CHAR"), new integer(0), new character(), new character("CheckCurrentChanged,RowObjectState,LastResultRow,FirstResultRow,QueryRowIdent"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DestroyStateless"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("DisconnectAppServer"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("ServerSubmitValidation"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataFieldDefs"), new character("CHARACTER"), new integer(0), new character(), new character("FIELD room-type AS INTEGER FORMAT \">>>>>>9\"    FIELD parent-type AS INTEGER FORMAT \">>>>>>9\"    FIELD description AS CHARACTER FORMAT \"x(20)\""));
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryContainer"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("IndexInformation"), new character("CHARACTER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("QueryContext"), new character("CHARACTER"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("FillBatchOnRepos"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("UpdateFromSource"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("AsynchronousSDO"), new character("LOGICAL"), new integer(), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("ToggleDataTargets"), new character("LOGICAL"), new integer(), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataLogicProcedure"), new character("CHARACTER"), new integer(), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataLogicObject"), new character("HANDLE"), new integer(), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataReadHandler"), new character("HANDLE"), new integer(), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataReadColumns"), new character("CHARACTER"), new integer(), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataReadBuffer"), new character("HANDLE"), new integer(), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("DataDelimiter"), new character("CHARACTER"), new integer(), new character(), new character("|"));
            ghAdmprops.unwrapTempTable().addNewField(new character("DataReadFormat"), new character("CHARACTER"), new integer(), new character(), new character("TrimNumeric"));
            ghAdmprops.unwrapTempTable().addNewField(new character("IsRowObjectExternal"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("IsRowObjUpdExternal"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            /* The following properties are used to store query manipluation strings made
               manually. If you change the query manually in code, you should set these
               properties. The filter window will retrieve these settings again when reapplying
               the filter, thus ensuring the original query is not corrupted by the Astra
               filter. Each property is chr(3) delimited to match the parameters that should
               be passed to each of the routines when reapplying them.
               
               Note that multiple entries are supported, seperated by chr(4), but this must
               be done manually - the set procedure does not do this automatically, but the
               filter will handle it when retrieving the values.
               To add multiple entries, be sure to do a get first and if a value exists, add
               a chr(4) before adding in your value and setting the new value.*/
            /* pcwhere + chr(3) + pcbuffer or empty or "?" + chr(3) + pcandor */
            ghAdmprops.unwrapTempTable().addNewField(new character("ManualAddQueryWhere"), new character("CHARACTER"));
            /* pccolumns + chr(3) + pcvalues + chr(3) + pcoperators */
            ghAdmprops.unwrapTempTable().addNewField(new character("ManualAssignQuerySelection"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("ManualSetQuerySort"), new character("CHARACTER"));
            ghAdmprops.unwrapTempTable().addNewField(new character("PromptOnDelete"), new character("LOGICAL"), new integer(0), new character(), new logical(true));
            ghAdmprops.unwrapTempTable().addNewField(new character("PromptColumns"), new character("CHAR"), new integer(0), new character(), new character(""));
            ghAdmprops.unwrapTempTable().addNewField(new character("DynamicData"), new character("LOGICAL"), new integer(0), new character(), new logical(false));
            ghAdmprops.unwrapTempTable().addNewField(new character("LastCommitErrorType"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("LastCommitErrorKeys"), new character("CHAR"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("RunDataLogicProxy"), new character("LOGICAL"), new integer(0), new character(), new unknown());
            ghAdmprops.unwrapTempTable().addNewField(new character("PrimarySDOSource"), new character("HANDLE"));
            ghAdmprops.unwrapTempTable().addNewField(new character("SchemaLocation"), new character("CHAR"), new integer(0), new character(), new character(""));
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 8.62
                  WIDTH              = 52.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ************************* Included-Libraries *********************** */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/smart.p"));
         }
         
         /* Use the old adm definition */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
            /* Now create the one record in this property temp-table, and store its
                   handle in ADM-DATA. The CHR(1) delimiters are to set aside spots
                   for UserProperties and UserLinks. */
            ghAdmprops.unwrapTempTable().tempTablePrepare("ADMProps");
            ghAdmpropsBuf.assign(ghAdmprops.unwrapTempTable().defaultBufferHandle());
            ghAdmpropsBuf.unwrapBuffer().bufferCreate();
            /* ... and save it. This ADM object is now open for business! */
            thisProcedure().unwrapADMData().setADMData(concat(valueOf(ghAdmpropsBuf), chr(1), chr(1)));
            /* from this point  {&ADM-PROPS-DEFINED} is true for all cases */
            cObjectName.assign("AppService,ASUsePrompt,ASInfo,ForeignFields,RowsToBatch,CheckCurrentChanged,RebuildOnRepos,ServerOperatingMode,DestroyStateless,DisconnectAppServer,ObjectName,UpdateFromSource,ToggleDataTargets,OpenOnInit,PromptOnDelete,PromptColumns");
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("InstanceProperties"));
            ghProp.unwrapBufferField().changeValue(cObjectName);
            cObjectName.assign(replaceAll(thisProcedure().unwrapFile().getFileName(), "\\", "/"));
            iStart.assign(plus(lastIndexOf(cObjectName, "/"), 1));
            cObjectName.assign(substring(cObjectName, iStart, minus(lastIndexOf(thisProcedure().unwrapFile().getFileName(), "."), iStart)));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
            ghProp.unwrapBufferField().changeValue(cObjectName);
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
            ghProp.unwrapBufferField().changeValue(new character("SmartDataObject"));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerType"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalVersion"));
            ghProp.unwrapBufferField().changeValue(new character(""));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalObjectName"));
            ghProp.unwrapBufferField().changeValue((_isNotEqual("", "") ? new character("") : thisProcedure().unwrapFile().getFileName()));
         }
         
         /* Starts super procedure */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/appserver.p"));
         }
         
         /* May have been retrieved from repository */
         /* {get Prop var [handle]} */ 
         cAppService.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAppService", targetProcedure()));
         
         /* NOTE: If REMOTE = true, we are running on an AppServer, so ignore the 
              AppService (partition) value, which is meaningful only to the client. 
              Also set the ASDivision property to Server.
              Note that AsDivision is set to 'Client' or blank in connectServer upon 
              in appserver.p */
         if ((SessionUtils.isRemote()).booleanValue())
         {
            cAppService.assign("");
            cAsdivision.assign("Server");
            cServerOperatingMode.assign(toUpperCase(SessionUtils.getServerOperatingMode()));
         }
         else
         {
            if (_isEqual(cAppService, ""))
            {
               cAppService.assign("");
            }
         }
         
         if (_isEqual(cAsdivision, ""))
         {
            cServerOperatingMode.assign("NONE");
         }
         else
         {
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setASDivision", targetProcedure(), "I", cAsdivision);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ServerOperatingMode"));
         ghProp.unwrapBufferField().changeValue(cServerOperatingMode);
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setAppService", targetProcedure(), "I", cAppService);
         
         /* _ADM-CODE-BLOCK-START _CUSTOM _INCLUDED-LIB-CUSTOM CUSTOM */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File        : appservercustom.i
             Purpose     : References the start of the custom super procedure.
                           Allows properties initialization.
         
             Syntax      : {src/adm2/custom/appservercustom.i}
         
             Description :
         
             Created     : 01/24/2001
             Notes       : Referenced in {src/adm2/appserver.i}
           ----------------------------------------------------------------------*/
         /*          This .W file was created with the Progress AppBuilder.      */
         /*----------------------------------------------------------------------*/
         /* ***************************  Definitions  ************************** */
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Include
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Include ASSIGN
                  HEIGHT             = 15
                  WIDTH              = 60.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Starts here the custom super procedure 
            Uncomment to run it */
         /*RUN start-super-proc ("adm2/custom/appservercustom.p":U).*/
         /* _UIB-CODE-BLOCK-END */
         /* _ADM-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/query.p"));
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/queryext.p"));
         }
         
         /* if not adm-load-from-repos */
         /* _ADM-CODE-BLOCK-START _CUSTOM _INCLUDED-LIB-CUSTOM CUSTOM */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File        : querycustom.i
             Purpose     : References the start of the custom super procedure.
                           Allows properties initialization.
         
             Syntax      : {src/adm2/custom/querycustom.i}
         
             Description :
         
             Created     : 06/03/1999
             Notes       : Referenced in {src/adm2/query.i}
           ----------------------------------------------------------------------*/
         /*          This .W file was created with the Progress AppBuilder.      */
         /*----------------------------------------------------------------------*/
         /* ***************************  Definitions  ************************** */
         /* _UIB-CODE-BLOCK-END */
         /* ********************  Preprocessor Definitions  ******************** */
         /* _UIB-PREPROCESSOR-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Include
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Include ASSIGN
                  HEIGHT             = 15
                  WIDTH              = 60.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Starts here the custom super procedure 
            Uncomment to run it */
         /*RUN start-super-proc ("adm2/custom/querycustom.p":U).*/
         /* _UIB-CODE-BLOCK-END */
         /* _ADM-CODE-BLOCK-END */
         ControlFlowOps.invokeIn("initProps", targetProcedure());
         
         /* _UIB-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         if (_not(glAdmloadFromRepos))
         {
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/data.p"));
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/dataext.p"));
            /* dataext.p is merely a simple "extension" of data.p.  This was necessary
                 because functions don't have there own action segement and data.p got
                 too big and had to be broken up.  All of the functions in dataext.p
                 are get and set property functions.  */
            ControlFlowOps.invokeWithMode("start-super-proc", "I", new character("adm2/dataextcols.p"));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceEvents"));
            ghProp.unwrapBufferField().changeValue(new character("dataAvailable,confirmContinue,isUpdatePending"));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DynamicData"));
         ghProp.unwrapBufferField().changeValue(new logical(false));
         cDataFieldDefs.assign("FIELD room-type AS INTEGER FORMAT \">>>>>>9\"    FIELD parent-type AS INTEGER FORMAT \">>>>>>9\"    FIELD description AS CHARACTER FORMAT \"x(20)\"");
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataFieldDefs"));
         ghProp.unwrapBufferField().changeValue(cDataFieldDefs);
         hDataQuery.assign(query1);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataHandle"));
         ghProp.unwrapBufferField().changeValue(hDataQuery);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataQueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         cQueryString /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "fixQueryString", targetProcedure(), "I", cQueryString));
         hDataQuery.unwrapQuery().prepare(cQueryString);
         hRowObject.assign(rowobject);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObject"));
         ghProp.unwrapBufferField().changeValue(hRowObject);
         /* Handle of RowObject buffer.*/
         hRowObject.assign(rowobjupd);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RowObjUpd"));
         ghProp.unwrapBufferField().changeValue(hRowObject);
         /* Row Update buffer handle. */
         hRowObject.assign(TempTableBuilder.asHandle(rowobject));
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowObjectTable", targetProcedure(), "I", hRowObject);
         /*  RowObject temp-table handle. */
         hRowObject.assign(TempTableBuilder.asHandle(rowobjupd));
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowObjUpdTable", targetProcedure(), "I", hRowObject);
         /*  RowObjUpd temp-table handle. */
         /* Don't let this object be an Update-Target if its fields are not updatable*/
         /* {get Prop var [handle]} */ 
         cColumns.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdatableColumns", targetProcedure()));
         
         /* Fetch this prop, set in query.i */
         if (_isEqual(cColumns, ""))
         {
            ControlFlowOps.invokeWithMode("modifyListProperty", "IIII", thisProcedure(), new character("REMOVE"), new character("SupportedLinks"), new character("Update-Target"));
         }
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Starts a super proc for THIS-PROCEDURE, then copies all existing<br>
    *              supers of THIS-PROCEDURE to supers of the newly ran super proc<br>
    *              (if it was newly ran by start-super-proc)
    */
   public void poisstartSuperProc(final character _pcProcName)
   {
      character pcProcName = TypeFactory.initInput(_pcProcName);
      character vcSuperProcsBefore = TypeFactory.character();
      handle vhSuperProc = TypeFactory.handle();
      /* This part copied from smrtprop.i start-super-proc */
      /*DEFINE INPUT PARAMETER pcProcName AS CHARACTER NO-UNDO.*/
      handle hProc = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         blockLabel0:
         {
            /* Debug Include {adbgmsg0.i} */
            if (_isEqual(goisDebug, "YES"))
            {
               if (_and(ghDebugWindowProc.isValid(), () -> isGreaterThan(lookup("PReceiveDebugMessage", ghDebugWindowProc.unwrapProcedure().internalEntries()), 0)))
               {
                  ControlFlowOps.invokeInWithMode("PReceiveDebugMessage", ghDebugWindowProc, "I", concat(EnvironmentOps.getSourceName(1), chr(1), "POISStartSuperProc running", chr(1), "PcProcName = ", (_notUnknown(pcProcName) ? valueOf(pcProcName) : new character("?")), chr(1)));
               }
               else
               {
                  messageBox(new Object[]
                  {
                     EnvironmentOps.getSourceName(new integer(1)),
                     new SkipEntity(1),
                     "POISStartSuperProc running",
                     new SkipEntity(true),
                     "PcProcName",
                     pcProcName,
                     new SkipEntity(true)
                  }, ALERT_INFO, BTN_OK, (String) null);
               }
            }
         }
         
         vcSuperProcsBefore.assign(thisProcedure().unwrapCommonProcedure().superProcedures());
         vhSuperProc.setUnknown();
         hProc.assign(ProcedureManager.firstProcedure());
         
         loopLabel0:
         while (_and(hProc.isValid(), () -> isNotEqual(hProc.unwrapFile().getFileName(), pcProcName)))
         {
            hProc.assign(hProc.unwrap().getNextSibling());
         }
         
         if (_not(hProc.isValid()))
         {
            ControlFlowOps.invokePersistentSet((pcProcName).toStringMessage(), hProc);
            vhSuperProc.assign(hProc);
         }
         
         /* 9.1 - added SEARCH-TARGET keyword */
         thisProcedure().unwrapCommonProcedure().addSuperProcedure(hProc, ProcedureManager.SEARCH_TARGET);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
            cType.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    *    Purpose:  Clone the current adm property temp-table<br>
    * Parameters: <br>
    *      Notes:  This is used to clone the dynamic property record for static <br>
    *              objects that does not call deleteProperties from destroyObject<br>
    *              in the target-procedure. This way the properties are kept alive <br>
    *              until destroyObject is completed without creating a memory leak,<br>
    *              since this TT is created in the object's widget-pool.
    */
   public void admCloneProps()
   {
      handle hReposBuffer = TypeFactory.handle();
      handle hPropTable = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hReposBuffer.assign(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))));
         
         if (_and(hReposBuffer.isValid(), () -> isNotEqual(hReposBuffer.unwrap().name(), "ADMProps")))
         {
            silent(() -> hReposBuffer.unwrapBuffer().findFirst(concat("WHERE Target = WIDGET-H(\"", valueOf(thisProcedure()), "\")")));
            
            if ((hReposBuffer.unwrapBuffer().available()).booleanValue())
            {
               TempTableBuilder.create(hPropTable);
               hPropTable.unwrapTempTable().createTableLike(hReposBuffer);
               hPropTable.unwrapTempTable().tempTablePrepare("ADMProps");
               hBuffer.assign(hPropTable.unwrapTempTable().defaultBufferHandle());
               hBuffer.unwrapBuffer().bufferCreate();
               hBuffer.unwrapBuffer().bufferCopy(hReposBuffer);
               /* delete the dynamic's property record */
               /* {fn <func-name> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionIn(logical.class, "deleteProperties", targetProcedure());
               /* ... and save it. This ADM object is now open for business! */
               thisProcedure().unwrapADMData().setADMData(concat(valueOf(hBuffer), chr(1), chr(1)));
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Procedure to start a super proc if it's not already running, <br>
    *              and to add it as a super proc in any case.<br>
    * Parameters:  Procedure name to make super.<br>
    * Notes:       NOTE: This presumes that we want only one copy of an ADM<br>
    *              super procedure running per session, meaning that they are <br>
    *              stateless and "multi-threaded". This is intended to be the case<br>
    *              for ours, but may not be true for all super procs.<br>
    *           -  The LAST-SUPER-PROCEDURE-PROP preprocessor allows classes<br>
    *              to specify a property of char or handle that stores<br>
    *              super-procedures that need to be kept at the bottom of the list <br>
    *              (defined by data.i and sbo.i if they are extended by other classes)
    */
   public void startSuperProc(final character _pcProcName)
   {
      character pcProcName = TypeFactory.initInput(_pcProcName);
      handle hProc = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hProc.assign(ProcedureManager.firstProcedure());
         
         loopLabel2:
         while (_and(hProc.isValid(), () -> isNotEqual(hProc.unwrapFile().getFileName(), pcProcName)))
         {
            hProc.assign(hProc.unwrap().getNextSibling());
         }
         
         if (_not(hProc.isValid()))
         {
            ControlFlowOps.invokePersistentSet((pcProcName).toStringMessage(), hProc);
         }
         
         thisProcedure().unwrapCommonProcedure().addSuperProcedure(hProc, ProcedureManager.SEARCH_TARGET);
         returnNormal();
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Sets all the object's properties that are related to the<br>
    *              Query, during initialization.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void initProps()
   {
      character cTable = TypeFactory.character();
      integer iTable = TypeFactory.integer();
      character cColumns = TypeFactory.character();
      character cDataCols = TypeFactory.character();
      character cUpdCols = TypeFactory.character();
      character cCalcData = TypeFactory.character();
      character cCalcUpd = TypeFactory.character();
      integer iNumData = TypeFactory.integer();
      integer iNumUpd = TypeFactory.integer();
      character cBuffers = TypeFactory.character("");
      character cAssignList = TypeFactory.character();
      integer iAssigns = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      integer iEntry = TypeFactory.integer();
      integer iCount = TypeFactory.integer();
      character cTables = TypeFactory.character();
      character[] cTableAssign = TypeFactory.characterExtent(19, "");
      character cDbEntry = TypeFactory.character();
      character cField = TypeFactory.character();
      character cQueryString = TypeFactory.character();
      /* Server side logic */
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      character cOpenQuery = TypeFactory.character();
      character cDbnames = TypeFactory.character();
      character cPhysicalTables = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* New for 9.1D to allow storage of this in the client SDO so the one-hit 
              appserver can be utilized also oat initialization */
         cQueryString.assign("");
         hQuery.assign(query0);
          /* Database query */hQuery.unwrapADMData().setADMData(concat((_notUnknown(hQuery.unwrapADMData().getADMData()) ? concat(hQuery.unwrapADMData().getADMData(), chr(1)) : new character("")), "STATIC"));
         
         loopLabel3:
         for (iTable.assign(1); _isLessThanOrEqual(iTable, hQuery.unwrapQuery().numBuffers()); iTable.increment())
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iTable));
            cBuffers.assign(concat(cBuffers, (_isEqual(iTable, 1) ? new character("") : new character(",")), valueOf(hBuffer)));
            cPhysicalTables.assign(concat(cPhysicalTables, (_isEqual(iTable, 1) ? new character("") : new character(",")), hBuffer.unwrapDatabaseInfo().getTable()));
            cDbnames.assign(concat(cDbnames, (_isEqual(iTable, 1) ? new character("") : new character(",")), hBuffer.unwrapDatabaseInfo().getDbName()));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         ghProp.unwrapBufferField().changeValue(cBuffers);
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDBNames", targetProcedure(), "I", cDbnames);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         ghProp.unwrapBufferField().changeValue(hQuery);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
         ghProp.unwrapBufferField().changeValue(cPhysicalTables);
         
         if (_isEqual(cQueryString, ""))
         {
            cOpenQuery.assign("OPEN QUERY Query-Main FOR EACH room-type NO-LOCK.");
            /* Next eliminate the "OPEN QUERY Query-Main" text. */
             /* NOTE: ' vs " parsing?? */iPos.assign(indexOf(cOpenQuery, " FOR "));
            /* No FOR keyword (reserved) found, check for PRESELECT (not reserved) */
            iPos.assign(_isEqual(iPos, 0) ? indexOf(cOpenQuery, " PRESELECT ") : iPos);
            cQueryString.assign(substring(cOpenQuery, plus(iPos, 1)));
         }
         
         /* Store the original query in the OpenQuery property and prepare the query 
              with it. 
              - setOpenQuery -> setQueryWhere -> prepareQuery */
         if (_not(new logical( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setOpenQuery", targetProcedure(), "I", cQueryString))))
         {
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "showMessage", targetProcedure(), "I", "5");
         }
         
         /* client */
         /* Define the AssignList property. This takes the ASSIGN-LIST preprocessor,
              which is in the form:
           RowObject.<ROfieldname><space>=<space><DBtable>.<DBfieldname><space><space>...
              and converts it to a series of lists of fields for each table:
           ROfieldname,DBfieldname[,...]{&adm-tabledelimiter}...
              This is a form usable by dynamic BUFFER-COPY and BUFFER-COMPARE.
           */
         cTables.assign(replaceAll("room-type", " ", ","));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         ghProp.unwrapBufferField().changeValue(cTables);
         /* List of DB table names */
         cAssignList.assign("");
         /* Start with the raw preproc value. */
         iAssigns.assign(numEntries(cAssignList, " "));
         
         loopLabel4:
         for (iEntry.assign(1); _isLessThanOrEqual(iEntry, iAssigns); iEntry.assign(plus(iEntry, 4)))
         {
            /* First get the DB table name and match it against the Tables list.  */
            cDbEntry.assign(entry(plus(iEntry, 2), cAssignList, " "));
            /* NOTE, we cannot use columnTable here because that will not work on 
                     a client, since that will resolve to query.p at this moment, because 
                     data.p, which has a non-db columnTable is not added as super yet! */
            cTable.assign(cDbEntry);
            /* remove field part from qualified fieldname */
            cTable.replaceEntry(numEntries(cTable, "."), ".", "");
            /* trim off remaining period */
            cTable.assign(rightTrim(cTable, "."));
            /* now lookup the [db.]table part of this in the table list */
            iTable.assign(lookup(cTable, cTables));
            
            if (_isGreaterThan(iTable, 0))
            {
               /* and this is the Database field name--ENTRY(3) if db qual, else 2:*/
               /* and this is the Database field name--ENTRY(3) if db qual, else 2:*/
               assignSingle(cTableAssign, iTable, concat(subscript(cTableAssign, iTable), (_isEqual(subscript(cTableAssign, iTable), "") ? new character("") : new character(",")), entry(2, entry(iEntry, cAssignList, " "), "."), ",", entry(numEntries(cDbEntry, "."), cDbEntry, ".")));
            }
         }
         
         /* END DO iEntry */
         /* Now put the list back together in the final form: */
         cAssignList.assign("");
         
         loopLabel5:
         for (iTable.assign(1); _isLessThanOrEqual(iTable, numEntries(cTables)); iTable.increment())
         {
            cAssignList.assign(concat(cAssignList, (_isEqual(iTable, 1) ? new character("") : new character(";")), subscript(cTableAssign, iTable)));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         ghProp.unwrapBufferField().changeValue(cAssignList);
         iNumUpd.assign(plus(iNumUpd, numEntries("room-type parent-type description", " ")));
         iNumData.assign(plus(iNumData, numEntries("room-type parent-type description", " ")));
         cUpdCols.assign(replaceAll("room-type parent-type description", " ", ","));
         cDataCols.assign(replaceAll("room-type parent-type description", " ", ","));
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         /*********************************************************************
         * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
         * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
         * below.  All Rights Reserved.                                       *
         *                                                                    *
         * The Initial Developer of the Original Code is PSC.  The Original   *
         * Code is Progress IDE code released to open source December 1, 2000.*
         *                                                                    *
         * The contents of this file are subject to the Possenet Public       *
         * License Version 1.0 (the "License"); you may not use this file     *
         * except in compliance with the License.  A copy of the License is   *
         * available as of the date of this notice at                         *
         * http://www.possenet.org/license.html                               *
         *                                                                    *
         * Software distributed under the License is distributed on an "AS IS"*
         * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
         * should refer to the License for the specific language governing    *
         * rights and limitations under the License.                          *
         *                                                                    *
         * Contributors:                                                      *
         *                                                                    *
         *********************************************************************/
         /*------------------------------------------------------------------------
             File       : adm2/colprep.i
             Purpose    : This include file is used in query.i to convert AB generated
                          preprocessors to comma-separated lists of columns 
                          separated by {&adm-tabledelimiter} for each table.  
          Preprocessors : &num - number part of AB generated preprocessors
                                 (FIRST, SECOND .. TENTH ) 
                          &enabledvar   - Char variable that accumulates enabled-fields
                          &datavar      - Chart variable that accumalates data-fields 
                          &enabledcount - Int var that counts database enabled-fields
                          &datavar      - Int var that counts database data-fields 
                          &adm-tabledelimiter - global delimiter used for column lists 
                                                that have entries per table         
                                                        
            Created     : November 16, 1999
         ----------------------------------------------------------------------*/
         /* &num <> first */
         /* if {&num}-table-in-query-{&query-name <> '' } */
         cColumns.assign(replaceAll("room-type parent-type description", " ", ","));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumns"));
         ghProp.unwrapBufferField().changeValue(cColumns);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(cUpdCols);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(cDataCols);
         
         loopLabel6:
         for (iCount.assign(1); _and(_isLessThanOrEqual(iCount, numEntries(cColumns)), or(isLessThan(iNumUpd, numEntries("room-type parent-type description", " ")), () -> isLessThan(iNumData, numEntries(cColumns)))); iCount.increment())
         {
            /* All fields are in data-fields so we don't care about enabled-fields */
            cField.assign(entry(iCount, cColumns));
            
            /* Replace {&adm-tabledelimiter} with commas in order to can-do */
            if (_not(matchesList(replaceAll(cDataCols, ";", ","), cField)))
            {
               cCalcData.assign(concat(cCalcData, (_isEqual(cCalcData, "") ? new character("") : new character(",")), cField));
               iNumData.assign(plus(iNumData, 1));
            }
            
            /* not can-do(datacols with commas,cfield) */
            /* Replace {&adm-tabledelimiter} with commas in order to can-do */
            if (_and(not(matchesList(replaceAll(cUpdCols, ";", ","), cField)), () -> isGreaterThan(lookup(cField, "room-type parent-type description", " "), 0)))
            {
               cCalcUpd.assign(concat(cCalcUpd, (_isEqual(cCalcUpd, "") ? new character("") : new character(",")), cField));
               iNumUpd.assign(plus(iNumUpd, 1));
            }
         }
         
         cDataCols.assign(concat(cDataCols, ";", cCalcData));
         cUpdCols.assign(concat(cUpdCols, ";", cCalcUpd));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(cUpdCols);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(cDataCols);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public logical deleteRecordStatic(final integer _piTableIndex_1)
   {
      integer piTableIndex_1 = TypeFactory.initInput(_piTableIndex_1);
      /*------------------------------------------------------------------------------
        Purpose:  Deletes a record from the specified table in the query.  This 
                  function is necessary in order to work around a limitation with the
                  BUFFER-DELETE() method for buffer handles: The method will fail if
                  there is delete validation defined on the target table.  The work
                  around is to use a static "DELETE {table}." statement instead.
          Notes:  Called internally by fetchDBRowForUpdate() in adm2/query.p
      ------------------------------------------------------------------------------*/
      logical lRet = TypeFactory.logical(true);
      
      return function("deleteRecordStatic", logical.class, TransactionType.FULL, new Block((Body) () -> 
      {
         switch (StringHelper.safeTrimTrailing((piTableIndex_1).toStringMessage()))
         {
            case "1":
               {
                  /*********************************************************************
                  * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
                  * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
                  * below.  All Rights Reserved.                                       *
                  *                                                                    *
                  * The Initial Developer of the Original Code is PSC.  The Original   *
                  * Code is Progress IDE code released to open source December 1, 2000.*
                  *                                                                    *
                  * The contents of this file are subject to the Possenet Public       *
                  * License Version 1.0 (the "License"); you may not use this file     *
                  * except in compliance with the License.  A copy of the License is   *
                  * available as of the date of this notice at                         *
                  * http://www.possenet.org/license.html                               *
                  *                                                                    *
                  * Software distributed under the License is distributed on an "AS IS"*
                  * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
                  * should refer to the License for the specific language governing    *
                  * rights and limitations under the License.                          *
                  *                                                                    *
                  * Contributors:                                                      *
                  *                                                                    *
                  *********************************************************************/
                  /*------------------------------------------------------------------------
                   File          : adm2/delrecst.i
                   Purpose       : This include file is used to produce DELETE
                                   statements for each table in the associated query. 
                                   The code is conditionally included when there is an 
                                   open query definition and when there is an "nth" 
                                   table referenced in the query.
                   Preprocessors : &num - number part of AB generated preprocessors
                                          (FIRST, SECOND .. TENTH )  
                   Created       : December 14, 1999
                   Referenced by : query.i
                  ----------------------------------------------------------------------*/
                  silent(() -> roomType.deleteRecord());
                  lRet.assign(not(ErrorManager.isError()));
               }
               break;
            case "2":
               {
                  break;
               }
            case "3":
               {
                  break;
               }
            case "4":
               {
                  break;
               }
            case "5":
               {
                  break;
               }
            case "6":
               {
                  break;
               }
            case "7":
               {
                  break;
               }
            case "8":
               {
                  break;
               }
            case "9":
               {
                  break;
               }
            case "10":
               {
                  break;
               }
            case "11":
               {
                  break;
               }
            case "12":
               {
                  break;
               }
            case "13":
               {
                  break;
               }
            case "14":
               {
                  break;
               }
            case "15":
               {
                  break;
               }
            case "16":
               {
                  break;
               }
            case "17":
               {
                  break;
               }
            case "18":
               {
                  break;
               }
         }
         
         returnNormal(lRet);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     wrapper for pre and postTransactionValidate procedures when<br>
    *              run from SBO.<br>
    * Parameters:  INPUT pcValType AS CHARACTER  "Pre" or "Post",<br>
    *              INPUT-OUTPUT TABLE FOR RowObjUpd.<br>
    * Notes:     - This is an override of the dynamic version in data.p and is <br>
    *              conditionally excluded if there is no RowObject TT include.
    */
   public void pushRowObjUpdTable(final TableParameter rowobjupdBuf2)
   {
      internalProcedure(new Block((Body) () -> 
      {
         TemporaryBuffer.associate(rowobjupdBuf2, rowobjupd, true, false);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     wrapper for pre and postTransactionValidate procedures when<br>
    *              run from SBO.<br>
    * Parameters:  INPUT pcValType AS CHARACTER  "Pre" or "Post",<br>
    *              INPUT-OUTPUT TABLE FOR RowObjUpd.<br>
    * Notes:     - This is an override of the dynamic version in data.p and is <br>
    *              conditionally excluded if there is no RowObject TT include.
    */
   public void pushTableAndValidate(final character _pcValType, final TableParameter rowobjupdBuf3)
   {
      character pcValType = TypeFactory.initInput(_pcValType);
      
      internalProcedure(new Block((Body) () -> 
      {
         TemporaryBuffer.associate(rowobjupdBuf3, rowobjupd, true, true);
         ControlFlowOps.invokeInWithMode("bufferValidate", targetProcedure(), "I", pcValType);
         returnNormal(ControlFlowOps.getReturnValue());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Procedure executed on a server side SmartDataObject.   <br>
    *              This is the equivalent of serverCommit, but can be run in <br>
    *              a not intialized object as it has input-ouput parameters for \<br>
    *              context.  <br>
    * Parameters: <br>
    *  INPUT-OUTPUT  pcContext  - in Contextfrom client <br>
    *                             out new context<br>
    *  INPUT-OUTPUT TABLE RowObjUpd - The Update version of the RowObject <br>
    *                                 Temp-Table<br>
    * <p>
    *  OUTPUT cMessages - a CHR(3) delimited string of accumulated messages from<br>
    *                     server.<br>
    *  OUTPUT cUndoIds  - list of any RowObject ROWIDs whose changes need to be <br>
    *                      undone as the result of errors in the form of:<br>
    *              "RowNumCHR(3)ADM-ERROR-STRING,RowNumCHR(3)ADM-ERROR-STRING,..."<br>
    * <p>
    * Notes:       If another user has modified the database records since the <br>
    *              original was read, the new database values are copied into the <br>
    *              RowObjUpd record and returned to Commit so the UI object can <br>
    *              display them.<br>
    *            - This is an override of the dynamic version in data.p and is <br>
    *              conditionally excluded if there is no RowObject TT include.
    */
   public void remoteCommit(final character pcContext, final TableParameter rowobjupdBuf4, final character pcMessages, final character pcUndoIds)
   {
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(pcMessages);
         TypeFactory.initOutput(pcUndoIds);
      }, 
      (Body) () -> 
      {
         TemporaryBuffer.associate(rowobjupdBuf4, rowobjupd, true, true);
         ControlFlowOps.invokeInWithMode("setContextAndInitialize", targetProcedure(), "I", pcContext);
         ControlFlowOps.invokeInWithMode("bufferCommit", targetProcedure(), "OO", pcMessages, pcUndoIds);
         pcContext /* {fn <func-name> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "obtainContextForClient", targetProcedure()));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *  Purpose:     Update procedure executed on the server side of a split <br>
    *               SmartDataObject, called from the client Commit function.<br>
    *               Commit passes a set of RowObjUpdate records both have changes to<br>
    *               be committed and their pre-change copies (before-images).  <br>
    *               commitRows verifies that the records have not been changed <br>
    *               since they were read, and then commits the changes to the <br>
    *               database.<br>
    *  Parameters:<br>
    *   INPUT-OUTPUT TABLE RowObjUpd - The Update version of the RowObject <br>
    *                                  Temp-Table<br>
    * <p>
    *   OUTPUT cMessages - a CHR(3) delimited string of accumulated messages from<br>
    *                       server.<br>
    *    OUTPUT cUndoIds  - list of any RowObject ROWIDs whose changes need to be <br>
    *                       undone as the result of errors in the form of:<br>
    *               "RowNumCHR(3)ADM-ERROR-STRING,RowNumCHR(3)ADM-ERROR-STRING,..."<br>
    * <p>
    * Notes:        If another user has modified the database records since the <br>
    *               original was read, the new database values are copied into the <br>
    *               RowObjUpd record and returned to Commit so the UI object can <br>
    *               display them.<br>
    *             - This is an override of the dynamic version in data.p and is <br>
    *               conditionally excluded if there is no RowObject TT include.
    */
   public void serverCommit(final TableParameter rowobjupdBuf5, final character pcMessages, final character pcUndoIds)
   {
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(pcMessages);
         TypeFactory.initOutput(pcUndoIds);
      }, 
      (Body) () -> 
      {
         TemporaryBuffer.associate(rowobjupdBuf5, rowobjupd, true, true);
         ControlFlowOps.invokeInWithMode("bufferCommit", targetProcedure(), "OO", pcMessages, pcUndoIds);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public handle getRowObjUpdStatic()
   {
      return function("getRowObjUpdStatic", handle.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: returns the static update buffer handle   
             Notes: Defined as private since it currently is used (in desperation) when 
                    default-buffer-handle is unknown in setRowObjUpdTable.
                    This problem occurs when an SBO runs locally; as 
                    serverCommitTransaction( input-output table-handle) makes the 
                    default-buffer-handle unknown. Maybe a side-effect of the fact that
                    the buffer is static on the server? 
         ------------------------------------------------------------------------------*/
         returnNormal(rowobjupd);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     DISABLE the User Interface<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Here we clean-up the user-interface by deleting<br>
    *              dynamic widgets we have created and/or hide<br>
    *              frames.  This procedure is usually called when<br>
    *              we are ready to "clean-up" after running.
    */
   public void disableUi()
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* Hide all frames. */
         if ((thisProcedure().unwrapProcedure().isPersistent()).booleanValue())
         {
            HandleOps.delete(thisProcedure());
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Super Override<br>
    * Parameters:<br>
    * Notes:       This procedure is not db-required so exists on both client and<br>
    *              server
    */
   public void initializeObject()
   {
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("openOnInit"));
         ghProp.unwrapBufferField().changeValue(new logical(false));
         /* 25/03/2003 - The initialization of the frame is taking too
                                  long - Testing with stopping the open query which
                                  is not necessary anyway, because the query does not
                                  have a valid string anyway
                                  */
         ControlFlowOps.invokeFunction("openQuery");
         ControlFlowOps.runSuper("initializeObject");
      }));
   }

   public character fwdGetAdmversion()
   {
      return function("fwdGetADMVersion", character.class, new Block((Body) () -> 
      {
         returnNormal("ADM2.2");
      }));
   }

   class HandleExpr0
   extends GenericExpression
   {
      public BaseDataType resolve()
      {
         return lhClientFunctionsSuperProcedure;
      }
   }

   class HandleExpr1
   extends GenericExpression
   {
      public BaseDataType resolve()
      {
         return lhClientFunctionsSuperProcedure;
      }
   }

   class HandleExpr2
   extends GenericExpression
   {
      public BaseDataType resolve()
      {
         return lhClientFunctionsSuperProcedure;
      }
   }

   class HandleExpr3
   extends GenericExpression
   {
      public BaseDataType resolve()
      {
         return lhClientFunctionsSuperProcedure;
      }
   }

   class HandleExpr4
   extends GenericExpression
   {
      public BaseDataType resolve()
      {
         return lhClientFunctionsSuperProcedure;
      }
   }

   class HandleExpr5
   extends GenericExpression
   {
      public BaseDataType resolve()
      {
         return lhClientFunctionsSuperProcedure;
      }
   }
}
