/*--------------------------------------------------------------------------
    File        : Queryext.java
   Purpose     : Support procedure for Query Object.  This is an extension
                 of query.p.  The extension is necessary to avoid an overflow
                 of the action segment on AS400. This extension file contains
                 all of the get and set property functions. These functions 
                 will be rolled back into query.p when segment size increases.

    Syntax      : adm2/query.p

    Modified    : Jan 22, 2001 Version 9.1C
    Modified    : 10/24/2001          Mark Davied (MIP)
                  Added new property FilterAvailable and it's get and set
                  functions.
    Modified    : 02/27/2002          Gikas A. Gikas
                  Incidental bug fixed while working on IZ 4050
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.util.ErrorManager.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/queryext.p).
 */
public class Queryext
{
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 4.57
            WIDTH              = 42.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* Preprocessors to define properties that can be retrieved
       directly from the property temp-table. */   
   /* if appserver aware add appserver properties */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/queryext.p).
    */
   public void execute()
   {
      externalProcedure(Queryext.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getAssignList()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Returns the list of updatable columns whose names have been 
                    modified in the SmartDataObject.
      
        Parameters: <none>
      
        Notes:      This string is of the form:
                    <RowObjectFieldName>,<DBFieldName>[,...][{&adm-tabledelimiter}...]
                    with a comma separated list of pairs of fields for each db table,
                    and {&adm-tabledelimiter} between the lists of pairs.
      ------------------------------------------------------------------------------*/
      character cList = TypeFactory.character();
      
      return function("getAssignList", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         cList.assign(ghProp.unwrapBufferField().value());
         returnNormal(cList);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getAuditEnabled()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns auditEnabled property value
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lAuditEnabled_1 = TypeFactory.logical();
      
      return function("getAuditEnabled", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AuditEnabled"));
         lAuditEnabled_1.assign(ghProp.unwrapBufferField().value());
         returnNormal(lAuditEnabled_1);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getBaseQuery()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the base query property 
          Notes: This is used internally by openQuery, and directly on client context 
                 management. Because setOpenQuery also setOpenQuery and wipes out 
                 all other query data it cannot be used when setting this when
                 received from server  
      ------------------------------------------------------------------------------*/
      character cBaseQuery = TypeFactory.character();
      
      return function("getBaseQuery", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
         cBaseQuery.assign(ghProp.unwrapBufferField().value());
         returnNormal(cBaseQuery);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getBufferHandles()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cBufferHandles = TypeFactory.character();
      
      return function("getBufferHandles", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         cBufferHandles.assign(ghProp.unwrapBufferField().value());
         returnNormal(cBufferHandles);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getCalculatedColumns()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns a comma-delimited list of the calculated columns for the 
                     SmartDataObject. 
        Parameters:  
             Notes:  Uses the DataColumsByTable property that stores fields for 
                     different tables delimited by {&adm-tabledelimiter} where the last entry may be 
                     calculated fields
        preprocessor - adm-tabledelimiter - defaults to ''       
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      character cTables = TypeFactory.character();
      
      return function("getCalculatedColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         
         if (_isGreaterThan(numEntries(cColumns, ";"), numEntries(cTables)))
         {
            cColumns.assign(entry(plus(numEntries(cTables), 1), cColumns, ";"));
         }
         else
         {
            cColumns.assign("");
         }
         
         returnNormal(cColumns);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getCheckLastOnOpen()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Returns the flag indicating whether a get-last should be performed 
                    on an open in order for fetchNext to be able to detect that we are 
                    on the last row. This is necessary to make the QueryPosition 
                    attribute reliable.   
      
        Parameters: <none>
      ------------------------------------------------------------------------------*/
      logical lCheck = TypeFactory.logical();
      
      return function("getCheckLastOnOpen", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CheckLastOnOpen"));
         lCheck.assign(ghProp.unwrapBufferField().value());
         returnNormal(lCheck);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataColumns()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns a comma-delimited list of the columnNames for the 
                     SmartDataObject. 
        Parameters:  
             Notes:   
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      
      return function("getDataColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumns"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         returnNormal(cColumns);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataColumnsByTable()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns a comma-delimited list of the columnNames delimited 
                     by {&adm-tabledelimiter} to identify which columns are from 
                     which table 
        Parameters:  
             Notes:  Use DataColums to get a comma separated list        
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      
      return function("getDataColumnsByTable", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         returnNormal(cColumns);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getDataIsFetched()
   {
      /*------------------------------------------------------------------------------
         Purpose: The SBO sets this to true in the SDO when it has fethed 
                  data on the SDOs behalf in order to prevent that the SDO does 
                  another server call to fetch the data it already has. 
                  This is checked in query.p dataAvailable and openQuery is skipped
                  if its true. It's immediately turned off after it is checked.    
          Notes: Duplicated in sboext 
      Note Date: 2002/04/14     
      ------------------------------------------------------------------------------*/
      logical lFetched = TypeFactory.logical();
      
      return function("getDataIsFetched", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataIsFetched"));
         lFetched.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFetched);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDbnames()
   {
      /*------------------------------------------------------------------------------
         Purpose:  Returns a comma delimited list of DBNames that corresponds 
                   to the Tables in the Query Objects
      Parameters:  <none>
           Notes: This does not have an xpTables preprocessor because getDBNames
                  is overridden in data.p.   
      ------------------------------------------------------------------------------*/
      character cNames = TypeFactory.character();
      
      return function("getDBNames", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DBNames"));
         cNames.assign(ghProp.unwrapBufferField().value());
         returnNormal(cNames);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getEnabledTables()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a list of the database tables which have enabled fields.
         Params:  <none>
          Notes:    
      ------------------------------------------------------------------------------*/
      character cTables = TypeFactory.character();
      character cEnabledTables = TypeFactory.character();
      character cUpdColumns = TypeFactory.character();
      integer iVar = TypeFactory.integer();
      
      return function("getEnabledTables", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         cUpdColumns.assign(ghProp.unwrapBufferField().value());
         
         loopLabel0:
         for (iVar.assign(1); _isLessThanOrEqual(iVar, numEntries(cTables)); iVar.increment())
         {
            if (_isNotEqual(entry(iVar, cUpdColumns, ";"), ""))
            {
               cEnabledTables.assign(concat(cEnabledTables, (_isEqual(cEnabledTables, "") ? new character("") : new character(",")), entry(iVar, cTables)));
            }
         }
         
         returnNormal(cEnabledTables);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getEntityFields()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cEntityFields_1 = TypeFactory.character();
      
      return function("getEntityFields", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EntityFields"));
         cEntityFields_1.assign(ghProp.unwrapBufferField().value());
         returnNormal(cEntityFields_1);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFetchAutoComment()
   {
      /*------------------------------------------------------------------------------
        Purpose: Set to true if auto comments should be retrieved with the 
                 data. 
          Notes: This is used in transferRows and transferDbRow.
              -  These comments will be automatically displayed by visual objects and
                 will be set to true by the visual object's LinkStateHandler if the 
                 property value is unknown.
      ------------------------------------------------------------------------------*/
      logical lFetchAutoComment = TypeFactory.logical();
      
      return function("getFetchAutoComment", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchAutoComment"));
         lFetchAutoComment.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFetchAutoComment);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFetchHasAudit()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true if Audit exists flag should be retrieved with the data 
          Notes: This is used in transferRows and transferDbRow 
      ------------------------------------------------------------------------------*/
      logical lFetchHasAudit = TypeFactory.logical();
      
      return function("getFetchHasAudit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasAudit"));
         lFetchHasAudit.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFetchHasAudit);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFetchHasComment()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns true if 'comments exists' flag should be retrieved with the 
                 data. 
          Notes: This is used in transferRows and transferDbRow 
      ------------------------------------------------------------------------------*/
      logical lFetchHasComment = TypeFactory.logical();
      
      return function("getFetchHasComment", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasComment"));
         lFetchHasComment.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFetchHasComment);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFetchOnOpen()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return what/whether a fetch should occur when the db query is opened
          Notes: A blank value means don't fetch on open, any other value is just 
                 being run as fetch + <property value>
               - Unknown value indicates default which is blank on 'server' and
                 'first' otherwise.
               - This is a replacement of the return that used to be in data.p 
                 fetchFirst. The blank gives the same effect as it prevents 
                 openQuery from calling fetchFirst.
                   
      Note Date: 2002/4/11             
      ------------------------------------------------------------------------------*/
      character cFetchOnOpen = TypeFactory.character();
      character cAsDivision = TypeFactory.character();
      logical lQueryCont = TypeFactory.logical();
      
      return function("getFetchOnOpen", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchOnOpen"));
         cFetchOnOpen.assign(ghProp.unwrapBufferField().value());
         
         if (_isUnknown(cFetchOnOpen))
         {
            /* If inside an sbo we want openQuery to fetchfirst (as always), 
                   so we do notneed ot chekc for the division */
            /* {get Prop var [handle]} */ 
            silent(() -> lQueryCont.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getQueryContainer", targetProcedure())));
            
            if (_not(isEqual(lQueryCont, true)))
            {
               /* {get Prop var [handle]} */ 
               cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAsDivision", targetProcedure()));
               
               if (_isEqual(cAsDivision, "Server"))
               {
                  cFetchOnOpen.assign("");
               }
            }
            
            if (_isUnknown(cFetchOnOpen))
            {
               cFetchOnOpen.assign("First");
            }
         }
         returnNormal(cFetchOnOpen);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFilterActive()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return whether a filter is active.  
          Notes: It may be set to true explicitly or use the Querycolumns   
      ------------------------------------------------------------------------------*/
      logical lFilterActive = TypeFactory.logical();
      character cQueryColumns_1 = TypeFactory.character();
      
      return function("getFilterActive", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterActive"));
         lFilterActive.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         cQueryColumns_1.assign(ghProp.unwrapBufferField().value());
         returnNormal(and(lFilterActive, () -> isNotEqual(cQueryColumns_1, "")));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFilterAvailable()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return whether a filter is available. 
          Notes: 
      ------------------------------------------------------------------------------*/
      logical lFilterAvailable = TypeFactory.logical();
      
      return function("getFilterAvailable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterAvailable"));
         lFilterAvailable.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFilterAvailable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFirstResultRow()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the FirstResultRow (unknown if first row hasn't been
                  fetched, 1 concatinated with the rowid if it has.)
          Notes:  
      ------------------------------------------------------------------------------*/
      character cFirstResultRow = TypeFactory.character();
      
      return function("getFirstResultRow", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
         cFirstResultRow.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFirstResultRow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getFirstRowNum()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the temp-table row number of the first row.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      integer iRow = TypeFactory.integer();
      
      return function("getFirstRowNum", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
         iRow.assign(ghProp.unwrapBufferField().value());
         returnNormal(iRow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getForeignFields()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the property which holds the mapping of field's in 
                     another DataSource to fields in this SMartDataObject's RowObject 
                     temp-table, to open a dependent query.
        
        Parameters:  <none>
        
        Notes:       The property format is a comma-separated list, consisting of the
                     first local db fieldname, followed by the matching source 
                     temp-table field name, followed by more pairs if there is more 
                     than one field to match.
      ------------------------------------------------------------------------------*/
      character cFields = TypeFactory.character();
      
      return function("getForeignFields", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignFields"));
         cFields.assign(ghProp.unwrapBufferField().value());
         returnNormal(cFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getForeignValues()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Retrieves the values of the most recently received ForeignField
                     values received by dataAvailable.  The values are character
                     strings formatted according to the field format specification and
                     they are separated by the {&adm-tabledelimiter} character.
      
        Parameters:  <none>
        
        Notes:
      ------------------------------------------------------------------------------*/
      character cValues = TypeFactory.character();
      
      return function("getForeignValues", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignValues"));
         cValues.assign(ghProp.unwrapBufferField().value());
         returnNormal(cValues);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getKeyFields()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the comma-separated KeyFields property.
         Params:  The indexInformation will be used to try to figure out the
                  default KeyFields list, but this is currently restricted to cases 
                  where: 
                  - The First Table in the join is the Only enabled table.
                  - All the fields of the index is present is the SDO.             
                  The following index may be selected.                 
                           
                  1. Primary index if unique.
                  2. First Unique index. 
                  
                  There's currently no check whether the field is mandatory.                                      
      ------------------------------------------------------------------------------*/
      character cKeyFields = TypeFactory.character();
      character cIndexInfo = TypeFactory.character();
      character cEnabledTables = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cUniqueList = TypeFactory.character();
      character cPrimaryList = TypeFactory.character();
      character cColumnList = TypeFactory.character();
      character cColumn = TypeFactory.character();
      integer iIdx = TypeFactory.integer();
      integer iFld = TypeFactory.integer();
      logical lPrimaryFound = TypeFactory.logical();
      integer iOkIndex = TypeFactory.integer();
      
      return function("getKeyFields", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("KeyFields"));
         cKeyFields.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cKeyFields, ""))
         {
            /* {get Prop var [handle]} */ 
            cEnabledTables.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledTables", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
            cTables.assign(ghProp.unwrapBufferField().value());
            
            /* Currently we only create a default KeyFields when 
                   ONE enabled table and it's the FIRST table */
            if (_and(isEqual(numEntries(cEnabledTables), 1), () -> isEqual(cEnabledTables, entry(1, cTables))))
            {
               /* {get Prop var [handle]} */ 
               cIndexInfo.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getIndexInformation", targetProcedure()));
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("KeyFields"));
               cKeyFields.assign(ghProp.unwrapBufferField().value());
               
               if (_and(notUnknown(cIndexInfo), () -> isEqual(cKeyFields, "")))
               {
                  /* Get the unique indexes from the IndexInformation function */
                  cUniqueList.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "indexInformation", targetProcedure(), "III", "unique", "yes", cIndexInfo));
                  /* only the first table's indexes*/
                  cUniqueList.assign(entry(1, cUniqueList, chr(2)));
                  /* Get the primary index(es) from the IndexInformation function */
                  cPrimaryList.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "indexInformation", targetProcedure(), "III", "primary", "yes", cIndexInfo));
                  /* only the first table's indexes*/
                  cPrimaryList.assign(entry(1, cPrimaryList, chr(2)));
                  
                  indexLoop:
                  for (iIdx.assign(1); _isLessThanOrEqual(iIdx, numEntries(cUniqueList, chr(1))); iIdx.increment())
                  {
                     cColumnList.assign(entry(iIdx, cUniqueList, chr(1)));
                     /* This is the entry we use if it's ok */
                     iOkIndex.assign(_isEqual(iOkIndex, 0) ? iIdx : iOkIndex);
                     
                     /* We never set primaryFound to false, because we want to bail out 
                                  on the first acceptable index AFTER the primary if the primary 
                                  could not be used.*/
                     if (_not(lPrimaryFound))
                     {
                        lPrimaryFound.assign(isEqual(cColumnList, cPrimaryList));
                     }
                     
                     loopLabel1:
                     for (iFld.assign(1); _isLessThanOrEqual(iFld, numEntries(cColumnList)); iFld.increment())
                     {
                        cColumn.assign(entry(iFld, cColumnList));
                        
                        /* If the field is qualifed it's not used in the SDO,
                                       so we don't use this index */
                        if (_isNotEqual(indexOf(cColumn, "."), 0))
                        {
                           iOkIndex.assign(0);
                        }
                     }
                     
                     /* if this index is ok and the primary has been found now or earlier,
                                  don't search anymore */
                     if (_and(isNotEqual(iOkIndex, 0), lPrimaryFound))
                     {
                        break indexLoop;
                     }
                  }
                  
                  /* do i = 1 to num-entries cFieldList */
                  /* Did we find an index? */
                  if (_isNotEqual(iOkIndex, 0))
                  {
                     cKeyFields.assign(entry(iOkIndex, cUniqueList, chr(1)));
                     /* store it for the future */
                     /* {set Prop value [handle]} */ 
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setKeyFields", targetProcedure(), "I", cKeyFields);
                  }
               }
            }
         }
         returnNormal(cKeyFields);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getKeyTableId()
   {
      /*------------------------------------------------------------------------------
        Purpose: Get the KeyTable identifier (Dynamics FiveLetterAcronym) property
          Notes: This is a unique Table identifier across databases for all tables 
                 used by the framework  
      ------------------------------------------------------------------------------*/
      character cKeyTableId = TypeFactory.character();
      
      return function("getKeyTableId", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("KeyTableId"));
         cKeyTableId.assign(ghProp.unwrapBufferField().value());
         returnNormal(cKeyTableId);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getLabel()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns the LABEL defined for the Dataobject   
          Notes: Null ? implies label is not set, so get the first enabled table
      ------------------------------------------------------------------------------*/
      character cLabel = TypeFactory.character();
      character cEnabledTables = TypeFactory.character();
      character cTables = TypeFactory.character();
      character cPhysicalTables = TypeFactory.character();
      integer iPos = TypeFactory.integer();
      
      return function("getLabel", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Label"));
         cLabel.assign(ghProp.unwrapBufferField().value());
         
         if (_isUnknown(cLabel))
         {
            /* {get Prop var [handle]} */ 
            cEnabledTables.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledTables", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
            cTables.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
            cPhysicalTables.assign(ghProp.unwrapBufferField().value());
            
            if (_isGreaterThan(cEnabledTables, ""))
            {
               iPos.assign(lookup(entry(1, cEnabledTables), cTables));
            }
            else
            {
               iPos.assign(1);
            }
            
            if (_isGreaterThan(iPos, 0))
            {
               cLabel.assign(entry(iPos, cPhysicalTables));
               
               /* capitalize unless ugly name */
               if (_and(and(isEqual(indexOf(cLabel, "_"), 0), () -> isEqual(indexOf(cLabel, "-"), 0)), () -> isGreaterThan(length(cLabel), 1)))
               {
                  silent(() -> cLabel.assign(concat(toUpperCase(substring(cLabel, 1, 1)), toLowerCase(substring(cLabel, 2)))));
               }
               
               if (_isNotEqual(cLabel, ""))
               {
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setLABEL", targetProcedure(), "I", cLabel);
               }
            }
            
            if (_or(isEqual(cLabel, ""), () -> isUnknown(cLabel)))
            {
               cLabel.assign(((character) ControlFlowOps.runSuper("getLabel")));
            }
         }
         returnNormal(cLabel);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getLastResultRow()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the LastResultRow (unknown if last row hasn't been
                  fetched, RowNum concatinated with the rowid if it has.)
          Notes:  
      ------------------------------------------------------------------------------*/
      character cLastResultRow = TypeFactory.character();
      
      return function("getLastResultRow", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
         cLastResultRow.assign(ghProp.unwrapBufferField().value());
         returnNormal(cLastResultRow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getLastRowNum()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the temp-table row number of the last row.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      integer iRow = TypeFactory.integer();
      
      return function("getLastRowNum", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
         iRow.assign(ghProp.unwrapBufferField().value());
         returnNormal(iRow);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getNavigationSource()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the handle of the query object's Navigation source.
        
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      character cSource = TypeFactory.character();
      
      return function("getNavigationSource", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NavigationSource"));
         cSource.assign(ghProp.unwrapBufferField().value());
         returnNormal(cSource);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getNavigationSourceEvents()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns a comma-separated list of the events this object 
                     needs to subscribe to in its NavigationSource.
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      character cEvents = TypeFactory.character();
      
      return function("getNavigationSourceEvents", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NavigationSourceEvents"));
         cEvents.assign(ghProp.unwrapBufferField().value());
         returnNormal(cEvents);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getNewBatchInfo()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a RowNum relationship defining the RowObject records of the
                  last batch retrieved.
          Notes:  i.e. RowNum > '100' 
      ------------------------------------------------------------------------------*/
      character cValue = TypeFactory.character();
      
      return function("getNewBatchInfo", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
         cValue.assign(ghProp.unwrapBufferField().value());
         returnNormal(cValue);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getOpenOnInit()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Returns TRUE if the query should be opened automatically when 
                    the object is initialized.
      
        Parameters: <none>
      ------------------------------------------------------------------------------*/
      logical lOpen = TypeFactory.logical();
      
      return function("getOpenOnInit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OpenOnInit"));
         lOpen.assign(ghProp.unwrapBufferField().value());
         returnNormal(lOpen);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getOpenQuery()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Returns the OPEN-QUERY preprocessor value, to allow it to be
                    manipulated by setQueryWhere, for example
        Parameters: <none>  
        Notes:      This is important because no matter how the query is dynamically
                    modified, it can be reset to its original state using this return
                    value in a QUERY-PREPARE method.
                  - getOpenQuery is called from the client on AppServer, so we are not
                    using the xp preprocessor.
                  - The actual value is stored in BaseQuery.. After the improvements
                    to make the Appserver use one hit perrequest setOpenQuery cannot
                    be used to store server context on the client, since it may be 
                    returned AFTER the query has been changed and wipe out all query 
                    data because it calls setQueryWhere('').    
      ------------------------------------------------------------------------------*/
      character cQuery = TypeFactory.character();
      
      return function("getOpenQuery", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
         cQuery.assign(ghProp.unwrapBufferField().value());
         returnNormal(cQuery);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getPhysicalTables()
   {
      /*------------------------------------------------------------------------------
         Purpose: Get the Physical names that corresponsd to the Tables that is used 
                  to build the BaseQuery.  
           Notes: The names used in the query is defined in the corresponding Tables 
                  property.  
                - Qualified with database name if the query is defined with dbname.
      ------------------------------------------------------------------------------*/
      character cPhysicalTables = TypeFactory.character();
      
      return function("getPhysicalTables", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
         cPhysicalTables.assign(ghProp.unwrapBufferField().value());
         returnNormal(cPhysicalTables);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getPositionForClient()
   {
      /*------------------------------------------------------------------------------
        Purpose: This property is set on the server and returned to the client so that
                 it is able to position correctly in the new batch of data. The client
                 need this information under certain cicumstances, for example when 
                 other settings decides that a full batch of data always is needed. 
          Notes: set on server, used in clientSendRows only  
      ------------------------------------------------------------------------------*/
      character cPositionForClient = TypeFactory.character();
      
      return function("getPositionForClient", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PositionForClient"));
         cPositionForClient.assign(ghProp.unwrapBufferField().value());
         returnNormal(cPositionForClient);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryColumns()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cQueryColumns_1 = TypeFactory.character();
      
      return function("getQueryColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         cQueryColumns_1.assign(ghProp.unwrapBufferField().value());
         returnNormal(cQueryColumns_1);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getQueryHandle()
   {
      /*------------------------------------------------------------------------------
        Purpose:    Returns the handle of the database query.
      
        Parameters: <none>
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      
      return function("getQueryHandle", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         returnNormal(hQuery);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getQueryOpen()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns TRUE if the Query Object's database query is currently 
                     open.
      
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      
      return function("getQueryOpen", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         returnNormal(_not(hQuery.isValid()) ? new logical(false) : hQuery.unwrapQuery().isOpen());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryPosition()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the QueryPosition property.
      
        Parameters:  <none>
        
        Notes:       Valid return values are:
                       FirstRecord, LastRecord, NotFirstOrLast or NoRecordAvailable
      ------------------------------------------------------------------------------*/
      /* The property does not have a field preprocessor to prevent it from being
           "set" directly, because it must also publish an event. So the code below
           must not use the {get} syntax. */
      character cPosition = TypeFactory.character();
      
      return function("getQueryPosition", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryPosition"));
         cPosition.assign(ghProp.unwrapBufferField().value());
         returnNormal(cPosition);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQuerySort()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return the sort phrase.   
          Notes: Does NOT return the first BY keyword.      
                 If a query is under work in the QueryString property this will 
                 return this BY-phrase, otherwise it will return the BY-phrase in the 
                 current or design query. 
                 This should make this property safe to use in qbf objects that 
                 may or may not have done query manipulation and may or may not 
                 have opened the query. 
                 It can also be used to retrieve the BY phrase before a setQueryWhere 
                 (which overrides setQuerySort) and reset it after. 
      ------------------------------------------------------------------------------*/
      character cQueryText = TypeFactory.character();
      character cSort = TypeFactory.character();
      
      return function("getQuerySort", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryText.assign(ghProp.unwrapBufferField().value());
         
         /* If not use the actual query */
         if (_isEqual(cQueryText, ""))
         {
            /* {get Prop var [handle]} */ 
            cQueryText.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
         }
         
         /* If not found so far use the designed query */
         if (_isEqual(cQueryText, ""))
         {
            /* {get Prop var [handle]} */ 
            cQueryText.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getOpenQuery", targetProcedure()));
         }
         
         cSort /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "sortExpression", targetProcedure(), "I", cQueryText));
         
         if (_isGreaterThan(cSort, ""))
         {
            /* remove the first BY keyword */
            returnNormal(substring(cSort, 3));
         }
         else
         {
            returnNormal("");
         }
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryString()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the QueryString attribute used as working storage for 
                     all query manipulation methods.
        Parameters:  <none>
        Notes:     - The method will always return a whereclause 
                     If the QueryString property has not been set it will use 
                     the current where clause - QueryWhere.
                     If there's no current use the design where clause - OpenQuery. 
                   - The openQuery will call prepareQuery with this property.     
      ------------------------------------------------------------------------------*/
      character cQueryString = TypeFactory.character();
      
      return function("getQueryString", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryString.assign(ghProp.unwrapBufferField().value());
         returnNormal(cQueryString);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getQueryWhere()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns the current where-clause for the database query.
                     Returns ? if the query handle is undefined.
      
        Parameters:  <none>
      ------------------------------------------------------------------------------*/
      handle hQuery = TypeFactory.handle();
      
      return function("getQueryWhere", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
         hQuery.assign(ghProp.unwrapBufferField().value());
         returnNormal((hQuery.isValid()).booleanValue() ? hQuery.unwrapQuery().prepareString() : new character());
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getRequiredProperties()
   {
      /*------------------------------------------------------------------------------
        Purpose: Gets the list of required properties   
          Notes: Currently used to identify the properties required to start a 
                 dynamic data object             
      ------------------------------------------------------------------------------*/
      character cRequiredProperties = TypeFactory.character();
      
      return function("getRequiredProperties", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RequiredProperties"));
         cRequiredProperties.assign(ghProp.unwrapBufferField().value());
         returnNormal(cRequiredProperties);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getTables()
   {
      /*------------------------------------------------------------------------------
         Purpose:  Returns a comma delimited list of tables in the Query Objects
      Parameters:  <none>
           Notes: The names in the list references to the tables as defined in the
                  dataobject and referenced in the query, so each entry must be unique. 
                  This means that the names may be buffer names that differ from the 
                  actual physical names.
                  The physical names is defined in the corresponding PhysicalTables 
                  property.  
                - Qualified with database name if the query is defined with dbname.
                - From 9.1B this property is a design time property while it earlier 
                  was resolved from the actual query. This was very expensive as the 
                  function then always needed to be resolved on the server and it is 
                  rather extensively used. 
                - Several other properties have table delimiters and are depending of 
                  the design time order of this property.              
                - The web2/webprop.i UNDEFINEs xpTables since it need to override 
                  getTables()    
      ------------------------------------------------------------------------------*/
      character cTables = TypeFactory.character();
      
      return function("getTables", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         returnNormal(cTables);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getTempTables()
   {
      /*------------------------------------------------------------------------------
         Purpose:  Returns a comma delimited list of temp tables in the Query Objects
      Parameters:  <none>
           Notes: The names in the list references those tables in the Physicaltables
                  proeprty that are temp tables.
      ------------------------------------------------------------------------------*/
      character cTempTables = TypeFactory.character();
      
      return function("getTempTables", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TempTables"));
         cTempTables.assign(ghProp.unwrapBufferField().value());
         returnNormal(cTempTables);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getTransferChildrenForAll()
   {
      /*------------------------------------------------------------------------------
        Purpose: This flag decides whether children for all records (of the batch) is 
                 to be transferred from the database. 
          Notes: Currently only supported for read event handlers during a fetch. 
                 The child SDO is only left with temp-table records for one parent 
                 when the fetch*batch is finished.   
      ------------------------------------------------------------------------------*/
      logical lTransferChildrenForAll = TypeFactory.logical();
      
      return function("getTransferChildrenForAll", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TransferChildrenForAll"));
         lTransferChildrenForAll.assign(ghProp.unwrapBufferField().value());
         returnNormal(lTransferChildrenForAll);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getUpdatableColumns()
   {
      /*------------------------------------------------------------------------------
        Purpose:     Returns a comma delimited list of the Updatable Columns for this
                     SmartDataObject.
        Parameters:  <none>
        Notes:       Uses the UpdatableColumnsByTable property which stores the
                     the columns in different tables delimited by {&adm-tabledelimiter} instead of 
                     comma in order to identify which columns are from which table 
                     in the event of a join. For example, if the query is a join of 
                     customer and order and the Name field from customer and the
                     OrderNum and OrderData field from Order are Updatable, then the 
                     property value will be equal to "Name{&adm-tabledelimiter}OrderNum,OrderDate".
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      
      return function("getUpdatableColumns", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         cColumns.assign(replaceAll(cColumns, ";", ","));
         
         loopLabel2:
         while (_isGreaterThan(indexOf(cColumns, ",,"), 0))
         {
            cColumns.assign(replaceAll(cColumns, ",,", ","));
         }
         
         /* There may be commas at the ends */
         returnNormal(trim(cColumns, ","));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getUpdatableColumnsByTable()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cColumns = TypeFactory.character();
      
      return function("getUpdatableColumnsByTable", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         cColumns.assign(ghProp.unwrapBufferField().value());
         returnNormal(cColumns);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getUseDbqualifier()
   {
      /*------------------------------------------------------------------------------
        Purpose: Return true if table references are qualified with db. 
          Notes: This property was originally implemented to be able to return 
                 getTables() correctly when getTables() was resolved from the query 
                 handle. 
                 From 9.1B the Tables property is set at design time and will thus 
                 always reflect the design time database qualification, so this 
                 function is now changed to be resolved from getTables, the very same 
                 function it was implemented to serve.
                 (It is also used in other functions).  
      ------------------------------------------------------------------------------*/
      character cTables = TypeFactory.character();
      
      return function("getUseDBQualifier", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         cTables.assign(ghProp.unwrapBufferField().value());
         returnNormal(isGreaterThan(numEntries(entry(1, cTables), "."), 1));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getWordIndexedFields()
   {
      return function("getWordIndexedFields", character.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: Return a comma separated list of word indexed fields 
             Notes: Qualified with database and table name.
                    We return database name from dynamic methods as all column* 
                    methods are able to respond to that.
                    This dbname is returned also if qualify with db name is false.
         ------------------------------------------------------------------------------*/
         returnNormal(replaceAll(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "indexInformation", targetProcedure(), "III", "WORD", false, new unknown()), chr(1), ","));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAssignList(final character _pcList_1)
   {
      character pcList_1 = TypeFactory.initInput(_pcList_1);
      
      return function("setAssignList", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AssignList"));
         ghProp.unwrapBufferField().changeValue(pcList_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAuditEnabled(final logical _lAuditEnabled_1)
   {
      logical lAuditEnabled_1 = TypeFactory.initInput(_lAuditEnabled_1);
      
      return function("setAuditEnabled", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AuditEnabled"));
         ghProp.unwrapBufferField().changeValue(lAuditEnabled_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setAutoCommit(final logical _plCommit_1)
   {
      logical plCommit_1 = TypeFactory.initInput(_plCommit_1);
      
      return function("setAutoCommit", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:     Passes along a "set AutoCommit" to the object's Update-Target.
           
           Parameters:
             plCommit - True or False to be set in the Update-Target procedure.
         
           Notes:       There is no AutoCommit property in a query object, but the 
                        AutoCommit setting may be passed from a SmartDataObject to its 
                        Data-Targets. If one of those targets is a query object that 
                        isn't also a SmartDataObject (e.g., a SmartDataBrowser that has 
                        its own db query), then the setting must be passed to the 
                        Update-Target, which is the actual DataObject. 
                        In a SmartDataObject the version of setAutoCommit in data.p
                        will be found in preference to this one and will set the 
                        actual property.
         ------------------------------------------------------------------------------*/
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "assignLinkProperty", targetProcedure(), "III", "Update-Target", "AutoCommit", (plCommit_1).booleanValue() ? new character("yes") : new character("No"));
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setBaseQuery(final character _pcBaseQuery_1)
   {
      character pcBaseQuery_1 = TypeFactory.initInput(_pcBaseQuery_1);
      
      return function("setBaseQuery", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose: Set the base query property 
             Notes: This is used internally by openQuery, and directly on client context 
                    management. Because setOpenQuery calls setQueryWhere and wipes out 
                    all other query data, it cannot be used to set the context when 
                    received from server.      
         ------------------------------------------------------------------------------*/
         if (_isEqual(pcBaseQuery_1, ""))
         {
            returnNormal(false);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
         ghProp.unwrapBufferField().changeValue(pcBaseQuery_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setBufferHandles(final character _pcBufferHandles_1)
   {
      character pcBufferHandles_1 = TypeFactory.initInput(_pcBufferHandles_1);
      
      return function("setBufferHandles", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BufferHandles"));
         ghProp.unwrapBufferField().changeValue(pcBufferHandles_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCheckLastOnOpen(final logical _pCheck_1)
   {
      logical pCheck_1 = TypeFactory.initInput(_pCheck_1);
      
      return function("setCheckLastOnOpen", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CheckLastOnOpen"));
         ghProp.unwrapBufferField().changeValue(pCheck_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataColumns(final character _pcColumns_3)
   {
      character pcColumns_3 = TypeFactory.initInput(_pcColumns_3);
      
      return function("setDataColumns", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:     Sets the comma-delimited list of the columnNames for the 
                        SmartDataObject. 
           Parameters:  
                Notes:   
         ------------------------------------------------------------------------------*/
         if (_isEqual(pcColumns_3, ""))
         {
            returnNormal(false);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumns"));
         ghProp.unwrapBufferField().changeValue(pcColumns_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataColumnsByTable(final character _pcColumns_3)
   {
      character pcColumns_3 = TypeFactory.initInput(_pcColumns_3);
      
      return function("setDataColumnsByTable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(pcColumns_3);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataIsFetched(final logical _plFetched_1)
   {
      logical plFetched_1 = TypeFactory.initInput(_plFetched_1);
      
      return function("setDataIsFetched", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataIsFetched"));
         ghProp.unwrapBufferField().changeValue(plFetched_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDbnames(final character _pcDbnames_1)
   {
      character pcDbnames_1 = TypeFactory.initInput(_pcDbnames_1);
      
      return function("setDBNames", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DBNames"));
         ghProp.unwrapBufferField().changeValue(pcDbnames_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setEntityFields(final character _cEntityFields_1)
   {
      character cEntityFields_1 = TypeFactory.initInput(_cEntityFields_1);
      
      return function("setEntityFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EntityFields"));
         ghProp.unwrapBufferField().changeValue(cEntityFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFetchAutoComment(final logical _plFetchAutoComment_1)
   {
      logical plFetchAutoComment_1 = TypeFactory.initInput(_plFetchAutoComment_1);
      
      return function("setFetchAutoComment", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchAutoComment"));
         ghProp.unwrapBufferField().changeValue(plFetchAutoComment_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFetchHasAudit(final logical _plFetchHasAudit_1)
   {
      logical plFetchHasAudit_1 = TypeFactory.initInput(_plFetchHasAudit_1);
      
      return function("setFetchHasAudit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasAudit"));
         ghProp.unwrapBufferField().changeValue(plFetchHasAudit_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFetchHasComment(final logical _plFetchHasComment_1)
   {
      logical plFetchHasComment_1 = TypeFactory.initInput(_plFetchHasComment_1);
      
      return function("setFetchHasComment", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchHasComment"));
         ghProp.unwrapBufferField().changeValue(plFetchHasComment_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFetchOnOpen(final character _pcFetchOnOpen_1)
   {
      character pcFetchOnOpen_1 = TypeFactory.initInput(_pcFetchOnOpen_1);
      
      return function("setFetchOnOpen", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchOnOpen"));
         ghProp.unwrapBufferField().changeValue(pcFetchOnOpen_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFilterActive(final logical _plFilterActive_1)
   {
      logical plFilterActive_1 = TypeFactory.initInput(_plFilterActive_1);
      
      return function("setFilterActive", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterActive"));
         ghProp.unwrapBufferField().changeValue(plFilterActive_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFilterAvailable(final logical _plFilterAvailable_1)
   {
      logical plFilterAvailable_1 = TypeFactory.initInput(_plFilterAvailable_1);
      /*------------------------------------------------------------------------------
        Purpose: Set a flag to indicate that filter is available.
          Notes: ALso update the FilterWindow property for StartFilter. 
      ------------------------------------------------------------------------------*/
      handle hFilterSource = TypeFactory.handle();
      character cFilterWindow = TypeFactory.character();
      handle hFilterContainer = TypeFactory.handle();
      handle hMyContainer = TypeFactory.handle();
      
      return function("setFilterAvailable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterAvailable"));
         ghProp.unwrapBufferField().changeValue(plFilterAvailable_1);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterWindow"));
         cFilterWindow.assign(ghProp.unwrapBufferField().value());
         
         if (_or(isEqual(cFilterWindow, ""), () -> isUnknown(cFilterWindow)))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterSource"));
            hFilterSource.assign(ghProp.unwrapBufferField().value());
            
            if ((hFilterSource.isValid()).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, hFilterSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
               hFilterContainer.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
               hMyContainer.assign(ghProp.unwrapBufferField().value());
               
               if (_and(isNotEqual(hMyContainer, hFilterContainer), () -> hFilterContainer.isValid()))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("FilterWindow"));
                  ghProp.unwrapBufferField().changeValue(hFilterContainer.unwrapFile().getFileName());
               }
               /* {set Prop value [handle]} */ 
            }
         }
         
         /* The toolbar is subscribing to filteravailble from the navigation link*/
         if ((plFilterAvailable_1).booleanValue())
         {
            NamedEventManager.publish("FilterState", targetProcedure(), "I", new Object[]
            {
               "FilterAvailable"
            });
         }
         
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFirstResultRow(final character _pcFirstResultRow_1)
   {
      character pcFirstResultRow_1 = TypeFactory.initInput(_pcFirstResultRow_1);
      
      return function("setFirstResultRow", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstResultRow"));
         ghProp.unwrapBufferField().changeValue(pcFirstResultRow_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFirstRowNum(final integer _piRowNum_1)
   {
      integer piRowNum_1 = TypeFactory.initInput(_piRowNum_1);
      
      return function("setFirstRowNum", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FirstRowNum"));
         ghProp.unwrapBufferField().changeValue(piRowNum_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setForeignFields(final character _pcFields_1)
   {
      character pcFields_1 = TypeFactory.initInput(_pcFields_1);
      
      return function("setForeignFields", logical.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:     Sets the Foreign Fields property of the object.
           
           Parameters: pcFields - comma-separated paired list of this objects db fields 
                                  and the RowObject fields they map to in the parent.
               Notes:   In the case where the current ForeignFields already have been 
                        used to add the key to the query, the fieldnames and the values 
                        will be removed from the QueryString by this call.
         ------------------------------------------------------------------------------*/
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "removeForeignKey", targetProcedure());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ForeignFields"));
         ghProp.unwrapBufferField().changeValue(pcFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setKeyFields(final character _pcKeyFields_1)
   {
      character pcKeyFields_1 = TypeFactory.initInput(_pcKeyFields_1);
      
      return function("setKeyFields", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("KeyFields"));
         ghProp.unwrapBufferField().changeValue(pcKeyFields_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setKeyTableId(final character _pcKeyTableId_1)
   {
      character pcKeyTableId_1 = TypeFactory.initInput(_pcKeyTableId_1);
      
      return function("setKeyTableId", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("KeyTableId"));
         ghProp.unwrapBufferField().changeValue(pcKeyTableId_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLastResultRow(final character _pcLastResultRow_1)
   {
      character pcLastResultRow_1 = TypeFactory.initInput(_pcLastResultRow_1);
      
      return function("setLastResultRow", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastResultRow"));
         ghProp.unwrapBufferField().changeValue(pcLastResultRow_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setLastRowNum(final integer _piLastRowNum_1)
   {
      integer piLastRowNum_1 = TypeFactory.initInput(_piLastRowNum_1);
      
      return function("setLastRowNum", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("LastRowNum"));
         ghProp.unwrapBufferField().changeValue(piLastRowNum_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setNavigationSource(final character _pcObject_1)
   {
      character pcObject_1 = TypeFactory.initInput(_pcObject_1);
      
      return function("setNavigationSource", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NavigationSource"));
         ghProp.unwrapBufferField().changeValue(pcObject_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setNewBatchInfo(final character _pcNewBatchInfo_1)
   {
      character pcNewBatchInfo_1 = TypeFactory.initInput(_pcNewBatchInfo_1);
      
      return function("setNewBatchInfo", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NewBatchInfo"));
         ghProp.unwrapBufferField().changeValue(pcNewBatchInfo_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setOpenOnInit(final logical _plOpen_1)
   {
      logical plOpen_1 = TypeFactory.initInput(_plOpen_1);
      
      return function("setOpenOnInit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("OpenOnInit"));
         ghProp.unwrapBufferField().changeValue(plOpen_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setOpenQuery(final character _pcQuery_1)
   {
      character pcQuery_1 = TypeFactory.initInput(_pcQuery_1);
      
      return function("setOpenQuery", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
         ghProp.unwrapBufferField().changeValue(pcQuery_1);
         /* Just use the setQueryWhere to blank to apply this property to the query.
              setQueryWhere has all the required logic to prepare and set other  
              query props to blank. */
         returnNormal( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryWhere", targetProcedure(), "I", ""));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPhysicalTables(final character _pcPhysicalTables_1)
   {
      character pcPhysicalTables_1 = TypeFactory.initInput(_pcPhysicalTables_1);
      
      return function("setPhysicalTables", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PhysicalTables"));
         ghProp.unwrapBufferField().changeValue(pcPhysicalTables_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPositionForClient(final character _pcPositionForClient_1)
   {
      character pcPositionForClient_1 = TypeFactory.initInput(_pcPositionForClient_1);
      
      return function("setPositionForClient", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PositionForClient"));
         ghProp.unwrapBufferField().changeValue(pcPositionForClient_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryColumns(final character _cQueryColumns_1)
   {
      character cQueryColumns_1 = TypeFactory.initInput(_cQueryColumns_1);
      
      return function("setQueryColumns", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryColumns"));
         ghProp.unwrapBufferField().changeValue(cQueryColumns_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryPosition(final character _pcPosition_1)
   {
      character pcPosition_1 = TypeFactory.initInput(_pcPosition_1);
      
      return function("setQueryPosition", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryPosition"));
         ghProp.unwrapBufferField().changeValue(pcPosition_1);
         NamedEventManager.publish("queryPosition", targetProcedure(), "I", new Object[]
         {
            pcPosition_1
         });
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQuerySort(final character _pcSort_1)
   {
      character pcSort_1 = TypeFactory.initInput(_pcSort_1);
      /*------------------------------------------------------------------------------
        Purpose:     Sets or resets the sorting criteria (BY phrase) of the database
                     query and stores the result in the QueryString property.
        Parameters:
          pcSort   - new sort (BY) clause.
                   - [BY ] column [DESCENDING | TOGGLE ] [ BY ... ]  
                   - TOGGLE sort option specifies that the column should be sorted
                     opposite of the current sort. The option can be specified for 
                     any column in the sort.      
                   - Supports RowObject as qualifier to specify the sort with the 
                     data object's column names.   
                   - ? means reset to Default sort (from BaseQuery).                  
        Notes:     - setQueryWhere wipes out anything done in setQuerySort.
                   - addQuerywhere does not. 
                   - Use resetSort to apply sort criteria AND reopen the query if 
                     it already is opened.
      ------------------------------------------------------------------------------*/
      character cQueryText = TypeFactory.character();
      character cBaseQuery = TypeFactory.character();
      
      return function("setQuerySort", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         cQueryText.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cQueryText, ""))
         {
            /* {get Prop var [handle]} */ 
            cQueryText.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryWhere", targetProcedure()));
            
            if (_or(isEqual(cQueryText, ""), () -> isUnknown(cQueryText)))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
               cQueryText.assign(ghProp.unwrapBufferField().value());
            }
            /* {get Prop var [handle]} */ 
         }
         
         /* no query under work */
         /* if ? get Default from BaseQuery */
         if (_isUnknown(pcSort_1))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
            cBaseQuery.assign(ghProp.unwrapBufferField().value());
            pcSort_1 /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "sortExpression", targetProcedure(), "I", cBaseQuery));
         }
         
         cQueryText.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newQuerySort", targetProcedure(), "III", cQueryText, pcSort_1, true));
         
         /* db columns */
         if (_isGreaterThan(cQueryText, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
            ghProp.unwrapBufferField().changeValue(cQueryText);
            returnNormal(true);
         }
         
         returnNormal(false);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryString(final character _pcQueryString_1)
   {
      character pcQueryString_1 = TypeFactory.initInput(_pcQueryString_1);
      
      return function("setQueryString", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryString"));
         ghProp.unwrapBufferField().changeValue(pcQueryString_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryWhere(final character _pcWhere_1)
   {
      character pcWhere_1 = TypeFactory.initInput(_pcWhere_1);
      /*------------------------------------------------------------------------------
        Purpose:    Append a WHERE clause to the basic static database query 
                    definition (on client or server side) or modifes the complete 
                    where clause.  
       
      Parameters:
          INPUT pcWhere - new WHERE clause (a logical expression without the 
                          leading WHERE keyword). 
        
        Notes:      NOTE that we always start with the base query definition 
                    to do this. If the where-clause parameter is blank, we simply 
                    reprepare the original query.
                    Criteria appended with the other query manipulation methods 
                    will be removed, INCLUDED sort criteria added with setQuerySort. 
        Parameters:
          pcWhere - new query expression or complete new where clause.  
                    - An expression without the leading WHERE keyword.
                      Will be added to the base query, if the base query has an 
                      expression the new expression wil be appended with AND 
                    - blank - reprepare the base query.
                    - a complete query with the FOR keyword.
                    
         Notes:     If pcWhere starts with "For " or "Preselect " then the existing 
                    where clause is completely replaced with pcWhere.
                    If pcWhere is blank, then the existing where clause is replaced
                    with the design-time where clause.
                    Otherwise pcWhere is added to the design-time where clause.
       ------------------------------------------------------------------------------*/
      character cQuery = TypeFactory.character();
      handle hQuery = TypeFactory.handle();
      character cAsDivision = TypeFactory.character();
      
      return function("setQueryWhere", logical.class, new Block((Body) () -> 
      {
         cQuery /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "newQueryWhere", targetProcedure(), "I", pcWhere_1));
         
         if (_notUnknown(cQuery))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryHandle"));
            hQuery.assign(ghProp.unwrapBufferField().value());
            
            if ((hQuery.isValid()).booleanValue())
            {
               /* prepares the query with the new string */
               returnNormal( /* {fnarg <func-name> <arg> [handle] */ ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "prepareQuery", targetProcedure(), "I", cQuery));
            }
            else
            {
               /* set QueryWhere may be called early to set context on server 
                        So we use the queryContext temporarily, which is only is used 
                        by createObjects. */
               /* {get Prop var [handle]} */ 
               cAsDivision.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getAsDivision", targetProcedure()));
               
               if (_isNotEqual(cAsDivision, "CLIENT"))
               {
                  if (_isEqual(cQuery, ""))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("BaseQuery"));
                     cQuery.assign(ghProp.unwrapBufferField().value());
                  }
                  /* {get Prop var [handle]} */ 
                  
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setQueryContext", targetProcedure(), "I", cQuery);
                  returnNormal(true);
               }
            }
         }
         /* cQuery <> ? */
         returnNormal(false);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setRequiredProperties(final character _pcProperties_1)
   {
      character pcProperties_1 = TypeFactory.initInput(_pcProperties_1);
      
      return function("setRequiredProperties", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("RequiredProperties"));
         ghProp.unwrapBufferField().changeValue(pcProperties_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setTables(final character _pcTables_1)
   {
      character pcTables_1 = TypeFactory.initInput(_pcTables_1);
      
      return function("setTables", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Tables"));
         ghProp.unwrapBufferField().changeValue(pcTables_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setTempTables(final character _pcTempTables_1)
   {
      character pcTempTables_1 = TypeFactory.initInput(_pcTempTables_1);
      
      return function("setTempTables", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TempTables"));
         ghProp.unwrapBufferField().changeValue(pcTempTables_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setTransferChildrenForAll(final logical _plTransferChildrenForAll_1)
   {
      logical plTransferChildrenForAll_1 = TypeFactory.initInput(_plTransferChildrenForAll_1);
      
      return function("setTransferChildrenForAll", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TransferChildrenForAll"));
         ghProp.unwrapBufferField().changeValue(plTransferChildrenForAll_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setUpdatableColumnsByTable(final character _pcColumns_3)
   {
      character pcColumns_3 = TypeFactory.initInput(_pcColumns_3);
      
      return function("setUpdatableColumnsByTable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdatableColumnsByTable"));
         ghProp.unwrapBufferField().changeValue(pcColumns_3);
         returnNormal(true);
      }));
   }

   public character fwdGetAdmversion()
   {
      return function("fwdGetADMVersion", character.class, new Block((Body) () -> 
      {
         returnNormal("ADM2.2");
      }));
   }
}
