/*--------------------------------------------------------------------------
    File        : Viewer.java
    Purpose     : Super procedure for PDO SmartViewer objects

    Syntax      : adm2/viewer.p

    Modified    : June 23, 2000 Version 9.1B
    Modified    : 10/25/2001      Mark Davies (MIP)
                  Field security - disabling of fields caused a problem
                  when trying to resolve an entry in the secured field list.
                  Added check to make sure that the list has that number
                  of entries in the list before attempting to retrieve it.
    Modofied    : 10/25/2001      Mark Davies (MIP)
                  Add security for Dynamic Combos and Dynamic Lookups.
    Modofied    : 10/26/2001      Mark Davies (MIP)
                  The widget names and value being sent to the combo and
                  lookup query builder is using a ',' comma as separator
                  and this causes problems for -E 'European numeric formats.
    Modified    : 11/14/2001       Mark Davies (MIP)
                  Added OR cSecuredFields = "":U when checking for security
                  to display and enable fields for NON Dynamics applications.
    Modified    : 11/14/2001        Mark Davies (MIP)
                  1. Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
                  2. Fixed IZ#2947 - FieldSecurity implementation incorrect in enabledFields.
    Modified    : 11/16/2001        Mark Davies (MIP)
                  Changed AllFieldHandles to conaint the PROCEDURE handle
                  of an SDF and not the FRAME handle.
    Modified    : 11/17/2001        Mark Davies (MIP)
                  Added 'CAN-QUERY' in initializeObject to check if the
                  TABLE attribute can be queried of the current widget 
    Modified    : 05/09/20002       Mark Davies (MIP)
                  Added override procedure for destroyObject to remove
                  Dynamic Lookup and Combo temp-table records.
                  Force a valueChanged event in Dynamic Combos on add.
                  Issue #4525
    Modified    : 05/10/20002       Mark Davies (MIP)
                  Fixed issue #2793 - Lookups should obtain Description Fields 
                  locally
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo._temp.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.persist.RecordBuffer.batch;
import static com.goldencode.p2j.util.ErrorManager.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/viewer.p).
 */
public class Viewer
{
   Ttdcombocopy_1_1.Buf ttdcombocopy = TemporaryBuffer.define(Ttdcombocopy_1_1.Buf.class, "ttdcombocopy", "ttDComboCopy", false);

   /* Create copies of the lookup and combo temp tables */   
   Ttlookupcopy_1_1.Buf ttlookupcopy = TemporaryBuffer.define(Ttlookupcopy_1_1.Buf.class, "ttlookupcopy", "ttLookupCopy", false);

   /* _UIB-CODE-BLOCK-END */   
   /* Dynamic Combo temp-table */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: ttdcombo.i
   
     Description:  Dynamic Combo Temp-Table Def Include
   
     Purpose:      Dynamic Combo Temp-Table Def Include
   
     Parameters:   <none>
   
     History:
     --------
     (v:010000)    Task:   101000015   UserRef:    
                   Date:   08/14/2001  Author:     Mark Davies
   
     Update Notes: Dynamic Combo Temp-Table Def Include
   
     (v:010001)    Task:   101000025   UserRef:    
                   Date:   08/30/2001  Author:     Mark Davies
   
     Update Notes: Added new field for KeyFormat
     
     Modified    : 05/09/2002        Mark Davies (MIP)
                   Added new field to ttDCombo called hViewer to contain the
                   handle a combo is on. Issue #4525
   
   -----------------------------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* MIP-GET-OBJECT-VERSION pre-processors
      The following pre-processors are maintained automatically when the object is
      saved. They pull the object and version from Roundtable if possible so that it
      can be displayed in the about window of the container */   
   /* MIP object identifying preprocessor */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.95
            WIDTH              = 40.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   Ttdcombo_1_1.Buf ttdcombo = TemporaryBuffer.define(Ttdcombo_1_1.Buf.class, "ttdcombo", "ttDCombo", false, false);

   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.95
            WIDTH              = 40.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* Lookup temp-table */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: ttlookup.i
   
     Description:  Lookup data build temp-table
   
     Purpose:      Lookup data build temp-table
   
     Parameters:   <none>
   
     History:
     --------
     (v:010000)    Task:        7065   UserRef:    
                   Date:   13/11/2000  Author:     Anthony Swindells
   
     Update Notes: Created from Template afteminclu.i
   
     (v:010002)    Task:    90000166   UserRef:    
                   Date:   24/07/2001  Author:     Mark Davies
   
     Update Notes: Add new field to lookup temp-table to indicate if more than one record could be found for lookup query.
   
     Modified    : 11/08/2001          Mark Davies (MIP)
                   Moved out of ICF structure
     Modified    : 05/09/2002        Mark Davies (MIP)
                   Added new field to ttLookup called hViewer to contain the
                   handle a lookup is on. Issue #4525
     Modified    : 05/10/2002        Mark Davies (MIP)
                   Added a field called lRefreshQuery to ttLookup temp-table to 
                   indicate if a lookup query needs to be refreshed before attempting
                   to display the values. Fixed issue #2793
   ------------------------------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* MIP-GET-OBJECT-VERSION pre-processors
      The following pre-processors are maintained automatically when the object is
      saved. They pull the object and version from Roundtable if possible so that it
      can be displayed in the about window of the container */   
   /* MIP object identifying preprocessor */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.95
            WIDTH              = 40.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   Ttlookup_1_1.Buf ttlookup = TemporaryBuffer.define(Ttlookup_1_1.Buf.class, "ttlookup", "ttLookup", false, false);

   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* Tell viewattr.i that this is the Super procedure. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : viewerexclcustom.i
       Purpose     : Defines EXCLUDE-xxx preprocessor variables to exclude
                     functions or internal procedures in the super procedure
                     where it is referenced.
       Syntax      : {src/adm2/custom/viewerexclcustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in src/adm2/viewer.p
     ------------------------------------------------------------------------*/   
   /* Combo temp-table */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: ttcombo.i
   
     Description:  Combo data build temp-table
   
     Purpose:      Combo data build temp-table
   
     Parameters:   <none>
   
     History:
     --------
     (v:010000)    Task:        5653   UserRef:    
                   Date:   29/05/2000  Author:     Anthony Swindells
   
     Update Notes: Created from Template afteminclu.i
   
     (v:010001)    Task:        6010   UserRef:    
                   Date:   13/06/2000  Author:     Anthony Swindells
   
     Update Notes: Add datatypt to temp-table
   
     (v:010002)    Task:        6421   UserRef:    
                   Date:   07/08/2000  Author:     Anthony Swindells
   
     Update Notes: Change primary key to be on handle and allow duplicate names - to support
                   generic code
   
   -------------------------------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* MIP-GET-OBJECT-VERSION pre-processors
      The following pre-processors are maintained automatically when the object is
      saved. They pull the object and version from Roundtable if possible so that it
      can be displayed in the about window of the container */   
   /* MIP object identifying preprocessor */   
   Ttcombodata_1_1.Buf ttcombodata = TemporaryBuffer.define(Ttcombodata_1_1.Buf.class, "ttcombodata", "ttComboData", false, false);

   /* _UIB-CODE-BLOCK-END */   
   /* NOTE: IN 9.1B beta, this is needed to provide SBO routines
        with the proper calling procedure. */   
   handle ghTargetProcedure = TypeFactory.handle();

   /* Global handle for SDF Cacghe Manager */   
   handle ghSdfcacheManager = TypeFactory.handle();

   /* _UIB-CODE-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Procedure
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: CODE-ONLY COMPILE
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Procedure ASSIGN
            HEIGHT             = 17.91
            WIDTH              = 56.2.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : viewprop.i
       Purpose     : Starts Super procedure viewer.p and defines basic PDO Viewer
                     properties
       Syntax      : {src/adm2/viewprop.i}
   
       Description :
   
       Modified    : May 19, 1999 Version 9.1A
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : viewerdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/viewerdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/viewprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* Include the next property file up the chain to get property FIELDs
        defined.      */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : dvisprop.i
       Purpose     : Property Include file for Data Visualization Objects 
   
       Syntax      : src\adm2\dvisprop.i
   
       Description :
   
       Modified    : May 26, 2000 -- Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : datavisdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/datavisdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/dvisprop.i}
     ------------------------------------------------------------------------*/   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the property temp-table. */   
   /* Include the next property file up the chain; we will add our field
        definitions to its list. As of 9.1A, if this visual object is also
        a SmartContainer, we include container properties as well. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visprop.i
       Purpose     : Starts Super procedure visual.p and defines basic properties
                     for visual SmartObjects in V9
       Syntax      : {src/adm2/visprop.i}
   
       Description :
   
       Modified    : May 19, 1999 Version 9.1A
       Modified    : 10/23/2001          Mark Davies (MIP)
                     Added two new fields for ResizeHorizontal and ResizeVertical
       Modified    : 11/02/2001          Mark Davies (MIP)
                     Replaced properties for FrameMinHeightChars and FrameMinWidthChars with MinHeight and MinWidth
       Modified    : 11/14/2001          Mark Davies (MIP)
                     Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
       Modified    : 02/27/2003          Neil Bell (MIP)
                     Added property 'SecuredTokens'                  
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visualdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/visualdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/visprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* This property contains the security for all objects found in the AllFieldHandles Property */    
   /* Include the next property file up the chain (in this case, the top
        of the chain); it starts the property temp-table definition and we
        add our field definitions to it. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver /* quick access to all handles on object, e.g. viewer, whether enabled or not */  /* quick access to all fields on object, e.g. viewer, whether enabled or not */ = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/viewer.p).
    */
   public void execute()
   {
      logical glIsDynamicsRunning = TypeFactory.logical();
      
      externalProcedure(Viewer.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         RecordBuffer.openScope(ttcombodata, ttlookup, ttdcombo, ttlookupcopy, ttdcombocopy);
         
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* else if appserver aware (adecomm/appserv.i) include appserver props */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* _UIB-CODE-BLOCK-END */
         /* ***************************  Main Block  *************************** */
         /* Clear out lookup/combo query temp-table */
         ttlookup.deleteAll();
         ttdcombo.deleteAll();
         silent(() -> glIsDynamicsRunning.assign(isEqual(new logical(ControlFlowOps.invokeDynamicFunctionIn("isICFRunning", targetProcedure())), true)));
         
         /* Get handle to SDF Cache Manager */
         if ((glIsDynamicsRunning).booleanValue())
         {
            silent(() -> ghSdfcacheManager.assign(ControlFlowOps.invokeDynamicFunctionInWithMode("getManagerHandle", targetProcedure(), "I", "SDFCacheManager")));
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     Gets initial values, enables fields, and allows entry of<br>
    *              values for a new record.     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       addRow creates the RowObject temp-table record
    */
   public void addRecord()
   {
      handle hUpdateTarget = TypeFactory.handle();
      character cTarget = TypeFactory.character();
      character cFields = TypeFactory.character();
      character cColValues = TypeFactory.character();
      handle hGaSource = TypeFactory.handle();
      character cUpdateNames = TypeFactory.character();
      integer iV = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("addRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
         cTarget.assign(ghProp.unwrapBufferField().value());
         hUpdateTarget.assign(handle.fromString(cTarget));
         
         /* If we're a GroupAssign-Target, then our Source has already done 
                  the add; otherwise we invoke addRow in the UpdateTarget.*/
         if ((hUpdateTarget.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTargetNames"));
            cUpdateNames.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
            cFields.assign(ghProp.unwrapBufferField().value());
            
            if (_and(isGreaterThan(cUpdateNames, ""), () -> isEqual(numEntries(cUpdateNames), 1)))
            {
               loopLabel0:
               for (iV.assign(1); _isLessThanOrEqual(iV, numEntries(cFields)); iV.increment())
               {
                  if (_isEqual(indexOf(entry(iV, cFields), "."), 0))
                  {
                     cFields.replaceEntry(iV, concat(cUpdateNames, ".", entry(iV, cFields)));
                  }
               }
            }
            ghTargetProcedure.assign(targetProcedure());
            cColValues.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "addRow", hUpdateTarget, "I", cFields));
            ghTargetProcedure.setUnknown();
            ControlFlowOps.invokeInWithMode("displayFields", targetProcedure(), "I", cColValues);
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignSource"));
            hGaSource.assign(ghProp.unwrapBufferField().value());
            
            if ((hGaSource.isValid()).booleanValue())
            {
               ControlFlowOps.invokeIn("DisplayRecord", targetProcedure());
            }
         }
         NamedEventManager.publish("addRecord", targetProcedure());
         
         /* In case of GroupAssign */
         if ((hUpdateTarget.isValid()).booleanValue())
         {
            ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", new unknown());
         }
         
         AdaptiveQuery query0 = new AdaptiveQuery();
         
         forEach("loopLabel1", new Block((Init) () -> 
         {
            query0.initialize(ttdcombo, "ttdcombo.hviewer = ?", null, "ttdcombo.hviewer asc", new Object[]
            {
               targetProcedure()
            }, LockType.NONE);
         }, 
         (Body) () -> 
         {
            /* For dynamic combos who are a parent to another dynamic combo
                     we need to force a valueChange to ensure that the child combos
                     are refreshed */
            query0.next();
            ControlFlowOps.invokeIn("valueChanged", ttdcombo.getHwidget());
         }));
         
         NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
         {
            "update"
         });
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Cancels the add, copy or update operation for the current row.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       cancelRecord in datavis.p (invoked by RUN SUPER) actually <br>
    *              cancels the update and for an add or copy, deletes the temp-table <br>
    *              record.
    */
   public void cancelRecord()
   {
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("cancelRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         NamedEventManager.publish("cancelRecord", targetProcedure());
         /* In case GroupAssign */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Copy a row    <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       copyRow creates the RowObject temp-table record
    */
   public void copyRecord()
   {
      handle hUpdateTarget = TypeFactory.handle();
      character cTarget = TypeFactory.character();
      character cFields = TypeFactory.character();
      character cColValues = TypeFactory.character();
      handle hGaSource = TypeFactory.handle();
      character cUpdateNames = TypeFactory.character();
      integer iV = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("copyRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
         cTarget.assign(ghProp.unwrapBufferField().value());
         hUpdateTarget.assign(handle.fromString(cTarget));
         
         /* If we're a GroupAssign-Target, then our Source has already done 
                  the add; otherwise we invoke addRow in the UpdateTarget.*/
         if ((hUpdateTarget.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTargetNames"));
            cUpdateNames.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
            cFields.assign(ghProp.unwrapBufferField().value());
            
            if (_isEqual(numEntries(cUpdateNames), 1))
            {
               loopLabel2:
               for (iV.assign(1); _isLessThanOrEqual(iV, numEntries(cFields)); iV.increment())
               {
                  if (_isEqual(indexOf(entry(iV, cFields), "."), 0))
                  {
                     cFields.replaceEntry(iV, concat(cUpdateNames, ".", entry(iV, cFields)));
                  }
               }
            }
            ghTargetProcedure.assign(targetProcedure());
            cColValues.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "copyRow", hUpdateTarget, "I", cFields));
            
            if (_isUnknown(cColValues))
            {
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setNewRecord", targetProcedure(), "I", false);
               /* {fn <func-name> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionIn(character.class, "showDataMessages", targetProcedure());
               returnNormal("ADM-ERROR");
            }
            
            ghTargetProcedure.setUnknown();
            ControlFlowOps.invokeInWithMode("displayFields", targetProcedure(), "I", cColValues);
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignSource"));
            hGaSource.assign(ghProp.unwrapBufferField().value());
            
            if ((hGaSource.isValid()).booleanValue())
            {
               ControlFlowOps.invokeIn("DisplayRecord", targetProcedure());
            }
         }
         NamedEventManager.publish("copyRecord", targetProcedure());
         
         /* In case of GroupAssign */
         if ((hUpdateTarget.isValid()).booleanValue())
         {
            ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", new unknown());
         }
         
         NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
         {
            "update"
         });
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Added this override procedure to remove any Dynamic Lookup and<br>
    *              Dynamic Combo temp-table records for the viewer being destroyed.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void destroyObject()
   {
      internalProcedure(new Block((Body) () -> 
      {
         ttdcombo.delete("hviewer = ?", targetProcedure());
         ttlookup.delete("hviewer = ?", targetProcedure());
         ControlFlowOps.runSuper("destroyObject");
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Disables fields in the ENABLED-FIELDS list (AppBuilder generated<br>
    *              preprocessor).<br>
    * Parameters:  INPUT pcFieldType AS CHARACTER  'All' or 'Create'<br>
    * Notes:       At present at least, this takes an argument which may be<br>
    *              "All" or "Create" to allow, e.g., cancelRecord to disable just<br>
    *              ADM-CREATE-FIELDS.<br>
    *              "Create" is currently not supported. <br>
    *              When a frame field is a SmartObject itself (for example, a <br>
    *              SmartField), disableField is run in the SmartObject to disable<br>
    *              it.
    */
   public void disableFields(final character _pcFieldType)
   {
      character pcFieldType = TypeFactory.initInput(_pcFieldType);
      character cEnableFields = TypeFactory.character();
      /* Field handles. */
      character cEnableObjects = TypeFactory.character();
      /* Object handles */
      integer iField = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      logical lEnabled = TypeFactory.logical();
      character cObjectsToDisable = TypeFactory.character();
      character cName = TypeFactory.character();
      handle hPopup = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
         lEnabled.assign(ghProp.unwrapBufferField().value());
         
         if ((lEnabled).booleanValue())
         {
            /* CreateFields not yet supported
                   IF pcFieldType = "Create":U THEN     _* Disable just Create-only fields *_
                    {get CreateHandles cEnableFields}.
                  ELSE 
                  */
            if (_isEqual(pcFieldType, "All"))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
               cEnableFields.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
               cEnableObjects.assign(ghProp.unwrapBufferField().value());
               /* {get Prop var [handle]} */ 
               cObjectsToDisable.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledObjFldsToDisable", targetProcedure()));
            }
            
            IntegerExpression resExpr0 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cEnableFields);
               }
            };
            ToClause toClause0 = new ToClause(iField, 1, resExpr0);
            
            doTo("loopLabel5", toClause0, new Block((Body) () -> 
            {
               hField.assign(handle.fromString(entry(iField, cEnableFields)));
               
               /* "Frame Field" May be a SmartObject procedure handle */
               if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
               {
                  silent(() -> ControlFlowOps.invokeIn("disableField", hField));
               }
               else
               {
                  if (_not(hField.unwrapWidget().isHidden()))
                  {
                     if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                     {
                         /* Editors must be sensitive, not R-O*/hField.unwrapWriteProtectable().setReadOnly(new logical(true));
                     }
                     else
                     {
                        hField.unwrapSensitive().setSensitive(new logical(false));
                     }
                     
                     hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                     
                     if ((hPopup.isValid()).booleanValue())
                     {
                        hPopup.unwrapSensitive().setSensitive(new logical(false));
                     }
                  }
 /* Skip fields hidden for multi-layout */;
               }
            }));
            
            /* END DO iField     */
            /* Disable no db fields */
            if (_isNotEqual(cObjectsToDisable, "(none)"))
            {
               IntegerExpression resExpr1 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return numEntries(cEnableObjects);
                  }
               };
               ToClause toClause1 = new ToClause(iField, 1, resExpr1);
               
               doTo("loopLabel6", toClause1, new Block((Body) () -> 
               {
                  hField.assign(handle.fromString(entry(iField, cEnableObjects)));
                  
                  if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                  {
                     silent(() -> cName /* {fn <func-name> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldName", hField)));
                  }
                  else
                  {
                     cName.assign(hField.unwrap().name());
                  }
                  
                  if (_or(isEqual(cObjectsToDisable, "(All)"), () -> matchesList(cObjectsToDisable, cName)))
                  {
                     /* "Frame Field" May be a SmartObject procedure handle */
                     if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                     {
                        silent(() -> ControlFlowOps.invokeIn("disableField", hField));
                     }
                     else
                     {
                        if (_not(hField.unwrapWidget().isHidden()))
                        {
                           if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                           {
                               /* Editors must be sensitive, not R-O*/hField.unwrapWriteProtectable().setReadOnly(new logical(true));
                           }
                           else
                           {
                              hField.unwrapSensitive().setSensitive(new logical(false));
                           }
                           
                           hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                           
                           if ((hPopup.isValid()).booleanValue())
                           {
                              hPopup.unwrapSensitive().setSensitive(new logical(false));
                           }
                        }
 /* Skip fields hidden for multi-layout */;
                     }
                  }
               }));
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("fieldsEnabled"));
            ghProp.unwrapBufferField().changeValue(new logical(false));
         }
         ControlFlowOps.runSuper("disableFields", pcFieldType);
         /* In case there's a GroupAssign */
         NamedEventManager.publish("disableFields", targetProcedure(), "I", new Object[]
         {
            pcFieldType
         });
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     "Displays" the current row values by moving them to the frame's<br>
    *              screen-values.<br>
    * Parameters:  INPUT pcColValues AS CHARACTER  CHR(1) - delimited list of the<br>
    *              BUFFER-VALUEs of the requested columns; the first entry in the<br>
    *              list is the RowIdent code.<br>
    * Notes:       When a frame field is a SmartObject (for example, a<br>
    *              SmartField), setDataValue is invoked in the SmartObject to set <br>
    *              its value.
    */
   public void displayFields(final character _pcColValues)
   {
      character pcColValues = TypeFactory.initInput(_pcColValues);
      handle hFrameField = TypeFactory.handle();
      integer iValue = TypeFactory.integer();
      integer iColCount = TypeFactory.integer();
      character cFieldHandles = TypeFactory.character();
      character cRowIdent = TypeFactory.character();
      character cValue = TypeFactory.character();
      handle hGasource = TypeFactory.handle();
      /* Used for Dynamic Combos */
      character cWidgetNames = TypeFactory.character();
      character cWidgetValues = TypeFactory.character();
      character cFieldName = TypeFactory.character();
      /* Field Security Check */
      character cAllFieldHandles = TypeFactory.character();
      character cSecuredFields = TypeFactory.character();
      integer iFieldPos = TypeFactory.integer();
      /* Used for RowUserProp - Row User Property functions */
      character cDataSourceNames = TypeFactory.character();
      character cRowUserProp = TypeFactory.character();
      character cRowUserPropName = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      character cUpdateTargetNames = TypeFactory.character();
      handle hDataSource = TypeFactory.handle();
      integer iPropLoop = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
         cFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
         cAllFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
         cSecuredFields.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignSource"));
         hGasource.assign(ghProp.unwrapBufferField().value());
         iColCount.assign(numEntries(cFieldHandles));
         /* Save off the row number, which is always the first value returned. */
         cRowIdent.assign(entry(1, pcColValues, chr(1)));
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", cRowIdent);
         
         if (_isNotEqual(cFieldHandles, ""))
         {
            ToClause toClause2 = new ToClause(iValue, 1, iColCount);
            
            doTo("loopLabel7", toClause2, new Block((Body) () -> 
            {
               cValue.assign(rightTrim(entry(plus(iValue, 1), pcColValues, chr(1))));
               /* A "Frame Field" may be a SmartObject procedure handle. */
               hFrameField.assign(handle.fromString(entry(iValue, cFieldHandles)));
               iFieldPos.assign(lookup(valueOf(hFrameField), cAllFieldHandles));
               
               if (_isEqual(hFrameField.unwrapType().getResourceType(), "PROCEDURE"))
               {
                  /* Make sure that we do not set the value of the data field here 
                          for Dynamic Combos - this takes care of the flashing problems */
                  if (_isEqual(lookup("dynamicCombo", hFrameField.unwrapProcedure().internalEntries()), 0))
                  {
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataValue", hFrameField, "I", cValue);
                     
                     /* If the SDF is hidden by security, hide it */
                     if (_and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "Hidden")))
                     {
                        silent(() -> ControlFlowOps.invokeIn("hideObject", hFrameField));
                     }
                  }
                  else
                  {
                     if (_and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "Hidden")))
                     {
                        /* {set Prop value [handle]} */
                        ControlFlowOps.invokeDynamicFunctionInWithMode("setSecured", hFrameField, "I", true);
                        silent(() -> ControlFlowOps.invokeIn("hideObject", hFrameField));
                     }
                  }
                  /* {set Prop value [handle]} */
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataModified", hFrameField, "I", false);
                  
                  /* Set Static SDF Values for Dynamic Combo */
                  if ((hFrameField.isValid()).booleanValue())
                  {
                     cFieldName.assign("");
                     /* {get Prop var [handle]} */
                     silent(() -> cFieldName.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldName", hFrameField)));
                     
                     if (_isNotEqual(cFieldName, ""))
                     {
                        if (_isEqual(cWidgetNames, ""))
                        {
                           cWidgetNames.assign(cFieldName);
                           cWidgetValues.assign(_or(isUnknown(cValue), () -> isEqual(cValue, "?")) ? new character("") : cValue);
                        }
                        else
                        {
                           cWidgetNames.assign(concat(cWidgetNames, chr(3), cFieldName));
                           cWidgetValues.assign(concat(cWidgetValues, chr(3), (_or(isUnknown(cValue), () -> isEqual(cValue, "?")) ? new character("") : cValue)));
                        }
                     }
                  }
               }
               else
               {
                  /* reset selection list */
                  if (_isEqual(hFrameField.unwrapType().getResourceType(), "SELECTION-LIST"))
                  {
                     hFrameField.unwrapWidget().setScreenValue(new character(""));
                  }
                  
                  /* Check Security for hidden fields */
                  if (_or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isNotEqual(entry(iFieldPos, cSecuredFields), "Hidden")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, "")))
                  {
                     if (_isEqual(hFrameField.unwrapType().getResourceType(), "COMBO-BOX"))
                     {
                        if (_isEqual(cValue, ""))
                        {
                           silent(() -> hFrameField.unwrapWidget().setScreenValue(new character(" ")));
                        }
                        else
                        {
                           hFrameField.unwrapWidget().setScreenValue(cValue);
                        }
                     }
                     else
                     {
                        hFrameField.unwrapWidget().setScreenValue(_notUnknown(pcColValues) ? (_or(isNotEqual(hFrameField.unwrapCommonField().getDataType(), "LOGICAL"), () -> isEqual(hFrameField.unwrapType().getResourceType(), "RADIO-SET")) ? (_and(isEqual(hFrameField.unwrapType().getResourceType(), "COMBO-BOX"), () -> isEqual(cValue, "")) ? new character( /* iz 5269 */" ") : cValue) : (_and(isEqual(hFrameField.unwrapType().getResourceType(), "FILL-IN"), () -> isEqual(cValue, "?")) ? cValue : _isEqual(cValue, "yes") ? entry(1, hFrameField.unwrapCommonField().getFormat(), "/") : entry(2, hFrameField.unwrapCommonField().getFormat(), "/"))) : (_or(isNotEqual(hFrameField.unwrapCommonField().getDataType(), "LOGICAL"), () -> isEqual(hFrameField.unwrapType().getResourceType(), "RADIO-SET")) ? new character("") : entry(2, hFrameField.unwrapCommonField().getFormat(), "/")));
                     }
                  }
                  /* For Combos and Selection lists */
                  hFrameField.unwrapWidget().setModified(new logical(false));
                  
                  /* This will be checked by update code. */
                  if (_isEqual(cWidgetNames, ""))
                  {
                     cWidgetNames.assign(hFrameField.unwrap().name());
                     cWidgetValues.assign(_or(isUnknown(hFrameField.unwrapWidget().getScreenValue()), () -> isEqual(hFrameField.unwrapWidget().getScreenValue(), "?")) ? new character("") : hFrameField.unwrapWidget().getScreenValue());
                  }
                  else
                  {
                     cWidgetNames.assign(concat(cWidgetNames, chr(3), hFrameField.unwrap().name()));
                     cWidgetValues.assign(concat(cWidgetValues, chr(3), (_or(isUnknown(hFrameField.unwrapWidget().getScreenValue()), () -> isEqual(hFrameField.unwrapWidget().getScreenValue(), "?")) ? new character("") : hFrameField.unwrapWidget().getScreenValue())));
                  }
               }
            }));
         }
         
         if ((new logical( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseRepository", targetProcedure()))).booleanValue())
         {
            if (_and(hDataSource.isValid(), () -> not(hGasource.isValid())))
            {
               if ((new logical( /* {fnarg <func-name> <arg> [handle] */ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "instanceOf", hDataSource, "I", "SBO"))).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
                  cDataSourceNames.assign(ghProp.unwrapBufferField().value());
                  /* {get Prop var [handle]} */ 
                  cUpdateTargetNames.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdateTargetNames", targetProcedure()));
                  
                  if (_isEqual(numEntries(cDataSourceNames), 1))
                  {
                     cQualifier.assign(cDataSourceNames);
                  }
                  else
                  {
                     if (_isEqual(numEntries(cUpdateTargetNames), 1))
                     {
                        cQualifier.assign(cUpdateTargetNames);
                     }
                  }
               }
               /* if datasource is SBO */
               cRowUserPropName.assign(_isGreaterThan(cQualifier, "") ? concat(cQualifier, ".RowUserProp") : new character("RowUserProp"));
               silent(() -> cRowUserProp /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnValue", hDataSource, "I", cRowUserPropName)));
               
               loopLabel8:
               for (iPropLoop.assign(1); _isLessThanOrEqual(iPropLoop, numEntries(cRowUserProp, chr(4))); iPropLoop.increment())
               {
                  if (_isEqual(entry(1, entry(iPropLoop, cRowUserProp, chr(4)), chr(3)), "gsmcmauto"))
                  {
                     messageBox(entry(new integer(2), entry(iPropLoop, cRowUserProp, chr(4)), chr(3)), ALERT_INFO, BTN_OK, "Comment(s) for Record");
                  }
               }
            }
            /* get lookup queries for specific data value and redisplay data */
            NamedEventManager.publish("getLookupQuery", targetProcedure(), "U", new Object[]
            {
               new TableParameter(ttlookup)
            });
            NamedEventManager.publish("getComboQuery", targetProcedure(), "U", new Object[]
            {
               new TableParameter(ttdcombo)
            });
            
            if ((new FindQuery(ttlookup, "ttlookup.hviewer = ?", null, "ttlookup.hviewer asc", new Object[]
            {
               targetProcedure()
            }, LockType.NONE).hasAny()).booleanValue())
            {
               silent(() -> ControlFlowOps.invokeIn("stripLookupFields", targetProcedure()));
            }
            
            if ((gshAstraAppserver.isValid()).booleanValue())
            {
               if (_or(new FindQuery(ttlookup, "ttlookup.hviewer = ? and ttlookup.lrefreshquery = true", null, "ttlookup.hviewer asc, ttlookup.lrefreshquery asc", new Object[]
               {
                  targetProcedure()
               }, LockType.NONE).hasAny(), () -> new FindQuery(ttdcombo, "ttdcombo.hviewer = ?", null, "ttdcombo.hviewer asc", new Object[]
               {
                  targetProcedure()
               }, LockType.NONE).hasAny()))
               {
                  AdaptiveQuery query3 = new AdaptiveQuery();
                  
                  forEach("loopLabel9", new Block((Init) () -> 
                  {
                     query3.initialize(ttlookup, "ttlookup.hviewer = ?", null, "ttlookup.hviewer asc", new Object[]
                     {
                        targetProcedure()
                     });
                  }, 
                  (Body) () -> 
                  {
                     /* create copies and pass those to the server. This will at least 
                                reduce data over to the AppServer if there are other lookups or 
                                combo records for other viewers */
                     query3.next();
                     ttlookupcopy.create();
                     RecordBuffer.copy(ttlookup, ttlookupcopy, true);
                     ttlookup.deleteRecord();
                  }));
                  
                  AdaptiveQuery query4 = new AdaptiveQuery();
                  
                  forEach("loopLabel10", new Block((Init) () -> 
                  {
                     query4.initialize(ttdcombo, "ttdcombo.hviewer = ?", null, "ttdcombo.hviewer asc", new Object[]
                     {
                        targetProcedure()
                     });
                  }, 
                  (Body) () -> 
                  {
                     /* now move the combo records */
                     query4.next();
                     ttdcombocopy.create();
                     RecordBuffer.copy(ttdcombo, ttdcombocopy, true);
                     ttdcombo.deleteRecord();
                  }));
                  
                  /* If the SDF Cache manager is running, get the data from there */
                  if ((ghSdfcacheManager.isValid()).booleanValue())
                  {
                     ControlFlowOps.invokeInWithMode("retrieveSDFCache", ghSdfcacheManager, "UUIII", new TableParameter(ttlookupcopy), new TableParameter(ttdcombocopy), cWidgetNames, cWidgetValues, targetProcedure());
                  }
                  else
                  {
 /* Get data from server */
                     ControlFlowOps.invokeRemoteWithMode("adm2/lookupqp.p", (handle) null, gshAstraAppserver, false, "UUIII", new TableParameter(ttlookupcopy), new TableParameter(ttdcombocopy), cWidgetNames, cWidgetValues, targetProcedure());
                  }
                  
                  AdaptiveQuery query5 = new AdaptiveQuery();
                  
                  forEach("loopLabel11", new Block((Init) () -> 
                  {
                     query5.initialize(ttlookupcopy, ((String) null), null, "ttlookupcopy.hwidget asc, ttlookupcopy.cwidgetname asc");
                  }, 
                  (Body) () -> 
                  {
                     /* Now we need to move the records back to the original temp-tables.
                                First move the lookup records */
                     query5.next();
                     ttlookup.create();
                     RecordBuffer.copy(ttlookupcopy, ttlookup, true);
                  }));
                  
                  AdaptiveQuery query6 = new AdaptiveQuery();
                  
                  forEach("loopLabel12", new Block((Init) () -> 
                  {
                     query6.initialize(ttdcombocopy, ((String) null), null, "ttdcombocopy.hwidget asc");
                  }, 
                  (Body) () -> 
                  {
                     /* now move the combo records */
                     query6.next();
                     ttdcombo.create();
                     RecordBuffer.copy(ttdcombocopy, ttdcombo, true);
                  }));
                  
                  /* Now clear the temp-tables */
                  ttlookupcopy.deleteAll();
                  ttdcombocopy.deleteAll();
               }
            }
            /* valid-handle(gshAstraAppsrver) */
            /* Display Lookup query/Combo results */
            NamedEventManager.publish("displayLookup", targetProcedure(), "I", new Object[]
            {
               new TableParameter(ttlookup)
            });
            NamedEventManager.publish("displayCombo", targetProcedure(), "I", new Object[]
            {
               new TableParameter(ttdcombo)
            });
         }
         ControlFlowOps.invokeIn("updateTitle", targetProcedure());
         silent(() -> ControlFlowOps.invokeIn("rowDisplay", targetProcedure()));
         /* Custom display checks. */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Enables fields in the ENABLED-FIELDS list (AppBuilder generated<br>
    *              preprocessor.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       When the frame field is a SmartObject (for example, a <br>
    *              SmartField), enableField is run in the SmartObject.
    */
   public void enableFields()
   {
      character cEnableFields = TypeFactory.character();
      /* Field handles. */
      character cEnableObjects = TypeFactory.character();
      /* Object handles */
      integer iField = TypeFactory.integer();
      handle hField = TypeFactory.handle();
      logical lEnabled = TypeFactory.logical();
      character cState = TypeFactory.character();
      character cNewRecord = TypeFactory.character();
      character cUpdateTarget = TypeFactory.character();
      handle hGasource = TypeFactory.handle();
      handle hPopup = TypeFactory.handle();
      /* Field Security Check */
      character cAllFieldHandles = TypeFactory.character();
      character cSecuredFields = TypeFactory.character();
      integer iFieldPos = TypeFactory.integer();
      character cObjectsToDisable = TypeFactory.character();
      character cName = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
         lEnabled.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignSource"));
         hGasource.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("AllFieldHandles"));
         cAllFieldHandles.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
         cSecuredFields.assign(ghProp.unwrapBufferField().value());
         
         /* Check the record state in the GA source to avoid timing problems
              when this method is called from queryPosition.
              The NewRecord value is not even propagated to GroupAssign-Target(s). */
         if ((hGasource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hGasource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("RecordState"));
            cState.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */
            cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", hGasource));
            ghProp.assign(handle.fromString(entry(1, hGasource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
            cUpdateTarget.assign(ghProp.unwrapBufferField().value());
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("RecordState"));
            cState.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */ 
            cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
            cUpdateTarget.assign(ghProp.unwrapBufferField().value());
         }
         
         if (_and(and(not(lEnabled), () -> isNotEqual(cUpdateTarget, "")), () -> or(isEqual(cState, "RecordAvailable"), () -> isNotEqual(cNewRecord, "No"))))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
            cEnableFields.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
            cEnableObjects.assign(ghProp.unwrapBufferField().value());
            /* {get Prop var [handle]} */ 
            cObjectsToDisable.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledObjFldsToDisable", targetProcedure()));
            
            IntegerExpression resExpr2 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cEnableFields);
               }
            };
            ToClause toClause3 = new ToClause(iField, 1, resExpr2);
            
            doTo("loopLabel13", toClause3, new Block((Body) () -> 
            {
               hField.assign(handle.fromString(entry(iField, cEnableFields)));
               iFieldPos.assign(lookup(valueOf(hField), cAllFieldHandles));
               
               if ((hField.isValid()).booleanValue())
               {
                  /* "Frame Field" May be a SmartObject procedure handle */
                  if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                  {
                     if (_or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, "")))
                     {
                        /* Check Security */
                        silent(() -> ControlFlowOps.invokeIn("enableField", hField));
                     }
                     else
                     {
                        if (_isEqual(entry(iFieldPos, cSecuredFields), "Hidden"))
                        {
                           silent(() -> ControlFlowOps.invokeIn("hideObject", hField));
                        }
                        else
                        {
                           silent(() -> ControlFlowOps.invokeIn("disableField", hField));
                        }
                     }
                  }
                  else
                  {
                     if (_not(hField.unwrapWidget().isHidden()))
                     {
                        /* Check Security */
                        if (_or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, "")))
                        {
                           hField.unwrapSensitive().setSensitive(new logical(true));
                           
                           if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                           {
                               /* Ed's must be sensitive, not R-O*/hField.unwrapWriteProtectable().setReadOnly(new logical(false));
                           }
                           
                           /* don't enable if read-only (can-query is ok as only fields can 
                                          have popups) */
                           if (_and(HandleOps.canQuery(hField, "read-only"), () -> not(hField.unwrapWriteProtectable().isReadOnly())))
                           {
                              hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                              
                              if ((hPopup.isValid()).booleanValue())
                              {
                                 hPopup.unwrapSensitive().setSensitive(new logical(true));
                              }
                           }
                        }
                     }
 /* Skip fields hidden for multi-layout */;
                  }
               }
            }));
            
            /* Looping through the fields */
            /* Enable no db fields */
            if (_isNotEqual(cObjectsToDisable, "(none)"))
            {
               IntegerExpression resExpr3 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return numEntries(cEnableObjects);
                  }
               };
               ToClause toClause4 = new ToClause(iField, 1, resExpr3);
               
               doTo("loopLabel14", toClause4, new Block((Body) () -> 
               {
                  hField.assign(handle.fromString(entry(iField, cEnableObjects)));
                  
                  if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                  {
                     silent(() -> cName /* {fn <func-name> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldName", hField)));
                  }
                  else
                  {
                     cName.assign(hField.unwrap().name());
                  }
                  
                  if (_or(isEqual(cObjectsToDisable, "(All)"), () -> matchesList(cObjectsToDisable, cName)))
                  {
                     iFieldPos.assign(0);
                     iFieldPos.assign(lookup(valueOf(hField), cAllFieldHandles));
                     
                     if ((hField.isValid()).booleanValue())
                     {
                        /* "Frame Field" May be a SmartObject procedure handle */
                        if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                        {
                           if (_or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, "")))
                           {
                              silent(() /* Check Security */ -> ControlFlowOps.invokeIn("enableField", hField));
                           }
                           else
                           {
                              if (_isEqual(entry(iFieldPos, cSecuredFields), "Hidden"))
                              {
                                 silent(() -> ControlFlowOps.invokeIn("hideObject", hField));
                              }
                              else
                              {
                                 silent(() -> ControlFlowOps.invokeIn("disableField", hField));
                              }
                           }
                        }
                        else
                        {
                           if (_not(hField.unwrapWidget().isHidden()))
                           {
                              /* Check Security */
                              if (_or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cSecuredFields), iFieldPos)), () -> isEqual(entry(iFieldPos, cSecuredFields), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cSecuredFields, "")))
                              {
                                 hField.unwrapSensitive().setSensitive(new logical(true));
                                 
                                 if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                                 {
                                     /* Ed's must be sensitive, not R-O*/hField.unwrapWriteProtectable().setReadOnly(new logical(false));
                                 }
                                 
                                 /* don't enable if read-only (can-query is ok as only fields can 
                                                 have popups) */
                                 if (_and(HandleOps.canQuery(hField, "read-only"), () -> not(hField.unwrapWriteProtectable().isReadOnly())))
                                 {
                                    hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                                    
                                    if ((hPopup.isValid()).booleanValue())
                                    {
                                       hPopup.unwrapSensitive().setSensitive(new logical(true));
                                    }
                                 }
                              }
                           }
 /* Skip fields hidden for multi-layout */;
                        }
                     }
                  }
               }));
            }
            
            /* Do for all enabled objects */
            /* If the list of enabled field handles isn't set, it may because this
                   object hasn't been fully initialized yet. So don't set the enabled
                   flag because we may be through here again. */
            if (_isNotEqual(cEnableFields, ""))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
               ghProp.unwrapBufferField().changeValue(new logical(true));
            }
            /* {set Prop value [handle]} */ 
         }
         else
         {
 /* Ensure that SDF fields is disabled if no record available */
            if (_and(and(not(lEnabled), () -> isNotEqual(cUpdateTarget, "")), () -> isEqual(cState, "NoRecordAvailable")))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
               cEnableFields.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
               cEnableObjects.assign(ghProp.unwrapBufferField().value());
               /* {get Prop var [handle]} */ 
               cObjectsToDisable.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getEnabledObjFldsToDisable", targetProcedure()));
               
               IntegerExpression resExpr4 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return numEntries(cEnableFields);
                  }
               };
               ToClause toClause5 = new ToClause(iField, 1, resExpr4);
               
               doTo("loopLabel15", toClause5, new Block((Body) () -> 
               {
                  hField.assign(handle.fromString(entry(iField, cEnableFields)));
                  iFieldPos.assign(0);
                  iFieldPos.assign(lookup(valueOf(hField), cAllFieldHandles));
                  
                  if ((hField.isValid()).booleanValue())
                  {
                     /* "Frame Field" May be a SmartObject procedure handle */
                     if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                     {
                        silent(() -> ControlFlowOps.invokeIn("disableField", hField));
                     }
                     else
                     {
                        if (_not(hField.unwrapWidget().isHidden()))
                        {
                           if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                           {
                               /* Editors must be sensitive, not R-O*/hField.unwrapWriteProtectable().setReadOnly(new logical(true));
                           }
                           else
                           {
                              hField.unwrapSensitive().setSensitive(new logical(false));
                           }
                           
                           hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                           
                           if ((hPopup.isValid()).booleanValue())
                           {
                              hPopup.unwrapSensitive().setSensitive(new logical(false));
                           }
                        }
 /* Skip fields hidden for multi-layout */;
                     }
                  }
               }));
               
               /* END DO iField     */
               if (_isNotEqual(cObjectsToDisable, "(none)"))
               {
                  IntegerExpression resExpr5 = new IntegerExpression()
                  {
                     public integer execute()
                     {
                        return numEntries(cEnableObjects);
                     }
                  };
                  ToClause toClause6 = new ToClause(iField, 1, resExpr5);
                  
                  doTo("loopLabel16", toClause6, new Block((Body) () -> 
                  {
                     hField.assign(handle.fromString(entry(iField, cEnableObjects)));
                     
                     if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                     {
                        silent(() -> cName /* {fn <func-name> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldName", hField)));
                     }
                     else
                     {
                        cName.assign(hField.unwrap().name());
                     }
                     
                     if (_or(isEqual(cObjectsToDisable, "(All)"), () -> matchesList(cObjectsToDisable, cName)))
                     {
                        iFieldPos.assign(0);
                        iFieldPos.assign(lookup(valueOf(hField), cAllFieldHandles));
                        
                        if ((hField.isValid()).booleanValue())
                        {
                           /* "Frame Field" May be a SmartObject procedure handle */
                           if (_isEqual(hField.unwrapType().getResourceType(), "PROCEDURE"))
                           {
                              silent(() -> ControlFlowOps.invokeIn("disableField", hField));
                           }
                           else
                           {
                              if (_not(hField.unwrapWidget().isHidden()))
                              {
                                 if (_isEqual(hField.unwrapType().getResourceType(), "EDITOR"))
                                 {
                                     /* Editors must be sensitive, not R-O*/hField.unwrapWriteProtectable().setReadOnly(new logical(true));
                                 }
                                 else
                                 {
                                    hField.unwrapSensitive().setSensitive(new logical(false));
                                 }
                                 
                                 hPopup /* {fnarg <func-name> <arg> [handle] */ .assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "popupHandle", targetProcedure(), "I", hField));
                                 
                                 if ((hPopup.isValid()).booleanValue())
                                 {
                                    hPopup.unwrapSensitive().setSensitive(new logical(false));
                                 }
                              }
 /* Skip fields hidden for multi-layout */;
                           }
                        }
                     }
                  }));
               }
            }
         }
         /* Disbling SDF if not data available */
         ControlFlowOps.runSuper("enableFields");
         NamedEventManager.publish("enableFields", targetProcedure());
         /* In case of GroupAssign */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *  Purpose: Run whenever a field is modified. <br>
    *           Sets DataModified  to true unless the ModifyFields property is <br>
    *           specifed to ignore this field. <br>
    * Paramter: Handle of a field whose value has changed <br>
    *    Notes: The passed field must be in the list of EnabledFields or <br>
    *           EnabledObjFlds.<br>
    *         - Called from valueChanged or SDF's setDataModified or Client APIs that<br>
    *           changes widget values.<br>
    *         - Will return true if this change triggered a DataModified change,<br>
    *           i.e that this is the FIRST change in the object.
    */
   public void fieldModified(final handle _phField)
   {
      handle phField = TypeFactory.initInput(_phField);
      logical lSetModified = TypeFactory.logical();
      character cEnabledHandles = TypeFactory.character();
      character cEnabledFields = TypeFactory.character();
      character cEnabledObjHdls = TypeFactory.character();
      character cEnabledObjFlds = TypeFactory.character();
      integer iFld = TypeFactory.integer();
      character cName = TypeFactory.character();
      character cModifyFields = TypeFactory.character();
      handle hUpdateTarget = TypeFactory.handle();
      character cUpdatableCols = TypeFactory.character();
      logical lResult = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         if (_not(phField.isValid()))
         {
            returnNormal();
         }
         
         /* if modified is set already just return */
         /* {get Prop var [handle]} */ 
         lResult.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", targetProcedure()));
         
         if ((lResult).booleanValue())
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
         lResult.assign(ghProp.unwrapBufferField().value());
         
         if (_not(lResult))
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ModifyFields"));
         cModifyFields.assign(ghProp.unwrapBufferField().value());
         
         /* Return if no fields should set DataModified.*/
         if (_isEqual(cModifyFields, "(None)"))
         {
            returnNormal();
         }
         
         /* If for some reason ModifyFields is not set, set default = (all) */
         if (_or(isEqual(cModifyFields, ""), () -> isUnknown(cModifyFields)))
         {
            cModifyFields.assign("(All)");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         cEnabledHandles.assign(ghProp.unwrapBufferField().value());
         iFld.assign(lookup(valueOf(phField), cEnabledHandles));
         
         if (_isGreaterThan(iFld, 0))
         {
            /* One of the EnabledFields has changed so (All) or (EnabledFields) 
                     should set DataModified */
            if (_or(isEqual(cModifyFields, "(All)"), () -> isEqual(cModifyFields, "(EnabledFields)")))
            {
               lSetModified.assign(true);
            }
            else
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
               cEnabledFields.assign(ghProp.unwrapBufferField().value());
               cName.assign(entry(iFld, cEnabledFields));
               
               /* if (updatable) check if the name is updatable in the updateTarget */
               if (_isEqual(cModifyFields, "(Updatable)"))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
                  hUpdateTarget.assign(ghProp.unwrapBufferField().value());
                  
                  if ((hUpdateTarget.isValid()).booleanValue())
                  {
                     /* {get Prop var [handle]} */
                     cUpdatableCols.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdatableColumns", hUpdateTarget));
                     
                     if (_isGreaterThan(lookup(cName, cUpdatableCols), 0))
                     {
                        lSetModified.assign(true);
                     }
                  }
               }
               else
               {
                  if (_isGreaterThan(lookup(cName, cModifyFields), 0))
                  {
                     lSetModified.assign(true);
                  }
               }
            }
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjHdls"));
            cEnabledObjHdls.assign(ghProp.unwrapBufferField().value());
            iFld.assign(lookup(valueOf(phField), cEnabledObjHdls));
            
            if (_isGreaterThan(iFld, 0))
            {
               /* '(All)' means that also local fields should set DataModified*/
               if (_isEqual(cModifyFields, "(All)"))
               {
                  lSetModified.assign(true);
               }
               else
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledObjFlds"));
                  cEnabledObjFlds.assign(ghProp.unwrapBufferField().value());
                  cName.assign(entry(iFld, cEnabledObjFlds));
                  
                  if (_isGreaterThan(lookup(cName, cModifyFields), 0))
                  {
                     lSetModified.assign(true);
                  }
               }
            }
         }
         
         /* else (field not found in enabledHandles) */
         if ((lSetModified).booleanValue())
         {
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataModified", targetProcedure(), "I", true);
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Initialization code specific to SmartDataViewers. <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Reads through list of enabled and displayed fields and<br>
    *              gets their handles to store in FieldHandles and EnabledHandles<br>
    *              properties.
    */
   public void initializeObject()
   {
      handle hFrameField = TypeFactory.handle();
      character cFieldHandles = TypeFactory.character("");
      character cEnabledHandles = TypeFactory.character("");
      character cEnabledFields = TypeFactory.character("");
      character cDisplayedFields = TypeFactory.character("");
      character cCreateHandles = TypeFactory.character("");
      character cTarget = TypeFactory.character();
      handle hContainerHandle = TypeFactory.handle();
      logical lSaveSource = TypeFactory.logical();
      handle hGaSource = TypeFactory.handle();
      character cGaMode = TypeFactory.character();
      integer iField = TypeFactory.integer();
      character cFieldName = TypeFactory.character();
      character cTableName = TypeFactory.character();
      handle hFrameProc = TypeFactory.handle();
      character cUibmode = TypeFactory.character();
      integer iSdfloop = TypeFactory.integer();
      character cContainerTargets = TypeFactory.character();
      handle hSdfhandle = TypeFactory.handle();
      handle hSdfframeHandle = TypeFactory.handle();
      character cNewRecord = TypeFactory.character();
      character cUpdateTargetNames = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("initializeObject");
         
         /* Invoke the standard behavior first. */
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
         cEnabledFields.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UIBMode"));
         cUibmode.assign(ghProp.unwrapBufferField().value());
         
         /* UIBmode gives errors because handles for SDFs is not added */
         if (_begins(cUibmode, "Design"))
         {
            /* If there are NO enabled fields, don't let the viewer be an */
            /* Update-Source or TableIO-Target. */
            if (_isEqual(cEnabledFields, ""))
            {
               ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", targetProcedure(), new character("REMOVE"), new character("SupportedLinks"), new character("Update-Source"));
               ControlFlowOps.invokeInWithMode("modifyListProperty", targetProcedure(), "IIII", targetProcedure(), new character("REMOVE"), new character("SupportedLinks"), new character("TableIO-Target"));
            }
            
            /* END DO cEnabled "" */
            returnNormal();
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
         cTarget.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
         hContainerHandle.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
         cDisplayedFields.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cUpdateTargetNames.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdateTargetNames", targetProcedure()));
         silent(() -> cContainerTargets.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkHandles", targetProcedure(), "I", "Container-Target")));
         hFrameField.assign(hContainerHandle);
         hFrameField.assign(hFrameField.unwrapTree().firstChild());
          /* Field Group */hFrameField.assign(hFrameField.unwrapTree().firstChild());
         cEnabledHandles.assign(fill(",", minus(numEntries(cEnabledFields), 1)));
         cFieldHandles.assign(fill(",", minus(numEntries(cDisplayedFields), 1)));
         
         LogicalOp whileClause0 = () -> hFrameField.isValid();
         
         doWhile("loopLabel17", whileClause0, new Block((Body) () -> 
         {
            /* Check for existance of SDFs */
            if (_and(isEqual(hFrameField.unwrapType().getResourceType(), "FRAME"), () -> isNotEqual(cContainerTargets, "")))
            {
               IntegerExpression resExpr6 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return numEntries(cContainerTargets);
                  }
               };
               ToClause toClause7 = new ToClause(iSdfloop, 1, resExpr6);
               
               doTo("sdfLoop", toClause7, new Block((Body) () -> 
               {
                  hSdfhandle.assign(handle.fromString(entry(iSdfloop, cContainerTargets)));
                  
                  if ((hSdfhandle.isValid()).booleanValue())
                  {
                     silent(() -> hSdfframeHandle.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getContainerHandle", hSdfhandle)));
                  }
                  
                  if (_isEqual(hFrameField, hSdfframeHandle))
                  {
                     hFrameProc.assign(hSdfhandle);
                     leave("sdfLoop");
                  }
               }));
               
               /* We rely on the fact that FieldName becomes unknown if this is a 
                        SmartFrame (unsupported) to avoid it messing up the list */
               if ((hFrameProc.isValid()).booleanValue())
               {
                  silent(() -> cFieldName.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getFieldName", hFrameProc)));
               }
               else
               {
                  cFieldName.setUnknown();
               }
            }
            else
            {
               hFrameProc.setUnknown();
               
               /* Signal no SMo here */
               /* As of 9.1B, the DisplayedFields can be qualified by
                        * the SDO Objectname if it's not just RowObject. */
               /* In 9.1C, when a widget is dynamic, the :TABLE attribute is not set
                        * for widgets on a frame. The table name should thus be stored in the
                        * private-data of the widget in the format "TableName,<table-name>"    */
               if ((hFrameField.unwrapDynamicResource().dynamic()).booleanValue())
               {
                  if (_isGreaterThan(lookup("TableName", hFrameField.unwrap().getPrivateData()), 0))
                  {
                     cTableName.assign(entry(plus(lookup("TableName", hFrameField.unwrap().getPrivateData()), 1), hFrameField.unwrap().getPrivateData()));
                  }
               }
               else
               {
                  if ((HandleOps.canQuery(hFrameField, "TABLE")).booleanValue())
                  {
                     cTableName.assign(hFrameField.unwrapDatabaseInfo().getTable());
                  }
               }
               cFieldName.assign(concat((_not(matchesList("RowObject,", cTableName)) ? concat(cTableName, ".") : new character("")), hFrameField.unwrap().name()));
            }
            
            /* not a SDF */
            if (_and(notUnknown(cFieldName), () -> isNotEqual(lookup(cFieldName, cDisplayedFields), 0)))
            {
               /* Displayed Fields */
               cFieldHandles.replaceEntry(lookup(cFieldName, cDisplayedFields), (_isUnknown(hFrameProc) ? valueOf(hFrameField) : valueOf(hFrameProc)));
               /* Enabled Fields */
               iField.assign(lookup(cFieldName, cEnabledFields));
               
               if (_isGreaterThan(iField, 0))
               {
                  if (_isGreaterThan(indexOf(cFieldName, "."), 0))
                  {
                     /* remove enabled DB fields that do not have an UpdateTarget */
                     cTarget.assign(entry(1, cFieldName, "."));
                     
                     if (_not(isGreaterThan(lookup(cTarget, cUpdateTargetNames), 0)))
                     {
                        cEnabledFields.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", iField, cEnabledFields, ","));
                        cEnabledHandles.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", iField, cEnabledHandles, ","));
                     }
                     else
                     {
                        cEnabledHandles.replaceEntry(iField, (_isUnknown(hFrameProc) ? valueOf(hFrameField) : valueOf(hFrameProc)));
                     }
                  }
                  else
                  {
                     cEnabledHandles.replaceEntry(iField, (_isUnknown(hFrameProc) ? valueOf(hFrameField) : valueOf(hFrameProc)));
                  }
               }
            }
            /* END DO IF LOOKUP */
            hFrameField.assign(hFrameField.unwrap().getNextSibling());
         }));
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
         ghProp.unwrapBufferField().changeValue(cEnabledFields);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         ghProp.unwrapBufferField().changeValue(cEnabledHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CreateHandles"));
         ghProp.unwrapBufferField().changeValue(cCreateHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
         ghProp.unwrapBufferField().changeValue(cFieldHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignSource"));
         hGaSource.assign(ghProp.unwrapBufferField().value());
         ControlFlowOps.invokeInWithMode("dataAvailable", targetProcedure(), "I", new unknown());
         
         /* See if there's a rec waiting. */
         /* Is this a GaTarget, if so get SaveSOurce, FieldsEnabled and NewRecord from the 
              GA source so we can find out whether we should be enabled (further down) or 
              display the current record 
              (if the GASource is in "Add" or "Copy" mode already */
         if ((hGaSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hGaSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectMode"));
            cGaMode.assign(ghProp.unwrapBufferField().value());
            
            LogicalOp whileClause1 = () -> hGaSource.isValid();
            
            doWhile("loopLabel18", whileClause1, new Block((Body) () -> 
            {
               /* {get Prop var [handle]} */
               cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", hGaSource));
               ghProp.assign(handle.fromString(entry(1, hGaSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignSource"));
               hGaSource.assign(ghProp.unwrapBufferField().value());
            }));
            
            /* synchronize the NewRecord value with our GA-Source */
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setNewRecord", targetProcedure(), "I", cNewRecord);
            
            if (_isNotEqual(cNewRecord, "NO"))
            {
               ControlFlowOps.invokeIn("displayRecord", targetProcedure());
            }
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("SaveSource"));
            lSaveSource.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         /* If we have NO tableio-source (?) or a Tableio-Source in Save mode 
              OR if our groupAssign-source is not in view mode the target also should be,
             (that would have been the case if this GAtarget had been initialized) */
         if (_or(and(isGreaterThan(cGaMode, ""), () -> isNotEqual(cGaMode, "view")), () -> not(isEqual(lSaveSource, false))))
         {
            ControlFlowOps.invokeIn("enableFields", targetProcedure());
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            ControlFlowOps.invokeInWithMode("disableFields", targetProcedure(), "I", new character("ALL"));
         }
         
         /* END ELSE DO IF not enableFields */
         /* Ensure only has entries for this viewer */
         ttcombodata.deleteAll();
         /* gather all master combo queries */
         NamedEventManager.publish("getComboQuery", targetProcedure(), "U", new Object[]
         {
            new TableParameter(ttcombodata)
         });
         
         /* build combo list-item pairs */
         if (_and(gshAstraAppserver.isValid(), () -> new FindQuery(ttcombodata, (String) null, null, "ttcombodata.hwidget asc", LockType.NONE).hasAny()))
         {
            ControlFlowOps.invokeRemoteWithMode("adm2/cobuildp.p", (handle) null, gshAstraAppserver, false, "U", new TableParameter(ttcombodata));
         }
         
         /* populate all master combos */
         NamedEventManager.publish("buildCombo", targetProcedure(), "I", new Object[]
         {
            new TableParameter(ttcombodata)
         });
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Receives messages when a GroupAssignTarget becomes<br>
    *              "active" (normally when it's viewed) or "inactive" (Hidden).<br>
    * Parameters:  pcState AS CHARACTER  'active'/'inactive'<br>
    *              The event is republished up the groupAssignSource and<br>
    *              the DataSource
    */
   public void linkState(final character _pcState)
   {
      character pcState = TypeFactory.initInput(_pcState);
      
      internalProcedure(new Block((Body) () -> 
      {
         NamedEventManager.publish("linkState", targetProcedure(), "I", new Object[]
         {
            pcState
         });
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose: Override to synchronize GATargets <br>
    * Parameters: pcstate -<br>
    *               add,remove,inactive,active<br>
    *             phObject - object to link to <br>
    *             pcLink   - Linkname (data-source)      <br>
    * Notes:   The SDO only calls this with 'inactive' when all data targets<br>
    *          that have a GaSOurce is hidden, using the getGroupAssingHidden<br>
    *          read-only property.
    */
   public void linkStateHandler(final character _pcState, final handle _phObject, final character _pcLink)
   {
      character pcState = TypeFactory.initInput(_pcState);
      handle phObject = TypeFactory.initInput(_phObject);
      character pcLink = TypeFactory.initInput(_pcLink);
      character cTargets = TypeFactory.character();
      handle hTarget = TypeFactory.handle();
      integer iTarget = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         if (_and(isEqual(pcLink, "DataSource"), () -> isEqual(pcState, "Add")))
         {
            /* Some DataSources do not yet support this property     */
            /* (ie SBO, passThru links thru containers etc.).        */
            /* Must be reviewed for complete solution                */
            /* {set Prop value [handle]} */
            silent(() -> ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setFetchAutoComment", phObject, "I", true));
         }
         
         ControlFlowOps.runSuper("linkStateHandler", pcState, phObject, pcLink);
         
         if (_and(matchesList("ACTIVE,INACTIVE", pcState), () -> isEqual(pcLink, "DataSource")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("GroupAssignTarget"));
            cTargets.assign(ghProp.unwrapBufferField().value());
            
            IntegerExpression resExpr7 = new IntegerExpression()
            {
               public integer execute()
               {
                  return numEntries(cTargets);
               }
            };
            ToClause toClause8 = new ToClause(iTarget, 1, resExpr7);
            
            doTo("loopLabel19", toClause8, new Block((Body) () -> 
            {
               hTarget.assign(handle.fromString(entry(iTarget, cTargets)));
               ControlFlowOps.invokeInWithMode("LinkStateHandler", hTarget, "III", pcState, phObject, pcLink);
            }));
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Locates a widget and retuns its handle and the handle of its<br>
    *              TARGET-PROCEDURE<br>
    * Parameters:  pcWidget AS CHARACTER<br>
    *              phWidget AS HANDLE<br>
    *              phTarget AS HANDLE<br>
    * Notes:       Contains code to locate a widget specific to viewers's when <br>
    *              the qualifier is 'Browse'
    */
   public void locateWidget(final character _pcWidget, final handle phWidget, final handle phTarget)
   {
      character pcWidget = TypeFactory.initInput(_pcWidget);
      character cDataTargets = TypeFactory.character();
      character cField = TypeFactory.character();
      character cObjectType = TypeFactory.character();
      character cQualifier = TypeFactory.character();
      handle hDataSource = TypeFactory.handle();
      handle hDataTarget = TypeFactory.handle();
      integer iDataTargets = TypeFactory.integer();
      logical lBrowsed = TypeFactory.logical();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(phWidget);
         TypeFactory.initOutput(phTarget);
      }, 
      (Body) () -> 
      {
         if (_and(isGreaterThan(numEntries(pcWidget, "."), 1), () -> isEqual(entry(1, pcWidget, "."), "Browse")))
         {
            cField.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", 1, pcWidget, "."));
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            
            if ((hDataSource.isValid()).booleanValue())
            {
               /* {get Prop var [handle]} */
               lBrowsed.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataQueryBrowsed", hDataSource));
               
               /* If the qualifier is Browse and the data source does not have a
                        browser data target, then there is no need to search further. */
               if (_and(begins(cQualifier, "Browse"), not(lBrowsed)))
               {
                  returnNormal();
               }
               
               ghProp.assign(handle.fromString(entry(1, hDataSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataTarget"));
               cDataTargets.assign(ghProp.unwrapBufferField().value());
               
               IntegerExpression resExpr8 = new IntegerExpression()
               {
                  public integer execute()
                  {
                     return numEntries(cDataTargets);
                  }
               };
               ToClause toClause9 = new ToClause(iDataTargets, 1, resExpr8);
               
               doTo("loopLabel20", toClause9, new Block((Body) () -> 
               {
                  silent(() -> hDataTarget.assign(handle.fromString(entry(iDataTargets, cDataTargets))));
                  
                  if (_and(hDataTarget.isValid(), () -> isNotEqual(hDataTarget, targetProcedure())))
                  {
                     ghProp.assign(handle.fromString(entry(1, hDataTarget.unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
                     cObjectType.assign(ghProp.unwrapBufferField().value());
                     
                     if (_isEqual(cObjectType, "SmartDataBrowser"))
                     {
                        silent(() -> 
                        {
                           phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", hDataTarget, "II", cField, "DATA"));
                           phTarget.assign(hDataTarget);
                        });
                     }
                     
                     if (_notUnknown(phWidget))
                     {
                        returnNormal();
                     }
                  }
               }));
            }
         }
         else
         {
            ControlFlowOps.runSuper("locateWidget", pcWidget, phWidget, phTarget);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     This procedure will check all the lookups in the current viewer<br>
    *              and try to get the displayed fields and linked field values from<br>
    *              it's data source before attempting to fetch the data from the <br>
    *              query on the server.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void stripLookupFields()
   {
      integer iLoop = TypeFactory.integer();
      integer iDataLoop = TypeFactory.integer();
      character cField = TypeFactory.character();
      character cValue = TypeFactory.character();
      handle hDataSource = TypeFactory.handle();
      character cTablesInQ = TypeFactory.character();
      character cDataSources = TypeFactory.character();
      character cFieldName = TypeFactory.character();
      character cMappedFields = TypeFactory.character();
      character cWidgetName = TypeFactory.character();
      character cViewerLinkedWidgets = TypeFactory.character();
      handle hContainer = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         /* If we don't have a valid Data Source then leave */
         if (_not(hDataSource.isValid()))
         {
            returnNormal();
         }
         
         /* Get a list of tables in the Data Source */
         silent(() -> cTablesInQ.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getTables", hDataSource)));
         cDataSources.assign("");
         
         /* Check if our Data Source is an SBO */
         if (_or(isUnknown(cTablesInQ), () -> isEqual(cTablesInQ, "?")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainer.assign(ghProp.unwrapBufferField().value());
            cDataSources.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getContainedDataObjects", hDataSource));
         }
         else
         {
            cDataSources.assign(valueOf(hDataSource));
         }
         
         IntegerExpression resExpr9 = new IntegerExpression()
         {
            public integer execute()
            {
               return numEntries(cDataSources);
            }
         };
         ToClause toClause10 = new ToClause(iDataLoop, 1, resExpr9);
         
         doTo("loopLabel21", toClause10, new Block((Body) () -> 
         {
            hDataSource.assign(handle.fromString(entry(iDataLoop, cDataSources)));
            
            if (_not(hDataSource.isValid()))
            {
               next("loopLabel21");
            }
            
            /* Get a list of tables in the Data Source */
            silent(() -> cTablesInQ.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getTables", hDataSource)));
            
            AdaptiveQuery query7 = new AdaptiveQuery();
            
            forEach("loopLabel22", new Block((Init) () -> 
            {
               query7.initialize(ttlookup, "ttlookup.hviewer = ?", null, "ttlookup.hviewer asc", new Object[]
               {
                  targetProcedure()
               }, LockType.EXCLUSIVE);
            }, 
            (Body) () -> 
            {
               /* We need to do this to ensure that we do not think that the
                    fields available in the data source is available for our 
                    lookup since the data source might be the exact same table
                    as the table the lookup is using - e.g. Object type maint with
                    a lookup on object type of extends_object_type_obj would result
                    in the lookup thinking it's got the object type of the extends_objec_type
                    lookup */
               query7.next();
               batch(() -> 
               {
                  ttlookup.setLrefreshquery(new logical(false));
                  ttlookup.setCfounddatavalues(new character(""));
               });
               /* First check if we have the MappedField property specified */
               silent(() -> cMappedFields.assign(ControlFlowOps.invokeDynamicFunctionIn("getMappedFields", ttlookup.getHwidget())));
               
               if (_or(or(ErrorManager.isError(), () -> isUnknown(cMappedFields)), () -> isEqual(cMappedFields, "?")))
               {
                  silent(() -> cMappedFields.assign(""));
               }
               
               if (_isNotEqual(cMappedFields, ""))
               {
                  /* The field list has the following order:
                             1. Key Field,
                             2. Displayed Field,
                             3. Linked Fields */
                  /* The MappedFields propery will also always include the displayed field
                             and is marked as <Displayed Field> */
                  /* Let's first set the value of the KeyField */
                  silent(() -> cValue.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataValue", ttlookup.getHwidget())));
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     cValue.assign("");
                  }
                  
                  ttlookup.setCfounddatavalues(cValue);
                  /* Now we will get the displayed field's value */
                  silent(() -> cField.assign(entry(minus(lookup("<Displayed Field>", cMappedFields), 1), cMappedFields)));
                  
                  if (_or(or(or(ErrorManager.isError(), () -> isEqual(cField, "")), () -> isUnknown(cField)), () -> isEqual(cField, "?")))
                  {
                     cField.assign("");
                     cValue.setUnknown();
                  }
                  
                  if (_isNotEqual(cField, ""))
                  {
                     silent(() -> cValue.assign(trim(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnStringValue", hDataSource, "I", cField))));
                  }
                  
                  /* The field could not be found in the Data Source */
                  if (_isUnknown(cValue))
                  {
                     ttlookup.setLrefreshquery(new logical(true));
                     returnNormal();
                  }
                  
                  ttlookup.setCfounddatavalues(concat(ttlookup.getCfounddatavalues(), chr(1), cValue));
                  
                  /* First check if we need to get more data - possible linked fields */
                  if (_isLessThanOrEqual(numEntries(rightTrim(ttlookup.getCfieldlist(), ",")), 2))
                  {
                     ttlookup.setLrefreshquery(new logical(false));
                     returnNormal();
                  }
                  
                  silent(() -> cViewerLinkedWidgets.assign(ControlFlowOps.invokeDynamicFunctionIn("getViewerLinkedWidgets", ttlookup.getHwidget())));
                  
                  if ((ErrorManager.isError()).booleanValue())
                  {
                     cViewerLinkedWidgets.assign("");
                  }
                  
                  /* If we don't have linked widgets but we have more than just 2 fields
                             in the field list then our data is corrupt - rather get out here 
                             and let the normal process continue as normal */
                  if (_isEqual(cViewerLinkedWidgets, ""))
                  {
                     ttlookup.setLrefreshquery(new logical(true));
                     returnNormal();
                  }
                  
                  IntegerExpression resExpr10 = new IntegerExpression()
                  {
                     public integer execute()
                     {
                        return numEntries(ttlookup.getCfieldlist());
                     }
                  };
                  ToClause toClause11 = new ToClause(iLoop, 3, resExpr10);
                  
                  doTo("loopLabel23", toClause11, new Block((Body) () -> 
                  {
                     silent(() -> cWidgetName.assign(entry(minus(iLoop, 2), cViewerLinkedWidgets)));
                     
                     if ((ErrorManager.isError()).booleanValue())
                     {
                        ttlookup.setLrefreshquery(new logical(true));
                        returnNormal();
                     }
                     
                     /* Get the Data Source field */
                     silent(() -> cField.assign(entry(minus(lookup(cWidgetName, cMappedFields), 1), cMappedFields)));
                     
                     if (_or(or(or(ErrorManager.isError(), () -> isEqual(cField, "")), () -> isEqual(cField, "?")), () -> isUnknown(cField)))
                     {
                        ttlookup.setLrefreshquery(new logical(true));
                        returnNormal();
                     }
                     
                     /* Get the Data Source field value */
                     if (_isNotEqual(cField, ""))
                     {
                        silent(() -> cValue.assign(trim(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnStringValue", hDataSource, "I", cField))));
                     }
                     
                     /* The field could not be found in the Data Source */
                     if (_isUnknown(cValue))
                     {
                        ttlookup.setLrefreshquery(new logical(true));
                        returnNormal();
                     }
                     
                     ttlookup.setCfounddatavalues(concat(ttlookup.getCfounddatavalues(), chr(1), cValue));
                  }));
               }
               else
               {
                  /* If we do not have mapped fields specified - do the asumption */
                  cFieldName.assign(_isGreaterThan(numEntries(ttlookup.getCwidgetname(), "."), 1) ? entry(2, ttlookup.getCwidgetname(), ".") : ttlookup.getCwidgetname());
                  
                  IntegerExpression resExpr11 = new IntegerExpression()
                  {
                     public integer execute()
                     {
                        return numEntries(ttlookup.getCfieldlist());
                     }
                  };
                  ToClause toClause12 = new ToClause(iLoop, 1, resExpr11);
                  
                  doTo("fieldLoop", toClause12, new Block((Body) () -> 
                  {
                     cField.assign(entry(iLoop, ttlookup.getCfieldlist()));
                     
                     if (_isEqual(cField, ""))
                     {
                        next("fieldLoop");
                     }
                     
                     if (_isGreaterThan(numEntries(cField, "."), 1))
                     {
                        if (_and(isEqual(entry(1, cField, "."), entry(1, cTablesInQ)), () -> isEqual(numEntries(cDataSources), 1)))
                        {
                           ttlookup.setLrefreshquery(new logical(true));
                           returnNormal();
                        }
                     }
                     
                     /* Check if the field we are looking for is infact from the table used 
                                  in the lookup */
                     if (_and(isGreaterThan(numEntries(cField, "."), 1), () -> isEqual(iDataLoop, 1)))
                     {
                        if (_and(and(isEqual(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnTable", hDataSource, "I", entry(2, cField, ".")), entry(1, cTablesInQ)), () -> isNotEqual(cFieldName, entry(2, cField, "."))), () -> isEqual(numEntries(cDataSources), iDataLoop)))
                        {
                           ttlookup.setLrefreshquery(new logical(true));
                           returnNormal();
                        }
                     }
                     /* First try to get the value of the field with the table name joined */
                     silent(() -> cValue.assign(trim(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnStringValue", hDataSource, "I", cField))));
                     
                     /* If the failed, then try to get the value for the field name only */
                     if (_or(or(ErrorManager.isError(), () -> isUnknown(cValue)), () -> isEqual(cValue, "?")))
                     {
                        cField.assign(_isGreaterThan(numEntries(cField, "."), 1) ? entry(2, cField, ".") : cField);
                        silent(() -> cValue.assign(trim(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnStringValue", hDataSource, "I", cField))));
                     }
                     
                     if (_or(isUnknown(cValue), () -> isEqual(cValue, "?")))
                     {
                        cValue.assign("");
                        ttlookup.setLrefreshquery(new logical(true));
                     }
                     
                     ttlookup.setCfounddatavalues(concat(ttlookup.getCfounddatavalues(), (_isEqual(iLoop, 1) ? new character("") : chr(1)), cValue));
                  }));
               }
            }));
         }));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Receives state messages for updates<br>
    * Parameters:  pcState AS CHARACTER  update state: <br>
    *              'UpdateBegin',<br>
    *              'UpdateComplete'<br>
    *              'Update'
    */
   public void updateState(final character _pcState)
   {
      character pcState = TypeFactory.initInput(_pcState);
      
      internalProcedure(new Block((Body) () -> 
      {
         switch (StringHelper.safeTrimTrailing((pcState).toStringMessage()).toUpperCase())
         {
            case "UPDATEBEGIN":
               {
                  ControlFlowOps.invokeIn("enableFields", targetProcedure());
                  /* In case enable fails for some reason.*/
                  NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
                  {
                     _isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR") ? new character("UpdateComplete") : new character("Update")
                  });
               }
               break;
            default:
               {
                  ControlFlowOps.runSuper("updateState", pcState);
               }
               break;
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *    Purpose:  Value-changed of frame anywhere event     <br>
    * Parameters:  <br>
    *      Notes: Static objects can reach this also by applying U10 event to run it.
    */
   public void valueChanged()
   {
      internalProcedure(new Block((Body) () -> 
      {
         if (_not(focus().isValid()))
         {
            returnNormal();
         }
         
         ControlFlowOps.invokeInWithMode("fieldModified", targetProcedure(), "I", focus());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Set object in viewMode    <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void viewRecord()
   {
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.invokeIn("disableFields", targetProcedure());
         NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
         {
            "view"
         });
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public logical getKeepChildPositions()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
              Notes:
      ------------------------------------------------------------------------------*/
      logical lKeepChildPositions = TypeFactory.logical();
      
      return function("getKeepChildPositions", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("KeepChildPositions"));
         lKeepChildPositions.assign(ghProp.unwrapBufferField().value());
         returnNormal(lKeepChildPositions);
      }));
   }

   /** getKeepChildPositions */
   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getTargetProcedure()
   {
      return function("getTargetProcedure", handle.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Needed by SBO routines to provide the actual caller object handle.
             Notes:  
         ------------------------------------------------------------------------------*/
         returnNormal(ghTargetProcedure);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setKeepChildPositions(final logical _plKeepChildPositions_1)
   {
      logical plKeepChildPositions_1 = TypeFactory.initInput(_plKeepChildPositions_1);
      
      return function("setKeepChildPositions", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("KeepChildPositions"));
         ghProp.unwrapBufferField().changeValue(plKeepChildPositions_1);
         returnNormal(true);
      }));
   }

   public character fwdGetAdmversion()
   {
      return function("fwdGetADMVersion", character.class, new Block((Body) () -> 
      {
         returnNormal("ADM2.2");
      }));
   }
}
