package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.room</code> (previously
 * <code>hotel.room</code>).
 * <p>
 * Hotel rooms
 */
@LegacyTable(name = "room", table = "room")
@LegacyIndexes(
{
   @LegacyIndex(name = "room-num", converted = "room_room_num", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "room-num")
   })
})
public class RoomImpl
implements Serializable,
           Persistable,
           Room
{
   /** Surrogate primary key */
   private Long id = null;

   /** Room Number */
   @LegacyField(fieldId = 1, name = "room-num", format = ">>>>9", initial = "0", columnLabel = "Room Number", label = "Room Number", order = 10, position = 2)
   private final integer roomNum;

   /** Floor */
   @LegacyField(fieldId = 2, name = "floor", format = ">9", initial = "0", columnLabel = "Floor", label = "Floor", order = 20, position = 3)
   private final integer floor;

   /** Room Type */
   @LegacyField(fieldId = 3, name = "room-type", format = ">9", initial = "0", columnLabel = "Room Type", label = "Room Type", order = 30, position = 4)
   private final integer roomType;

   /** Active */
   @LegacyField(fieldId = 4, name = "active", format = "yes/no", initial = "true", columnLabel = "Active", label = "Active", order = 40, position = 5)
   private final logical active;

   /**
    * Default constructor.
    */
   public RoomImpl()
   {
      roomNum = new integer(0);
      floor = new integer(0);
      roomType = new integer(0);
      active = new logical(true);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new RoomImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      RoomImpl from = (RoomImpl) old;
      setId(from.id);
      setRoomNum(from.roomNum);
      setFloor(from.floor);
      setRoomType(from.roomType);
      setActive(from.active);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Room Number
    * 
    * @return  Room Number
    */
   public integer getRoomNum()
   {
      return (new integer(roomNum));
   }

   /**
    * Setter: Room Number
    * 
    * @param   roomNum
    *          Room Number
    */
   public void setRoomNum(NumberType roomNum)
   {
      this.roomNum.assign(roomNum);
   }

   /**
    * Getter: Floor
    * 
    * @return  Floor
    */
   public integer getFloor()
   {
      return (new integer(floor));
   }

   /**
    * Setter: Floor
    * 
    * @param   floor
    *          Floor
    */
   public void setFloor(NumberType floor)
   {
      this.floor.assign(floor);
   }

   /**
    * Getter: Room Type
    * 
    * @return  Room Type
    */
   public integer getRoomType()
   {
      return (new integer(roomType));
   }

   /**
    * Setter: Room Type
    * 
    * @param   roomType
    *          Room Type
    */
   public void setRoomType(NumberType roomType)
   {
      this.roomType.assign(roomType);
   }

   /**
    * Getter: Active
    * 
    * @return  Active
    */
   public logical isActive()
   {
      return (new logical(active));
   }

   /**
    * Setter: Active
    * 
    * @param   active
    *          Active
    */
   public void setActive(logical active)
   {
      this.active.assign(active);
   }
}
