package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.room_type</code> (previously
 * <code>hotel.room-type</code>).
 * <p>
 * Room types, e.g. standard or luxury rooms
 */
@LegacyTable(name = "room-type", table = "room_type")
@LegacyIndexes(
{
   @LegacyIndex(name = "type", converted = "room_type_type", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "room-type")
   })
})
public class RoomTypeImpl
implements Serializable,
           Persistable,
           RoomType
{
   /** Surrogate primary key */
   private Long id = null;

   /** Room Type */
   @LegacyField(fieldId = 1, name = "room-type", format = ">9", initial = "0", columnLabel = "Room Type", label = "Room Type", order = 10, position = 2)
   private final integer roomType;

   /** Parent Type */
   @LegacyField(fieldId = 2, name = "parent-type", format = ">9", initial = "0", columnLabel = "Parent Type", label = "Parent Type", order = 20, position = 3)
   private final integer parentType;

   /** Description */
   @LegacyField(fieldId = 3, name = "description", format = "x(20)", columnLabel = "Description", label = "Description", order = 30, position = 4)
   private final character description;

   /** Image */
   @LegacyField(fieldId = 4, name = "image", format = "x(256)", columnLabel = "Image", label = "Image", order = 40, position = 5)
   private final character image;

   /** Maximum Persons */
   @LegacyField(fieldId = 5, name = "max-persons", format = "->,>>>,>>9", initial = "0", columnLabel = "Maximum Persons", label = "Maximum Persons", order = 50, position = 6)
   private final integer maxPersons;

   /**
    * Default constructor.
    */
   public RoomTypeImpl()
   {
      roomType = new integer(0);
      parentType = new integer(0);
      description = new character("");
      image = new character("");
      maxPersons = new integer(0);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new RoomTypeImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      RoomTypeImpl from = (RoomTypeImpl) old;
      setId(from.id);
      setRoomType(from.roomType);
      setParentType(from.parentType);
      setDescription(from.description);
      setImage(from.image);
      setMaxPersons(from.maxPersons);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Room Type
    * 
    * @return  Room Type
    */
   public integer getRoomType()
   {
      return (new integer(roomType));
   }

   /**
    * Setter: Room Type
    * 
    * @param   roomType
    *          Room Type
    */
   public void setRoomType(NumberType roomType)
   {
      this.roomType.assign(roomType);
   }

   /**
    * Getter: Parent Type
    * 
    * @return  Parent Type
    */
   public integer getParentType()
   {
      return (new integer(parentType));
   }

   /**
    * Setter: Parent Type
    * 
    * @param   parentType
    *          Parent Type
    */
   public void setParentType(NumberType parentType)
   {
      this.parentType.assign(parentType);
   }

   /**
    * Getter: Description
    * 
    * @return  Description
    */
   public character getDescription()
   {
      return (new character(description));
   }

   /**
    * Setter: Description
    * 
    * @param   description
    *          Description
    */
   public void setDescription(Text description)
   {
      this.description.assign(description);
   }

   /**
    * Getter: Image
    * 
    * @return  Image
    */
   public character getImage()
   {
      return (new character(image));
   }

   /**
    * Setter: Image
    * 
    * @param   image
    *          Image
    */
   public void setImage(Text image)
   {
      this.image.assign(image);
   }

   /**
    * Getter: Maximum Persons
    * 
    * @return  Maximum Persons
    */
   public integer getMaxPersons()
   {
      return (new integer(maxPersons));
   }

   /**
    * Setter: Maximum Persons
    * 
    * @param   maxPersons
    *          Maximum Persons
    */
   public void setMaxPersons(NumberType maxPersons)
   {
      this.maxPersons.assign(maxPersons);
   }
}
