/*--------------------------------------------------------------------------
    File        : Browser.java
    Purpose     : Super procedure for PDO SmartBrowser objects

    Syntax      : adm2/browser.p

    Modified    : August 8, 2000 -- Version 9.1B
    Modified    : 09/29/2001        Mark Davies (MIP)
                  Added security to Excel and Crysal Exports.
    Modified    : 10/19/2001        Mark Davies (MIP)
                  Security for hiding secured fields works for a browse on an
                  object controller with a standard layout, but does not
                  work when you design it with a browser, viewer and toolbars
                  on one container. To rectify this I took out the container
                  name being passed to the field security check API.
    Modified    : 11/13/2001        Mark Davies (MIP)
                  Check for valid-handle for all gsh... variables
    Modified    : 02/27/2002        Mark Davies (MIP)
                  Fix for issue #4086 - Changes made to cater for SBO caused
                  an error.
  ------------------------------------------------------------------------*/

package com.goldencode.hotel.common.adm2;

import com.goldencode.p2j.util.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.KeyReader.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;
import static com.goldencode.p2j.util.SelfManager.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.util.ErrorManager.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in common/adm2/browser.p).
 */
public class Browser
{
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* This is needed by some procedures so that another procedure we call
        can identify what the original TARGET-PROCEDURE is. The value is 
        returned by getTargetProcedure. */   
   handle ghTargetProcedure = TypeFactory.handle();

   /* _UIB-CODE-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Procedure
      Compile into: adm2
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: CODE-ONLY COMPILE
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Procedure ASSIGN
            HEIGHT             = 19.48
            WIDTH              = 59.8.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ************************* Included-Libraries *********************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : brsprop.i
       Purpose     : Basic Property definition include file for 
                     SmartDataBrowse Objects
   
       Syntax      : {src/adm2/brsprop.i}
   
       Description :
   
       Modified    : August 18, 1999 Version 9.1A
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : browserdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/browserdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/brsprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* SearchField is added as a property for non-DB browsers.
      ForeignFields is added for DB Browsers. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : dvisprop.i
       Purpose     : Property Include file for Data Visualization Objects 
   
       Syntax      : src\adm2\dvisprop.i
   
       Description :
   
       Modified    : May 26, 2000 -- Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : datavisdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/datavisdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/dvisprop.i}
     ------------------------------------------------------------------------*/   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the property temp-table. */   
   /* Include the next property file up the chain; we will add our field
        definitions to its list. As of 9.1A, if this visual object is also
        a SmartContainer, we include container properties as well. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visprop.i
       Purpose     : Starts Super procedure visual.p and defines basic properties
                     for visual SmartObjects in V9
       Syntax      : {src/adm2/visprop.i}
   
       Description :
   
       Modified    : May 19, 1999 Version 9.1A
       Modified    : 10/23/2001          Mark Davies (MIP)
                     Added two new fields for ResizeHorizontal and ResizeVertical
       Modified    : 11/02/2001          Mark Davies (MIP)
                     Replaced properties for FrameMinHeightChars and FrameMinWidthChars with MinHeight and MinWidth
       Modified    : 11/14/2001          Mark Davies (MIP)
                     Renamed property 'DisplayFieldsSecurity' to 'FieldSecurity'
       Modified    : 02/27/2003          Neil Bell (MIP)
                     Added property 'SecuredTokens'                  
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : visualdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/visualdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/visprop.i}
     ------------------------------------------------------------------------*/   
   /* This is the procedure to execute to set InstanceProperties at design time. */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 8
            WIDTH              = 60.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Include the file which defines prototypes for all of the super
        procedure's entry points. Also, start or attach to the super procedure.
        Skip start-super-proc if we *are* the super procedure. 
        And skip including the prototypes if we are *any* super procedure. */   
   /* These preprocessors tell at compile time which properties can be
        retrieved directly from the temp-table. */   
   /* This property contains the security for all objects found in the AllFieldHandles Property */    
   /* Include the next property file up the chain (in this case, the top
        of the chain); it starts the property temp-table definition and we
        add our field definitions to it. */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smrtprop.i
       Purpose     : Starts smart.p super procedure and defines general
                     SmartObject properties and other values.
       Syntax      : {src/adm2/smrtprop.i}
   
       Description :
   
       Modified    : February 4, 2000 - Version 9.1B
     ------------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress UIB.             */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*--------------------------------------------------------------------------
       File        : smartdefscustom.i
       Purpose     : Entry point to define:
                       New definitions
                       {&xcInstanceProperties}
                       {&ADM-PROPERTY-DLG}
       Syntax      : {src/adm2/custom/smartdefscustom.i}
   
       Modified    : 06/03/1999
       
       Notes       : Referenced in {src/adm2/smrtprop.i}
     ------------------------------------------------------------------------*/   
   /* Service Managers */   
   /*********************************************************************
   * Copyright (C) 2000-2002 by Progress Software Corporation ("PSC"),  *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*---------------------------------------------------------------------------------
     File: globals.i
   
     Description:  ICF Global Shared Variables
   
     Purpose:      ICF Global Shared Variables
                   The global shared variables available within the Dynamics environment.
                   
                   NOTE: (Dynamic Only )
                   Shared variables do not span the client / server boundary. On the
                   client these are set-up in afstartupp.p and on the server in the agent
                   as_startup.p
                   Also note if we include the guts of a procedure as an internal
                   procedure for shared client/server code, we sometime need to
                   ensure the shared variables are not redefined.
   
     Parameters:   <none>
   
     History:
     --------
   
     Modified: 11/08/2001 - Mark Davies (MIP)
               Copied code for ICF afglobals.i and added here to allow
               ICF changed adm code to compile. afglobals.i now includes
               this file for definitions.
               Any ADM code will now reference globals.i which excludes any other
               ICF specific includes.
   -----------------------------------------------------------*/   
   /*                   This .W file was created with the Progress UIB.             */   
   /*-------------------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* ********************  Preprocessor Definitions  ******************** */   
   /* _UIB-PREPROCESSOR-BLOCK-END */   
   /* *********************** Procedure Settings ************************ */   
   /* Settings for THIS-PROCEDURE
      Type: Include
      Allow: 
      Frames: 0
      Add Fields to: Neither
      Other Settings: INCLUDE-ONLY
    */   
   /* *************************  Create Window  ************************** */   
   /* DESIGN Window definition (used by the UIB) 
     CREATE WINDOW Include ASSIGN
            HEIGHT             = 5.57
            WIDTH              = 64.4.
   _* END WINDOW DEFINITION *_
                                                                           */   
   /* ***************************  Main Block  *************************** */   
   /* Used whereever Crystal is used and when defining the tt_datasource temp-table 
      to define the maximum columns supported when printing to crystal.
      See browsercustom.p and afcrpplip2p.p for example uses
   */   
   /* Used for filter checks */   
   /* The table delimiter used in assignlist, *ColumnsByTable props */   
   handle gshAstraAppserver /* quick access to all handles on object, e.g. viewer, whether enabled or not */  /* quick access to all fields on object, e.g. viewer, whether enabled or not */ = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshastraappserver", true, new handle());

   /* Handle to Astra Application Server Partition */   
   handle gshSessionManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsessionmanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshRimanager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrimanager", true, new handle());

   /* Handle Astra Session Manager */   
   handle gshSecurityManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshsecuritymanager", true, new handle());

   /* Handle Astra Security Manager */   
   handle gshProfileManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshprofilemanager", true, new handle());

   /* Handle Astra Profile Manager */   
   handle gshRepositoryManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshrepositorymanager", true, new handle());

   /* Handle Astra Repository Manager */   
   handle gshTranslationManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshtranslationmanager", true, new handle());

   /* Handle Astra Translation Manager */   
   handle gshWebManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshwebmanager", true, new handle());

   /* Handle Astra Web Manager */   
   character gscSessionId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gscsessionid", true, new character(""));

   /* Unique session id */   
   decimal gsdSessionObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionobj", true, new decimal(0));

   /* Unique session id */   
   handle gshFinManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshfinmanager", true, new handle());

   /* Handle AstraFin Generic Manager */   
   handle gshGenManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshgenmanager", true, new handle());

   /* Handle AstraFramework Generic Manager */   
   handle gshAgnManager = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gshagnmanager", true, new handle());

   /* Handle AstraGen Generic Manager */   
   decimal gsdTempUniqueId = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdtempuniqueid", true, new decimal(0));

   /* A number for unique objects */   
   decimal gsdUserObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsduserobj", true, new decimal(0));

   /* User Object ID */   
   decimal gsdRenderTypeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdrendertypeobj", true, new decimal(0));

   /* Render Type Object ID */   
   decimal gsdSessionScopeObj = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "gsdsessionscopeobj", true, new decimal(0));

   /* Render Type Object ID */   
   /*********************************************************************
   * Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
   * 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
   * below.  All Rights Reserved.                                       *
   *                                                                    *
   * The Initial Developer of the Original Code is PSC.  The Original   *
   * Code is Progress IDE code released to open source December 1, 2000.*
   *                                                                    *
   * The contents of this file are subject to the Possenet Public       *
   * License Version 1.0 (the "License"); you may not use this file     *
   * except in compliance with the License.  A copy of the License is   *
   * available as of the date of this notice at                         *
   * http://www.possenet.org/license.html                               *
   *                                                                    *
   * Software distributed under the License is distributed on an "AS IS"*
   * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
   * should refer to the License for the specific language governing    *
   * rights and limitations under the License.                          *
   *                                                                    *
   * Contributors:                                                      *
   *                                                                    *
   *********************************************************************/   
   /*------------------------------------------------------------------------
       File        : globalscustom.i
       Purpose     : The developer can create his own version of this to 
                     allow 'global' definitions to be added to the 
                     framework.   
       Syntax      : {src/adm2/custom/globalscustom.i}
   
       Description :
   
       Created     : 06/03/1999
       Notes       : Referenced in {src/adm2/globals.i}
     ----------------------------------------------------------------------*/   
   /*          This .W file was created with the Progress AppBuilder.      */   
   /*----------------------------------------------------------------------*/   
   /* ***************************  Definitions  ************************** */   
   /* _UIB-CODE-BLOCK-END */   
   /* define the ADM Version and broker handle for all SmartObjects */   
   /* Note that adm-props-from-repository is documented as possible to define
        as a variable for flexible override, so it cannot be used for conditional 
        compile.  */   
   /* adm-props-from-repository is supported to be defined as a variable,
       so it is not used for conditional compile */   
   handle ghProp = TypeFactory.handle();

   /**
    * External procedure (converted to Java from the 4GL source code
    * in common/adm2/browser.p).
    */
   public void execute()
   {
      externalProcedure(Browser.this, new Block((Init) () -> 
      {
         gsdSessionObj.setPrecision(9);
         gsdTempUniqueId.setPrecision(9);
         gsdUserObj.setPrecision(9);
         gsdRenderTypeObj.setPrecision(9);
         gsdSessionScopeObj.setPrecision(9);
      }, 
      (Body) () -> 
      {
         /* _UIB-CODE-BLOCK-END */
         /* *********************** Procedure Settings ************************ */
         /* Settings for THIS-PROCEDURE
            Type: Method-Library
            Allow: 
            Frames: 0
            Add Fields to: Neither
            Other Settings: INCLUDE-ONLY
          */
         /* *************************  Create Window  ************************** */
         /* DESIGN Window definition (used by the UIB) 
           CREATE WINDOW Method-Library ASSIGN
                  HEIGHT             = 14.86
                  WIDTH              = 57.2.
         _* END WINDOW DEFINITION *_
                                                                                 */
         /* ***************************  Main Block  *************************** */
         /* Include the file which defines prototypes for all of the super
              procedure's entry points. Also, start or attach to the super procedure.
              Skip start-super-proc if we *are* the super procedure. 
              And skip including the prototypes if we are *any* super procedure. */
         /* These preprocessors let the get and set methods know at compile time
             which property values are located in the temp-table and which must
             be accessed through the property functions.
          */
         /* for support of dynamic objects */
         /* physical name - no path */
         /* for support of runtime parameters passed from the menus */
         /* Properties for super procedure management */
         /* The name of the super procedure; can be a CSV list */ 
 /* STATELESS, STATEFUL; the mode that all supers for the object will be launched in */ 
 /* Runtime property storing handles to the object's super procedures, defined in SuperProcedure. */
         /* Layout support for dynamic objects */
         /* contains a layout position that allows an object to be laid out correctly. */
         /* determine if we need to create the prop fields here or from the */
         /* repository (if ICF is running ) */
         /* COMMENT THE FOLLOWING LINE TO USE THE "<class>prop.i" DEFINITIONS */
         /* This temp-table defines all the propertt fields for an object.
             This include file contributes the DEFINE statement header and
             all basic smart object properties. Each other property class include file
             adds its own fields and then the parent object ends the statement.
             Define the fields for smart objects only, not for their super procedures.
          */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         /* else if appserver aware (adecomm/appserv.i) include appserver props */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
         
         /* _UIB-CODE-BLOCK-END */
         if (_not(handle.fromString(entry(1, thisProcedure().unwrapADMData().getADMData(), chr(1))).isValid()))
         {
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public character getObjectType()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the Type of the SmartObject, such as "SmartDataObject", etc.
                  For an ADM Super procedure, this function returns "SUPER".
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cType = TypeFactory.character();
      
      return function("getObjectType", character.class, new Block((Body) () -> 
      {
         cType.assign("SUPER");
         returnNormal(cType);
      }));
   }

   /** *********************  Internal Procedures ********************** */
   /**
    * Purpose:     SmartDataBrowser version of addRecord which inserts a new blank <br>
    *              row after the currently selected one if any.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Initial values are not displayed here but from ROW-ENTRY trigger <br>
    *              (see brsentry.i).
    */
   public void addRecord()
   {
      handle hBrowse = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      handle hSource = TypeFactory.handle();
      handle hSearchField = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("addRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TableIOSource"));
         hSource.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
         hSearchField.assign(ghProp.unwrapBufferField().value());
         
         /* If the Browse was not enabled (perhaps because it was empty),
                  enable it. */
         if ((hBrowse.unwrapWriteProtectable().isReadOnly()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
            hFrame.assign(ghProp.unwrapBufferField().value());
            apply(hFrame, "ENTRY");
            hBrowse.unwrapWriteProtectable().setReadOnly(new logical(false));
         }
         
         if ((hSearchField.isValid()).booleanValue())
         {
            hSearchField.unwrapSensitive().setSensitive(new logical(false));
         }
         
         hQuery.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
         
         if (_not(hQuery.unwrapQuery().isOpen()))
         {
            hQuery.unwrapQuery().queryOpen();
         }
         
         hBrowse.unwrapBrowse().insertRow("AFTER");
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseInitted"));
         ghProp.unwrapBufferField().changeValue(new logical(false));
         /* Signal for ROW-ENTRY trigger code */
         NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
         {
            "update"
         });
         returnNormal();
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public void adjustReposition()
   {
      handle hDataSource = TypeFactory.handle();
      handle hDataQuery = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      handle hRowObject = TypeFactory.handle();
      rowid rRowid = TypeFactory.rowid();
      integer iLastRowNum = TypeFactory.integer();
      integer iNumEntries = TypeFactory.integer();
      logical lFetchOnRepos = TypeFactory.logical();
      logical lRebuild = TypeFactory.logical();
      character cObjectType = TypeFactory.character();
      character cDataSource = TypeFactory.character();
      integer iLoop = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         /* Handle of the browse widget. */
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         /* Proc. handle of our SDO/SBO. */
         if ((hDataSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hDataSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectType"));
            cObjectType.assign(ghProp.unwrapBufferField().value());
         }
         
         if (_isEqual(cObjectType, "SmartBusinessObject"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
            cDataSource.assign(ghProp.unwrapBufferField().value());
            hDataSource /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "dataObjectHandle", hDataSource, "I", cDataSource));
         }
 /* we really want the SDO */;
         
         if (_and(hBrowse.isValid(), () -> hDataSource.isValid()))
         {
            ghTargetProcedure.assign(targetProcedure());
            /* {get Prop var [handle]} */
            hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", hDataSource));
            ghTargetProcedure.setUnknown();
            iNumEntries.assign(hBrowse.unwrapNumEntries().getNumEntries());
            
            if (_and(and(and(isGreaterThan(iNumEntries, 0), () -> isLessThan(iNumEntries, hBrowse.unwrapDown().getDown())), () -> hDataQuery.isValid()), () -> isEqual( /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()), "NO")))
            {
               /* {get Prop var [handle]} */
               iLastRowNum.assign(ControlFlowOps.invokeDynamicFunctionIn(integer.class, "getLastRowNum", hDataSource));
               /* {get Prop var [handle]} */
               hRowObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", hDataSource));
               rRowid.assign(hRowObject.unwrapBuffer().rowID());
               
               if (_notUnknown(rRowid))
               {
                  if (_notUnknown(iLastRowNum))
                  {
                     loopLabel0:
                     for (iLoop.assign(1); _isLessThanOrEqual(iLoop, iNumEntries); iLoop.increment())
                     {
                        if ((hBrowse.unwrapBrowse().isRowSelected(iLoop)).booleanValue())
                        {
                           break loopLabel0;
                        }
                     }
                     
                     hBrowse.unwrapBrowse().setRepositionedRow(minus(hBrowse.unwrapDown().getDown(), (int64.maximum(new int64(0), minus(iNumEntries, iLoop)))), "ALWAYS");
                     hDataQuery.unwrapQuery().queryRepositionByID(rRowid);
                  }
                  else
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchOnReposToEnd"));
                     lFetchOnRepos.assign(ghProp.unwrapBufferField().value());
                     
                     if ((lFetchOnRepos).booleanValue())
                     {
                        NamedEventManager.publish("fetchBatch", targetProcedure(), "I", new Object[]
                        {
                           true
                        });
                        hBrowse.unwrapBrowse().setRepositionedRow(iNumEntries, "always");
                        hDataQuery.unwrapQuery().queryRepositionByID(rRowid);
                        ControlFlowOps.invokeInWithMode("DataAvailable", hDataSource, "I", new character("value-changed"));
                     }
                  }
               }
            }
            /* {get Prop var [handle]} */
            lRebuild.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getRebuildOnRepos", hDataSource));
            /* This is not very well though out and the down logic doesn't really work 
                  for long, but we need to get rid of the 'always' setting used above, and 
                  it is here in an attempt to keep the same setting after 
                  fetchRowident -> fetchDataSet ('Batchend') and also at initialization.  
                  The down / 2 is here just to try to keep old behavior, but we set it to 
                  1 for SDOs with rebuild as the half way down setting makes no sense and 
                  causes the above logic to have problems getting the last recod at bottom
                   
                  The point is: don't hesitate to get rid of this, for example when improving
                  browse repositioning...  */
            hBrowse.unwrapBrowse().setRepositionedRow((lRebuild).booleanValue() ? new integer(1) : new integer(divide(hBrowse.unwrapDown().getDown(), 2)), "CONDITIONAL");
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Applies "ENTRY" to the first enabled column <br>
    *              in the browse (unless pcCellName is specified).<br>
    * Parameters:  pcCellName AS CHARACTER  optional fieldname; if specified,<br>
    *              the browse column of that name will be positioned to.
    */
   public void applyCellEntry(final character _pcCellName)
   {
      character pcCellName = TypeFactory.initInput(_pcCellName);
      handle hBrowse = TypeFactory.handle();
      handle hCell = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hCell.assign(hBrowse.unwrapBrowse().getFirstColumn());
         
         loopLabel1:
         while ((hCell.isValid()).booleanValue())
         {
            if (_and(not(hCell.unwrapWriteProtectable().isReadOnly()), () -> or(isUnknown(pcCellName), () -> isEqual(hCell.unwrap().name(), pcCellName))))
            {
               apply(hCell, "ENTRY");
               returnNormal();
            }
            
            hCell.assign(hCell.unwrapBrowseColumn().getNextColumn());
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Applies "ENTRY" to the first enabled column in the browse <br>
    *              (unless pcField is specified) if columns are enabled in the <br>
    *              browse, or in the first child of the SmartDataBrowser's frame.<br>
    * Parameters:  pcField AS CHARACTER  optional fieldname; if specified,<br>
    *              the column field of that name will be positioned to.
    */
   public void applyEntry(final character _pcField_1)
   {
      character pcField_1 = TypeFactory.initInput(_pcField_1);
      character cEnabledFields = TypeFactory.character();
      logical lFieldsEnabled = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
         lFieldsEnabled.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
         cEnabledFields.assign(ghProp.unwrapBufferField().value());
         
         if (_and(isNotEqual(lookup(pcField_1, cEnabledFields), 0), lFieldsEnabled))
         {
            ControlFlowOps.invokeInWithMode("applyCellEntry", targetProcedure(), "I", pcField_1);
         }
         else
         {
            ControlFlowOps.runSuper("applyEntry", pcField_1);
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     This procedure adds the value input to the browse's <br>
    *              MAX-DATA-GUESS attribute<br>
    * Parameters:  INPUT piMaxGuess AS INTEGER<br>
    * Notes:
    */
   public void assignMaxGuess(final integer _piMaxGuess)
   {
      integer piMaxGuess = TypeFactory.initInput(_piMaxGuess);
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hBrowse.unwrapBrowse().setMaxDataGuess(new integer(plus(hBrowse.unwrapBrowse().getMaxDataGuess(), piMaxGuess)));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public void calcWidth()
   {
      decimal dMaxWidth = TypeFactory.decimal();
      decimal dWidth = TypeFactory.decimal();
      handle hColumn = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      decimal dHeight = TypeFactory.decimal();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         /* We need to make the browse visible in order to get the width for its columns */
         hBrowse.unwrapWidget().setHidden(new logical(false));
         hColumn.assign(hBrowse.unwrapBrowse().getFirstColumn());
         
         if ((hBrowse.unwrapBrowse().hasRowMarkers()).booleanValue())
         {
            dWidth.assign(2);
         }
         
         loopLabel2:
         while ((hColumn.isValid()).booleanValue())
         {
            dWidth.assign(plus(plus(dWidth, hColumn.unwrapSizeable().getWidthChars()), 1));
            hColumn.assign(hColumn.unwrapBrowseColumn().getNextColumn());
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MaxWidth"));
         dMaxWidth.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         dHeight.assign(ControlFlowOps.invokeDynamicFunctionIn(decimal.class, "getHeight", targetProcedure()));
         dWidth.assign(decimal.minimum(dWidth, dMaxWidth));
         ControlFlowOps.invokeInWithMode("resizeObject", targetProcedure(), "II", dHeight, plus(dWidth, 3.5));
         silent(() -> hBrowse.unwrapSizeable().setWidthChars(dWidth));
         ControlFlowOps.invokeInWithMode("resizeObject", targetProcedure(), "II", dHeight, plus(dWidth, 3.5));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Deletes a currently selected NEW row from the Browse's viewport.  <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Published from dataSource on cancel of a new row. <br>
    *              The 9.1B check to see if the updateSource was a browse did <br>
    *              not work for SBOs, where the SDO has no updateSource and the <br>
    *              SBO has several. It's too much of a hassle to have some <br>
    *              kind of property to keep track of this.  <br>
    *              Publish seems to be a more future proof and dynamic way to do <br>
    *              this as it should work also when we decide to support multiple <br>
    *              or 'switchable' updateSoruces for SDOs. <br>
    *              As the method ensures that this really is new should make it <br>
    *              robust enough as the query only can have one browse.
    */
   public void cancelNew()
   {
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.unwrapBrowse().isNewRow()).booleanValue())
         {
            ControlFlowOps.invokeIn("deleteComplete", targetProcedure());
         }
         
         returnNormal();
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public void cancelRecord()
   {
      character cNew = TypeFactory.character();
      logical lSelfQuery = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         cNew.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
         ControlFlowOps.runSuper("cancelRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lSelfQuery.assign(ghProp.unwrapBufferField().value());
         
         if ((lSelfQuery).booleanValue())
         {
            /* if new get rid of the inserted row 
                   Not needed if we have a datasource as the SDO publishes cancelNew */
            if (_isNotEqual(cNew, "No"))
            {
 /* Could be Add or Copy or No */
               ControlFlowOps.invokeIn("cancelNew", targetProcedure());
            }
            else
            {
               ControlFlowOps.invokeInWithMode("displayFields", targetProcedure(), "I", new unknown());
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of copyRecord which inserts a new <br>
    *              blank row after the currently selected one if any.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Initial values are not displayed here but from ROW-ENTRY trigger <br>
    *              (see brsentry.i).
    */
   public void copyRecord()
   {
      handle hBrowse = TypeFactory.handle();
      handle hDataQuery = TypeFactory.handle();
      handle hRowObject = TypeFactory.handle();
      handle hSearchField = TypeFactory.handle();
      logical lQueryObject = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObject.assign(ghProp.unwrapBufferField().value());
         
         if (_not(lQueryObject))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
            hBrowse.assign(ghProp.unwrapBufferField().value());
            hDataQuery.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
            hRowObject.assign(hDataQuery.unwrapQuery().bufferHandle(1));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", valueOf(hRowObject.unwrapBuffer().rowID()));
         }
         
         ControlFlowOps.runSuper("copyRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
         hSearchField.assign(ghProp.unwrapBufferField().value());
         
         if ((hSearchField.isValid()).booleanValue())
         {
            hSearchField.unwrapSensitive().setSensitive(new logical(false));
         }
         
         hBrowse.unwrapBrowse().insertRow("AFTER");
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseInitted"));
         ghProp.unwrapBufferField().changeValue(new logical(false));
         /* Signal for ROW-ENTRY trigger code */
         NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
         {
            "update"
         });
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Create popup menu item<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       In order to translate popup menu items, create an override of this<br>
    *              procedure and use the Globalization Manager API to translate <br>
    *              these items as Text.  This should be rewritten to define the <br>
    *              popup with the Toolbar and Menu Designer rather than <br>
    *              hardcoding it here, that would allow for translations as well.
    */
   public void createPopupItem(final handle _phPopupMenu, final logical _lSubMenu, final character _pcLabel, final character _pcName, final logical _plChecked, final logical _plToggle, final handle phMenuItem)
   {
      handle phPopupMenu = TypeFactory.initInput(_phPopupMenu);
      logical lSubMenu = TypeFactory.initInput(_lSubMenu);
      character pcLabel = TypeFactory.initInput(_pcLabel);
      character pcName = TypeFactory.initInput(_pcName);
      logical plChecked = TypeFactory.initInput(_plChecked);
      logical plToggle = TypeFactory.initInput(_plToggle);
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(phMenuItem);
      }, 
      (Body) () -> 
      {
         if ((lSubMenu).booleanValue())
         {
            DynamicWidgetFactory.createSubMenu(phMenuItem);
            phMenuItem.unwrapWidget().setParentHandle(phPopupMenu);
            phMenuItem.unwrapCommonField().setLabel(pcLabel);
            phMenuItem.unwrap().name(pcName);
         }
         else
         {
            if ((plToggle).booleanValue())
            {
               DynamicWidgetFactory.createMenuItem(phMenuItem);
               phMenuItem.unwrapWidget().setParentHandle(phPopupMenu);
               phMenuItem.unwrapCommonField().setLabel(pcLabel);
               phMenuItem.unwrap().name(pcName);
               phMenuItem.unwrapMenuItem().setToggleBox(plToggle);
               phMenuItem.unwrapWidget().setChecked(plChecked);
            }
            else
            {
               DynamicWidgetFactory.createMenuItem(phMenuItem);
               phMenuItem.unwrapWidget().setParentHandle(phPopupMenu);
               phMenuItem.unwrapCommonField().setLabel(pcLabel);
               phMenuItem.unwrap().name(pcName);
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Creates browser popup menu<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Called by initializeObject
    */
   public void createPopupMenu()
   {
      handle hBrowse = TypeFactory.handle();
      handle hColsMovable = TypeFactory.handle();
      handle hColsSortable = TypeFactory.handle();
      handle hDefaultCol = TypeFactory.handle();
      handle hDefaultSort = TypeFactory.handle();
      handle hExitItem = TypeFactory.handle();
      handle hPopupMenu = TypeFactory.handle();
      handle hRemoveCol = TypeFactory.handle();
      handle hRemoveSort = TypeFactory.handle();
      handle hResetCol = TypeFactory.handle();
      handle hResetSort = TypeFactory.handle();
      handle hResetSubMenu = TypeFactory.handle();
      handle hRuleItem = TypeFactory.handle();
      handle hSaveColSettings = TypeFactory.handle();
      handle hSaveSorting = TypeFactory.handle();
      logical lMovable = TypeFactory.logical();
      logical lSortable = TypeFactory.logical();
      logical lUseRepos = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         lMovable.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getColumnsMovable", targetProcedure()));
         /* {get Prop var [handle]} */ 
         lSortable.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getColumnsSortable", targetProcedure()));
         /* {get Prop var [handle]} */ 
         lUseRepos.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseRepository", targetProcedure()));
         DynamicWidgetFactory.createMenu(hPopupMenu);
         hPopupMenu.unwrapMenu().setPopupOnly(new logical(true));
         hPopupMenu.unwrapWidget().setTitle(new character("Browser Menu"));
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("&Move Columns"), new character("miColsMovable"), lMovable, new logical(true), hColsMovable);
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("&Sort Columns"), new character("miColsSortable"), lSortable, new logical(true), hColsSortable);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MovableHandle"));
         ghProp.unwrapBufferField().changeValue(hColsMovable);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SortableHandle"));
         ghProp.unwrapBufferField().changeValue(hColsSortable);
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(true), new character("&Reset"), new character("smReset"), new logical(false), new logical(false), hResetSubMenu);
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hResetSubMenu, new logical(false), new character("&Column Positions and Sizes"), new character("miResetCol"), new logical(false), new logical(false), hResetCol);
         
         if ((lUseRepos).booleanValue())
         {
            ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hResetSubMenu, new logical(false), new character("&Sort Column"), new character("miResetSort"), new logical(false), new logical(false), hResetSort);
         }
         
         DynamicWidgetFactory.createMenuItem(hRuleItem);
         hRuleItem.unwrapWidget().setParentHandle(hResetSubMenu);
         hRuleItem.unwrapSubType().setSubType(new character("RULE"));
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hResetSubMenu, new logical(false), new character("&Default Column Positions and Sizes"), new character("miDefaultCol"), new logical(false), new logical(false), hDefaultCol);
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hResetSubMenu, new logical(false), new character("De&fault Sort Column"), new character("miDefaultSort"), new logical(false), new logical(false), hDefaultSort);
         
         /* Movable and Sortable are mutually exclusive and Sortable takes precedence */
         if (_and(lMovable, lSortable))
         {
            hColsMovable.unwrapWidget().setChecked(new logical(false));
         }
         
         if ((lUseRepos).booleanValue())
         {
            DynamicWidgetFactory.createMenuItem(hRuleItem);
            hRuleItem.unwrapWidget().setParentHandle(hPopupMenu);
            hRuleItem.unwrapSubType().setSubType(new character("RULE"));
            ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("Save &Column Positions and Sizes"), new character("miSaveColumnSettings"), new logical(false), new logical(false), hSaveColSettings);
            ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("Save &Sort Column"), new character("miSaveSorting"), new logical(false), new logical(false), hSaveSorting);
            ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("&Remove Column Positions and Sizes"), new character("miRemoveCol"), new logical(false), new logical(false), hRemoveCol);
            ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("Re&move Sort Column"), new character("miRemoveSort"), new logical(false), new logical(false), hRemoveSort);
         }
         
         /* if lUseRepos */
         DynamicWidgetFactory.createMenuItem(hRuleItem);
         hRuleItem.unwrapWidget().setParentHandle(hPopupMenu);
         hRuleItem.unwrapSubType().setSubType(new character("RULE"));
         ControlFlowOps.invokeInWithMode("createPopupItem", targetProcedure(), "IIIIIIO", hPopupMenu, new logical(false), new character("E&xit"), new character("miExit"), new logical(false), new logical(false), hExitItem);
         hBrowse.unwrapWidget().setPopupMenu(hPopupMenu);
         registerTriggerPersistentIn(new EventList("VALUE-CHANGED", hColsMovable), "movableValueChanged", targetProcedure());
         registerTriggerPersistentIn(new EventList("VALUE-CHANGED", hColsSortable), "sortableValueChanged", targetProcedure());
         registerTriggerPersistentIn(new EventList("CHOOSE", hResetCol), "resetColumnSettings", targetProcedure());
         registerTriggerPersistentIn(new EventList("CHOOSE", hDefaultCol), "resetDefaultSettings", targetProcedure());
         /* TRUE parameter indicates that sort should be reset to default sort */
         registerTriggerPersistentIn(new EventList("CHOOSE", hDefaultSort), "resetSortColumn", targetProcedure(), new logical(true));
         
         if ((lUseRepos).booleanValue())
         {
            /* FALSE parameter indicates that sort should not be resert to default sort */
            registerTriggerPersistentIn(new EventList("CHOOSE", hResetSort), "resetSortColumn", targetProcedure(), new logical(false));
            /* FALSE parameter indicates that the profile should be saved rather 
                   than deleted */
            registerTriggerPersistentIn(new EventList("CHOOSE", hSaveColSettings), "updateColumnSettings", targetProcedure(), new logical(false));
            /* FALSE parameter indicates that the profile should be saved rather
                   than deleted */
            registerTriggerPersistentIn(new EventList("CHOOSE", hSaveSorting), "updateSortColumn", targetProcedure(), new logical(false));
            registerTriggerPersistentIn(new EventList("CHOOSE", hRemoveCol), "removeColumnSettings", targetProcedure());
            registerTriggerPersistentIn(new EventList("CHOOSE", hRemoveSort), "removeSortColumn", targetProcedure());
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Create the browser's search field<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Called from initializeObject<br>
    *              This resorts the query by the SearchField.
    */
   public void createSearchField()
   {
      character cSearchField = TypeFactory.character();
      character cSearchLabel = TypeFactory.character();
      handle hBrowse = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      handle hSearchField = TypeFactory.handle();
      handle hSearchLabel = TypeFactory.handle();
      logical lHideOnInit = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchField"));
         cSearchField.assign(ghProp.unwrapBufferField().value());
         
         /* If there is a SearchField, then allocate the first line of the frame
              to it and define the label and fill-in for it. */
         if (_and(isNotEqual(cSearchField, ""), () -> notUnknown(cSearchField)))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
            hFrame.assign(ghProp.unwrapBufferField().value());
            
            /* Frame handle to put the widgets in. */
            /* Currently we only support dbfields as search/sort field */
            if (_isGreaterThan( /* {fnarg <func-name> <arg> [handle] */ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDbColumn", hDataSource, "I", cSearchField), ""))
            {
               hBrowse.unwrapSizeable().setHeightChars(minus(hBrowse.unwrapSizeable().getHeightChars(), 1));
                /* Shorten the browse */hBrowse.unwrapWidget().setRow(new decimal(new integer(2)));
                /* and place at row 2 */cSearchLabel.assign(concat( /* {fnarg <func-name> <arg> [handle] */ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnLabel", hDataSource, "I", cSearchField), ": "));
               DynamicWidgetFactory.createText(hSearchLabel /* Label for the field */);
               hSearchLabel.unwrapWidget().setScreenValue(cSearchLabel);
               hSearchLabel.unwrapCommonField().setFormat(concat("X(", valueOf(length(cSearchLabel)), ")"));
               hSearchLabel.unwrapWidget().setRow(new decimal(new integer(1)));
               hSearchLabel.unwrapSizeable().setWidthChars(FontTable.getTextWidthChars(cSearchLabel));
               hSearchLabel.unwrapSizeable().setHeightChars(new decimal(new integer(1)));
               hSearchLabel.unwrapWidget().setColumn(new decimal(new integer(2)));
               hSearchLabel.unwrapWidget().setFrameHandle(hFrame);
               hSearchLabel.unwrapWidget().setScreenValue(cSearchLabel);
               hSearchLabel.unwrapWidget().setHidden(new logical(false));
               DynamicWidgetFactory.createFillIn(hSearchField);
               hSearchField.unwrapCommonField().setDataType(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnDataType", hDataSource, "I", cSearchField));
               hSearchField.unwrapCommonField().setFormat(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "columnFormat", hDataSource, "I", cSearchField));
               hSearchField.unwrapWidget().setRow(new decimal(new integer(1)));
               hSearchField.unwrapWidget().setColumn(plus(hSearchLabel.unwrapWidget().getColumn(), hSearchLabel.unwrapSizeable().getWidthChars()));
               hSearchField.unwrapWidget().setFrameHandle(hFrame);
               hSearchField.unwrapWidget().setVisible(new logical(true));
               hSearchField.unwrapSensitive().setSensitive(new logical(true));
               hSearchField.unwrapWidget().setSideLabelHandle(hSearchLabel);
               registerTriggerPersistentIn(new EventList("ANY-PRINTABLE", hSearchField), "searchTrigger", targetProcedure());
               /* reopens only if different from before, sets QuerySort if query is not 
                        opened yet */
               /* {fnarg <func-name> <arg> [handle] */
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "resortQuery", hDataSource, "I", concat("RowObject.", cSearchField));
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("HideOnInit"));
               lHideOnInit.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
               ghProp.unwrapBufferField().changeValue(hSearchField);
               
               if ((lHideOnInit).booleanValue())
               {
                  ControlFlowOps.invokeIn("hideObject", targetProcedure());
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of dataAvailable.<br>
    *              Repositions itself in response to a query reposition in its <br>
    *              DataSource.<br>
    * Parameters:  pcRelative AS CHARACTER<br>
    *              Possible values are:<br>
    *              FIRST    - Repositions to the first row in the browse<br>
    *              LAST     - Repositions to the last row in the browse<br>
    *              PREV     - Repositions to the prev row in the browse<br>
    *              NEXT     - Repositions to the next row in the browse<br>
    *              REPOSITIONED - A navigation has taken place, but the query has <br>
    *                             been repositioned so the browse is already <br>
    *                             on the correct record <br>
    *              DIFFERENT- Returns or invokes next up on the chain <br>
    *                        (If database based browse query.p will publish <br>
    *                         dataavail, is not important for SDO browsing)<br>
    *                                         <br>
    * Notes:       This procedure is here rather than in browser.p because it<br>
    *              must sometimes use query.p, and sometimes datavis.p, as its<br>
    *              super procedure, depending on whether the SmartDataBrowser has <br>
    *              its own database query.
    */
   public void dataAvailable(final character _pcRelative)
   {
      character pcRelative = TypeFactory.initInput(_pcRelative);
      handle hBrowse = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      logical lSelfQuery = TypeFactory.logical();
      logical lSourceQuery = TypeFactory.logical((Boolean) null);
      character cRowIdent = TypeFactory.character();
      character cFields = TypeFactory.character();
      handle hSource = TypeFactory.handle();
      character cDisplayed = TypeFactory.character();
      character cColumns = TypeFactory.character();
      logical lInitialized = TypeFactory.logical();
      logical lNewMode = TypeFactory.logical();
      handle hUpdateTarget = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("objectinitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         if (_not(lInitialized))
         {
            returnNormal();
         }
         
         /* Do nothing if source sdo is not initialized */
         /* {get Prop var [handle]} */ 
         hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         if ((hSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("objectinitialized"));
            lInitialized.assign(ghProp.unwrapBufferField().value());
            
            if (_not(lInitialized))
            {
               returnNormal();
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lSelfQuery.assign(ghProp.unwrapBufferField().value());
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, sourceProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
            lSourceQuery.assign(ghProp.unwrapBufferField().value());
         });
         
         /* If this is a browse with its own query, we have to differentiate 
              between events coming from this browser through the VALUE-CHANGED trigger, 
              which we want to respond to, and the event published by our super procedure,
              query.p, which we need to ignore to avoid an infinite loop. So brschnge.i
              sends a special parameter of VALUE-CHANGED for that trigger,which we 
              convert to DIFFERENT. Otherwise we ignore DIFFERENT if it comes from 
              "ourself" or when we browse an SDO. We also ignore 'RESET' from an SDO. */
         if (_or(not(lSelfQuery), () -> and(lSelfQuery, () -> isEqual(thisProcedure(), sourceProcedure()))))
         {
            if (_isEqual(pcRelative, "DIFFERENT"))
            {
               /* The SBO publishes dataAvailable 'DIFFERENT' at start up.*/
               /* {get Prop var [handle]} */ 
               hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
               
               if ((hDataSource.isValid()).booleanValue())
               {
                  cRowIdent.assign(entry(1, ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colValues", hDataSource, "I", ""), chr(1)));
                  /* {set Prop value [handle]} */ 
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", cRowIdent);
               }
               
               /* This block is also entered when a browser is initialized / started up,
                        in which case it will exit this procedure without updating the window
                        title at startup. This is to fix issue 4703 */
               ControlFlowOps.invokeIn("updateTitle", targetProcedure());
               returnNormal();
            }
            else
            {
               if (_isGreaterThan(lookup(pcRelative, "SAME,RESET"), 0))
               {
                  /* A record in new mode will not be present in the browser yet, unless 
                           the browse is the update source, so in that case we avoid this as 
                           refresh/display would reposition the query and make the DataSource
                           loose track of its new unsaved record. (The problem was encountered
                           when an SBO publishes dataAvailable after a failed add)*/
                  if ((hSource.isValid()).booleanValue())
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
                     hUpdateTarget.assign(ghProp.unwrapBufferField().value());
                     
                     if (_or(not(hUpdateTarget.isValid()), () -> isNotEqual(hUpdateTarget, hSource)))
                     {
                        /* {get Prop var [handle]} */
                        lNewMode.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getNewMode", hSource));
                     }
                  }
                  
                  if (_not(lNewMode))
                  {
                     /* This is necessary because BROWSER:REFRESH does not refresh */
                     /* the current row */
                     ControlFlowOps.invokeIn("displayRecord", targetProcedure());
                     ControlFlowOps.invokeIn("refreshBrowse", targetProcedure());
                  }
               }
            }
         }
         
         /* If the browser has its own db query and no Foreign Fields, and this
              request didn't come from itself, then this is a request from an SDO
              being used for update to refresh the browse row with new values. */
         if (_and(lSelfQuery, () -> isNotEqual(sourceProcedure(), thisProcedure())))
         {
            /* {get Prop var [handle]} */ 
            cFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", targetProcedure()));
            
            if (_and(isEqual(cFields, ""), () -> hSource.isValid()))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
               cDisplayed.assign(ghProp.unwrapBufferField().value());
               silent(() -> cColumns.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colValues", hSource, "I", cDisplayed)));
               ControlFlowOps.invokeInWithMode("displayFields", targetProcedure(), "I", cColumns);
               returnNormal();
            }
         }
         
         /* END DO IF SelfQuery and no FF */
         /* IF this browser has a db query and the source of the event also does
              (i.e., is not a Nav Panel) and we didn't get the event from ourself...*/
         if (_and(and(and(lSelfQuery, lSourceQuery), () -> isNotEqual(sourceProcedure(), thisProcedure())), () -> isNotEqual(lookup(pcRelative, "FIRST,NEXT,PREV,LAST"), 0)))
         {
            pcRelative.assign("DIFFERENT");
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         switch (StringHelper.safeTrimTrailing((pcRelative).toStringMessage()).toUpperCase())
         {
            case "FIRST":
               {
                  /* applying 'home' in a column goes to start of column */
                  if (_and(focus().isValid(), () -> isEqual(focus().unwrapWidget().getParentHandle(), hBrowse)))
                  {
                     apply(hBrowse, "CTRL-HOME");
                  }
                  else
                  {
                     apply(hBrowse, "HOME");
                  }
               }
               break;
            case "NEXT":
               {
                  silent(() -> hBrowse.unwrapBrowse().selectNextRow());
               }
               break;
            case "PREV":
               {
                  silent(() -> hBrowse.unwrapBrowse().selectPreviousRow());
               }
               break;
            case "LAST":
               {
                  /* Applying 'End' in a column goes to end of column */
                  if (_and(focus().isValid(), () -> isEqual(focus().unwrapWidget().getParentHandle(), hBrowse)))
                  {
                     apply(hBrowse, "CTRL-END");
                  }
                  else
                  {
                     apply(hBrowse, "END");
                  }
               }
               break;
         }
         ControlFlowOps.invokeIn("updateTitle", targetProcedure());
         silent(() -> ControlFlowOps.invokeIn("rowDisplay", targetProcedure()));
         /* Custom display checks. */
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         if ((hDataSource.isValid()).booleanValue())
         {
            /* colValues will return ALL rowids for an SBO when no fields is passed, 
                   but getRowident has logic to filter away the abundant ones.. */
            cRowIdent.assign(entry(1, ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colValues", hDataSource, "I", ""), chr(1)));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", cRowIdent);
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Runs persistently from default action.   <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:    The trigger is defined in setActionEvent .
    */
   public void defaultAction()
   {
      character cActionEvent = TypeFactory.character();
      logical lExit = TypeFactory.logical();
      character cLaunch = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FolderWindowToLaunch"));
         cLaunch.assign(ghProp.unwrapBufferField().value());
         
         if (_isGreaterThan(cLaunch, ""))
         {
            ControlFlowOps.invokeInWithMode("LaunchFolderWindow", targetProcedure(), "I", new character("View"));
         }
         else
         {
            /* {get Prop var [handle]} */ 
            cActionEvent.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getActionEvent", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyExitOnAction"));
            lExit.assign(ghProp.unwrapBufferField().value());
            
            if (_isNotEqual(cActionEvent, ""))
            {
               NamedEventManager.publish(cActionEvent, targetProcedure());
            }
            
            /* Is the default action supposed to close the browse ? */
            if ((lExit).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyActionOnExit"));
               ghProp.unwrapBufferField().changeValue(new logical(false));
               ControlFlowOps.invokeIn("exitObject", targetProcedure());
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Deletes the currently selected row from the Browse's viewport.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       If the SmartDataBrowser is updateable and has its own query it <br>
    *              will be invoked from deleteRecord which in this case does <br>
    *              nothing as the row has already been deleted.
    */
   public void deleteComplete()
   {
      handle hBrowse = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      character cForeignFields = TypeFactory.character();
      logical lSelfQuery = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lSelfQuery.assign(ghProp.unwrapBufferField().value());
         
         if ((lSelfQuery).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            /* {get Prop var [handle]} */
            cForeignFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", targetProcedure()));
         }
         
         if ((lSelfQuery).booleanValue())
         {
            if (_and(isNotEqual(cForeignFields, ""), () -> isEqual(hDataSource, sourceProcedure())))
            {
               returnNormal();
            }
         }
         
         if (_isEqual(hBrowse.unwrapWidget().getNumSelectedRows(), 1))
         {
            hBrowse.unwrapBrowse().deleteCurrentRow();
         }
         
         /* Multiple browser is not really supported, but this was necessary to 
              ensure that a 9.1B fix that changed the place where this was published
              from data.p did not make them behave even worse...
              It fixes a core bug that makes the first row available AND 
              ensures that a row is selected after delete AND dataVailable behaves ok.    
              Note that this is depending of the fact that current code also  
              prevents the delete to take place with more than one selected row.
              If multiple browsers is to be supported to be able to DELETE 
              more than one row this may behave really badly... */
         /* aviod error 2108 with empty multi-select add/cancel */
         /* aviod error 2108 with empty multi-select add/cancel */
         if (_and(and(hBrowse.unwrapMultiple().isMultiple(), () -> notUnknown(hBrowse.unwrapBrowse().getFocusedRow())), () -> isGreaterThan(hBrowse.unwrapBrowse().visibleIterations(), 0)))
         {
            hBrowse.unwrapBrowse().selectFocusedRow();
         }
         
         returnNormal();
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public void deleteRecord()
   {
      character cQueryPos = TypeFactory.character();
      character cRowIdent = TypeFactory.character("");
      handle hBrowse = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      integer iCnt = TypeFactory.integer();
      logical lQueryObj = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if (_isNotEqual(hBrowse.unwrapWidget().getNumSelectedRows(), 1))
         {
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "showMessage", targetProcedure(), "I", "1");
            /* Must select a single row for deletion. */
            returnNormal("ADM-ERROR");
         }
         
         silent(() -> hBrowse.unwrapBrowse().selectFocusedRow());
         /* In case it's not yet explicitly selected.*/
         hQuery.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObj.assign(ghProp.unwrapBufferField().value());
         
         if ((lQueryObj).booleanValue())
         {
            loopLabel3:
            for (iCnt.assign(1); _isLessThanOrEqual(iCnt, hQuery.unwrapQuery().numBuffers()); iCnt.increment())
            {
               hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iCnt));
               cRowIdent.assign(concat(cRowIdent, ",", valueOf(hBuffer.unwrapBuffer().rowID())));
            }
         }
         else
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
            cRowIdent.assign(valueOf(hBuffer.unwrapBuffer().rowID()));
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            /* {get Prop var [handle]} */
            cQueryPos.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", hDataSource));
         }
         
         /* Else Do - QueryObject = No */
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", cRowIdent);
         ControlFlowOps.runSuper("deleteRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         if ((lQueryObj).booleanValue())
         {
            ControlFlowOps.invokeIn("deleteComplete", targetProcedure());
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of destroyObject which lets its<br>
    *              Data source know that it is no longer being browsed.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       This is needed because only one browser may browse a<br>
    *              SmartDataObject query at a time.
    */
   public void destroyObject()
   {
      logical lQueryObject = TypeFactory.logical();
      handle hDataSource = TypeFactory.handle();
      logical lAction = TypeFactory.logical();
      handle hSearchField = TypeFactory.handle();
      handle hSearchLabel = TypeFactory.handle();
      handle hPopupMenu = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghTargetProcedure.assign(targetProcedure());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObject.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyActionOnExit"));
         lAction.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
         hSearchField.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         silent(() -> hPopupMenu.assign(hBrowse.unwrapWidget().getPopupMenu()));
         
         /* Do this only if the object does not have its own database query. */
         if (_not(lQueryObject))
         {
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            /* {set Prop value [handle]} */
            silent(() -> ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataQueryBrowsed", hDataSource, "I", false));
         }
         
         /* Is the exit supposed to behave as a double-click or return?
             (publish the ActionEvent) */
         if ((lAction).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyExitOnAction"));
            ghProp.unwrapBufferField().changeValue(new logical(false));
            ControlFlowOps.invokeIn("defaultAction", targetProcedure());
         }
         
         /* if lAction */
         if ((hSearchField.isValid()).booleanValue())
         {
            hSearchLabel.assign(hSearchField.unwrapWidget().getSideLabelHandle());
            HandleOps.delete(hSearchField);
            HandleOps.delete(hSearchLabel);
         }
         
         /* Destroy the browser's Popup Menu */
         if ((hPopupMenu.isValid()).booleanValue())
         {
            HandleOps.delete(hPopupMenu);
         }
         
         ControlFlowOps.runSuper("destroyObject");
         ghTargetProcedure.setUnknown();
         /* 9.1B: reset local TargetProcedure var. */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Makes the Browse control read-only.<br>
    * Parameters:  pcFields AS CHARACTER  'All' or 'Create'<br>
    * Notes:       'Create' is not supported yet.<br>
    *              Move away focus from the Browse before turning off the<br>
    *              READ-ONLY attribute.
    */
   public void disableFields(final character _pcFields)
   {
      character pcFields = TypeFactory.initInput(_pcFields);
      handle hBrowse = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      logical lEnabled = TypeFactory.logical();
      character cFields = TypeFactory.character();
      logical lSelected = TypeFactory.logical();
      handle hColumn = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         /* NOTE: No specific support yet for just disabling 'Create' fields. */
         if (_and(hBrowse.isValid(), () -> isEqual(pcFields, "All")))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
            cFields.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
            lEnabled.assign(ghProp.unwrapBufferField().value());
            
            if (_and(isNotEqual(cFields, ""), lEnabled) /* There are enabled fields to disable */)
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
               hFrame.assign(ghProp.unwrapBufferField().value());
               /* Was a row selected? */
               lSelected.assign(isGreaterThan(hBrowse.unwrapWidget().getNumSelectedRows(), 0));
               apply(hFrame, "ENTRY");
               /* Setting each column to read-only avoids problems with the column 
                        remaining visually enabled when this procedure is called without  
                        row-leave/leave been fired (for example the toolbar which has no focus). */
               silent(() -> hBrowse.unwrapWriteProtectable().setReadOnly(new logical(true)));
               hColumn.assign(hBrowse.unwrapBrowse().getFirstColumn());
               
               loopLabel4:
               while ((hColumn.isValid()).booleanValue())
               {
                  if ((matchesList(cFields, hColumn.unwrap().name())).booleanValue())
                  {
                     silent(() -> hColumn.unwrapWriteProtectable().setReadOnly(new logical(true)));
                  }
                  
                  hColumn.assign(hColumn.unwrapBrowseColumn().getNextColumn());
               }
               
               if ((lSelected).booleanValue())
               {
                  silent(() -> hBrowse.unwrapBrowse().selectFocusedRow());
               }
               
               /* The APPLY LEAVE is necessary to overcome a problem with the browse whereby any triggers on the 
                         browse cannot be programtically fired until the user executes a keyboard or mouse event. The APPLY
                         HOME,CTRL-HOME,END,CTR-END in dataAvailable were not firing because of this.     */
               apply(hBrowse, "LEAVE");
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
               ghProp.unwrapBufferField().changeValue(new logical(false));
            }
         }
         ControlFlowOps.runSuper("disableFields", pcFields);
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Displays the current row values by moving them to the browse's<br>
    *              screen-values.<br>
    * Parameters:  pcColValues AS CHARACTER  CHR(1) delimited list of <br>
    *              BUFFER-VALUES of the requested fields; the first value in the<br>
    *              list is the RowIdent value.
    */
   public void displayFields(final character _pcColValues)
   {
      character pcColValues = TypeFactory.initInput(_pcColValues);
      handle hFrameField = TypeFactory.handle();
      integer iValue = TypeFactory.integer();
      character cRowIdent = TypeFactory.character();
      integer iColCount = TypeFactory.integer();
      character cFields = TypeFactory.character();
      character cFieldHandles = TypeFactory.character();
      logical lSelfQuery = TypeFactory.logical();
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      handle hField = TypeFactory.handle();
      character cField = TypeFactory.character();
      character cBufferHandles = TypeFactory.character();
      character cBufferList = TypeFactory.character();
      character cNewRecord = TypeFactory.character();
      logical lStaticDisp = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lSelfQuery.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
         cFields.assign(ghProp.unwrapBufferField().value());
         
         /* Use static display for locally calculated fields */
         if (_isGreaterThan(lookup("<calc>", cFields), 0))
         {
            /* Conditionally compiled from browser.i if @ found in browser fields 
                 (local calc field) */
            silent(() -> ControlFlowOps.invokeInWithMode("DisplayFieldsStatic", targetProcedure(), "I", pcColValues));
            lStaticDisp.assign(not(ErrorManager.isError()));
         }
         
         if (_not(lStaticDisp))
         {
            if (_or(not(lSelfQuery), () -> and(lSelfQuery, () -> isNotEqual(cNewRecord, "no"))))
            {
               iColCount.assign(numEntries(pcColValues, chr(1)));
               /* Save off the row ident string, which is always the first value returned. */
               cRowIdent.assign(entry(1, pcColValues, chr(1)));
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", cRowIdent);
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
               cFieldHandles.assign(ghProp.unwrapBufferField().value());
               
               if (_isNotEqual(cFieldHandles, ""))
               {
                  loopLabel5:
                  for (iValue.assign(2); _isLessThanOrEqual(iValue, iColCount); iValue.increment())
                  {
                     cField.assign(entry(minus(iValue, 1), cFields));
                     
                     /* If this is a calc field, ignore it and don't display anything */
                     if (_isNotEqual(cField, "<Calc>"))
                     {
                        silent(() -> 
                        {
                           hFrameField.assign(handle.fromString(entry(minus(iValue, 1), cFieldHandles)));
                           hFrameField.unwrapWidget().setScreenValue(_notUnknown(pcColValues) ? trim(entry(iValue, pcColValues, chr(1))) : _isNotEqual(hFrameField.unwrapCommonField().getDataType(), "LOGICAL") ? new character("") : new character());
                           hFrameField.unwrapWidget().setModified(new logical(false));
                        });
                     }
                  }
               }
            }
 /* Checked by update code. */            else
            {
               /* {get Prop var [handle]} */ 
               hQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getQueryHandle", targetProcedure()));
               
               if (_not(hQuery.unwrapQuery().isOpen()))
               {
                  returnNormal();
               }
               
               loopLabel6:
               for (iValue.assign(1); _isLessThanOrEqual(iValue, hQuery.unwrapQuery().numBuffers()); iValue.increment())
               {
                  hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iValue));
                  cBufferHandles.assign(concat(cBufferHandles, ",", valueOf(hBuffer)));
                  cBufferList.assign(concat(cBufferList, ",", hBuffer.unwrap().name()));
               }
               
               cBufferHandles.assign(leftTrim(cBufferHandles, ","));
               cBufferList.assign(leftTrim(cBufferList, ","));
               hQuery.unwrapQuery().getCurrent();
               hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
               
               if ((hBuffer.unwrapBuffer().available()).booleanValue())
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
                  cFieldHandles.assign(ghProp.unwrapBufferField().value());
                  
                  if (_isNotEqual(cFieldHandles, ""))
                  {
                     loopLabel7:
                     for (iValue.assign(1); _isLessThanOrEqual(iValue, numEntries(cFieldHandles)); iValue.increment())
                     {
                        hFrameField.assign(handle.fromString(entry(iValue, cFieldHandles)));
                        
                        if (_notUnknown(hFrameField.unwrapDatabaseInfo().getTable()))
                        {
                           hBuffer.assign(handle.fromString(entry(lookup(hFrameField.unwrapDatabaseInfo().getTable(), cBufferList), cBufferHandles)));
                           hField.assign(hBuffer.unwrapBuffer().bufferField(hFrameField.unwrap().name()));
                           hFrameField.unwrapWidget().setScreenValue(new character(_notUnknown(hField.unwrapBufferField().value()) ? hField.unwrapBufferField().value() : _isNotEqual(hField.unwrapCommonField().getDataType(), "LOGICAL") ? new character("") : new character()));
                           hFrameField.unwrapWidget().setModified(new logical(false));
                        }
                     }
                  }
               }
            }
         }
         ControlFlowOps.invokeIn("updateTitle", targetProcedure());
         silent(() -> ControlFlowOps.invokeIn("rowDisplay", targetProcedure()));
         /* Custom display checks. */
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Makes the Browse control updatable.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       The Browse is made updateable only if some field are enabled<br>
    *              and the SmartDataBrowser is link to an Update target.
    */
   public void enableFields()
   {
      logical lSelected = TypeFactory.logical();
      handle hBrowse = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      handle hColumn = TypeFactory.handle();
      logical lEnabled = TypeFactory.logical();
      character cDisplayedFields = TypeFactory.character();
      character cFields = TypeFactory.character();
      character cFieldSecurity = TypeFactory.character();
      character cState = TypeFactory.character();
      character cTarget = TypeFactory.character();
      integer iFieldPos = TypeFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
            cFields.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
            lEnabled.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("RecordState"));
            cState.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
            cTarget.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
            cDisplayedFields.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
            cFieldSecurity.assign(ghProp.unwrapBufferField().value());
            
            /* There are fields to enable */
            if (_and(and(isNotEqual(cFields, ""), () -> not(lEnabled)), () -> isNotEqual(cTarget, "")) /* and an Update-Target */)
            {
               lSelected.assign(isEqual(hBrowse.unwrapWidget().getNumSelectedRows(), 1));
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
               hFrame.assign(ghProp.unwrapBufferField().value());
               apply(hFrame, "ENTRY");
               /* Turn off read-only for each column 
                        See comments in disable fields why read-only is true */
               hColumn.assign(hBrowse.unwrapBrowse().getFirstColumn());
               
               loopLabel8:
               while ((hColumn.isValid()).booleanValue())
               {
                  iFieldPos.assign(lookup(hColumn.unwrap().name(), cDisplayedFields));
                  
                  if (_and(matchesList(cFields, hColumn.unwrap().name()), () -> or(or(and(and(isNotEqual(iFieldPos, 0), () -> isGreaterThanOrEqual(numEntries(cFieldSecurity), iFieldPos)), () -> isEqual(entry(iFieldPos, cFieldSecurity), "")), () -> isEqual(iFieldPos, 0)), () -> isEqual(cFieldSecurity, ""))))
                  {
                     hColumn.unwrapWriteProtectable().setReadOnly(new logical(false));
                  }
                  
                  hColumn.assign(hColumn.unwrapBrowseColumn().getNextColumn());
               }
               
               hBrowse.unwrapWriteProtectable().setReadOnly(new logical(false));
               
               if ((lSelected).booleanValue())
               {
                  silent(() -> hBrowse.unwrapBrowse().selectFocusedRow());
               }
               
               apply(hBrowse, "ENTRY");
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
               ghProp.unwrapBufferField().changeValue(new logical(true));
            }
            
            if (_isEqual(cState, "NoRecordAvailable"))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
               hFrame.assign(ghProp.unwrapBufferField().value());
               apply(hFrame, "ENTRY");
            }
         }
         ControlFlowOps.runSuper("enableFields");
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:   Override as visual assumes EnabledObjFlds is fields.  <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void enableObject()
   {
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hBrowse.unwrapSensitive().setSensitive(new logical(true));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectEnabled"));
         ghProp.unwrapBufferField().changeValue(new logical(true));
         /* We also run enable_UI from here. */
         silent(() -> ControlFlowOps.invokeIn("enable_UI", targetProcedure()));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Changes/Resets Browse's states depending on action which occurs.<br>
    * Parameters:  pcState AS CHARACTER <br>
    *              Possible values are: 'BatchStart', 'BatchEnd', 'NextStart'<br>
    *              'NextEnd', 'PrevStart', 'PrevEnd'.<br>
    * Notes:       Used internally.
    */
   public void fetchDataSet(final character _pcState)
   {
      character pcState = TypeFactory.initInput(_pcState);
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         switch (StringHelper.safeTrimTrailing((pcState).toStringMessage()).toUpperCase())
         {
            case "LASTSTART":
               {
                  hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "ALWAYS");
                  hBrowse.unwrapBrowse().setRefreshable(new logical(false));
               }
               break;
            case "BATCHSTART":
               {
                  hBrowse.unwrapBrowse().setRefreshable(new logical(false));
               }
               break;
            case "LASTEND":
               {
                  hBrowse.unwrapBrowse().setRefreshable(new logical(true));
                  
                  /* This is/was a workaround to "wake-up" an updatable browse when running 
                           on an Appserver. If the cell had focus before fetchLast it did not 
                           detect that anything had changed and all rows became invisible. 
                           This particular condition can be detected as hBrowse:num-entries 
                           returned 0, (the browse must have a split personality), but by doing 
                           this in all cases we get an additional benefit as an updatable browse 
                           will keep focus in the current cell */
                  if (_isEqual(focus(), hBrowse))
                  {
                     apply(hBrowse, "ENTRY");
                  }
               }
               break;
            case "BATCHEND":
               {
                  hBrowse.unwrapBrowse().setRefreshable(new logical(true));
                  ControlFlowOps.invokeIn("adjustReposition", targetProcedure());
               }
               break;
            case "PREVSTART":
               {
                  hBrowse.unwrapBrowse().setRepositionedRow(1, "ALWAYS");
               }
               break;
            case "NEXTSTART":
               {
                  hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "ALWAYS");
               }
               break;
            case "NEXTEND":
            case "PREVEND":
               {
                  hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "CONDITIONAL");
               }
               break;
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void filterActive(final logical _plActive_1)
   {
      logical plActive_1 = TypeFactory.initInput(_plActive_1);
      handle hToolbarSource = UndoableFactory.handle();
      character cToolbarSources = UndoableFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         cToolbarSources.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkHandles", targetProcedure(), "I", "toolbar-source"));
         
         if (_isNotEqual(cToolbarSources, ""))
         {
            hToolbarSource.assign(handle.fromString(entry(1, cToolbarSources)));
            
            if ((plActive_1).booleanValue())
            {
               ControlFlowOps.invokeInWithMode("updateState", hToolbarSource, "I", new character("FilterTickOn"));
            }
            else
            {
               ControlFlowOps.invokeInWithMode("updateState", hToolbarSource, "I", new character("FilterTickOff"));
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Initialize Column Settings and properties for a browser<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Sets FieldHandles and EnabledHandles properties. This used to be <br>
    *              in initializeObject. <br>
    *              Called from inititializeObject.
    */
   public void initializeColumnSettings()
   {
      character cColDataEntry = TypeFactory.character();
      character cColumnBgcolors = TypeFactory.character();
      character cColumnFgcolors = TypeFactory.character();
      character cColumnLabelBgcolors = TypeFactory.character();
      character cColumnLabelFgcolors = TypeFactory.character();
      character cColumnLabelFonts = TypeFactory.character();
      character cColumnLabels = TypeFactory.character();
      character cColumnWidths = TypeFactory.character();
      character cColumnFonts = TypeFactory.character();
      character cDefaultColumnData = TypeFactory.character();
      character cEnabledFields = TypeFactory.character("");
      character cEnabledHandles = TypeFactory.character("");
      character cFieldHandles = TypeFactory.character("");
      character cFieldSecurity = TypeFactory.character();
      character cSavedColumnData = TypeFactory.character();
      decimal dAllWidth = TypeFactory.decimal();
      decimal dTotalWidth = TypeFactory.decimal();
      handle hBrowse = TypeFactory.handle();
      handle hColumn = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      handle hLastColumn = TypeFactory.handle();
      integer iColDataEntry = TypeFactory.integer();
      integer iFieldPos = TypeFactory.integer();
      logical lUseRepository = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
         cEnabledFields.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldSecurity"));
         cFieldSecurity.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SavedColumnData"));
         cSavedColumnData.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         lUseRepository.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseRepository", targetProcedure()));
         
         if ((lUseRepository).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnBGColors"));
            cColumnBgcolors.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnFGColors"));
            cColumnFgcolors.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnLabelBGColors"));
            cColumnLabelBgcolors.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnLabelFGColors"));
            cColumnLabelFgcolors.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnLabelFonts"));
            cColumnLabelFonts.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnLabels"));
            cColumnLabels.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnWidths"));
            cColumnWidths.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseColumnFonts"));
            cColumnFonts.assign(ghProp.unwrapBufferField().value());
         }
         
         /* if using repository and dynamic */
         if ((hBrowse.isValid()).booleanValue())
         {
            silent(() -> hColumn.assign(hBrowse.unwrapBrowse().getFirstColumn()));
         }
         
         dTotalWidth.assign(0);
         dAllWidth.assign(0);
         
         loopLabel9:
         while ((hColumn.isValid()).booleanValue())
         {
            iFieldPos.assign(plus(iFieldPos, 1));
            cFieldHandles.assign(concat(cFieldHandles, (_isNotEqual(cFieldHandles, "") ? new character(",") : new character("")), valueOf(hColumn)));
            
            if (_isNotEqual(lookup(hColumn.unwrap().name(), cEnabledFields), 0))
            {
               cEnabledHandles.assign(concat(cEnabledHandles, (_isNotEqual(cEnabledHandles, "") ? new character(",") : new character("")), valueOf(hColumn)));
            }
            
            silent(() -> 
            {
               hColumn.unwrapBrowseColumn().setColumnBgColor(_and(isGreaterThanOrEqual(numEntries(cColumnBgcolors, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnBgcolors, chr(5)), "?")) ? new integer(entry(iFieldPos, cColumnBgcolors, chr(5))) : hColumn.unwrapBrowseColumn().getColumnBgColor());
               hColumn.unwrapBrowseColumn().setColumnFgColor(_and(isGreaterThanOrEqual(numEntries(cColumnFgcolors, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnFgcolors, chr(5)), "?")) ? new integer(entry(iFieldPos, cColumnFgcolors, chr(5))) : hColumn.unwrapBrowseColumn().getColumnFgColor());
               hColumn.unwrapBrowseElement().setLabelBgColor(_and(isGreaterThanOrEqual(numEntries(cColumnLabelBgcolors, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnLabelBgcolors, chr(5)), "?")) ? new integer(entry(iFieldPos, cColumnLabelBgcolors, chr(5))) : hColumn.unwrapBrowseElement().getLabelBgColor());
               hColumn.unwrapBrowseElement().setLabelFgColor(_and(isGreaterThanOrEqual(numEntries(cColumnLabelFgcolors, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnLabelFgcolors, chr(5)), "?")) ? new integer(entry(iFieldPos, cColumnLabelFgcolors, chr(5))) : hColumn.unwrapBrowseElement().getLabelFgColor());
               hColumn.unwrapWidget().setLabelFont(_and(isGreaterThanOrEqual(numEntries(cColumnLabelFonts, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnLabelFonts, chr(5)), "?")) ? new integer(entry(iFieldPos, cColumnLabelFonts, chr(5))) : hColumn.unwrapWidget().getLabelFont());
               hColumn.unwrapCommonField().setLabel(_and(isGreaterThanOrEqual(numEntries(cColumnLabels, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnLabels, chr(5)), "?")) ? entry(iFieldPos, cColumnLabels, chr(5)) : hColumn.unwrapCommonField().getLabel());
               hColumn.unwrapSizeable().setWidthChars(_and(isGreaterThanOrEqual(numEntries(cColumnWidths, chr(5)), iFieldPos), () -> isNotEqual(entry(iFieldPos, cColumnWidths, chr(5)), "?")) ? new decimal(entry(iFieldPos, cColumnWidths, chr(5))) : hColumn.unwrapSizeable().getWidthChars());
               hColumn.unwrapWidget().setFont(_isGreaterThanOrEqual(numEntries(cColumnFonts, chr(5)), iFieldPos) ? new integer(entry(iFieldPos, cColumnFonts, chr(5))) : hColumn.unwrapWidget().getFont());
               hColumn.unwrapWidget().setAutoResize(new logical(true));
               hColumn.unwrapWidget().setResizable(new logical(true));
            });
            cDefaultColumnData.assign(concat(cDefaultColumnData, (_isGreaterThan(numEntries(cDefaultColumnData, chr(3)), 0) ? chr(3) : new character("")), hColumn.unwrap().name(), chr(4), valueOf(hColumn.unwrapSizeable().getWidthPixels())));
            
            loopLabel10:
            for (iColDataEntry.assign(1); _isLessThanOrEqual(iColDataEntry, numEntries(cSavedColumnData, chr(3))); iColDataEntry.increment())
            {
               cColDataEntry.assign(entry(iColDataEntry, cSavedColumnData, chr(3)));
               
               if (_isEqual(entry(1, cColDataEntry, chr(4)), hColumn.unwrap().name()))
               {
                  silent(() -> hColumn.unwrapSizeable().setWidthPixels(new integer(entry(2, cColDataEntry, chr(4)))));
               }
            }
            
            /* do iNumCol to entries in saved column data */
            dAllWidth.assign(plus(dAllWidth, hColumn.unwrapSizeable().getWidthPixels()));
            
            /* Hide/Disable Secured columns */
            if (_and(isGreaterThanOrEqual(numEntries(cFieldSecurity), iFieldPos), () -> isNotEqual(entry(iFieldPos, cFieldSecurity), "")))
            {
               if (_isEqual(entry(iFieldPos, cFieldSecurity), "Hidden"))
               {
                  hColumn.unwrapWidget().setVisible(new logical(false));
               }
               else
               {
                  silent(() -> hColumn.unwrapWriteProtectable().setReadOnly(new logical(true)));
               }
            }
            else
            {
               dTotalWidth.assign(plus(dTotalWidth, hColumn.unwrapSizeable().getWidthPixels()));
            }
            hLastColumn.assign(hColumn);
            hColumn.assign(hColumn.unwrapBrowseColumn().getNextColumn());
         }
         
         /* If a column is now hidden due to security, add the width
                of that column to the width of the last visible column to
                use up all the original available space */
         if (_isLessThan(dTotalWidth, dAllWidth))
         {
            hLastColumn.unwrapSizeable().setWidthPixels(new integer(plus(hLastColumn.unwrapSizeable().getWidthPixels(), minus(dAllWidth, dTotalWidth))));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldHandles"));
         ghProp.unwrapBufferField().changeValue(cFieldHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledHandles"));
         ghProp.unwrapBufferField().changeValue(cEnabledHandles);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultColumnData"));
         ghProp.unwrapBufferField().changeValue(cDefaultColumnData);
         ErrorManager.setError(new logical(false));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of initializeObject which executes <br>
    *              code mainly to set properties values.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       May create a dynamic 'Search' FILL-IN if the SearchField<br>
    *              property is set.
    */
   public void initializeObject()
   {
      handle hDataSource = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      integer iMaxGuess = TypeFactory.integer();
      character cEnabledFields = TypeFactory.character("");
      character cCreateHandles = TypeFactory.character("");
      character cUibmode = TypeFactory.character();
      logical lSaveSource = TypeFactory.logical();
      logical lQueryObject = TypeFactory.logical();
      character cTarget = TypeFactory.character();
      logical lQueryBrowsed = TypeFactory.logical();
      character cFields = TypeFactory.character();
      handle hDataQuery = TypeFactory.handle();
      handle hDataBuffer = TypeFactory.handle();
      handle hTableio = TypeFactory.handle();
      integer iField = TypeFactory.integer();
      character cField = TypeFactory.character();
      handle hField = TypeFactory.handle();
      logical lDynamic = TypeFactory.logical(false);
      character cValidation = TypeFactory.character("");
      logical lCalcWidth = TypeFactory.logical();
      handle hQueryRowObject = TypeFactory.handle();
      handle hQueryHandle = TypeFactory.handle();
      logical lScrollRemote = TypeFactory.logical();
      handle hTarget = TypeFactory.handle();
      logical lRowDisplay = TypeFactory.logical();
      rowid rRowid = TypeFactory.rowid();
      logical lPopupActive = TypeFactory.logical();
      logical lMovable = TypeFactory.logical();
      logical lSortable = TypeFactory.logical();
      logical lFitLastColumn = TypeFactory.logical();
      logical lUseRepository = TypeFactory.logical();
      character cDataSourceNames = TypeFactory.character();
      handle hSdo = TypeFactory.handle();
      character cTooltip = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* NEW FOR 9.1B: stash the TARGET-PROCEDURE handle away so the
              Data-Source can identify it if it's an SBO */
         ghTargetProcedure.assign(targetProcedure());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UIBMode"));
         cUibmode.assign(ghProp.unwrapBufferField().value());
         /* First determine whether this is a "dynamic" browser (actually an
              empty static browser to be filled in at runtime). */
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         /* Handle of the browse widget. */
         /* Retrieves column positions and sizes, if they exist, from the profile 
              manager for this browser instance */
         if ((gshProfileManager.isValid()).booleanValue())
         {
            ControlFlowOps.invokeIn("readSavedSettings", targetProcedure());
         }
         
         /* The sort column used to be set here.  It is now set from the SDO initialization */
         hBrowse.unwrapBrowse().setAllowColumnSearching(new logical(true));
         
         if (_isEqual(hBrowse.unwrapWidget().getNumColumns(), 0))
         {
            lDynamic.assign(true);
            
            /* Signal to code below. */
            /* Get Displayed and Enabled fields at runtime. */
            if (_not(hDataSource.isValid()))
            {
               returnError();
            }
            
            /* Can't do anything without it. */
            /* {get Prop var [handle]} */
            silent(() -> hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", hDataSource)));
            
            /*Handle of the SDO's query. */
            if ((hDataQuery.isValid()).booleanValue())
            {
               hQueryRowObject.assign(hDataQuery.unwrapQuery().bufferHandle(1));
               rRowid.assign(hQueryRowObject.unwrapBuffer().rowID());
               silent(() -> hBrowse.unwrapQueryAssociable().setQueryAsHandle(hDataQuery));
               ErrorManager.setError(new logical(false));
               hDataBuffer.assign(hDataQuery.unwrapQuery().bufferHandle(1));
               
               /* SDO Buffer handle.  */
               /* synch the browse with the data source */
               if (_notUnknown(rRowid))
               {
                  hDataQuery.unwrapQuery().queryRepositionByID(rRowid);
               }
               
               /* If the browse became visible when the QUERY was attached above then 
                        turn off fit-last-column since the first column will expand to the
                        full length when using add-like below
                        (the reason that this is down here was that there was a run 
                        dataAvailable after the repos, which theoretically may change
                        visibility.. it can probably be move up closer if needed..  )   */
               if ((hBrowse.unwrapWidget().isVisible()).booleanValue())
               {
                  lFitLastColumn.assign(hBrowse.unwrapBrowse().isFitLastColumn());
                  hBrowse.unwrapBrowse().setFitLastColumn(new logical(false));
               }
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
            cEnabledFields.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
            cFields.assign(ghProp.unwrapBufferField().value());
            
            /* If DisplayedFields is blank, this means take all fields from the SDO.
                   If EnabledFields is blank, this means take all updatable fields
                   from the SDO, IF there is a Tableio-Source. */
            if (_and(isEqual(cFields, ""), () -> hDataSource.isValid()))
            {
               /* If the DataSource is in fact an SBO, this function is not
                          available, which means effectively that a dynamic SDB built
                          against an SBO *must* have a column list (otherwise the SBO
                          has no idea which SDO to attach the SDB to). */
               /* {get Prop var [handle]} */
               silent(() -> cFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getDataColumns", hDataSource)));
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
               ghProp.unwrapBufferField().changeValue(cFields);
            }
            
            loopLabel11:
            for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cFields)); iField.increment())
            {
               /* Add the requested columns to the browse. */
               silent(() -> hField.assign(hDataBuffer.unwrapBuffer().bufferField(entry(iField, cFields))));
               
               if ((hField.isValid()).booleanValue())
               {
                  if (_notUnknown(hField.unwrapBufferField().validateExpression()))
                  {
                     cValidation.assign(hField.unwrapBufferField().validateExpression());
                     hField.unwrapBufferField().changeValidateExpression(new character(""));
                     hBrowse.unwrapBrowse().addLikeColumn(hField);
                     hField.unwrapBufferField().changeValidateExpression(cValidation);
                  }
                  else
                  {
                     hBrowse.unwrapBrowse().addLikeColumn(hField);
                  }
               }
            }
            
            /* END DO iField */
            /* set fit-last-column (expandable) back if turned off temporarily above*/
            if ((lFitLastColumn).booleanValue())
            {
               hBrowse.unwrapBrowse().setFitLastColumn(new logical(true));
            }
            
            /* {get Prop var [handle]} */ 
            lUseRepository.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getUseRepository", targetProcedure()));
            
            if (_and(isEqual(cEnabledFields, ""), not(lUseRepository)))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("TableIOSource"));
               hTableio.assign(ghProp.unwrapBufferField().value());
               
               if (_and(hTableio.isValid(), () -> hDataSource.isValid()))
               {
                  /* {get Prop var [handle]} */
                  cEnabledFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getUpdatableColumns", hDataSource));
                  
                  loopLabel12:
                  for (iField.assign(1); _isLessThanOrEqual(iField, numEntries(cEnabledFields)); iField.increment())
                  {
                     cField.assign(entry(iField, cEnabledFields));
                     
                     if (_isEqual(lookup(cField, cFields), 0))
                     {
                        cEnabledFields.assign(replaceAll(concat(",", cEnabledFields, ","), concat(",", cField, ","), ","));
                        cEnabledFields.assign(substring(cEnabledFields, 2, minus(length(cEnabledFields), 2), "CHARACTER"));
                     }
                  }
                  
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("EnabledFields"));
                  ghProp.unwrapBufferField().changeValue(cEnabledFields);
               }
            }
            
            /* END DO IF cEnabledFields "" */
            /* If there are any enabled fields, set the enabled prop to yes,
                   because they will be explitly disabled by initializeObject if needed. */
            if (_isNotEqual(cEnabledFields, ""))
            {
               /* Now enable the appropriate columns. 
                        The Browse must be made updatable first. */
               hBrowse.unwrapWriteProtectable().setReadOnly(new logical(false));
               hField.assign(hBrowse.unwrapBrowse().getFirstColumn());
               
               loopLabel13:
               while ((hField.isValid()).booleanValue())
               {
                  if (_isNotEqual(lookup(hField.unwrap().name(), cEnabledFields), 0))
                  {
                     hField.unwrapWriteProtectable().setReadOnly(new logical(false));
                  }
                  
                  hField.assign(hField.unwrapBrowseColumn().getNextColumn());
               }
               
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
               ghProp.unwrapBufferField().changeValue(new logical(true));
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("TOOLTIP"));
            cTooltip.assign(ghProp.unwrapBufferField().value());
            hBrowse.unwrapWidget().setTooltip(cTooltip);
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CalcWidth"));
         lCalcWidth.assign(ghProp.unwrapBufferField().value());
         
         if (_and(not(begins(cUibmode, "Design")), lCalcWidth))
         {
            hBrowse.unwrapBrowse().setExpandable(new logical(false));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObject.assign(ghProp.unwrapBufferField().value());
         
         /* We need to set CheckLastOnOpen to TRUE so that LastDBRowIdent is set
              when the query is opened, we can then check the value of LastDBRowIdent
              in fetchNext to determine if we have just moved to the last row in 
              the query, if we have then QueryPosition can be set properly */
         if ((lQueryObject).booleanValue())
         {
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setCheckLastOnOpen", targetProcedure(), "I", true);
         }
         else
         {
            if ((hDataSource.isValid()).booleanValue())
            {
               /* {get Prop var [handle]} */
               lQueryBrowsed.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataQueryBrowsed", hDataSource));
               
               if ((lQueryBrowsed).booleanValue())
               {
                  ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "showMessage", targetProcedure(), "I", "SDO query cannot be browsed by more than one SmartDataBrowser.");
                  returnNormal("ADM-ERROR");
               }
               
               /* {set Prop value [handle]} */
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataQueryBrowsed", hDataSource, "I", true);
            }
         }
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ScrollRemote"));
         lScrollRemote.assign(ghProp.unwrapBufferField().value());
         /* Check also for rowDisplay in custom super. */
         hTarget.assign(handle.fromString(entry(1, targetProcedure().unwrapCommonProcedure().superProcedures())));
         
         if (_and(isNotEqual(hTarget, thisProcedure()), () -> isNotEqual( /* Make sure it's not just browser.p */lookup("rowDisplay", hTarget.unwrapProcedure().internalEntries()), 0)))
         {
            lRowDisplay.assign(true);
         }
         
         if (_or(lScrollRemote, lRowDisplay))
         {
            registerTriggerPersistentIn(new EventList("ROW-DISPLAY", hBrowse), "rowDisplay", targetProcedure());
         }
         
         ControlFlowOps.runSuper("initializeObject");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            returnNormal("ADM-ERROR");
         }
         
         /* If this object or its parent is in design mode, then don't
              open the query or do any of the rest of this initialization code. */
         if (_begins(cUibmode, "Design"))
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SaveSource"));
         lSaveSource.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("UpdateTarget"));
         cTarget.assign(ghProp.unwrapBufferField().value());
         
         if (_or(not(lSaveSource), () -> isEqual(cTarget, "")))
         {
            ControlFlowOps.invokeInWithMode("disableFields", targetProcedure(), "I", new character("All"));
            /* This may need to be reflected in the tableiosource */
            NamedEventManager.publish("resetTableio", targetProcedure());
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectMode"));
            ghProp.unwrapBufferField().changeValue(new character("Modify"));
         }
 /* {set Prop value [handle]} */ ;
         
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObject.assign(ghProp.unwrapBufferField().value());
         hBrowse.unwrapBrowse().setCreateOnAdd(new logical(true));
         
         if ((lQueryObject).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            cFields.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getForeignFields", targetProcedure()));
            
            /* This browser is browsing its own db query. If there's no other DataSource
                   dependency and we're not in design mode, open that query. */
            if (_or(and(not(hDataSource.isValid()), () -> not(begins(cUibmode, "Design"))), () -> isEqual(cFields, "")))
            {
                /* {fn <func-name> [handle] */ ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
            }
            else
            {
               if ((hDataSource.isValid()).booleanValue())
               {
                  ControlFlowOps.invokeInWithMode("dataAvailable", targetProcedure(), "I", new character("DIFFERENT"));
               }
            }
            hDataQuery.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
            
            if (_isGreaterThan(hDataQuery.unwrapQuery().getNumResults(), 0))
            {
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("RecordState"));
               ghProp.unwrapBufferField().changeValue(new character("RecordAvailable"));
            }
            /* {set Prop value [handle]} */ 
         }
         else
         {
            if ((hDataSource.isValid()).booleanValue())
            {
               /* Reassign the Browse to use the Data Object's query. */
               /* {get Prop var [handle]} */
               silent(() -> hDataQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", hDataSource)));
               
               /* NO-ERROR for design*/
               if ((hDataQuery.isValid()).booleanValue())
               {
                  /* Else go ahead and attach the query and set the source's property.*/
                  if (_not(lDynamic))
                  {
                     silent(() -> hBrowse.unwrapQueryAssociable().setQueryAsHandle(hDataQuery));
                     ErrorManager.setError(new logical(false));
                  }
 /* If dynamic, this has been done above. */;
               }
            }
         }
         
         /* END DO for non-QueryObject */
         /* Initialize the list of handles of the browse fields, 
              for displayFields and updateRecord to use. 
              Also set a similar list of the field names for updateRecord to use. 
              Also a list of Enabled Field handles for enable/disableFields,
                and a list of any fields to be enabled for Create. */
         if ((hBrowse.isValid()).booleanValue())
         {
            /* SKip all this at design time or when there's no query yet. */
            /* Get the handle to the RowObject temp-table in the query and
                   set QueryRowObject from it */
            hQueryHandle.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
            
            if ((hQueryHandle.isValid()).booleanValue())
            {
               hQueryRowObject.assign(hQueryHandle.unwrapQuery().bufferHandle(1));
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowObject"));
            ghProp.unwrapBufferField().changeValue(hQueryRowObject);
            /* set column data (such as label, width, etc.) */
            silent(() -> ControlFlowOps.invokeIn("initializeColumnSettings", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("CreateHandles"));
            ghProp.unwrapBufferField().changeValue(cCreateHandles);
            ControlFlowOps.invokeIn("createSearchField", targetProcedure());
            
            if ((lCalcWidth).booleanValue())
            {
               ControlFlowOps.invokeIn("calcWidth", targetProcedure());
            }
            
            /* We initially set MAX-DATA-GUESS to Rows to Batch, MAX-DATA-GUESS
                   later gets adjusted by calls to assignMaxGuess from data.p's 
                   batch processing. In preview mode the SDO is not available.  */
            if (_and(not(begins(cUibmode, "Preview")), () -> hDataSource.isValid()))
            {
               /* {get Prop var [handle]} */
               iMaxGuess.assign(ControlFlowOps.invokeDynamicFunctionIn(integer.class, "getRowsToBatch", hDataSource));
               
               /* Setting max-data-guess to 0 gives a very strange effect, but in 
                        if RowsToBatch is 0 we have all the records, so LastRowNum should 
                        provide a guess (if the SDO is started)  */
               if (_isEqual(iMaxGuess, 0))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
                  cDataSourceNames.assign(ghProp.unwrapBufferField().value());
                  hSdo.setUnknown();
                  
                  if (_isGreaterThan(cDataSourceNames, ""))
                  {
                     silent(() -> hSdo /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "dataObjectHandle", hDataSource, "I", cDataSourceNames)));
                     
                     if ((hSdo.isValid()).booleanValue())
                     {
                        /* {get Prop var [handle]} */
                        iMaxGuess.assign(ControlFlowOps.invokeDynamicFunctionIn(integer.class, "getLastRowNum", hSdo));
                     }
                  }
                  
                  /* No-error as it still may get here with an SBO if something
                             is wrong with datasourcenames (mismatch or blank) 
                             Even if that is very wrong, this error wont give any clue ..  */
                  if (_and(isEqual(iMaxGuess, 0), () -> not(hSdo.isValid())))
                  {
                     /* {get Prop var [handle]} */
                     silent(() -> iMaxGuess.assign(ControlFlowOps.invokeDynamicFunctionIn(integer.class, "getLastRowNum", hDataSource)));
                  }
               }
               
               if (_isGreaterThan(iMaxGuess, 0))
               {
                  hBrowse.unwrapBrowse().setMaxDataGuess(iMaxGuess);
               }
            }
            /* if not design mode */
            /* Sets column positions and sizes based on savedColumnData */
            ControlFlowOps.invokeIn("resetColumnSettings", targetProcedure());
            /* Column movable and allow-column-searching are mutually exclusive, 
                   column sorting (implemented with column-searching) takes precedence */
            /* {get Prop var [handle]} */ 
            lMovable.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getColumnsMovable", targetProcedure()));
            /* {get Prop var [handle]} */ 
            lSortable.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getColumnsSortable", targetProcedure()));
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("PopupActive"));
            lPopupActive.assign(ghProp.unwrapBufferField().value());
            hBrowse.unwrapBrowse().setColumnMovable((lSortable).booleanValue() ? new logical(false) : lMovable);
            hBrowse.unwrapBrowse().setAllowColumnSearching(lSortable);
            
            if (_and(lMovable, lSortable))
            {
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setColumnsMovable", targetProcedure(), "I", false);
            }
            
            if ((lPopupActive).booleanValue())
            {
               ControlFlowOps.invokeIn("createPopupMenu", targetProcedure());
            }
         }
         /* END IF VALID-HANDLE(hBrowse) */
         ghTargetProcedure.setUnknown();
         /* 9.1B: reset local storage of this. */
         ControlFlowOps.invokeIn("adjustReposition", targetProcedure());
         /* we need to tell ourselves that we have data...  */
         ControlFlowOps.invokeInWithMode("dataAvailable", targetProcedure(), "I", new character("DIFFERENT"));
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void launchFolderWindow(final character _pcContainerMode)
   {
      character pcContainerMode = TypeFactory.initInput(_pcContainerMode);
      character cFolderWindow = TypeFactory.character();
      handle hContainerSource = TypeFactory.handle();
      handle hFolderWindow = TypeFactory.handle();
      handle hWindowHandle = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      character cProcedureType = TypeFactory.character();
      logical lMultiInstanceActivated = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FolderWindowToLaunch"));
         cFolderWindow.assign(ghProp.unwrapBufferField().value());
         
         if (_isGreaterThan(cFolderWindow, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
            hContainerSource.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
            hWindowHandle.assign(ghProp.unwrapBufferField().value());
            
            if ((gshSessionManager.isValid()).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
               
               if ((hContainerSource.isValid()).booleanValue())
               {
                  /* {get Prop var [handle]} */
                  lMultiInstanceActivated.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getMultiInstanceActivated", hContainerSource));
               }
               
               /* pcLogicalName     */
               ControlFlowOps.invokeInWithMode("launchContainer", gshSessionManager, "IIIIIIIIIIIOO", cFolderWindow, new character(""), new character(""), not(lMultiInstanceActivated), new character(""), new character(""), new character(""), pcContainerMode, hWindowHandle, hContainerSource, targetProcedure(), hFolderWindow, cProcedureType);
            }
 /* pcProcedureType   */            else
            {
               if ((hContainerSource.isValid()).booleanValue())
               {
                  ControlFlowOps.invokeInWithMode("constructObject", hContainerSource, "IIIO", cFolderWindow, hWindowHandle, concat("ContainerMode", chr(4), pcContainerMode), hFolderWindow);
                  
                  if ((hFolderWindow.isValid()).booleanValue())
                  {
                     ControlFlowOps.invokeIn("initializeObject", hFolderWindow);
                  }
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *    Purpose: Override in order to run profile sort before the SDO is initialized  <br>
    * Parameters: See smart.p   <br>
    *      Notes:
    */
   public void linkStateHandler(final character _pcState, final handle _phObject, final character _pcLink)
   {
      character pcState = TypeFactory.initInput(_pcState);
      handle phObject = TypeFactory.initInput(_phObject);
      character pcLink = TypeFactory.initInput(_pcLink);
      
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.runSuper("linkStateHandler", pcState, phObject, pcLink);
         
         if (_and(isEqual(pcState, "add"), () -> isEqual(pcLink, "DataSource")))
         {
            ControlFlowOps.invokeInWithMode("resetSortColumn", targetProcedure(), "I", new logical(false));
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Locates a widget and retuns its handle and the handle of its<br>
    *              TARGET-PROCEDURE<br>
    * Parameters:  pcWidget AS CHARACTER<br>
    *              phWidget AS HANDLE<br>
    *              phTarget AS HANDLE<br>
    * Notes:       Contains code to locate a widget specific to browser's when <br>
    *              the qualifier is 'Browse'
    */
   public void locateWidget(final character _pcWidget, final handle phWidget, final handle phTarget)
   {
      character pcWidget = TypeFactory.initInput(_pcWidget);
      character cField = TypeFactory.character();
      
      internalProcedure(new Block((Init) () -> 
      {
         TypeFactory.initOutput(phWidget);
         TypeFactory.initOutput(phTarget);
      }, 
      (Body) () -> 
      {
         if (_and(isGreaterThan(numEntries(pcWidget, "."), 1), () -> isEqual(entry(1, pcWidget, "."), "Browse")))
         {
            silent(() -> 
            {
               cField.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "deleteEntry", targetProcedure(), "III", 1, pcWidget, "."));
               phWidget.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "internalWidgetHandle", targetProcedure(), "II", cField, "DATA"));
               phTarget.assign(targetProcedure());
            });
            
            if (_notUnknown(phWidget))
            {
               returnNormal();
            }
         }
         else
         {
            ControlFlowOps.runSuper("locateWidget", pcWidget, phWidget, phTarget);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Handles value-changed event for the movable popup menu item<br>
    * Parameters:  <br>
    * Notes:       Movable and sortable are mutually exclusive
    */
   public void movableValueChanged()
   {
      handle hMovableItem = TypeFactory.handle();
      handle hSortableItem = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MovableHandle"));
         hMovableItem.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SortableHandle"));
         hSortableItem.assign(ghProp.unwrapBufferField().value());
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setColumnsMovable", targetProcedure(), "I", hMovableItem.unwrapWidget().isChecked());
         
         if (_and(hSortableItem.unwrapWidget().isChecked(), () -> hMovableItem.unwrapWidget().isChecked()))
         {
            hSortableItem.unwrapWidget().setChecked(new logical(false));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setColumnsSortable", targetProcedure(), "I", false);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Moves and resizes columns based on column data being passed to it<br>
    * Parameters:  pcColumnData AS CHAR<br>
    *              Format of pcColumnData:<br>
    *       column1name CHR(4) column1width CHR(3) column2name CHR(4) column2width etc...<br>
    * Notes:
    */
   public void moveAndResizeColumns(final character _pcColumnData)
   {
      character pcColumnData = TypeFactory.initInput(_pcColumnData);
      character cColumnEntry = TypeFactory.character();
      handle hBrowse = TypeFactory.handle();
      handle hColumn = TypeFactory.handle();
      integer iColumnPos = TypeFactory.integer();
      integer iNumCols = TypeFactory.integer();
      handle hRo = TypeFactory.handle();
      rowid rCurRowid = TypeFactory.rowid();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         /* Issue 12777 is caused by REFRESHABLE=YES which is commented out later on */
         /* so this line must be commented also. Please do not remove commented code */
         /* as it serves as a prevention mechanism for anyone adding the same code */
         /* again and causing regressions */
         /*   hBrowse:REFRESHABLE = NO.  */
         /* fix for issues 12635, 12777... */
         /* The following code is a workaround for a core bug in the browse */
         /* widget which causes the browse query to release the current query */
         /* buffer as a result of direct manipulation of certain attributes. */
         /* So, we need to save the current Rowid, if any, and re-find it later */
         silent(() -> hRo.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle().unwrapQuery().bufferHandle()));
         
         if (_and(hRo.isValid(), () -> hRo.unwrapBuffer().available()))
         {
            rCurRowid.assign(hRo.unwrapBuffer().rowID());
         }
         
         loopLabel14:
         for (iNumCols.assign(1); _isLessThanOrEqual(iNumCols, numEntries(pcColumnData, chr(3))); iNumCols.increment())
         {
            cColumnEntry.assign(entry(iNumCols, pcColumnData, chr(3)));
            hColumn.assign(hBrowse.unwrapBrowse().getFirstColumn());
            iColumnPos.assign(1);
            
            loopLabel15:
            while ((hColumn.isValid()).booleanValue())
            {
               if (_isEqual(hColumn.unwrap().name(), entry(1, cColumnEntry, chr(4))))
               {
                  silent(() -> hBrowse.unwrapBrowse().moveColumn(iColumnPos, iNumCols));
                  hColumn.unwrapSizeable().setWidthPixels(new integer(entry(2, cColumnEntry, chr(4))));
                  break loopLabel15;
               }
               
               hColumn.assign(hColumn.unwrapBrowseColumn().getNextColumn());
               iColumnPos.assign(plus(iColumnPos, 1));
            }
         }
         
         /* do iNumCols to column data entries */
         if ((hRo.isValid()).booleanValue())
         {
            if (_notUnknown(rCurRowid))
            {
               if (_isNotEqual(rCurRowid, hRo.unwrapBuffer().rowID()))
               {
                   /* IF a valid buffer existed in the beginning... */hRo.unwrapBuffer().findByRowID(rCurRowid, LockType.NONE);
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     trigger code for OFF-END trigger of SmartdataBrowse<br>
    * Notes:       This procedure contains the code that was previously held in<br>
    *              src/adm2/brsoffnd.i.
    */
   public void offEnd()
   {
      handle hDataSource = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      handle hFocus = TypeFactory.handle();
      logical lQuery = TypeFactory.logical();
      logical lModified_1 = TypeFactory.logical();
      character cNewRecord = TypeFactory.character();
      integer iLastRowNum = TypeFactory.integer();
      character cMsg = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_not(lQuery))
         {
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            
            /* if the browser does not manage it's own query and it doesn't have */
            /* a DataSource or the DataSource has no records, then we don't need */
            /* to do anything here */
            if (_or(not(hDataSource.isValid()), () -> begins(new character( /* {fn <func-name> [handle] */ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", hDataSource)), "NoRecord")))
            {
               returnNormal();
            }
            
            iLastRowNum.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkProperty", targetProcedure(), "II", "DATA-SOURCE", "LastRowNum"));
            
            if (_notUnknown(iLastRowNum))
            {
               returnNormal();
            }
            
            /* If we have already fetched the last row then return */
            /* {get Prop var [handle]} */ 
            lModified_1.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", targetProcedure()));
            
            if ((lModified_1).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
               cMsg.assign((_isEqual(cNewRecord, "no") ? new character("12") : _isEqual(cNewRecord, "add") ? new character("13") : new character("14")));
               /* {fnarg <func-name> <arg> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "showMessage", targetProcedure(), "I", cMsg);
               ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", new unknown());
               returnConsume();
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
            hBrowse.assign(ghProp.unwrapBufferField().value());
            hFocus.assign(focus());
            ControlFlowOps.invokeInWithMode("fetchDataSet", targetProcedure(), "I", new character("BatchStart"));
            
            if (_isEqual(keyFunction(keyCode(KeyReader.getLabel())), "PAGE-DOWN"))
            {
               hBrowse.unwrapBrowse().setRepositionedRow(1, "ALWAYS");
            }
            else
            {
               hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "ALWAYS");
            }
            
            NamedEventManager.publish("fetchBatch", targetProcedure(), "I", new Object[]
            {
               true
            });
            /* yes -- move forwards */
            ControlFlowOps.invokeInWithMode("fetchDataSet", targetProcedure(), "I", new character("BatchEnd"));
            hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "CONDITIONAL");
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowReset"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            /* Traditionally the browser just used run dataAvailable from the events,
                  but this is now in a super-procedure and the sbo need to know the
                  target, so we subscribe the source temporarily. */
            NamedEventManager.subscribe(hDataSource, "DataAvailable", targetProcedure());
            NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
            {
               "DIFFERENT"
            });
            NamedEventManager.unsubscribe(hDataSource, "DataAvailable", targetProcedure());
            
            if ((hFocus.isValid()).booleanValue())
            {
               if (_isEqual(hFocus.unwrapWidget().getParentHandle(), hBrowse))
               {
                  ControlFlowOps.invokeInWithMode("applyCellEntry", targetProcedure(), "I", _isEqual(keyFunction(keyCode(KeyReader.getLabel())), "TAB") ? new character() : hFocus.unwrap().name());
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Trigger code for OFF-HOME trigger of SmartdataBrowse<br>
    * Notes:       This procedure contains the code that was previously held in<br>
    *              src/adm2/brsoffhm.i.
    */
   public void offHome()
   {
      handle hDataSource = TypeFactory.handle();
      handle hFocus = TypeFactory.handle();
      logical lQuery = TypeFactory.logical();
      logical lModified_1 = TypeFactory.logical();
      character cNewRecord = TypeFactory.character();
      handle hCell = TypeFactory.handle();
      handle hCellEntry = TypeFactory.handle();
      integer iCount = TypeFactory.integer();
      integer iFirstRowNum = TypeFactory.integer();
      handle hBrowse = TypeFactory.handle();
      character cMsg = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_not(lQuery))
         {
            iFirstRowNum.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "linkProperty", targetProcedure(), "II", "DATA-SOURCE", "FirstRowNum"));
            
            if (_notUnknown(iFirstRowNum))
            {
               returnNormal();
            }
            
            /* If we have already fetched the first row then return */
            /* {get Prop var [handle]} */ 
            lModified_1.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", targetProcedure()));
            
            if ((lModified_1).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
               cMsg.assign((_isEqual(cNewRecord, "no") ? new character("12") : _isEqual(cNewRecord, "add") ? new character("13") : new character("14")));
               ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", new unknown());
               returnConsume();
            }
            
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
            hBrowse.assign(ghProp.unwrapBufferField().value());
            hFocus.assign(focus());
            ControlFlowOps.invokeInWithMode("fetchDataSet", targetProcedure(), "I", new character("BatchStart"));
            
            if (_isEqual(keyFunction(keyCode(KeyReader.getLabel())), "PAGE-UP"))
            {
               hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "ALWAYS");
            }
            else
            {
               hBrowse.unwrapBrowse().setRepositionedRow(1, "ALWAYS");
            }
            
            NamedEventManager.publish("fetchBatch", targetProcedure(), "I", new Object[]
            {
               false
            });
            /* no -- move backwards */
            ControlFlowOps.invokeInWithMode("fetchDataSet", targetProcedure(), "I", new character("BatchEnd"));
            hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "CONDITIONAL");
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowReset"));
            ghProp.unwrapBufferField().changeValue(new logical(true));
            /* {get Prop var [handle]} */ 
            hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
            
            if ((hDataSource.isValid()).booleanValue())
            {
               /* Traditionally the browser just used run dataAvailable from the events,
                       but this is now ina super-procedure and the sbo need to know the
                       target, so we subscribe the source temporarily. */
               NamedEventManager.subscribe(hDataSource, "DataAvailable", targetProcedure());
               NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
               {
                  "DIFFERENT"
               });
               NamedEventManager.unsubscribe(hDataSource, "DataAvailable", targetProcedure());
            }
            
            if (_isEqual(keyFunction(keyCode(KeyReader.getLabel())), "BACK-TAB"))
            {
               hCell.assign(hBrowse.unwrapBrowse().getFirstColumn());
               
               loopLabel16:
               for (iCount.assign(1); _isLessThanOrEqual(iCount, hBrowse.unwrapWidget().getNumColumns()); iCount.increment())
               {
                  if (_not(hCell.unwrapWriteProtectable().isReadOnly()))
                  {
                     hCellEntry.assign(hCell);
                  }
                  
                  hCell.assign(hCell.unwrapBrowseColumn().getNextColumn());
               }
            }
            else
            {
               if ((hFocus.isValid()).booleanValue())
               {
                  if (_isEqual(hFocus.unwrapWidget().getParentHandle(), hBrowse))
                  {
                     hCellEntry.assign(hFocus);
                  }
               }
            }
            
            if ((hCellEntry.isValid()).booleanValue())
            {
               ControlFlowOps.invokeInWithMode("applyCellEntry", targetProcedure(), "I", hCellEntry.unwrap().name());
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:  Event handler for the 'END' event    <br>
    * Parameters:  &lt;none&gt;<br>
    *  Notes:   This event need to call fetchLast in the DataSource for the normal <br>
    *           case when it is a true user event, but APPLY 'END' is also the only <br>
    *           way to make the browse respond to a fetchLast in the DataSource. <br>
    *           In the latter case QueryPosition is already set in the datasource <br>
    *           and can thus be used to avoid an extra (or endless..) loop.
    */
   public void onEnd()
   {
      handle hBrowse = TypeFactory.handle();
      logical lQuery = TypeFactory.logical();
      character cQueryPos = TypeFactory.character();
      handle hSource = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQuery.assign(ghProp.unwrapBufferField().value());
         hBrowse.unwrapBrowse().setRefreshable(new logical(false));
         
         if ((lQuery).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            cQueryPos.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", targetProcedure()));
            
            if (_isEqual(lookup(cQueryPos, "LastRecord,OnlyRecord"), 0))
            {
               ControlFlowOps.invokeIn("fetchLast", targetProcedure());
            }
         }
         else
         {
            if ((new logical(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "isOkToFetch", targetProcedure(), "I", "LAST"))).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
               /* The SBO fetchFirst need to know who this is */
               ghTargetProcedure.assign(targetProcedure());
               ControlFlowOps.invokeIn("fetchLast", hSource);
               ghTargetProcedure.setUnknown();
            }
         }
         /* isOkToFetch = TRUE */
         hBrowse.unwrapBrowse().setRefreshable(new logical(true));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *    Purpose:  Event handler for the 'HOME' event    <br>
    * Parameters:  &lt;none&gt;<br>
    *      Notes: This event need to call fetchFirst in the DataSource for the normal <br>
    *             case when it is a true user event, but APPLY 'HOME' is also the only <br>
    *             way to make the browse respond to a fetchFirst in the DataSource. <br>
    *             In the latter case QueryPosition is already set in the datasource <br>
    *             and can thus be used to avoid an extra (or endless..) loop.
    */
   public void onHome()
   {
      handle hBrowse = TypeFactory.handle();
      logical lQuery = TypeFactory.logical();
      character cQueryPos = TypeFactory.character();
      handle hSource = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQuery.assign(ghProp.unwrapBufferField().value());
         hBrowse.unwrapBrowse().setRefreshable(new logical(false));
         
         if ((lQuery).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            cQueryPos.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", targetProcedure()));
            
            if (_isEqual(lookup(cQueryPos, "FirstRecord,OnlyRecord"), 0))
            {
               ControlFlowOps.invokeIn("fetchFirst", targetProcedure());
            }
         }
         else
         {
            if ((new logical(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "isOkToFetch", targetProcedure(), "I", "FIRST"))).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
               /* The SBO fetchFirst need to know who this is */
               ghTargetProcedure.assign(targetProcedure());
               ControlFlowOps.invokeIn("fetchFirst", hSource);
               ghTargetProcedure.setUnknown();
            }
         }
         /* isOkToFetch = TRUE */
         hBrowse.unwrapBrowse().setRefreshable(new logical(true));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *    Purpose: Event handler for the Browse VALUE-CHANGED event.<br>
    * Parameters:  &lt;none&gt;<br>
    *      Notes: Different from the valueChanged procedure which fires for <br>
    *             value changes in the fields (VALUE-CHANGED on browse ANYWHERE) <br>
    *           - This event should NOT be overriden as a place to trap change of <br>
    *             record. The fact that it is fired from the value-changed trigger <br>
    *             means that it only runs when a user changes the row in the browse <br>
    *             (it does also backfire from events like home and end). It does not <br>
    *             run in all cases where a record is changed. (reposition for example)            <br>
    *           - DataAvailable is the event that traps the change of record.
    */
   public void onValueChanged()
   {
      handle hBrowse = TypeFactory.handle();
      logical lQuery = TypeFactory.logical();
      character cNewRecord = TypeFactory.character();
      handle hDataSource = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if (_not(hBrowse.isValid()))
         {
            returnConsume();
         }
         
         /* {get Prop var [handle]} */ 
         cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
         
         if (_isEqual(cNewRecord, "NO"))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
            lQuery.assign(ghProp.unwrapBufferField().value());
            
            if ((lQuery).booleanValue())
            {
 /* Browser has its own dbquery */
               ControlFlowOps.invokeWithMode("dataAvailable", "I", new character("VALUE-CHANGED"));
            }
            else
            {
               if ((new logical(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "isOkToFetch", targetProcedure(), "I", "VALUE-CHANGED"))).booleanValue())
               {
                  /* {get Prop var [handle]} */ 
                  hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
                  /* Traditionally the browser just used run dataAvailable from the events,
                           but this is now in a super-procedure and the sbo need to know the
                           target, so we subscribe the source temporarily.  */
                  NamedEventManager.subscribe(hDataSource, "DataAvailable", targetProcedure());
                  NamedEventManager.publish("dataAvailable", targetProcedure(), "I", new Object[]
                  {
                     "DIFFERENT"
                  });
                  NamedEventManager.unsubscribe(hDataSource, "DataAvailable", targetProcedure());
               }
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Reads column positions and sizes from profile into <br>
    *              SavedColumnData property.  <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void readSavedSettings()
   {
      character cBrowseName = TypeFactory.character();
      character cColumnData = TypeFactory.character();
      character cContainerName = TypeFactory.character();
      character cProfileKey = TypeFactory.character();
      handle hContainerSource = TypeFactory.handle();
      logical lProfileExists = TypeFactory.logical();
      rowid rId = TypeFactory.rowid();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cBrowseName.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hContainerSource.assign(ghProp.unwrapBufferField().value());
         
         if ((hContainerSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LogicalObjectName"));
            cContainerName.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         if (_and(isEqual(cContainerName, ""), () -> hContainerSource.isValid()))
         {
            cContainerName.assign(hContainerSource.unwrapFile().getFileName());
         }
         
         cProfileKey.assign(concat(cContainerName, cBrowseName));
         ControlFlowOps.invokeInWithMode("checkProfileDataExists", gshProfileManager, "IIIIIO", new character("Browser"), new character("Columns"), cProfileKey, new logical(false), new logical(true), lProfileExists);
         
         if ((lProfileExists).booleanValue())
         {
            rId.setUnknown();
            ControlFlowOps.invokeInWithMode("getProfileData", gshProfileManager, "IIIIUO", new character("Browser"), new character("Columns"), cProfileKey, new logical(false), rId, cColumnData);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("savedColumnData"));
            ghProp.unwrapBufferField().changeValue(cColumnData);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:    Published from dataSource when a display is not enough.      <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void refreshBrowse()
   {
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         silent(() -> hBrowse.unwrapWidget().refresh());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Remove column position and size settings from the browser and <br>
    *              from the repository if a profile is stored for this browser.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Invoked by the Remove Column Positins and Sizes popup menu item
    */
   public void removeColumnSettings()
   {
      internalProcedure(new Block((Body) () -> 
      {
         ControlFlowOps.invokeIn("resetDefaultSettings", targetProcedure());
         /* Send flag to indicate that the profile should be deleted if it exists */
         ControlFlowOps.invokeInWithMode("updateColumnSettings", targetProcedure(), "I", new logical(true));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Remove column sort from the browser's data source and <br>
    *              from the repository if a profile is stored for this browser.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Invoked by the Remove Sort Column popup menu item
    */
   public void removeSortColumn()
   {
      internalProcedure(new Block((Body) () -> 
      {
         /* Send flag to indicate that the default sort should be used */
         ControlFlowOps.invokeInWithMode("resetSortColumn", targetProcedure(), "I", new logical(true));
         /* Send flag to indicate that the profile should be deleted if it exists */
         ControlFlowOps.invokeInWithMode("updateSortColumn", targetProcedure(), "I", new logical(true));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Resets column settings based on SavedColumnData <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Invoked from the Reset Column Positions and Sizes popup menu<br>
    *              item and from initializeObject to set the initial <br>
    *              column positions and sizes.
    */
   public void resetColumnSettings()
   {
      character cColumnData = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SavedColumnData"));
         cColumnData.assign(ghProp.unwrapBufferField().value());
         
         if (_isEqual(cColumnData, ""))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultColumnData"));
            cColumnData.assign(ghProp.unwrapBufferField().value());
         }
         
         /* if saved column data blank */
         ControlFlowOps.invokeInWithMode("moveAndResizeColumns", targetProcedure(), "I", cColumnData);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Resets default column settings based on DefaultColumnData <br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Invoked from the Reset Default Column Positions and Sizes <br>
    *              popup menu item.
    */
   public void resetDefaultSettings()
   {
      character cDefaultData = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultColumnData"));
         cDefaultData.assign(ghProp.unwrapBufferField().value());
         ControlFlowOps.invokeInWithMode("moveAndResizeColumns", targetProcedure(), "I", cDefaultData);
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of resetRecord.<br>
    *              Redisplays the original field values retrieved from the <br>
    *              Data Object or directly from the query associated with<br>
    *              the SmartDataBrowser.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:       Invoked when the Reset button is pressed on an Update SmartPanel.
    */
   public void resetRecord()
   {
      logical lSelfQuery = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lSelfQuery.assign(ghProp.unwrapBufferField().value());
         
         if (_not(lSelfQuery))
         {
            ControlFlowOps.runSuper("resetRecord");
         }
         else
         {
            ControlFlowOps.invokeInWithMode("displayFields", targetProcedure(), "I", new unknown());
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataModified", targetProcedure(), "I", new unknown());
            NamedEventManager.publish("resetRecord", targetProcedure());
            /* In case GroupAssign */
            ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", new unknown());
         }
         
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Resets sort column <br>
    * Parameters:  plDefault AS LOGICAL<br>
    * Notes:       Sort Column is not stored as a property in the browser since <br>
    *              sorting is done for an SDO, not a browser.  <br>
    *              <br>
    *              This procedure handles the following:<br>
    *              - Setting the sort column initially from profile data (when <br>
    *                invoked from initializeObject)<br>
    *              - Resetting the sort column from profile data (when invoked<br>
    *                from Reset Sort Column popup menu item)<br>
    *              - Resetting the sort column to the default sort criteria <br>
    *                defined in the SDO's base query (when invoked from<br>
    *                Reset Default Sort Column popup menu item - plDefault is TRUE)<br>
    *              <br>
    *              setSort is not used because setSort is designed to work with the<br>
    *              START-SEARCH trigger and it will change the SDO sort from <br>
    *              ascending to descending and vice versa if the same column is <br>
    *              chosen.  This should not happen here where just the sort from <br>
    *              the profile should be used.
    */
   public void resetSortColumn(final logical _plDefault)
   {
      logical plDefault = TypeFactory.initInput(_plDefault);
      character cBrowseName = TypeFactory.character();
      character cContainerName = TypeFactory.character();
      character cDataSourceName = TypeFactory.character();
      character cProfileKey = TypeFactory.character();
      character cSortColumn = TypeFactory.character();
      character cSourceNames = TypeFactory.character();
      handle hContainerSource = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      logical lInitialized = TypeFactory.logical();
      logical lProfileExists = TypeFactory.logical();
      rowid rId = TypeFactory.rowid();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cBrowseName.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hContainerSource.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         /* If the sort column is not being reset to the default sort column then
              the profile data is checked */
         if (_not(plDefault))
         {
            if ((hContainerSource.isValid()).booleanValue())
            {
               ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("LogicalObjectName"));
               cContainerName.assign(ghProp.unwrapBufferField().value());
            }
            /* {get Prop var [handle]} */ 
            
            if (_and(isEqual(cContainerName, ""), () -> hContainerSource.isValid()))
            {
               cContainerName.assign(hContainerSource.unwrapFile().getFileName());
            }
            
            cProfileKey.assign(concat(cContainerName, cBrowseName));
            
            if ((gshProfileManager.isValid()).booleanValue())
            {
               ControlFlowOps.invokeInWithMode("checkProfileDataExists", gshProfileManager, "IIIIIO", new character("SDO"), new character("Sorting"), cProfileKey, new logical(false), new logical(true), lProfileExists);
               
               if ((lProfileExists).booleanValue())
               {
                  rId.setUnknown();
                  ControlFlowOps.invokeInWithMode("getProfileData", gshProfileManager, "IIIIUO", new character("SDO"), new character("Sorting"), cProfileKey, new logical(false), rId, cSortColumn);
                  
                  if (_isEqual(cSortColumn, ""))
                  {
                     lProfileExists.assign(false);
                  }
               }
            }
         }
         
         /* if not Default */
         /* If a profile does not exist and this is called from initializeObject 
              then just return rather than attempting to set the sort column */
         if (_and(not(lProfileExists), not(lInitialized)))
         {
            returnNormal();
         }
         
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
         cSourceNames.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, hDataSource.unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cDataSourceName.assign(ghProp.unwrapBufferField().value());
         
         /* The SBO does not have a sort API, so if SourceNames is defined we need to 
              get the handle of the actual Source */
         if (_and(and(notUnknown(cSourceNames), () -> isNotEqual(cSourceNames, "")), () -> isNotEqual(cSourceNames, cDataSourceName)))
         {
            hDataSource /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "DataObjectHandle", hDataSource, "I", cSourceNames));
         }
         
         /* If a profile does not exist and this is not being called from 
              initializeObject then the sort column needs to be reset from the 
              default sort criteria of the SDO's base query (? = default)  */
         if (_and(not(lProfileExists), lInitialized))
         {
            cSortColumn.setUnknown();
         }
         else
         {
 /* if not qualification then assume rowobject is referenced  */
            if (_and(isGreaterThan(cSortColumn, ""), () -> isLessThanOrEqual(numEntries(cSortColumn, "."), 1)))
            {
               cSortColumn.assign(concat("RowObject.", cSortColumn));
            }
         }
         
         if ((hDataSource.isValid()).booleanValue())
         {
            /* {fnarg <func-name> <arg> [handle] */
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "resortQuery", hDataSource, "I", cSortColumn);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Changes the size of the browse widget in a SmartDataBrowser   <br>
    * Parameters:  pd_height AS DECIMAL  The desired height (in rows)<br>
    *              pd_width  AS DECIMAL  The desired width (in columns)
    */
   public void resizeBrowse(final decimal _pdHeight, final decimal _pdWidth)
   {
      decimal pdHeight = TypeFactory.initInput(_pdHeight);
      decimal pdWidth = TypeFactory.initInput(_pdWidth);
      handle hBrowse = TypeFactory.handle();
      handle hFieldGroup = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         pdHeight.assign(decimal.maximum(pdHeight, new decimal(2.0)));
         pdWidth.assign(decimal.maximum(pdWidth, new decimal(2.0)));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hFieldGroup.assign(hBrowse.unwrapWidget().getParentHandle());
          /* Field Group */hFrame.assign(hFieldGroup.unwrapWidget().getParentHandle());
         
         /* The frame ! */
         if (_isLessThanOrEqual(plus(pdWidth, hBrowse.unwrapWidget().getColumn()), hFrame.unwrapSizeable().getWidthChars()))
         {
            hBrowse.unwrapSizeable().setWidthChars(pdWidth);
         }
         
         if (_isLessThanOrEqual(plus(pdHeight, hBrowse.unwrapWidget().getRow()), hFrame.unwrapSizeable().getHeightChars()))
         {
            hBrowse.unwrapSizeable().setHeightChars(pdHeight);
         }
         
         hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "CONDITIONAL");
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Changes the size of a SmartDataBrowser.<br>
    * Parameters:  pd_height AS DECIMAL  The desired height (in rows)<br>
    *              pd_width  AS DECIMAL  The desired width (in columns)
    */
   public void resizeObject(final decimal _pdHeight, final decimal _pdWidth)
   {
      decimal pdHeight = TypeFactory.initInput(_pdHeight);
      decimal pdWidth = TypeFactory.initInput(_pdWidth);
      decimal dSearchSize = TypeFactory.decimal();
      handle hBrowse = TypeFactory.handle();
      handle hFieldGroup = TypeFactory.handle();
      handle hFrame = TypeFactory.handle();
      handle hSearchField = TypeFactory.handle();
      handle hSearchLabel = TypeFactory.handle();
      handle htmpWidget = TypeFactory.handle();
      logical otherWidget = TypeFactory.logical();
      /* 1st Child   */
      /* Fix for issue 5633 - the browse widget loses the current */
      /* query record when certain sizing attributes change (EXPANDABLE or HEIGHT). */
      /* So, we need to save the current Rowid, if any, and re-find it later */
      handle hRo = TypeFactory.handle();
      rowid rCurRowid = TypeFactory.rowid();
      
      internalProcedure(new Block((Body) () -> 
      {
         pdHeight.assign(decimal.maximum(pdHeight, new decimal(2.0)));
         pdWidth.assign(decimal.maximum(pdWidth, new decimal(2.0)));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hFieldGroup.assign(hBrowse.unwrapWidget().getParentHandle());
          /* Field Group */hFrame.assign(hFieldGroup.unwrapWidget().getParentHandle());
          /* The frame ! */htmpWidget.assign(hFieldGroup.unwrapTree().firstChild());
         silent(() -> hRo.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle().unwrapQuery().bufferHandle()));
         
         if (_and(hRo.isValid(), () -> hRo.unwrapBuffer().available()))
         {
            rCurRowid.assign(hRo.unwrapBuffer().rowID());
         }
         
         /******* issue 5633 (first part) ********/
         /* If the frame is NOT size-to-fit then we don't allow anything to
              be resized.  We can tell if size-to-fit has been turned off
              because SCROLLABLE is turned on.                                    */
         if ((hFrame.unwrapWidget().isScrollable()).booleanValue())
         {
            returnNormal();
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
         hSearchField.assign(ghProp.unwrapBufferField().value());
         
         if ((hSearchField.isValid()).booleanValue())
         {
            hSearchLabel.assign(hSearchField.unwrapWidget().getSideLabelHandle());
            dSearchSize.assign(plus(hSearchField.unwrapWidget().getColumn(), hSearchField.unwrapSizeable().getWidthChars()));
         }
         
         LogicalOp whileClause0 = () -> htmpWidget.isValid();
         
         repeatWhile("searchForSiblings", whileClause0, new Block((Body) () -> 
         {
            if (_isNotEqual(htmpWidget, hBrowse))
            {
               if (_or(or(or(isNotEqual(htmpWidget.unwrapType().getResourceType(), "BUTTON"), () -> isNotEqual(htmpWidget.unwrapXY().getX(), 4)), () -> isNotEqual(htmpWidget.unwrapXY().getY(), 4)), () -> isNotEqual(htmpWidget.unwrapWidget().getTooltip(), "SmartObject Options")))
               {
                  if (_and(isNotEqual(htmpWidget, hSearchLabel), () -> isNotEqual(htmpWidget, hSearchField)))
                  {
                     otherWidget.assign(true);
                     leave("searchForSiblings");
                  }
               }
            }
            htmpWidget.assign(htmpWidget.unwrap().getNextSibling());
         }));
         
         /* While looking for siblings */
         /* If the width is getting smaller, do the browse first else the frame */
         if (_isLessThan(pdWidth, hBrowse.unwrapSizeable().getWidthChars()))
         {
            silent(() -> 
            {
               if (_not(otherWidget))
               {
                  hBrowse.unwrapSizeable().setWidthChars(minus(pdWidth, minus(hBrowse.unwrapWidget().getColumn(), 1)));
               }
               
               hFrame.unwrapSizeable().setWidthChars(decimal.maximum(pdWidth, dSearchSize));
            });
         }
         else
         {
            silent(() -> 
            {
               hFrame.unwrapSizeable().setWidthChars(decimal.maximum(pdWidth, dSearchSize));
               
               if (_not(otherWidget))
               {
                  hBrowse.unwrapSizeable().setWidthChars(minus(pdWidth, minus(hBrowse.unwrapWidget().getColumn(), 1)));
               }
            });
         }
         silent(() -> 
         {
            if (_not(otherWidget))
            {
               hBrowse.unwrapSizeable().setHeightChars(minus(pdHeight, minus(hBrowse.unwrapWidget().getRow(), 1)));
            }
         });
         
         /* Error 6422 is given because the browse requires minimum 2 rows in viewport.*/
         if (_and(ErrorManager.isError(), () -> isEqual(ErrorManager.getErrorNumber(1), 6422)))
         {
            /* Make the browse as low as allowed */
            hBrowse.unwrapDown().setDown(new integer(2));
            pdHeight.assign(plus(hBrowse.unwrapSizeable().getHeightChars(), minus(hBrowse.unwrapWidget().getRow(), 1)));
         }
         
         silent(() -> hFrame.unwrapSizeable().setHeightChars(pdHeight));
         hBrowse.unwrapBrowse().setRepositionedRow(hBrowse.unwrapDown().getDown(), "CONDITIONAL");
         
         /* Fix for issue 5633 (second part) */
         if ((hRo.isValid()).booleanValue())
         {
            if (_notUnknown(rCurRowid))
            {
               if (_isNotEqual(rCurRowid, hRo.unwrapBuffer().rowID()))
               {
                   /* IF a valid buffer existed in the beginning... */hRo.unwrapBuffer().findByRowID(rCurRowid, LockType.NONE);
               }
            }
         }
         /* ...attempt to find saved */
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "adjustVisibleRowids", targetProcedure());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Event Handler for the ROW-DISPLAY event.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:
    */
   public void rowDisplay()
   {
      handle hBuffer = TypeFactory.handle();
      character cRowids = TypeFactory.character();
      logical lReset = TypeFactory.logical();
      logical lScrollRemote = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ScrollRemote"));
         lScrollRemote.assign(ghProp.unwrapBufferField().value());
         
         if ((lScrollRemote).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowObject"));
            hBuffer.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowids"));
            cRowids.assign(ghProp.unwrapBufferField().value());
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowReset"));
            lReset.assign(ghProp.unwrapBufferField().value());
            
            /* If the handle to the query buffer has not yet been set then reurn */
            if (_not(hBuffer.isValid()))
            {
               returnNormal();
            }
            
            /* If the count is zero, clear the rowid list */
            if ((lReset).booleanValue())
            {
               cRowids.assign(fill(",", minus(numEntries(cRowids), 1)));
            }
            
            /* If the ROWID is not unknown, add it to the list of ROWIDs 
                   (we remove the first entry to ensure the list doesn't grow out
                   of proportions, since we cannot access the DOWN attribute in this
                   trigger, the number of entries in the list is adjusted in 
                   adjustVisibleRowids called from resizeObject or viewObject)*/
            if (_notUnknown(hBuffer.unwrapBuffer().rowID()))
            {
               cRowids.replaceEntry(1, "");
               cRowids.assign(substring(cRowids, 2));
               cRowids.assign(concat(cRowids, (_isEqual(cRowids, "") ? new character("") : new character(",")), valueOf(hBuffer.unwrapBuffer().rowID())));
            }
            
            /* Increment the row counter */
            lReset.assign(false);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowReset"));
            ghProp.unwrapBufferField().changeValue(lReset);
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowids"));
            ghProp.unwrapBufferField().changeValue(cRowids);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     This is the procedure run from the trigger on the SearchField<br>
    *              for the SmartDataBrowser, if one has been defined. It repositions <br>
    *              the query to the first row GE the value requested <br>
    *              (SearchValue property).<br>
    * Parameters:  &lt;none&gt;
    */
   public void searchTrigger()
   {
      handle hSource = TypeFactory.handle();
      character cSearchField = TypeFactory.character();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* First we must capture the keystroke which fired the trigger. */
         apply(lastKey());
         /* {get Prop var [handle]} */ 
         hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchField"));
         cSearchField.assign(ghProp.unwrapBufferField().value());
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "findRowWhere", hSource, "III", cSearchField, self().unwrapWidget().getScreenValue(), ">=");
         /* Now we must throw away the krystroke event to avoid getting double
              characters in the fill-in. */
         returnConsume();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Sets the down attribute for the browse and resizes the browse <br>
    *              appropriately.<br>
    * Parameters:  piNumDown AS INTEGER<br>
    * Notes:       Called from initializeObject for dynamic SmartDataBrowsers.
    */
   public void setDown(final integer _piNumDown_1)
   {
      integer piNumDown_1 = TypeFactory.initInput(_piNumDown_1);
      handle hBrowse = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hBrowse.unwrapWidget().setVisible(new logical(true));
         /* Added 3 as 2 has been reported to cause problems ???? */
         ControlFlowOps.invokeInWithMode("resizeObject", targetProcedure(), "II", plus(multiply(piNumDown_1, plus(hBrowse.unwrapBrowse().getRowHeightChars(), decimal.fromLiteral("0.2"))), 3), hBrowse.unwrapSizeable().getWidthChars());
         
         /* There is a core bug where if down is already set to the number we are trying to 
              set it to then setting down doesn't have any affect on height - which is what
              we need to have adjusted to get rid of any extra space at the bottom.  So when 
              down is equal to what we are about to set it to then we need to set it twice for
              it to have the correct affect. */
         if (_isEqual(hBrowse.unwrapDown().getDown(), piNumDown_1))
         {
            silent(() -> hBrowse.unwrapDown().setDown(new integer(minus(piNumDown_1, 1))));
         }
         
         silent(() -> hBrowse.unwrapDown().setDown(piNumDown_1));
         ControlFlowOps.invokeInWithMode("resizeObject", targetProcedure(), "II", plus(minus(hBrowse.unwrapWidget().getRow(), 1), hBrowse.unwrapSizeable().getHeightChars()), hBrowse.unwrapSizeable().getWidthChars());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Handles value-changed event for the sortable popup menu item<br>
    * Parameters:  <br>
    * Notes:       Movable and sortable are mutually exclusive
    */
   public void sortableValueChanged()
   {
      handle hMovableItem = TypeFactory.handle();
      handle hSortableItem = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MovableHandle"));
         hMovableItem.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SortableHandle"));
         hSortableItem.assign(ghProp.unwrapBufferField().value());
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setColumnsSortable", targetProcedure(), "I", hSortableItem.unwrapWidget().isChecked());
         
         if (_and(hSortableItem.unwrapWidget().isChecked(), () -> hMovableItem.unwrapWidget().isChecked()))
         {
            hMovableItem.unwrapWidget().setChecked(new logical(false));
            /* {set Prop value [handle]} */ 
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setColumnsMovable", targetProcedure(), "I", false);
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:    Start Search Trigger    <br>
    * Parameters: phBrowse - The Browse handle <br>
    *                        Persistent triggers have early binding so we cannot<br>
    *                        pass current-column.  <br>
    * Notes:
    */
   public void startSearch(final handle _phBrowse)
   {
      handle phBrowse = TypeFactory.initInput(_phBrowse);
      handle hColumn = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         hColumn.assign(phBrowse.unwrapBrowse().getCurrentColumn());
         /* {set Prop value [handle]} */ 
         ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setSORT", targetProcedure(), "I", hColumn.unwrap().name());
         apply(phBrowse, "END-SEARCH");
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Procedure to handle browse toolbar actions.<br>
    * Parameters:  pcValue AS CHARACTER - action<br>
    * Notes:
    */
   public void toolbar(final character _pcValue)
   {
      character pcValue = TypeFactory.initInput(_pcValue);
      handle hDataSource = TypeFactory.handle();
      logical lAutoCommit = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         switch (StringHelper.safeTrimTrailing((pcValue).toStringMessage()).toUpperCase())
         {
            case "VIEW":
            case "COPY":
            case "MODIFY":
            case "ADD":
               {
                  ControlFlowOps.invokeInWithMode("launchFolderWindow", targetProcedure(), "I", pcValue);
               }
               break;
            case "DELETE":
               {
                  /* {get Prop var [handle]} */ 
                  hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
                  
                  /* We currently use autocimmit on delete for the toolbar link as this is 
                         used from an objectContainer that does not have commit/undo actions*/
                  if ((hDataSource.isValid()).booleanValue())
                  {
                     /* {get Prop var [handle]} */
                     lAutoCommit.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getAutoCommit", hDataSource));
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setAutoCommit", hDataSource, "I", true);
                     ControlFlowOps.invokeIn("deleteRecord", targetProcedure());
                     /* {set Prop value [handle]} */
                     ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setAutoCommit", hDataSource, "I", lAutoCommit);
                  }
               }
               break;
            default:
               {
                  ControlFlowOps.runSuper("toolbar", pcValue);
               }
               break;
         }
         ErrorManager.setError(new logical(false));
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Saves/Deletes column positions and sizes for the profile data<br>
    * Parameters:  plDeleteProfile AS LOGICAL<br>
    * Notes:
    */
   public void updateColumnSettings(final logical _plDeleteProfile)
   {
      logical plDeleteProfile = TypeFactory.initInput(_plDeleteProfile);
      character cBrowseName = TypeFactory.character();
      character cColumnData = TypeFactory.character();
      character cContainerName = TypeFactory.character();
      character cProfileKey = TypeFactory.character();
      handle hBrowse = TypeFactory.handle();
      handle hColumn = TypeFactory.handle();
      handle hContainerSource = TypeFactory.handle();
      logical lProfileExists = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cBrowseName.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hContainerSource.assign(ghProp.unwrapBufferField().value());
         
         if ((hContainerSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LogicalObjectName"));
            cContainerName.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         if (_and(isEqual(cContainerName, ""), () -> hContainerSource.isValid()))
         {
            cContainerName.assign(hContainerSource.unwrapFile().getFileName());
         }
         
         cProfileKey.assign(concat(cContainerName, cBrowseName));
         cColumnData.assign("");
         hColumn.assign(hBrowse.unwrapBrowse().getFirstColumn());
         
         if (_not(plDeleteProfile))
         {
            loopLabel17:
            while ((hColumn.isValid()).booleanValue())
            {
               cColumnData.assign(concat(cColumnData, (_isGreaterThan(numEntries(cColumnData, chr(3)), 0) ? chr(3) : new character("")), hColumn.unwrap().name(), chr(4), valueOf(hColumn.unwrapSizeable().getWidthPixels())));
               hColumn.assign(hColumn.unwrapBrowseColumn().getNextColumn());
            }
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SavedColumnData"));
         ghProp.unwrapBufferField().changeValue(cColumnData);
         
         if ((gshProfileManager.isValid()).booleanValue())
         {
            if ((plDeleteProfile).booleanValue())
            {
               ControlFlowOps.invokeInWithMode("checkProfileDataExists", gshProfileManager, "IIIIIO", new character("Browser"), new character("Columns"), cProfileKey, new logical(false), new logical(true), lProfileExists);
            }
            
            /* If the profile exists and needs to be deleted then run setProfileData
                   to delete the profile OR if the profile does not need to be deleted
                   then run setProfileData to create/update profile */
            if (_or(and(plDeleteProfile, lProfileExists), () -> not(plDeleteProfile)))
            {
               ControlFlowOps.invokeInWithMode("setProfileData", gshProfileManager, "IIIIIII", new character("Browser"), new character("Columns"), cProfileKey, new unknown(), cColumnData, plDeleteProfile, new character("PER"));
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of updateRecord.<br>
    *              If the SmartDataBrowser is browsing a DataObject query and not <br>
    *              its own, this sets the RowIdent property to the RowObject <br>
    *              ROWID, because the DataObject will be looking for it to <br>
    *              identify which row is being updated. <br>
    *              Otherwise the RowIndent property is set to the ROWID(s)<br>
    *              of the database record(s).<br>
    * Parameters:  &lt;none&gt;
    */
   public void updateRecord()
   {
      handle hBrowse = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      handle hParent = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      integer iBuffer = TypeFactory.integer();
      character cRowIdent = TypeFactory.character("");
      character cNewRecord = TypeFactory.character();
      logical lQueryObj = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObj.assign(ghProp.unwrapBufferField().value());
         hQuery.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
         
         if (_not(lQueryObj))
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(1));
         }
         else
         {
            /* If it's a new record (Add/Copy) then RowIdent will be set correctly. */
            if (_isEqual(cNewRecord, "No"))
            {
               loopLabel18:
               for (iBuffer.assign(1); _isLessThanOrEqual(iBuffer, hQuery.unwrapQuery().numBuffers()); iBuffer.increment())
               {
                  hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iBuffer));
                  cRowIdent.assign(concat(cRowIdent, ",", (hBuffer.unwrapBuffer().available()).booleanValue() ? valueOf(hBuffer.unwrapBuffer().rowID()) : new character("")) /* Leave first element blank - no TT */);
               }
               
               /* END DO iBuffer */
               /* {set Prop value [handle]} */ 
               ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setRowIdent", targetProcedure(), "I", cRowIdent);
            }
 /* RowIdent is the db ROWIDs. */;
         }
         /* END DO QueryObject NE No */
         ControlFlowOps.runSuper("updateRecord");
         
         if (_isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR"))
         {
            if ((focus().isValid()).booleanValue())
            {
               hParent.assign(focus().unwrapWidget().getParentHandle());
            }
            
            if (_isNotEqual(hParent, hBrowse))
            {
               ControlFlowOps.invokeInWithMode("applyEntry", targetProcedure(), "I", new unknown());
            }
            
            returnNormal("ADM-ERROR");
         }
         else
         {
            if (_and(lQueryObj, () -> isNotEqual(cNewRecord, "No")))
            {
               /* {fn <func-name> [handle] */ 
               ControlFlowOps.invokeDynamicFunctionIn(logical.class, "openQuery", targetProcedure());
               apply(hBrowse, "END");
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Saves/Deletes column positions and sizes for the profile data<br>
    * Parameters:  plDeleteProfile AS LOGICAL<br>
    * Notes:
    */
   public void updateSortColumn(final logical _plDeleteProfile)
   {
      logical plDeleteProfile = TypeFactory.initInput(_plDeleteProfile);
      character cBrowseName = TypeFactory.character();
      character cContainerName = TypeFactory.character();
      character cProfileKey = TypeFactory.character();
      character cQuerySort = TypeFactory.character();
      handle hContainerSource = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      logical lProfileExists = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cBrowseName.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
         hContainerSource.assign(ghProp.unwrapBufferField().value());
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         if ((hContainerSource.isValid()).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("LogicalObjectName"));
            cContainerName.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         if (_and(isEqual(cContainerName, ""), () -> hContainerSource.isValid()))
         {
            cContainerName.assign(hContainerSource.unwrapFile().getFileName());
         }
         
         cProfileKey.assign(concat(cContainerName, cBrowseName));
         
         if (_not(plDeleteProfile))
         {
            /* {get Prop var [handle]} */
            cQuerySort.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQuerySort", hDataSource));
         }
         
         /* if not delete */
         if ((gshProfileManager.isValid()).booleanValue())
         {
            if ((plDeleteProfile).booleanValue())
            {
               ControlFlowOps.invokeInWithMode("checkProfileDataExists", gshProfileManager, "IIIIIO", new character("SDO"), new character("Sorting"), cProfileKey, new logical(false), new logical(true), lProfileExists);
            }
            
            /* If the profile exists and needs to be deleted then run setProfileData
                   to delete the profile OR if the profile does not need to be deleted
                   and the sort column is not blank then run setProfileData to 
                   create/update profile */
            if (_or(and(plDeleteProfile, lProfileExists), () -> and(not(plDeleteProfile), () -> isNotEqual(cQuerySort, ""))))
            {
               ControlFlowOps.invokeInWithMode("setProfileData", gshProfileManager, "IIIIIII", new character("SDO"), new character("Sorting"), cProfileKey, new unknown(), cQuerySort, plDeleteProfile, new character("PER"));
            }
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of updateState.<br>
    * Parameters:  pcState AS CHARACTER<br>
    *              Possible values are:<br>
    *              UpdateBegin      - Enables columns in the browse and passes<br>
    *                                 updateState to other objects<br>
    *              Update           - Disables the browse. It means that some other<br>
    *                                 object has just started an update<br>
    *              UpdateComplete   - Enables the browse<br>
    *    Notes:   See datavis.p
    */
   public void updateState(final character _pcState)
   {
      character pcState = TypeFactory.initInput(_pcState);
      handle hBrowse = TypeFactory.handle();
      handle hSearchField = TypeFactory.handle();
      logical lModified_1 = TypeFactory.logical();
      character cNewRecord = TypeFactory.character();
      logical lCanNavigate = TypeFactory.logical();
      handle hSource = TypeFactory.handle();
      handle hDataSource = TypeFactory.handle();
      handle hCurrSource = TypeFactory.handle();
      handle hRequester = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* If we are the publisher's (dataSource's) currentUpdateSource then we 
              actually are the original publisher of this event, so just return */
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         
         /* SBOs does run, so SOURCE is incorrect, so unless we know the 
              SOURCE is the dataSource check the special trick property   */
         if (_isNotEqual(hDataSource, sourceProcedure()))
         {
            silent(() -> hRequester /* {fn <func-name> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getTargetProcedure", sourceProcedure())));
         }
         else
         {
            hRequester.assign(sourceProcedure());
         }
         
         /* {get Prop var [handle]} */
         silent(() -> hCurrSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getCurrentUpdateSource", hRequester)));
         
         if (_isEqual(targetProcedure(), hCurrSource))
         {
            returnNormal();
         }
         
         switch (StringHelper.safeTrimTrailing((pcState).toStringMessage()).toUpperCase())
         {
            case "UPDATEBEGIN":
               {
                  ControlFlowOps.invokeIn("enableFields", targetProcedure());
                  /* In case enable fails for some reason.*/
                  NamedEventManager.publish("updateState", targetProcedure(), "I", new Object[]
                  {
                     _isEqual(ControlFlowOps.getReturnValue(), "ADM-ERROR") ? new character("UpdateComplete") : new character("Update")
                  });
               }
               break;
            case "UPDATE":
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
                  hBrowse.assign(ghProp.unwrapBufferField().value());
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
                  hSearchField.assign(ghProp.unwrapBufferField().value());
                  /* {get Prop var [handle]} */ 
                  lModified_1.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", targetProcedure()));
                  /* {get Prop var [handle]} */ 
                  cNewRecord.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getNewRecord", targetProcedure()));
                  
                  if (_and(not(lModified_1), () -> isEqual(cNewRecord, "no")))
                  {
                     if ((hSearchField.isValid()).booleanValue())
                     {
                        hSearchField.unwrapSensitive().setSensitive(new logical(false));
                     }
                     
                     hBrowse.unwrapSensitive().setSensitive(new logical(false));
                  }
               }
               break;
            case "UPDATECOMPLETE":
               {
                  /* We might get this as a run from an SBO, in which case the 
                           TargetProcedure has been set in order to identify the source */
                  /* {get Prop var [handle]} */
                  silent(() -> hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getTargetProcedure", sourceProcedure())));
                  
                  if (_not(hSource.isValid()))
                  {
                     hSource.assign(sourceProcedure());
                  }
                  
                  /* canNavigate returns false if updates prevents us from navigating.
                           We must set the gh- var so the SBO can identify the real target
                           from the source-procedure. */
                  lCanNavigate.setUnknown();
                  ghTargetProcedure.assign(targetProcedure());
                  silent(() -> lCanNavigate /* {fn <func-name> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "canNavigate", hSource)));
                  ghTargetProcedure.setUnknown();
                  
                  /* Active denials only (unknown is yes) */
                  if (_not(isEqual(lCanNavigate, false)))
                  {
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
                     hBrowse.assign(ghProp.unwrapBufferField().value());
                     ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
                     hSearchField.assign(ghProp.unwrapBufferField().value());
                     
                     if ((hSearchField.isValid()).booleanValue())
                     {
                        hSearchField.unwrapSensitive().setSensitive(new logical(true));
                     }
                     
                     hBrowse.unwrapSensitive().setSensitive(new logical(true));
                  }
               }
               break;
         }
         ControlFlowOps.runSuper("updateState", pcState);
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     SmartDataBrowser version of updateTitle.<br>
    * Notes:       Setting up window title for a object controller is different than<br>
    *              for a viewer. This code deals with specifically getting the info <br>
    *              from the broswer's data-source's.
    */
   public void updateTitle()
   {
      handle hSource = TypeFactory.handle();
      character cWindowTitleField = TypeFactory.character();
      character cWindowTitleColumn = TypeFactory.character();
      character cWindowTitleValue = TypeFactory.character();
      character cWindowName = TypeFactory.character();
      character cWindowTitle = TypeFactory.character();
      handle hContainerHandle = TypeFactory.handle();
      handle hSourceSource = TypeFactory.handle();
      handle hContainerSource = TypeFactory.handle();
      
      internalProcedure(new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("WindowTitleField"));
         cWindowTitleField.assign(ghProp.unwrapBufferField().value());
         
         if ((hSource.isValid()).booleanValue())
         {
            /* {get Prop var [handle]} */
            hSourceSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", hSource));
         }
         
         if (_and(isNotEqual(cWindowTitleField, ""), () -> hSourceSource.isValid()))
         {
            cWindowTitleColumn.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(character.class, "colValues", hSourceSource, "I", cWindowTitleField));
            
            if (_isEqual(numEntries(cWindowTitleColumn, chr(1)), 2))
            {
               cWindowTitleValue.assign(entry(2, cWindowTitleColumn, chr(1)));
               
               if (_notUnknown(cWindowTitleValue))
               {
                  ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
                  ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerSource"));
                  hContainerSource.assign(ghProp.unwrapBufferField().value());
                  
                  if ((hContainerSource.isValid()).booleanValue())
                  {
                     /* {get Prop var [handle]} */
                     cWindowName.assign(ControlFlowOps.invokeDynamicFunctionIn("getWindowName", hContainerSource));
                     ghProp.assign(handle.fromString(entry(1, hContainerSource.unwrapADMData().getADMData(), chr(1))));
                     ghProp.assign(ghProp.unwrapBuffer().bufferField("ContainerHandle"));
                     hContainerHandle.assign(ghProp.unwrapBufferField().value());
                  }
                  
                  if (_isEqual(indexOf(cWindowName, " - "), 0))
                  {
                     cWindowTitle.assign(cWindowName);
                  }
                  else
                  {
                     cWindowTitle.assign(substring(cWindowName, 1, minus(indexOf(cWindowName, " - "), 1)));
                  }
                  
                  cWindowTitle.assign(concat(cWindowTitle, " - ", cWindowTitleValue));
                  
                  if ((hContainerHandle.isValid()).booleanValue())
                  {
                     hContainerHandle.unwrapWidget().setTitle(cWindowTitle);
                  }
               }
            }
         }
         returnNormal();
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    *    Purpose: Event handler for VALUE-CHANGED of browse anywhere event.<br>
    * Parameters: &lt;none&gt; <br>
    *      Notes: Sets DataModified, which is the all important flag that tells <br>
    *             that data need to be saved.<br>
    *         -   This is NOT the VALUE-CHANGED event handler of the browse. Its <br>
    *             event handler is onValueChanged. <br>
    *             They have nothing in common as the browse's event triggers from <br>
    *             a user change of record while this event is for changes to <br>
    *             values in columns.
    */
   public void valueChanged()
   {
      character cDisplayedFields = TypeFactory.character();
      logical lEnabled = TypeFactory.logical();
      logical lModified_1 = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DisplayedFields"));
         cDisplayedFields.assign(ghProp.unwrapBufferField().value());
         
         /* Ignore the event if it wasn't a browse field. */
         if (_isNotEqual(lookup(focus().unwrap().name(), cDisplayedFields), 0))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("FieldsEnabled"));
            lEnabled.assign(ghProp.unwrapBufferField().value());
            
            if ((lEnabled).booleanValue())
            {
               /* {get Prop var [handle]} */ 
               lModified_1.assign(ControlFlowOps.invokeDynamicFunctionIn(logical.class, "getDataModified", targetProcedure()));
               
               if (_not(lModified_1))
               {
                  /* {set Prop value [handle]} */ 
                   /* Don't send the updateState event more than once. */ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "setDataModified", targetProcedure(), "I", true);
               }
            }
 /* Only if browse enabled for input. */;
         }
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * Purpose:     Override to run setDown to set the DOWN attribute for the browse <br>
    *              when it is being viewed and to fix problems caused by the browse's<br>
    *              implicit visualization behavior.<br>
    * Parameters:  &lt;none&gt;<br>
    * Notes:     - setDown must be called from viewObject as opposed to <br>
    *              initializeObject to support the object being on a page other<br>
    *              than zero, because set down requires a visible browse. <br>
    *              This sets DOWN when the object is actually being viewed rather <br>
    *              than when it is initialized which could happen before it is <br>
    *              viewed if it is on a page other than zero.  <br>
    *            - The browser implicit sets the buffer to its currently selected <br>
    *              row when it is visualized. This may cause problems if the <br>
    *              DataSource is being manipulated by another object; An <br>
    *              update-target may have called AddRow in the DataSource, which <br>
    *              creates a RowObject while the browse still has the old row<br>
    *              selected.
    */
   public void viewObject()
   {
      integer iNumDown = TypeFactory.integer();
      rowid rRowid = TypeFactory.rowid();
      handle hDataSource = TypeFactory.handle();
      handle hDataHandle = TypeFactory.handle();
      handle hRowObject = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      logical lTargetNotSet = TypeFactory.logical();
      
      internalProcedure(new Block((Body) () -> 
      {
         lTargetNotSet.assign(isNotEqual(ghTargetProcedure, targetProcedure()));
         ghTargetProcedure.assign(targetProcedure());
         /* {get Prop var [handle]} */ 
         hDataSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((lTargetNotSet).booleanValue())
         {
            ghTargetProcedure.setUnknown();
         }
         
         /* Log the DataSource's current rowid for comparison further down. 
              Implicit browser behavior when viewing a hidden browse may change it  */
         if ((hDataSource.isValid()).booleanValue())
         {
            /* Use datahandle to ensure that SBOs returns correct info. */
            /* {get Prop var [handle]} */
            silent(() -> hDataHandle.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataHandle", hDataSource)));
            
            if ((hDataHandle.isValid()).booleanValue())
            {
               hRowObject.assign(hDataHandle.unwrapQuery().bufferHandle(1));
               rRowid.assign(hRowObject.unwrapBuffer().rowID());
            }
         }
         /* valid datasource */
         ControlFlowOps.runSuper("viewObject");
         
         /* Reset buffer in case the visualization changed it */
         if (_and(and(and(hDataSource.isValid(), () -> hRowObject.isValid()), () -> notUnknown(rRowid)), () -> isNotEqual(rRowid, hRowObject.unwrapBuffer().rowID())))
         {
            silent(() -> hRowObject.unwrapBuffer().findByRowID(rRowid));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NumDown"));
         iNumDown.assign(ghProp.unwrapBufferField().value());
         
         if (_isGreaterThan(iNumDown, 0))
         {
            ControlFlowOps.invokeInWithMode("setDown", targetProcedure(), "I", iNumDown);
         }
         
         /* {fn <func-name> [handle] */ 
         ControlFlowOps.invokeDynamicFunctionIn(logical.class, "adjustVisibleRowids", targetProcedure());
      }));
   }

   /** _UIB-CODE-BLOCK-END */
   /**
    * ***********************  Function Implementations****************
    */
   public logical adjustVisibleRowids()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Adjusts the number of entries in the visibleRowids to the number of 
                  records in the browse. 
          Notes:  The visibleRowids is used on scroll-notify when ScrollRemote
                  is true. It stores the list of the rowids in the viewport. The list
                  is updated in the rowDisplay procedure fired on the ROW-DISPLAY event   
                - The logic here is to avoid having rowDisplay adding to the list 
                  and exceeding Progress limitations when not cleared. Since DOWN 
                  cannot be referenced in ROW-DISPLAY this logic does the counting 
                  and adjusts the number of entries in the property while RowDisplay 
                  always removes an entry when it adds an entry.
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      character cRowids = TypeFactory.character();
      character cRowList = TypeFactory.character();
      integer iEntries = TypeFactory.integer();
      integer iEntry = TypeFactory.integer();
      
      return function("adjustVisibleRowids", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         iEntries.assign(hBrowse.unwrapDown().getDown());
         
         if (_isGreaterThan(iEntries, 0))
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowids"));
            cRowids.assign(ghProp.unwrapBufferField().value());
            
            /* increase */
            if (_isLessThan(numEntries(cRowids), iEntries))
            {
               cRowList.assign(concat(fill(",", minus(minus(iEntries, numEntries(cRowids)), 1)), cRowids));
            }
            else
            {
               if (_isGreaterThan(numEntries(cRowids), iEntries))
               {
                  loopLabel19:
                  for (iEntry.assign(1); _isLessThanOrEqual(iEntry, iEntries); iEntry.increment())
                  {
                     cRowList.assign(concat(entry(plus(minus(numEntries(cRowids), iEntry), 1), cRowids), (_isEqual(cRowList, "") ? new character("") : new character(",")), cRowList));
                  }
               }
            }
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowids"));
            ghProp.unwrapBufferField().changeValue(cRowList);
         }
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character colValues(final character _pcViewColList_1)
   {
      character pcViewColList_1 = TypeFactory.initInput(_pcViewColList_1);
      /*------------------------------------------------------------------------------
        Purpose:  Formats a row of formatted values from the current row of the
                  SmartDataBrowser query for the specified column list.
         Params:  pcViewColList AS CHARACTER -- comma-separated list of 
                  requested columns.
          Notes:  Only applies for a SmartDataBrowser with its own query.
                  Passes back a CHR(1) delimited list with the RowIdent as the 
                  first value in all cases and fields values 
                  (in the order they are requested).
      ------------------------------------------------------------------------------*/
      character cColValues = TypeFactory.character("");
      integer iCol = TypeFactory.integer();
      handle hColumn = TypeFactory.handle();
      integer iColCount = TypeFactory.integer();
      handle hQuery = TypeFactory.handle();
      handle hBuffer = TypeFactory.handle();
      integer iTable = TypeFactory.integer();
      integer iArray = TypeFactory.integer();
      character cName = TypeFactory.character();
      character cRowIdent = TypeFactory.character();
      
      return function("colValues", character.class, new Block((Body) () -> 
      {
         iColCount.assign(numEntries(pcViewColList_1));
         /* NOTE: This function is valid only if the Browser has its own db query,
              in which case it will include the Query super procedure and this will
              be defined. Otherwise we don't do anything with this request. */
         silent(() -> hQuery.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getQueryHandle", targetProcedure())));
         
         if (_not(hQuery.isValid()))
         {
            returnNormal(new character());
         }
         
         loopLabel20:
         for (iTable.assign(1); _isLessThanOrEqual(iTable, hQuery.unwrapQuery().numBuffers()); iTable.increment())
         {
            hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iTable));
            cRowIdent.assign(concat(cRowIdent, ",", valueOf(hBuffer.unwrapBuffer().rowID())));
         }
         
         cColValues.assign(concat(cColValues, cRowIdent, chr(1)));
         
         ToClause toClause0 = new ToClause(iCol, 1, iColCount);
         
         doTo("iColLoop", toClause0, new Block((Body) () -> 
         {
            loopLabel21:
            for (iTable.assign(1); _isLessThanOrEqual(iTable, hQuery.unwrapQuery().numBuffers()); iTable.increment())
            {
               cName.assign(entry(iCol, pcViewColList_1));
               iArray.assign(_isGreaterThan(numEntries(cName, "["), 1) ? new integer(entry(1, entry(2, cName, "["), "]")) : new integer(0));
               cName.assign(entry(1, cName, "["));
               hBuffer.assign(hQuery.unwrapQuery().bufferHandle(iTable));
               silent(() -> hColumn.assign(hBuffer.unwrapBuffer().bufferField(cName)));
               
               if ((hColumn.isValid()).booleanValue())
               {
                  cColValues.assign(concat(cColValues, rightTrim(valueOf(hColumn.unwrapBufferField().value(iArray))), (_isNotEqual(iCol, iColCount) ? chr(1) : new character(""))));
                  next("iColLoop");
               }
            }
            
            /* END DO iTable      */
            ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "showMessage", targetProcedure(), "I", concat("Column ", entry(iCol, pcViewColList_1), "not found in Browser."));
            returnNormal(new character());
         }));
         
         /* END DO iCol        */
         returnNormal(cColValues);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getActionEvent()
   {
      /*------------------------------------------------------------------------------
        Purpose: return the event to publish on Default-action of the browse. 
      Parameter: 
          Notes: setActionEvent(pcEvent) defines the persistent trigger that runs 
                 defaultAction and also subscribes the source-procedure.  
      ------------------------------------------------------------------------------*/
      character cActionEvent = TypeFactory.character();
      
      return function("getActionEvent", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ActionEvent"));
         cActionEvent.assign(ghProp.unwrapBufferField().value());
         returnNormal(cActionEvent);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getApplyActionOnExit()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns TRUE if exit is suppoesed to select the row in the browse.
          Notes: Currently used by the SmartSelect.
      ------------------------------------------------------------------------------*/
      logical lApply = TypeFactory.logical();
      
      return function("getApplyActionOnExit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyActionOnExit"));
         lApply.assign(ghProp.unwrapBufferField().value());
         returnNormal(lApply);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getApplyExitOnAction()
   {
      /*------------------------------------------------------------------------------
        Purpose: Returns TRUE if DEFAULT-ACTION is to exit the browse.
          Notes: Currently used by the SmartSelect.
                 The logic is not performed in the trigger, but in the defaultAction
                 procedure that gets defined as a persitent DEFAULT-ACTION event
                 when setActionEvent is defined. 
                 Local DEFAULT-ACTION events could be set up to run defaultAction.            
      ------------------------------------------------------------------------------*/
      logical lApply = TypeFactory.logical();
      
      return function("getApplyExitOnAction", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyExitOnAction"));
         lApply.assign(ghProp.unwrapBufferField().value());
         returnNormal(lApply);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getBgcolor()
   {
      /*------------------------------------------------------------------------------
        Purpose:  returns the Background color of the browse
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iBgcolor = TypeFactory.integer((Long) null);
      handle hBrowse = TypeFactory.handle();
      
      return function("getBGColor", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            iBgcolor.assign(hBrowse.unwrapWidget().getBgColor());
         }
         
         returnNormal(iBgcolor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getBrowseHandle()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the browse control.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      
      return function("getBrowseHandle", handle.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
            hBrowse.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(hBrowse);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getCalcWidth()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns logical value that determines whether the width of the browse
                  is calculated to the exact width it should be for the fields it contains
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lCalcWidth = TypeFactory.logical();
      
      return function("getCalcWidth", logical.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("CalcWidth"));
            lCalcWidth.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(lCalcWidth);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getColumnsMovable()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns logical value that determines whether the browser's columns
                  are movable
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lMovable = TypeFactory.logical();
      
      return function("getColumnsMovable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColumnsMovable"));
         lMovable.assign(ghProp.unwrapBufferField().value());
         returnNormal(lMovable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getColumnsSortable()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns logical value that determines whether the browser's columns
                  are sortable
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lSortable = TypeFactory.logical();
      
      return function("getColumnsSortable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColumnsSortable"));
         lSortable.assign(ghProp.unwrapBufferField().value());
         returnNormal(lSortable);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDataSignature()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns a character string value which is a list of integers 
                  corresponding to the datatypes of the fields in the object 
                  Temp-Table, for use in comparing objects for equivalence.
         Params:  <none>
          Notes:  A SmartDataObject and a SmartDataBrowser which have the same 
                  DataSignature will be compatible; the SDO's query can be used 
                  by the SmartDataBrowser. The integer values are the same codes 
                  used in the _dtype field in the schema. 
                  In the case where the SmartDataBrowser has its own db query, 
                  this function will return Unknown.
                  Used internally.
      ------------------------------------------------------------------------------*/
      character cSignature = TypeFactory.character("");
      integer iCol = TypeFactory.integer();
      handle hRowObject = TypeFactory.handle();
      character cDataType = TypeFactory.character();
      handle hColumn = TypeFactory.handle();
      logical lQueryObj = TypeFactory.logical();
      
      return function("getDataSignature", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryObject"));
         lQueryObj.assign(ghProp.unwrapBufferField().value());
         
         if ((lQueryObj).booleanValue())
         {
            returnNormal(new character());
         }
         
         /* {get Prop var [handle]} */ 
         silent(() -> hRowObject.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getRowObject", targetProcedure())));
         
         if ((hRowObject.isValid()).booleanValue())
         {
            loopLabel22:
            for (iCol.assign(1); _isLessThanOrEqual(iCol, hRowObject.unwrapBuffer().numFields()); iCol.increment())
            {
               hColumn.assign(hRowObject.unwrapBuffer().bufferField(iCol));
               cDataType.assign(hColumn.unwrapCommonField().getDataType());
               /* Note: Float/Double reserved for possible future use. */
               cSignature.assign(concat(cSignature, (_isEqual(cDataType, "CHARACTER") ? new character("1") : _isEqual(cDataType, "DATE") ? new character("2") : _isEqual(cDataType, "LOGICAL") ? new character("3") : _isEqual(cDataType, "INTEGER") ? new character("4") : _isEqual(cDataType, "DECIMAL") ? new character("5") : _or(isEqual(cDataType, "FLOAT"), () -> isEqual(cDataType, "DOUBLE")) ? new character("6") : _isEqual(cDataType, "RECID") ? new character("7") : _isEqual(cDataType, "RAW") ? new character("8") : _isEqual(cDataType, "ROWID") ? new character("9") : new character("0"))));
            }
         }
         else
         {
            cSignature.setUnknown();
         }
         
         /* No RowObject means dynamic browse. */
         returnNormal(cSignature);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getDefaultColumnData()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the list of default column data
         Params:  <none>
          Notes:  Format is column1name CHR(4) width CHR(3) column2name CHR(4) width
      ------------------------------------------------------------------------------*/
      character cData = TypeFactory.character();
      
      return function("getDefaultColumnData", character.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultColumnData"));
            cData.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(cData);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getFetchOnReposToEnd()
   {
      /*------------------------------------------------------------------------------
        Purpose: returns true if the browse should fetch more data from the server to 
                 fill the batch when repositioing to the end of a batch
          Notes: This gives the correct visual appearance, but will require an 
                 additional request to the server.
      ------------------------------------------------------------------------------*/
      logical lFetchOnRepos = TypeFactory.logical();
      
      return function("getFetchOnReposToEnd", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchOnReposToEnd"));
         lFetchOnRepos.assign(ghProp.unwrapBufferField().value());
         returnNormal(lFetchOnRepos);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getFgcolor()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Retrieves the Foreground Color of the browse
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iFgcolor = TypeFactory.integer((Long) null);
      handle hBrowse = TypeFactory.handle();
      
      return function("getFGColor", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            iFgcolor.assign(hBrowse.unwrapWidget().getFgColor());
         }
         
         returnNormal(iFgcolor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getFolderWindowToLaunch()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      character cTemp = UndoableFactory.character();
      
      return function("getFolderWindowToLaunch", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FolderWindowToLaunch"));
         cTemp.assign(ghProp.unwrapBufferField().value());
         returnNormal(cTemp);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public decimal getMaxWidth()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns maximum value used for setting the width of the browse when
                  CalcWidth is TRUE.  
          Notes:  
      ------------------------------------------------------------------------------*/
      decimal dMaxWidth = TypeFactory.decimal();
      
      return function("getMaxWidth", decimal.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("MaxWidth"));
            dMaxWidth.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(dMaxWidth);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getMovableHandle()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the movable popup menu item.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hMovableItem = TypeFactory.handle();
      
      return function("getMovableHandle", handle.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("MovableHandle"));
            hMovableItem.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(hMovableItem);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getNumDown()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the number of rows that are displayed DOWN in the browse   
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iNumDown = TypeFactory.integer();
      
      return function("getNumDown", integer.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("NumDown"));
            iNumDown.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(iNumDown);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getPopupActive()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns logical value that determines whether the browser popup
                  menu is active
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lActive = TypeFactory.logical();
      
      return function("getPopupActive", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PopupActive"));
         lActive.assign(ghProp.unwrapBufferField().value());
         returnNormal(lActive);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getQueryRowObject()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the RowObject Temp-Table associated with the
                  Browse's Query.
          Notes:  Note that this function cannot calculate this at run time. This 
                  function is called from within a ROW-DISPLAY event and that 
                  precludes us from getting access to the BROWSE handle here.
                  Thus, setQueryRowObject sets the value of this handle.
      ------------------------------------------------------------------------------*/
      handle hQueryRowObject = TypeFactory.handle();
      
      return function("getQueryRowObject", handle.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowObject"));
         hQueryRowObject.assign(ghProp.unwrapBufferField().value());
         returnNormal(hQueryRowObject);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getSavedColumnData()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the list of saved column data
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cData = TypeFactory.character();
      
      return function("getSavedColumnData", character.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("SavedColumnData"));
            cData.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(cData);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getScrollRemote()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Gets the value of the ScrollRemote attribute  
          Notes:  
      ------------------------------------------------------------------------------*/
      logical lScrollRemote = TypeFactory.logical();
      
      return function("getScrollRemote", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ScrollRemote"));
         lScrollRemote.assign(ghProp.unwrapBufferField().value());
         returnNormal(lScrollRemote);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getSearchField()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the name of a field on which searching is enabled; if
                  set, space is allocated for a fill-in to accept a value to be
                  repositioned to.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cField = TypeFactory.character();
      
      return function("getSearchField", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchField"));
         cField.assign(ghProp.unwrapBufferField().value());
         returnNormal(cField);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public integer getSeparatorFgcolor()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      integer iSeparatorFgcolor = TypeFactory.integer((Long) null);
      handle hBrowse = TypeFactory.handle();
      
      return function("getSeparatorFGColor", integer.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            silent(() -> iSeparatorFgcolor.assign(hBrowse.unwrapBrowse().getSeparatorFgColor()));
         }
         
         returnNormal(iSeparatorFgcolor);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getSeparators()
   {
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      logical lInitialized = TypeFactory.logical();
      logical lSeparators = TypeFactory.logical();
      
      return function("getSeparators", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         if ((lInitialized).booleanValue())
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
            hBrowse.assign(ghProp.unwrapBufferField().value());
            lSeparators.assign(hBrowse.unwrapBrowse().isSeparators());
         }
         else
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("Separators"));
            lSeparators.assign(ghProp.unwrapBufferField().value());
         }
         /* {get Prop var [handle]} */ 
         
         returnNormal(lSeparators);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getSortableHandle()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the handle of the sortable popup menu item.
         Params:  <none>
      ------------------------------------------------------------------------------*/
      handle hSortableItem = TypeFactory.handle();
      
      return function("getSortableHandle", handle.class, new Block((Body) () -> 
      {
         silent(() -> 
         {
            ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
            ghProp.assign(ghProp.unwrapBuffer().bufferField("SortableHandle"));
            hSortableItem.assign(ghProp.unwrapBufferField().value());
         });
         returnNormal(hSortableItem);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public handle getTargetProcedure()
   {
      return function("getTargetProcedure", handle.class, new Block((Body) () -> 
      {
         /*------------------------------------------------------------------------------
           Purpose:  Returns the value of the super procedure's current TARGET-PROCEDURE,
                     when this is needed by a routine called from the super proc.
              
         ------------------------------------------------------------------------------*/
         returnNormal(ghTargetProcedure);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getTooltip()
   {
      /*------------------------------------------------------------------------------
        Purpose:  Returns the browser's tooltip
         Params:  <none>
      ------------------------------------------------------------------------------*/
      character cTooltip = TypeFactory.character();
      
      return function("getTooltip", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TOOLTIP"));
         cTooltip.assign(ghProp.unwrapBufferField().value());
         returnNormal(cTooltip);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character getVisibleRowids()
   {
      /*------------------------------------------------------------------------------
        Purpose:  get method for VisibleRowids property
          
      ------------------------------------------------------------------------------*/
      character cVisible = TypeFactory.character();
      
      return function("getVisibleRowids", character.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowids"));
         cVisible.assign(ghProp.unwrapBufferField().value());
         returnNormal(cVisible);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical getVisibleRowReset()
   {
      /*------------------------------------------------------------------------------
        Purpose:  get method for VisibleRowReset property
          
      ------------------------------------------------------------------------------*/
      logical lReset = TypeFactory.logical();
      
      return function("getVisibleRowReset", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowReset"));
         lReset.assign(ghProp.unwrapBufferField().value());
         returnNormal(lReset);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical isOkToFetch(final character _pcRequestedBatch_1)
   {
      character pcRequestedBatch_1 = TypeFactory.initInput(_pcRequestedBatch_1);
      /*------------------------------------------------------------------------------
        Purpose:  Indicates whether or not the current Data Object row has changed or
                  is not available. 
                  Input Parameter "pcRequestedBatch".
                     Values: FIRST, LAST are the only values that cause different results.
          Notes:  It is used by the Browser event procedures (on***) to determine if data
                  needs to be fetched depending on current batch state.
      ------------------------------------------------------------------------------*/
      character cRowident = TypeFactory.character();
      handle hRowObject = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      handle hBrowse = TypeFactory.handle();
      handle hSource = TypeFactory.handle();
      character cQueryPos = TypeFactory.character();
      logical lok = TypeFactory.logical();
      character cSourceName = TypeFactory.character();
      character cObjectName = TypeFactory.character();
      
      return function("isOkToFetch", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         hQuery.assign(hBrowse.unwrapQueryAssociable().getQueryAsHandle());
         hRowObject.assign(hQuery.unwrapQuery().bufferHandle(1));
         lok.assign(notUnknown(hRowObject.unwrapBuffer().rowID()));
         
         /* if no row is available, we don't need to fetch anything */
         if ((lok).booleanValue())
         {
            /* {get Prop var [handle]} */ 
            cRowident.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getRowIdent", targetProcedure()));
            
            /* if the batch request is for First or Last, make sure that are other records*/
            /* available for fetch and that the corresponding record (First or Last) is not */
            /* in the current batch */
            if (_isGreaterThan(lookup(pcRequestedBatch_1, "FIRST,LAST"), 0))
            {
               /* {get Prop var [handle]} */ 
               hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
               ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
               cSourceName.assign(ghProp.unwrapBufferField().value());
               ghProp.assign(handle.fromString(entry(1, hSource.unwrapADMData().getADMData(), chr(1))));
               ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
               cObjectName.assign(ghProp.unwrapBufferField().value());
               
               /* Get the handle of the actual Source if the DataSource is an SBO */
               if (_and(and(notUnknown(cSourceName), () -> isNotEqual(cSourceName, "")), () -> isNotEqual(cSourceName, cObjectName)))
               {
                  hSource /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "DataObjectHandle", hSource, "I", cSourceName));
               }
               
               /* {get Prop var [handle]} */
               cQueryPos.assign(ControlFlowOps.invokeDynamicFunctionIn(character.class, "getQueryPosition", hSource));
               lok.assign(and(isEqual(lookup(cQueryPos, concat(pcRequestedBatch_1, "Record,OnlyRecord")), 0), () -> or(isEqual(lookup(valueOf(hRowObject.unwrapBuffer().rowID()), entry(1, cRowident), ";"), 0), () -> isUnknown(ControlFlowOps.invokeDynamicFunctionIn(concat("get", pcRequestedBatch_1, "RowNum"), hSource)))));
            }
            else
            {
               lok.assign(isEqual(lookup(valueOf(hRowObject.unwrapBuffer().rowID()), entry(1, cRowident), ";"), 0));
            }
         }
         returnNormal(lok);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character rowVisible(final character _pcRowids_2, final handle _phQryBuffer_1)
   {
      character pcRowids_2 = TypeFactory.initInput(_pcRowids_2);
      handle phQryBuffer_1 = TypeFactory.initInput(_phQryBuffer_1);
      /*------------------------------------------------------------------------------
        Purpose:    Determines whether the currently visible records in the browse 
                    are either the first or last records in the batch.
        Parameters: pcRowids - comma separated list of ROWIDs that are currently 
                    visible.
      ------------------------------------------------------------------------------*/
      handle hBuffer = TypeFactory.handle();
      handle hQuery = TypeFactory.handle();
      character cReturn = TypeFactory.character("NONE");
      integer iFirst = TypeFactory.integer();
      integer iLast = TypeFactory.integer();
      character cLastRow = TypeFactory.character();
      
      return function("rowVisible", character.class, new Block((Body) () -> 
      {
         /* Just in case this function gets called without knowing the correct
              buffer */
         if (_or(not(phQryBuffer_1.isValid()), () -> isNotEqual(phQryBuffer_1.unwrapType().getResourceType(), "BUFFER")))
         {
            returnNormal(cReturn);
         }
         
         /* Define a buffer on the query buffer and a query based on the new buffer.
              We cannot reposition the existing query as it has a browse based on it. */
         RecordBuffer.create(hBuffer, phQryBuffer_1);
         QueryWrapper.createQuery(hQuery);
         /* Add the buffer to the query, prepare the query and open the query */
         hQuery.unwrapQuery().addBuffer(hBuffer);
         hQuery.unwrapQuery().prepare(concat("FOR EACH ", hBuffer.unwrap().name()));
         hQuery.unwrapQuery().queryOpen();
         /* Get the last record in the query */
         hQuery.unwrapQuery().getLast();
         
         /* Check if the last record's ROWID is the in the list of ROWIDs */
         if ((matchesList(pcRowids_2, valueOf(hBuffer.unwrapBuffer().rowID()))).booleanValue())
         {
            cLastRow.assign(valueOf(hBuffer.unwrapBuffer().rowID()));
            cReturn.assign("LAST");
         }
         
         /* See if the first record is visible. 
              Note that it is possible that both may be visible and we need to 
              deal with this. */
         hQuery.unwrapQuery().getFirst();
         
         if ((matchesList(pcRowids_2, valueOf(hBuffer.unwrapBuffer().rowID()))).booleanValue())
         {
            if (_isNotEqual(cReturn, "LAST"))
            {
               cReturn.assign("FIRST");
            }
            else
            {
               iFirst.assign(lastIndexOf(pcRowids_2, valueOf(hBuffer.unwrapBuffer().rowID())));
               iLast.assign(lastIndexOf(pcRowids_2, cLastRow));
               
               /* cReturn is already last so all we do is check if we
                        have to make it first. */
               if (_isGreaterThan(iFirst, iLast))
               {
                  cReturn.assign("FIRST");
               }
            }
         }
         
         /* If we don't have a last, see if the first record is visible */
         if (_isNotEqual(cReturn, "LAST"))
         {
         }
         
         /* Close the query and delete the objects */
         hQuery.unwrapQuery().queryClose();
         HandleOps.delete(hQuery);
         HandleOps.delete(hBuffer);
         /* Return the return value. */
         returnNormal(cReturn);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setActionEvent(final character _pcEvent_1)
   {
      character pcEvent_1 = TypeFactory.initInput(_pcEvent_1);
      /*------------------------------------------------------------------------------
        Purpose: subscribe an event to Default-action in the browse. 
      Parameter: pcEvent - event that will be published on default-action of the 
                           browse. The caller must also subscribe to the 
                           event.
         Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      
      return function("setActionEvent", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ActionEvent"));
         ghProp.unwrapBufferField().changeValue(pcEvent_1);
         registerTriggerPersistentIn(new EventList("default-action", hBrowse), "defaultAction", targetProcedure());
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setApplyActionOnExit(final logical _plApply_2)
   {
      logical plApply_2 = TypeFactory.initInput(_plApply_2);
      
      return function("setApplyActionOnExit", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyActionOnExit"));
         ghProp.unwrapBufferField().changeValue(plApply_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setApplyExitOnAction(final logical _plApply_2)
   {
      logical plApply_2 = TypeFactory.initInput(_plApply_2);
      
      return function("setApplyExitOnAction", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ApplyExitOnAction"));
         ghProp.unwrapBufferField().changeValue(plApply_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setBgcolor(final integer _piBgcolor_1)
   {
      integer piBgcolor_1 = TypeFactory.initInput(_piBgcolor_1);
      /*------------------------------------------------------------------------------
        Purpose: Sets the background color of the browse
         Params: piBGColor AS INTEGER 
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      
      return function("setBGColor", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            hBrowse.unwrapWidget().setBgColor(piBgcolor_1);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BGCOLOR"));
         ghProp.unwrapBufferField().changeValue(piBgcolor_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setCalcWidth(final logical _plCalcWidth_1)
   {
      logical plCalcWidth_1 = TypeFactory.initInput(_plCalcWidth_1);
      
      return function("setCalcWidth", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("CalcWidth"));
         ghProp.unwrapBufferField().changeValue(plCalcWidth_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColumnsMovable(final logical _plMovable_1)
   {
      logical plMovable_1 = TypeFactory.initInput(_plMovable_1);
      /*------------------------------------------------------------------------------
        Purpose:  Sets a logical value that determines whether the browser's 
                  columns are movable and if the browse object has been initialized
                  and its handle is valid, COLUMN-MOVABLE is set.
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      logical lInitialized = TypeFactory.logical();
      
      return function("setColumnsMovable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColumnsMovable"));
         ghProp.unwrapBufferField().changeValue(plMovable_1);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         if (_and(lInitialized, () -> hBrowse.isValid()))
         {
            if ((plMovable_1).booleanValue())
            {
               hBrowse.unwrapBrowse().setAllowColumnSearching(new logical(false));
            }
            
            hBrowse.unwrapBrowse().setColumnMovable(plMovable_1);
         }
         /* if initialized and browse valid */
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setColumnsSortable(final logical _plSortable_1)
   {
      logical plSortable_1 = TypeFactory.initInput(_plSortable_1);
      /*------------------------------------------------------------------------------
        Purpose:  Sets a logical value that determines whether the browser's 
                  columns are sortable and if the browse object has been initialized
                  and its handle is valid, ALLOW-COLUMN-SEARCHING is set to turn 
                  the START-SEARCH trigger on or off which is used to sort the columns.
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      logical lInitialized = TypeFactory.logical();
      
      return function("setColumnsSortable", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ColumnsSortable"));
         ghProp.unwrapBufferField().changeValue(plSortable_1);
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("BrowseHandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectInitialized"));
         lInitialized.assign(ghProp.unwrapBufferField().value());
         
         if (_and(lInitialized, () -> hBrowse.isValid()))
         {
            if ((plSortable_1).booleanValue())
            {
               hBrowse.unwrapBrowse().setColumnMovable(new logical(false));
            }
            
            hBrowse.unwrapBrowse().setAllowColumnSearching(plSortable_1);
         }
         /* if initialized and browse valid */
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDataModified(final logical _lModified_1)
   {
      logical lModified_1 = TypeFactory.initInput(_lModified_1);
      /*------------------------------------------------------------------------------
        Purpose:  SmartDataBrowser version of setDataModified.
         Params:  lModified AS LOGICAL -- TRUE or FALSE 
          Notes:  The purpose of this version is to enable/disable the search 
                  field (if defined) to disable the search capability while there 
                  are updates in progress in the SmartDataBrowser.
      ------------------------------------------------------------------------------*/
      handle hSearchField = TypeFactory.handle();
      
      return function("setDataModified", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchHandle"));
         hSearchField.assign(ghProp.unwrapBufferField().value());
         
         if (_and(hSearchField.isValid(), () -> notUnknown(lModified_1)))
         {
            hSearchField.unwrapSensitive().setSensitive(not(lModified_1));
         }
         
         returnNormal((logical) ControlFlowOps.runSuper("setDataModified", lModified_1));
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setDefaultColumnData(final character _pcData_2)
   {
      character pcData_2 = TypeFactory.initInput(_pcData_2);
      
      return function("setDefaultColumnData", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DefaultColumnData"));
         ghProp.unwrapBufferField().changeValue(pcData_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFetchOnReposToEnd(final logical _plFetchOnRepos_1)
   {
      logical plFetchOnRepos_1 = TypeFactory.initInput(_plFetchOnRepos_1);
      
      return function("setFetchOnReposToEnd", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FetchOnReposToend"));
         ghProp.unwrapBufferField().changeValue(plFetchOnRepos_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFgcolor(final integer _piFgcolor_1)
   {
      integer piFgcolor_1 = TypeFactory.initInput(_piFgcolor_1);
      /*------------------------------------------------------------------------------
        Purpose: Sets the foreground color of the browse
         Params: piFGColor AS INTEGER 
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      
      return function("setFGColor", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            hBrowse.unwrapWidget().setFgColor(piFgcolor_1);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FGCOLOR"));
         ghProp.unwrapBufferField().changeValue(piFgcolor_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setFolderWindowToLaunch(final character _pcTemp_1)
   {
      character pcTemp_1 = TypeFactory.initInput(_pcTemp_1);
      
      return function("setFolderWindowToLaunch", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("FolderWindowToLaunch"));
         ghProp.unwrapBufferField().changeValue(pcTemp_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setMaxWidth(final decimal _pdMaxWidth_1)
   {
      decimal pdMaxWidth_1 = TypeFactory.initInput(_pdMaxWidth_1);
      
      return function("setMaxWidth", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MaxWidth"));
         ghProp.unwrapBufferField().changeValue(pdMaxWidth_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setMovableHandle(final handle _phMovable_1)
   {
      handle phMovable_1 = TypeFactory.initInput(_phMovable_1);
      
      return function("setMovableHandle", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("MovableHandle"));
         ghProp.unwrapBufferField().changeValue(phMovable_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setNumDown(final integer _piNumDown_1)
   {
      integer piNumDown_1 = TypeFactory.initInput(_piNumDown_1);
      
      return function("setNumDown", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("NumDown"));
         ghProp.unwrapBufferField().changeValue(piNumDown_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setPopupActive(final logical _plActive_1)
   {
      logical plActive_1 = TypeFactory.initInput(_plActive_1);
      
      return function("setPopupActive", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("PopupActive"));
         ghProp.unwrapBufferField().changeValue(plActive_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setQueryRowObject(final handle _phQueryRowObject_1)
   {
      handle phQueryRowObject_1 = TypeFactory.initInput(_phQueryRowObject_1);
      
      return function("setQueryRowObject", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("QueryRowObject"));
         ghProp.unwrapBufferField().changeValue(phQueryRowObject_1);
         returnNormal(false);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSavedColumnData(final character _pcData_2)
   {
      character pcData_2 = TypeFactory.initInput(_pcData_2);
      
      return function("setSavedColumnData", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SavedColumnData"));
         ghProp.unwrapBufferField().changeValue(pcData_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setScrollRemote(final logical _plScrollRemote_1)
   {
      logical plScrollRemote_1 = TypeFactory.initInput(_plScrollRemote_1);
      
      return function("setScrollRemote", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ScrollRemote"));
         ghProp.unwrapBufferField().changeValue(plScrollRemote_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSearchField(final character _pcField_1)
   {
      character pcField_1 = TypeFactory.initInput(_pcField_1);
      
      return function("setSearchField", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SearchField"));
         ghProp.unwrapBufferField().changeValue(pcField_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSeparatorFgcolor(final integer _piSeparatorFgcolor_1)
   {
      integer piSeparatorFgcolor_1 = TypeFactory.initInput(_piSeparatorFgcolor_1);
      /*------------------------------------------------------------------------------
         Purpose: Sets the separator color of the browse
          Params: piSeparatorFGColor AS INTEGER 
           Notes:  
       ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      
      return function("setSeparatorFGColor", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            silent(() -> hBrowse.unwrapBrowse().setSeparatorFgColor(piSeparatorFgcolor_1));
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SeparatorFGColor"));
         ghProp.unwrapBufferField().changeValue(piSeparatorFgcolor_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSeparators(final logical _plSeparators_1)
   {
      logical plSeparators_1 = TypeFactory.initInput(_plSeparators_1);
      /*------------------------------------------------------------------------------
        Purpose:  
          Notes:  
      ------------------------------------------------------------------------------*/
      handle hBrowse = TypeFactory.handle();
      
      return function("setSeparators", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("Browsehandle"));
         hBrowse.assign(ghProp.unwrapBufferField().value());
         
         if ((hBrowse.isValid()).booleanValue())
         {
            hBrowse.unwrapBrowse().setSeparators(plSeparators_1);
         }
         
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SEPARATORS"));
         ghProp.unwrapBufferField().changeValue(plSeparators_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSort(final character _pcColumnName_1)
   {
      character pcColumnName_1 = TypeFactory.initInput(_pcColumnName_1);
      /*------------------------------------------------------------------------------
        Purpose:    Set sortcolumn and sort the data source accordingly       
        Parameters: pcColumnName -  Name of browse column.
        Notes:      
      ------------------------------------------------------------------------------*/
      handle hSource = TypeFactory.handle();
      character cSourceNames = TypeFactory.character();
      logical lSort = TypeFactory.logical();
      character cSourceName = TypeFactory.character();
      
      return function("setSort", logical.class, new Block((Body) () -> 
      {
         /* {get Prop var [handle]} */ 
         hSource.assign(ControlFlowOps.invokeDynamicFunctionIn(handle.class, "getDataSource", targetProcedure()));
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("DataSourceNames"));
         cSourceNames.assign(ghProp.unwrapBufferField().value());
         ghProp.assign(handle.fromString(entry(1, hSource.unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("ObjectName"));
         cSourceName.assign(ghProp.unwrapBufferField().value());
         
         /* The SBO does not have a sort API, so if SourceNames is defined we need to 
              get the handle of the actual Source */
         if (_and(and(notUnknown(cSourceNames), () -> isNotEqual(cSourceNames, "")), () -> isNotEqual(cSourceNames, cSourceName)))
         {
            hSource /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(handle.class, "DataObjectHandle", hSource, "I", cSourceNames));
         }
         
         if ((hSource.isValid()).booleanValue())
         {
            ControlFlowOps.invokeInWithMode("changeCursor", targetProcedure(), "I", new character("WAIT"));
            /* Qualify with RowObject name and specify to toggle current sort */
            pcColumnName_1.assign(concat("RowObject.", pcColumnName_1, " TOGGLE"));
            lSort /* {fnarg <func-name> <arg> [handle] */.assign(ControlFlowOps.invokeDynamicFunctionInWithMode(logical.class, "resortQuery", hSource, "I", pcColumnName_1));
            ControlFlowOps.invokeInWithMode("changeCursor", targetProcedure(), "I", new character(""));
         }
         
         /* valid-handle(hSource)*/
         returnNormal(lSort);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setSortableHandle(final handle _phSortable_1)
   {
      handle phSortable_1 = TypeFactory.initInput(_phSortable_1);
      
      return function("setSortableHandle", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("SortableHandle"));
         ghProp.unwrapBufferField().changeValue(phSortable_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setTooltip(final character _pcTooltip_1)
   {
      character pcTooltip_1 = TypeFactory.initInput(_pcTooltip_1);
      
      return function("setTooltip", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("TOOLTIP"));
         ghProp.unwrapBufferField().changeValue(pcTooltip_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setVisibleRowids(final character _pcRowids_2)
   {
      character pcRowids_2 = TypeFactory.initInput(_pcRowids_2);
      
      return function("setVisibleRowids", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowids"));
         ghProp.unwrapBufferField().changeValue(pcRowids_2);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public logical setVisibleRowReset(final logical _plReset_1)
   {
      logical plReset_1 = TypeFactory.initInput(_plReset_1);
      
      return function("setVisibleRowReset", logical.class, new Block((Body) () -> 
      {
         ghProp.assign(handle.fromString(entry(1, targetProcedure().unwrapADMData().getADMData(), chr(1))));
         ghProp.assign(ghProp.unwrapBuffer().bufferField("VisibleRowReset"));
         ghProp.unwrapBufferField().changeValue(plReset_1);
         returnNormal(true);
      }));
   }

   /**
    * _UIB-CODE-BLOCK-END
    */
   public character stripCalcs(final character _cClause_1)
   {
      character cClause_1 = TypeFactory.initInput(_cClause_1);
      /*------------------------------------------------------------------------------
        Purpose:  Strips the expression portion of expression @ variable 
          Notes:  
      ------------------------------------------------------------------------------*/
      character cPart = TypeFactory.character();
      character cNewClause = TypeFactory.character();
      character cNewClause2 = TypeFactory.character();
      integer iCount = TypeFactory.integer();
      integer iPos = TypeFactory.integer();
      
      return function("stripCalcs", character.class, new Block((Body) () -> 
      {
         /* If there are no "@" in the clause, don't change it. */
         if (_isLessThanOrEqual(numEntries(cClause_1, "@"), 1))
         {
            returnNormal(cClause_1);
         }
         
         loopLabel23:
         for (iCount.assign(1); _isLessThanOrEqual(iCount, numEntries(cClause_1, "@")); iCount.increment())
         {
            cPart.assign(entry(iCount, cClause_1, "@"));
            /* Find the position of the first "(" in the clause */
            iPos.assign(indexOf(cPart, "("));
            
            /* If there is an "(", replace the rest of the string with a <calc>" */
            if (_isGreaterThan(iPos, 0))
            {
               cPart.assign(concat(substring(cPart, 1, minus(iPos, 1)), "<calc>"));
            }
            
            /* Add this part to the clause */
            cNewClause.assign(concat(cNewClause, cPart));
         }
         
         loopLabel24:
         for (iCount.assign(1); _isLessThanOrEqual(iCount, numEntries(cNewClause, " ")); iCount.increment())
         {
            /* Add the entry to the clause */
            cNewClause2.assign(concat(cNewClause2, entry(iCount, cNewClause, " "), " "));
            
            /* If this entry is a <calc> constant */
            if (_isEqual(entry(iCount, cNewClause, " "), "<calc>"))
            {
               iCount.assign(plus(iCount, 1));
            }
         }
         
         returnNormal(trim(cNewClause2));
      }));
   }

   public character fwdGetAdmversion()
   {
      return function("fwdGetADMVersion", character.class, new Block((Body) () -> 
      {
         returnNormal("ADM2.2");
      }));
   }
}
